Package: kardl
Type: Package
Title: Make Symmetric and Asymmetric ARDL Estimations
Version: 2.0.5
Authors@R: 
  c(person(given = "Huseyin",
           family = "karamelikli",
           role = c("aut", "cre"),
           email = "hakperest@gmail.com",
           comment = c(ORCID = "0000-0001-7622-0972")),
    person(given = "Huseyin Utku",
           family = "Demir",
           role = "aut",
           comment = c(ORCID = "0000-0002-9140-0362")) 
           )
Maintainer: Huseyin karamelikli <hakperest@gmail.com>
Description: Implements estimation procedures for Autoregressive Distributed Lag (ARDL) 
    and Nonlinear ARDL (NARDL) models, which allow researchers to investigate both 
    short- and long-run relationships in time series data under mixed orders of integration. 
    The package supports simultaneous modeling of symmetric and asymmetric regressors, 
    flexible treatment of short-run and long-run asymmetries, and automated equation handling. 
    It includes several cointegration testing approaches such as the Pesaran-Shin-Smith F 
    and t bounds tests, and narayan test. 
    Methodological foundations are provided in Pesaran, Shin, and Smith (2001) 
    <doi:10.1016/S0304-4076(01)00049-5> and Shin, Yu, and Greenwood-Nimmo (2014, ISBN:9780123855079).
License: GPL-3
Encoding: UTF-8
Imports: stats, msm, lmtest,lifecycle, nlWaldTest, car, ggplot2, utils
Suggests: knitr, rmarkdown, magrittr, rlang, tidyr, dplyr, MASS,
        sandwich, testthat (>= 3.0.0)
NeedsCompilation: no
VignetteBuilder: knitr
Packaged: 2026-07-21 13:34:32 UTC; huseyin
Author: Huseyin karamelikli [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7622-0972>),
  Huseyin Utku Demir [aut] (ORCID:
    <https://orcid.org/0000-0002-9140-0362>)
Depends: R (>= 3.5.0)
Config/testthat/edition: 3
URL: https://karamelikli.github.io/kardl/,
        https://github.com/karamelikli/kardl
BugReports: https://github.com/karamelikli/kardl/issues
RoxygenNote: 7.3.1
Config/roxygen2/version: 8.0.0
Repository: CRAN
Date/Publication: 2026-07-22 09:10:02 UTC
Built: R 4.6.0; ; 2026-07-22 13:45:14 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(bootstrap,default)
S3method(bootstrap,kardl_lm)
S3method(bootstrap,kardl_longrun)
S3method(kardl_extract,default)
S3method(kardl_extract,kardl_boot)
S3method(kardl_extract,kardl_lm)
S3method(kardl_extract,kardl_mplier)
S3method(kardl_extract,kardl_symmetric)
S3method(kardl_extract,kardl_test)
S3method(kardl_extract,kardl_test_summary)
S3method(kardl_longrun,default)
S3method(kardl_longrun,kardl_lm)
S3method(makemodel,grid)
S3method(makemodel,grid_custom)
S3method(makemodel,quick)
S3method(makemodel,user)
S3method(model_criterion,default)
S3method(model_criterion,lm)
S3method(mplier,default)
S3method(mplier,kardl_lm)
S3method(mplier,kardl_longrun)
S3method(narayan,default)
S3method(narayan,kardl_lm)
S3method(narayan,kardl_longrun)
S3method(plot,kardl_long_run)
S3method(plot,kardl_mplier)
S3method(predict,kardl_lm)
S3method(print,kardl_boot)
S3method(print,kardl_hypotheses)
S3method(print,kardl_lm)
S3method(print,kardl_long_run)
S3method(print,kardl_mplier)
S3method(print,kardl_symmetric)
S3method(print,kardl_test_summary)
S3method(print,summary.kardl_boot)
S3method(print,summary.kardl_mplier)
S3method(print,summary.kardl_symmetric)
S3method(print,summary_kardl_longrun)
S3method(pssf,default)
S3method(pssf,kardl_lm)
S3method(pssf,kardl_longrun)
S3method(psst,default)
S3method(psst,kardl_lm)
S3method(psst,kardl_longrun)
S3method(summary,kardl_boot)
S3method(summary,kardl_longrun)
S3method(summary,kardl_mplier)
S3method(summary,kardl_symmetric)
S3method(summary,kardl_test)
S3method(symmetrytest,default)
S3method(symmetrytest,kardl_lm)
S3method(symmetrytest,kardl_longrun)
export(bootstrap)
export(ecm)
export(kardl)
export(kardl_extract)
export(kardl_get)
export(kardl_longrun)
export(kardl_reset)
export(kardl_set)
export(lmerge)
export(model_criterion)
export(mplier)
export(narayan)
export(parse_formula_vars)
export(parse_model_specs)
export(pssf)
export(psst)
export(symmetrytest)
import(ggplot2)
import(lifecycle)
import(lmtest)
import(msm)
import(nlWaldTest)
import(stats)
importFrom(car,linearHypothesis)
importFrom(lifecycle,deprecated)
importFrom(lmtest,coeftest)
importFrom(msm,deltamethod)
importFrom(stats,vcov)
importFrom(utils,capture.output)
importFrom(utils,flush.console)
importFrom(utils,methods)
importFrom(utils,tail)

# KARDL <img src="man/figures/KARDL.png" align="right"/>

[![R-CMD-check](https://github.com/karamelikli/kardl/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/karamelikli/kardl/actions/workflows/R-CMD-check.yaml) 
[![GitHub version](https://img.shields.io/github/v/release/karamelikli/kardl)](https://github.com/karamelikli/kardl/releases) 
[![License: GPL-3](https://img.shields.io/badge/license-GPL--3-blue.svg)](https://opensource.org/licenses/GPL-3.0) 
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/kardl)](https://cran.r-project.org/package=kardl) 
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)

## Introduction

The `kardl` package is an R tool for estimating symmetric and asymmetric Autoregressive Distributed Lag (ARDL) and Nonlinear ARDL (NARDL) models, designed for econometricians and researchers analyzing cointegration and dynamic relationships in time series data. It offers flexible model specifications, allowing users to include deterministic variables, asymmetric effects for short- and long-run dynamics, and trend components. The package supports customizable lag structures, model selection criteria (AIC, BIC, AICc, HQ), and parallel processing for computational efficiency. Key features include:

- **Flexible Formula Specification**: Use `Asymmetric()`, `Lasymmetric()`, and `Sasymmetric()` to model asymmetric effects in short- and long-run dynamics, and `deterministic()` for dummy variables.
- **Lag Optimization**: Choose between automatic lag selection (`"quick"`, `"grid"`, `"grid_custom"`) or user-defined lags.
- **Dynamic Analysis**: Compute long-run coefficients, perform cointegration tests (PSS F, PSS t, Narayan), and ECM estimation.

This vignette demonstrates how to use the `kardl()` function to estimate an asymmetric ARDL model, perform diagnostic tests, and visualize results, using economic data from Turkey. 

## Installation

`kardl` in R can easily be installed from its CRAN repository:

``` r

install.packages("kardl")
library(kardl)
```

Alternatively, you can use the `devtools` package to load directly from GitHub:

``` r

# Install required packages
install.packages(c("stats", "msm", "lmtest", "nlWaldTest", "car", "strucchange", "utils"))
# Install kardl from GitHub
install.packages("devtools")
devtools::install_github("karamelikli/kardl")
```

Load the package:

``` r

library(kardl)
```

## Unique Features and Methodological Contributions

The `kardl` package implements several methodological extensions and improvements for ARDL/NARDL modelling that go beyond standard implementations available in R and other software:

- **different_asym_lag** option: Enables different lag orders for the positive and negative partial sum decompositions in NARDL models. This provides greater flexibility than the common symmetric lag restriction used in most existing packages.
- **Narayan cointegration test** (`narayan()`): A dedicated small-sample bounds test (Narayan, 2005) with automatic handling of critical values for cases II–V. While the test exists in the literature, its seamless integration into a full ARDL/NARDL workflow is unique in R.
- **Symmetry tests** (`symmetrytest()`): Comprehensive Wald tests for both short-run and long-run symmetry in NARDL models.
- **Bootstrap confidence intervals for dynamic multipliers**: Robust uncertainty quantification around short-run, long-run, and impact multipliers through resampling methods, fully integrated with asymmetric multiplier estimation.
- Additional advanced capabilities include highly flexible formula parsing for mixed symmetric/asymmetric regressors, multiple model selection criteria (AIC, BIC, AICc, HQ), parallel processing support, and extensive post-estimation tools tailored for asymmetric analysis.

These features make `kardl` particularly suitable for researchers needing fine-grained control over asymmetric dynamics and small-sample inference.


## Estimating an asymmetric ARDL Model

This example estimates an asymmetric ARDL model to analyze the impact of petrol prices and driving patterns on road fatalities in the UK, using the built-in `Seatbelts` dataset with variables for DriversKilled, PetrolPrice, drivers, kms, and a seatbelt law dummy variable.

### Step 1: Data Preparation

The `Seatbelts` dataset contains monthly data on road casualties in Great Britain from 1969 to 1984. It is a built-in R time series dataset that can be used directly.

Note: The `Seatbelts` dataset is a built-in R dataset included in the `datasets` package.
The data can be accessed directly without any conversion.


### Step 2: Define the Model Formula

We define the model formula using R's formula syntax, incorporating asymmetric effects and deterministic variables. We use `asymmetric()` for variables with both short- and long-run asymmetry, `lasymmetric()` for long-run asymmetry, `sasymmetric()` for short-run asymmetry, and `deterministic()` for fixed dummy variables. The `trend` term includes a linear time trend in the model.

``` r
# Define the model formula
my_formula <- DriversKilled ~ PetrolPrice + drivers + asymmetric(PetrolPrice + drivers) + deterministic(law) +
  trend
```
Indeed, the formula syntax is flexible, allowing for various combinations of asymmetric and deterministic variables. The following variations of the formula are equivalent and will yield the same model specification:

``` r
same_formula <- y ~ asymmetric(x1) +
  sasymmetric(x2 + x3) +
  lasymmetric(x4 + x5) +
  deterministic(dummy1) + trend
same_formula <- y ~ asymmetric(x1) +
  sasymmetric(x2 + x3) +
  lasymmetric(x4 + x5) +
  deterministic(dummy1) + trend
same_formula <- y ~ asym(x1) + sasym(x2 + x3) + lasym(x4 + x5) +
  det(dummy1) + trend
same_formula <- y ~ a(x1) + s(x2 + x3) + l(x4 + x5) + d(dummy1) + trend
```




### Step 3: Model Estimation

We estimate the ARDL model using different `mode` settings to demonstrate flexibility in lag selection. The `kardl()` function supports various modes: `"grid"`, `"grid_custom"`, `"quick"`, or a user-defined lag vector.

#### Using `mode = "grid"`
The `"grid"` mode evaluates all lag combinations up to `maxlag` and provides console feedback.

``` r
# Set model options
kardl_set(criterion = "BIC", different_asym_lag = TRUE, data = Seatbelts)
# Estimate model with grid mode
kardl_model <- kardl(
  data = Seatbelts, formula = my_formula,
  maxlag = 4, mode = "grid"
)
# View results
kardl_model
```
Summary of the model provides detailed information about the estimated coefficients, standard errors, t-values, and significance levels.

``` r
# Display model summary
summary(kardl_model)
```


#### Using User-Defined Lags

Specify custom lags to bypass automatic lag selection:

``` r
kardl_model2 <- kardl(
  data = Seatbelts, my_formula,
  mode = c(2, 1, 1, 3, 0)
)
# View results
kardl_extract(kardl_model2,"opt_lag")

```
``` r
# Display model summary
summary(kardl_model2)
```



#### Using All Variables
Use the `.` operator to include all variables except the dependent variable:

``` r
kardl_set(data = Seatbelts)
kardl(formula = DriversKilled ~ . + deterministic(law), mode = "grid")
```

#### Visualizing Lag Criteria

The `lag_criteria` component contains lag combinations and their criterion values. We visualize these to compare model selection criteria (AIC, BIC, HQ).

``` r
library(dplyr)
library(tidyr)
library(ggplot2)
# Convert lag_criteria to a data frame
lag_criteria <- as.data.frame(kardl_extract(kardl_model, "lag_criteria"))
colnames(lag_criteria) <- c("lag", "AIC", "BIC", "AICc", "HQ")
lag_criteria <- lag_criteria |> mutate(across(c(AIC, BIC, HQ), as.numeric))

# Pivot to long format
lag_criteria_long <- lag_criteria |>
  select(-AICc) |>
  pivot_longer(
    cols = c(AIC, BIC, HQ),
    names_to = "Criteria",
    values_to = "Value"
  )

# Find minimum values
min_values <- lag_criteria_long |>
  group_by(Criteria) |>
  slice_min(order_by = Value) |>
  ungroup()

# Plot
ggplot(
  lag_criteria_long,
  aes(x = lag, y = Value, color = Criteria, group = Criteria)
) +
  geom_line() +
  geom_point(
    data = min_values, aes(x = lag, y = Value),
    color = "red", size = 3, shape = 8
  ) +
  geom_text(
    data = min_values, aes(x = lag, y = Value, label = lag),
    vjust = 1.5, color = "black", size = 3.5
  ) +
  labs(
    title = "Lag Criteria Comparison",
    x = "Lag Configuration",
    y = "Criteria Value"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))
```

#### Error Correction Model (ECM) Estimation

The `ecm()` function estimates a Restricted ECM for cointegration testing. We specify the same formula and lag structure as in the ARDL model.
``` r
ecm_model <- ecm(
  data = Seatbelts, formula = my_formula,
  maxlag = 4, mode = "grid_custom"
)
# View results
summary(ecm_model)
```


### Step 4: Long-Run Coefficients

We calculate long-run coefficients using `kardl_longrun()`, which standardizes coefficients by dividing them by the negative of the dependent variable’s long-run parameter.

``` r
# Long-run coefficients
my_long <- kardl_longrun(kardl_model)
my_long
```

The `summary()` function provides detailed information about the long-run coefficients, including standard errors, t-values, and significance levels.

``` r
# Summary of long-run coefficients
summary(my_long)
```


### Step 5: Symmetry Test

The `symmetrytest()` function performs Wald tests to assess short- and long-run asymmetry in the model.

``` r
ast <- Seatbelts |>
  kardl(
    DriversKilled ~ PetrolPrice + drivers + asymmetric(PetrolPrice + drivers) +
      deterministic(law) + trend,
    mode = c(1, 2, 3, 0, 1),
    data = _
  ) |>
  symmetrytest()
ast
```

Summary of the symmetry test provides detailed results for both long-run and short-run asymmetry tests, including F-values, p-values, hypotheses, and test decisions.

``` r
# Summary of symmetry test
summary(ast)
```



### Step 6: Cointegration Tests

We perform cointegration tests to assess long-term relationships using `pssf()`, `psst()`, and `narayan()`.

#### PSS F Bound Test

The `pssf()` function tests for cointegration using the Pesaran, Shin, and Smith F Bound test.

``` r
test_result <- kardl_model |> pssf(case = 3, signif_level = "0.05")
test_result
```

Summary of the PSS F Bound test provides detailed information about the test statistic, critical values, hypotheses, and decision regarding cointegration.

``` r
summary(test_result)
```

#### PSS t Bound Test

The `psst()` function tests the significance of the lagged dependent variable’s coefficient.

``` r
test_result <- kardl_model |> psst(case = 3, signif_level = "0.05")
test_result
```
Summary of the PSS t Bound test provides detailed information about the test statistic, critical values, hypotheses, and decision regarding cointegration.
``` r
summary(test_result)
```

#### Narayan Test

The `narayan()` function is tailored for small sample sizes. It tests for cointegration using critical values optimized for small samples.

``` r
test_result <- kardl_model |> narayan(case = 3, signif_level = "0.05")
test_result
```
Summary of the Narayan test provides detailed information about the test statistic, critical values, hypotheses, and decision regarding cointegration.

``` r
summary(test_result)
```



### Step 7: Dynamic Multipliers

The `mplier()` function calculates dynamic multipliers for the model, showing how changes in independent variables affect the dependent variable over time.
``` r
multipliers <- kardl_model |> mplier()
# View multipliers of the model
head(kardl_extract(multipliers, "multipliers"))
# View long-run multipliers
kardl_extract(multipliers, "omega")
# View short-run multipliers
head(kardl_extract(multipliers, "lambda"))
```

Plotting dynamic multipliers for specific variables can be done using the `plot()` function, which visualizes the response of the dependent variable to changes in independent variables over time.
``` r
plot(multipliers, variables = c("PetrolPrice", "drivers"))
```

 
 To handle a large number of variables, you can specify a subset of variables to plot or use `variables = "all"` to visualize all dynamic multipliers.
 
 Bootstrap confidence intervals for dynamic multipliers can be calculated using the `bootstrap()` function, which provides robust estimates of uncertainty around the multipliers.
``` r
bootstrap_results <- kardl_model |>
  bootstrap(horizon = 12, replications = 10)
# View bootstrap summary
summary(bootstrap_results)
```

Visualize bootstrap results for specific variables to understand the variability and confidence intervals of the dynamic multipliers.

``` r
plot(bootstrap_results, variables = "PetrolPrice")
```
 


### Step 8: Customizing asymmetric Variables

We demonstrate how to customize prefixes and suffixes for asymmetric variables using `kardl_set()`.

``` r
# Set custom prefixes and suffixes
kardl_reset()
kardl_set(asym_prefix = c("asyP_", "asyN_"), asym_suffix = c("_PP", "_NN"))
kardl_custom <- kardl(data = Seatbelts, my_formula)
kardl_custom
```

## Key Functions and Parameters

- **`kardl(data, model, maxlag, mode, ...)`**:
  - `data`: A time series dataset (e.g., a data frame with DriversKilled, PetrolPrice, drivers).
  - `formula`: A formula specifying the long-run equation, e.g., `y ~ x + z + asymmetric(z) + lasymmetric(x2 + x3) + sasymmetric(x3 + x4) + deterministic(dummy1 + dummy2) + trend`. Supports:
    - `asymmetric()`: asymmetric effects for both short- and long-run dynamics.
    - `lasymmetric()`: Long-run asymmetric variables.
    - `sasymmetric()`: Short-run asymmetric variables.
    - `deterministic()`: Fixed dummy variables.
    - `trend`: Linear time trend.
  - `maxlag`: Maximum number of lags (default: 4). Use smaller values (e.g., 2) for small datasets, larger values (e.g., 8) for long-term dependencies.
  - `mode`: Estimation mode:
    - `"quick"`: Verbose output for interactive use.
    - `"grid"`: Verbose output with lag optimization.
    - `"grid_custom"`: Silent, efficient execution.
    - User-defined vector (e.g., `c(1, 2, 4, 5)` or `c(DriversKilled = 2, PetrolPrice_POS = 3, PetrolPrice_NEG = 1, drivers = 3)`).
  - Returns a list with components: `inputs`, `finalModel`, `start_time`, `end_time`, `properLag`, `time_span`, `opt_lag`, `lag_criteria`, `type` ("kardlmodel").

- **`kardl_set(...)`**: Configures options like `criterion` (AIC, BIC, AICc, HQ), `different_asym_lag`, `asym_prefix`, `Sasymuffix`, `short_coef`, and `long_coef`. Use `kardl_get()` to retrieve settings and `kardl_reset()` to restore defaults.

- **`kardl_longrun(model)`**: Calculates standardized long-run coefficients, returning `type` ("kardl_longrun"), `coef`, `delta_se`, `results`, and `starsDesc`.

- **`symmetrytest(model)`**: Performs Wald tests for short- and long-run asymmetry, returning `Lhypotheses`, `Lwald`, `Shypotheses`, `Swald`, and `type` ("symmetrytest").

- **`pssf(model, case, signif_level)`**: Performs the Pesaran, Shin, and Smith F Bound test for cointegration, supporting cases 1–5 and significance levels ("auto", 0.01, 0.025, 0.05, 0.1, 0.10).

- **`psst(model, case, signif_level)`**: Performs the PSS t Bound test, focusing on the lagged dependent variable’s coefficient.

- **`narayan(model, case, signif_level)`**: Conducts the Narayan test for cointegration, optimized for small samples (cases 2–5).

- **`ecm(data, model, maxlag, mode,  ...)`**: Conducts the Restricted ECM test for cointegration, with similar parameters to `kardl()` and case/significance level options.

For detailed documentation, use `?kardl`, `?kardl_set`, `?kardl_longrun`, `?symmetrytest`, `?pssf`, `?psst`, `?narayan`, or `?ecm`.

# Options

The options for the KARDL package are set by the `kardl_set()` function in R. The default values are set in the `kardl_set` list. You can change the options by using the `kardl_set()` function with the desired parameters. The following options are available:

| Option Name | Default | Description |
|------------------------|------------------------|------------------------|
| formula | NULL | The formula to be used for the model estimation |
| data | NULL | The data to be used for the model estimation |
| maxlag | 4 | The maximum number of lags to be considered for the model estimation |
| mode | "quick" | The mode of the model estimation, can be "quick", "grid", "grid_custom" or a user-defined vector |
| criterion | "AIC" | The criterion for model selection, can be "AIC", "BIC", "HQ" or a user-defined function |
| different_asym_lag | FALSE | If TRUE, the asymmetry lags will be different for positive and negative shocks |
| asym_prefix | c() | Prefix for asymmetry variables, default is empty |
| asym_suffix | c("\_POS", "\_NEG") | Suffix for asymmetry variables, default is "\_POS" and "\_NEG" |
| long_coef | "L{lag}.{varName}" | Prefix for long-run coefficients, default is "L1." |
| short_coef | "L{lag}.d.{varName}" | Prefix for short-run coefficients, default is "L1.d." |
| batch | "1/1" | Batch size for parallel processing, default is "1/1" |
| print_wrap | NULL | If not NULL, the output will be wrapped to the specified number of characters |

The details of the options are as follows:

### 1. model

`formula` is a formula object specifying the model to be estimated. The default value is `NULL`, which means that the user must provide a model formula when calling the `kardl()` function.

#### Details

The `model` parameter defines the structure of the ARDL or NARDL model to be estimated. It should include the dependent variable on the left side of the formula and the independent variables, asymmetric components, deterministic variables, and trend (if applicable) on the right side. The formula can include: - `Asymmetric()`: To specify variables with asymmetric effects in both short- and long -run dynamics. - `Lasymmetric()`: To specify variables with asymmetric effects only in the long-run -dynamics. - `Sasymmetric()`: To specify variables with asymmetric effects only in the short-run -dynamics. - `Deterministic()`: To include fixed dummy variables (e.g., seasonal d -ummies, event dummies). - `trend`: To include a linear time trend in the model. When constructing the `model` formula, ensure that: - All variables used in the formula are present in the `data` provided. - The formula is syntactically correct and follows R's formula conventions. - The use of asymmetric and deterministic functions is appropriate for the research question and data characteristics.

### 2. data

`data` is a data frame or time series object containing the variables to be used in the model estimation. The default value is `NULL`, which means that the user must provide a dataset when calling the `kardl()` function.

#### Details

The `data` parameter is essential for the `kardl()` function to perform model estimation. It should contain all the variables specified in the model formula, including the dependent variable and any independent variables, asymmetric components, and deterministic variables defined in the formula. The trend will be generated automatically if specified in the formula. Input data can be in the form of a data frame, tibble, or time series object (e.g., `ts`, `xts`, `zoo`).

When providing the `data`, ensure that: - The dataset is clean and free of missing values for the variables used in the model. - The variables are appropriately formatted (e.g., numeric for continuous variables). - The time series data is ordered correctly, especially if the analysis involves lagged variables.

### 3. maxlag

`maxlag` is an integer value specifying the maximum number of lags to be considered for the model estimation. The default value is `4`.

#### Details

The `maxlag` parameter sets the upper limit for the number of lags that the `kardl()` function will evaluate when optimizing the lag structure of the model. This is particularly important when using modes like `"grid"` or `"grid_custom"`, where the function systematically tests different lag combinations up to the specified maximum. When choosing a value for `maxlag`, consider the following: - **Data Frequency**: For monthly data, a `maxlag` of 4 is often sufficient to capture short-term dynamics. For quarterly data, a lower `maxlag` ( e.g., 2) may be appropriate, while for daily data, a higher `maxlag` (e.g., 8 or more) might be necessary. - **Sample Size**: A larger `maxlag` increases the number of parameters to estimate, which can be problematic with small sample sizes. Ensure that the sample size is adequate to support the number of lags being considered. - **Model Complexity**: Higher `maxlag` values lead to more complex models, which may overfit the data. Balance the need for capturing dynamics with the risk of overfitting. - **Computational Resources**: Evaluating a large number of lag combinations can be computationally intensive. Consider the available resources and time constraints when setting `maxlag`.

### 4. mode

`mode` is a character string or numeric vector specifying the mode of the model estimation. The default value is `"quick"`. The available options are:\
- **"quick"**: This mode provides a fast estimation of the model without optimizing the lags. It is suitable for initial explorations or when the user has a predefined lag structure. - **"grid"**: This mode performs a grid search over all possible lag combinations up to the specified `maxlag`. It provides verbose output, including the lag criteria for each combination, and is useful for thorough lag optimization. - **"grid_custom"**: Similar to `"grid"`, but with silent execution. It is more efficient for large datasets or when the user wants to avoid console output during the lag optimization process. - **User-defined vector**: The user can specify a custom lag structure by providing a numeric vector (e.g., `c(1, 2, 4, 5)`) or a named vector (e.g., `c(DriversKilled = 2, PetrolPrice_POS = 3, PetrolPrice_NEG = 1, drivers = 3)`). This allows for complete control over the lag selection process.

#### Details

The `mode` parameter determines how the `kardl()` function approaches the estimation of the ARDL or NARDL model. Each mode has its advantages and is suited to different scenarios: - Use `"quick"` for rapid assessments when the lag structure is already known or when computational speed is a priority. - Use `"grid"` for comprehensive lag optimization, especially when the optimal lag structure is unknown. This mode is ideal for exploratory analysis and model selection. - Use `"grid_custom"` for efficient lag optimization without console output, particularly for large datasets or when running multiple models in batch mode. - Use a user-defined vector when the user has specific knowledge about the appropriate lags for each variable, allowing for tailored model specifications. When using `"grid"` or `"grid_custom"`, ensure that the `maxlag` parameter is set appropriately to balance the thoroughness of the search with computational feasibility.

### 5. criterion

`criterion` is a character string specifying the criterion to be used for selecting the optimal lags. The default value is `"AIC"`. The available options are:

- **User defined function**: For detailed information on the model selection criteria used in the methods, see the documentation for the `model_criterion` function.
- **AIC**: Akaike Information Criterion (AIC). This criterion balances model fit and complexity, favoring models with a lower AIC value. It is widely used in time series analysis and model selection.
- **BIC**: Schwarz Criterion (SC), also known as the Bayesian Information Criterion (BIC). This criterion imposes a stricter penalty for model complexity compared to AIC, often leading to simpler models when data size is large.
- **AICc**: Corrected Akaike Information Criterion. This is a modification of AIC that accounts for small sample sizes. It is more reliable than AIC when the number of observations is limited.
- **HQ**: Hannan-Quinn Criterion. This criterion is similar to AIC and BIC but uses a logarithmic penalty term that grows more slowly than BIC. It is often used in econometric applications.


#### Details

For detailed information on the model selection criteria used in the methods, see the documentation for the `model_criterion` function.

The choice of the criterion can significantly impact the selected lag length and, consequently, the performance of the model. Each criterion has its strengths and is suited to specific scenarios:

- Use `"AIC"` for general purposes, especially when prioritizing a good fit over simplicity.
- Use `"BIC"` when you prefer a more parsimonious model, particularly with large datasets.
- Use `"AICc"` when working with small sample sizes to avoid overfitting.
- Use `"HQ"` for a balance between AIC and BIC, often in econometrics or time series models.

Ensure that the selected criterion aligns with the goals of your analysis and the characteristics of your data.

#### Examples

##### Default criterion (AIC)

``` r

kardl_set(criterion = "AIC")
kardl(data, my_formula)
```

##### Using BIC for lag selection

``` r

kardl_set(criterion = "BIC")
kardl(data, my_formula)
```

##### Using AICc for small sample sizes

``` r

kardl_set(criterion = "AICc")
data %>% kardl(my_formula, data=.)
```

##### Using HQ criterion

``` r

kardl_set(criterion = "HQ")
kardl(data, my_formula)
```

### 6. different_asym_lag

`different_asym_lag` is a logical value (`TRUE` or `FALSE`) indicating whether positive and negative asymmetric variables should be assigned different lags during the estimation process. The default value is `FALSE`, meaning that both positive and negative components will use the same lag.

#### Details

Asymmetric decomposition separates a variable into its positive and negative changes. In some models, it may be desirable to assign different lags to these components to capture distinct dynamic behaviors. Setting `different_asym_lag = TRUE` allows the function to optimize lags for positive and negative components independently. When `different_asym_lag = FALSE`, both components will share the same lag.

This parameter is particularly useful when:

- Positive and negative changes in a variable are expected to have different impacts over time.
- The user wants more flexibility in modeling asymmetric responses.

**Attention!**

- When `different_asym_lag = TRUE`, ensure that the model has sufficient data to estimate separate lags reliably.
- For models with limited observations or a high number of variables, `different_asym_lag = FALSE` may be more robust and computationally efficient.

#### Examples

##### Using the same lag for positive and negative components (default)

``` r

kadrl_set(different_asym_lag = FALSE)
kardl(data, my_formula)
```

##### Assigning different lags for positive and negative components

``` r

kardl_set(different_asym_lag = TRUE)
kardl(data, my_formula)
```

### 7. asym_prefix

`asym_prefix` is a character vector specifying the prefixes used for naming asymmetric variables created during positive and negative decomposition. The default value is an empty vector `c()`, indicating that no prefixes are added by default.

When specified, the prefixes are added to the beginning of variable names to represent the positive and negative decomposition:

- The first element in the vector corresponds to the positive decomposition.
- The second element in the vector corresponds to the negative decomposition.

#### Details

Asymmetric decomposition is used to analyze the separate effects of positive and negative changes in a variable. For example, given a variable `X`, prefixes can be used to generate `POS_X` and `NEG_X` for the positive and negative components, respectively.

By default, no prefixes are applied (`asym_prefix = c()`). However, users can define custom prefixes by providing a vector with two elements. For example:

- `kardl_set(asym_prefix = c("POS_", "NEG_"))` results in variable names such as `POS_X` and `NEG_X`.
- `kardl_set(asym_prefix = c("Increase_", "Decrease_"))` results in variable names such as `Increase_X` and `Decrease_X`.

**Attention!**

- The vector must contain exactly two elements: the first for the positive decomposition and the second for the negative decomposition.
- If prefixes are used in combination with suffixes (via `asym_suffix`), ensure that the resulting variable names are meaningful and do not conflict.

### Examples

#### Using default (no prefixes)

``` r

kardl_set( asym_prefix = c())
```

#### Custom prefixes for positive and negative decomposition

``` r

kardl_set(asym_prefix = c("POS_", "NEG_"))
```

#### Combining prefixes with suffixes

``` r

kardl_set( asym_prefix = c("Change_", "Fall_"), asym_suffix = c("_High", "_Low"))
```

### 8. asym_suffix

`asym_suffix` is a character vector specifying the suffixes used for naming asymmetric variables created during positive and negative decomposition. The default value is `c("_POS", "_NEG")`, where:

- `"_POS"` is the suffix appended to variables representing the positive decomposition.
- `"_NEG"` is the suffix appended to variables representing the negative decomposition.

##### Details

The order of the suffixes is important:

- The first element in the vector corresponds to the positive decomposition.
- The second element in the vector corresponds to the negative decomposition.

Asymmetric decomposition is commonly used in models to separate the effects of positive and negative changes in a variable. For example, given a variable `X`, the decomposition may result in `X_POS` and `X_NEG` to represent its positive and negative components, respectively.

By default, the suffixes `"_POS"` and `"_NEG"` are used, but users can customize them as needed by providing a custom vector. For example:

- `asym_suffix = c("_Increase", "_Decrease")` results in variable names such as `X_Increase` and `X_Decrease`.
- `asym_suffix = c("_Up", "_Down")` results in variable names such as `X_Up` and `X_Down`.

**Attention!**

- Ensure that the custom suffix vector has exactly two elements:
  - The first element must always represent the positive decomposition.
  - The second element must always represent the negative decomposition.
- Providing more or fewer elements or incorrect ordering may cause errors in variable naming.

### 9. long_coef

`long_coef` is a character string specifying the prefix format for naming long-run coefficients in the model output. The default value is `"L{lag}.{varName}"`, where:

- `{lag}` is a placeholder for the lag number.
- `{varName}` is a placeholder for the variable name.

This format generates names like `L1.X` for the first lag of variable `X`.

#### Details

Long-run coefficients represent the long-term relationships between the dependent variable and independent variables in an ARDL or NARDL model. The `long_coef` parameter allows users to customize how these coefficients are named in the output, making it easier to identify and interpret them. The default format `"L{lag}.{varName}"` is widely used and provides clear information about the lag and variable associated with each coefficient. Users can modify the format by changing the `long_coef` string. For example:

- `long_coef = "LongRun_{varName}_Lag{lag}"` results in names like `LongRun_X_Lag1`.
- `long_coef = "LR_{varName}_L{lag}"` results in names like `LR_X_L1`.

**Attention!**

- Ensure that the placeholders `{lag}` and `{varName}` are included in the custom format to maintain clarity in the coefficient names.
- Avoid using special characters or spaces in the format that may cause issues in R variable naming conventions.

### 10. short_coef

`short_coef` is a character string specifying the prefix format for naming short-run coefficients in the model output. The default value is `"L{lag}.d.{varName}"`, where:

- `{lag}` is a placeholder for the lag number.
- `{varName}` is a placeholder for the variable name.

This format generates names like `L1.d.X` for the first lag of the differenced variable `X`.

#### Details

Short-run coefficients capture the immediate effects of changes in independent variables on the dependent variable in an ARDL or NARDL model. The `short_coef` parameter allows users to customize how these coefficients are named in the output, facilitating easier identification and interpretation. The default format `"L{lag}.d.{varName}"` is commonly used and provides clear information about the lag, differencing, and variable associated with each coefficient. Users can modify the format by changing the `short_coef` string. For example:

- `short_coef = "ShortRun_{varName}_Lag{lag}"` results in names like `ShortRun_X_Lag1`.
- `short_coef = "SR_{varName}_L{lag}"` results in names like `SR_X_L1`.

**Attention!**

- Ensure that the placeholders `{lag}` and `{varName}` are included in the custom format to maintain clarity in the coefficient names.
- Avoid using special characters or spaces in the format that may cause issues in R variable naming conventions.

### 11. batch

`batch` is a character string specifying the batch size for parallel processing during model estimation. The default value is `"1/1"`, indicating that the model estimation will be executed as a single job without batching.

#### Details

The `batch` parameter is particularly useful when dealing with large datasets or complex models that require significant computational resources. By specifying a batch size, users can divide the model estimation process into smaller, more manageable segments, which can be processed in parallel. The format for the `batch` parameter is `"m/n"`, where:

- `m` is the number of batches to be processed in parallel.
- `n` is the total number of batches.

For example, setting `batch = "2/4"` would divide the estimation into 4 batches, with 2 batches being processed simultaneously. This can significantly reduce computation time, especially for models with extensive lag structures or large numbers of variables.

**Attention!**

- Ensure that the specified batch size is appropriate for the available computational resources (e.g., CPU cores, memory).
- Overloading the system with too many parallel processes may lead to performance degradation or failures.
- The effectiveness of batching may vary depending on the specific model and dataset characteristics.

#### Examples

##### Default batch size (no batching)

``` r
kardl_set(batch = "1/1")
kardl(data, my_formula)
```

##### Using a batch size of 2 out of 4

``` r
kardl_set(batch = "2/4")
kardl(data, my_formula)
```

##### Using a batch size of 3 out of 6

``` r
kardl_set(batch = "3/6")
kardl(data, my_formula)
```

### 12. print_wrap

`print_wrap` is an optional parameter that specifies the maximum number of characters per line for console output. The default value is `NULL`, which means that the output will not be wrapped and will be displayed in its entirety.
 
#### Examples

##### Default (no wrapping)

``` r
kardl_set(print_wrap = NULL)
```

##### Custom wrapping (e.g., 80 characters per line)
``` r
kardl_set(print_wrap = 80L)
```




## Contributing to kardl

Thank you for considering contributing to the `kardl` package!

### Life Cycle Statement

This package is **in a stable state of development, with active subsequent development primarily in response to user feedback.**

The core functionality for linear and nonlinear ARDL/NARDL estimation, dynamic multipliers, symmetry tests, Narayan cointegration test, and bootstrap methods is considered stable and reliable.

Future development will mainly focus on: 
- Fixing bugs reported by users 
- Addressing feature requests (e.g., additional post-estimation tools, new test options, or performance improvements) 
- Maintaining compatibility with newer R versions and dependencies 
- Minor enhancements based on community needs

Major new methodological features will only be added if they align with clear user demand or important advancements in the econometric literature.

### How to Contribute

- Report bugs or suggest features by opening an issue on GitHub.
- Submit pull requests for bug fixes or improvements.
- All contributions are welcome and will be properly acknowledged.

For questions or discussions, please open an issue.

## Conclusion

The `kardl` package is a versatile tool for econometric analysis, offering robust support for symmetric and asymmetric ARDL/NARDL modeling, cointegration tests. Its flexible formula specification, lag optimization, and support for parallel processing make it ideal for studying complex economic relationships. For more information, visit <https://github.com/karamelikli/kardl> or contact the authors at [hakperest\@gmail.com](mailto:hakperest@gmail.com).

## Disclosures

- The logo created by **canva.com** free account.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  echo = TRUE, message = FALSE, warning = FALSE, fig.width = 16,
  fig.height = 8,
  out.width = "100%",
  dpi = 150
)

## ----srr-tags, eval = FALSE, echo = FALSE-------------------------------------
# #' srr tags for the kardl package
# #'
# #' This package follows structured statistical reporting rules using the
# #' \code{srrstats} framework. The standards governing implementation and
# #' documentation are summarized below.
# #'
# #' @srrstats {G1.0} The package documentation and accompanying materials cite
# #'           the ARDL bounds-testing and nonlinear ARDL literature on which the
# #'           implemented estimators, asymmetry decomposition, bounds tests, and
# #'           dynamic multipliers are based.
# #' @srrstats {G1.1} The package is described as an implementation and extension
# #'           of ARDL and NARDL workflows, supporting mixed symmetric and
# #'           asymmetric regressors, flexible lag selection, and dynamic
# #'           multiplier methods.
# #' @srrstats {G1.2} The README, NEWS file, and package website describe the
# #'           development status, recent changes, and future maintenance plans
# #'           of the package.
# #' @srrstats {G1.3} Key statistical concepts such as ARDL, NARDL, ECM, bounds
# #'           testing, short-run asymmetry, long-run asymmetry, and dynamic
# #'           multipliers are defined in function documentation and vignettes.
# #' @srrstats {G1.4} All user-facing functions and S3 methods are documented
# #'           using \code{roxygen2}, and the generated Rd files are maintained
# #'           as part of the package documentation workflow.
# #' @srrstats {G1.4a} Internal helper functions are documented where necessary
# #'           and are excluded from the public help index using \code{@noRd}
# #'           or marked as internal.

## ----install-cran, eval=FALSE-------------------------------------------------
# install.packages("kardl")
# library(kardl)

## ----install, eval=FALSE------------------------------------------------------
# # Install required packages
# install.packages(c(
#   "stats", "msm", "lmtest", "nlWaldTest", "car", "strucchange",
#   "utils", "ggplot2"
# ))
# # Install kardl from GitHub
# install.packages("devtools")
# devtools::install_github("karamelikli/kardl")

## ----load---------------------------------------------------------------------
library(kardl)

## ----data-prepare-------------------------------------------------------------
# Define the model formula
my_formula <- DriversKilled ~ PetrolPrice + drivers +
  asymmetric(PetrolPrice + drivers) + deterministic(law) + trend

## ----eval=FALSE---------------------------------------------------------------
# same_formula <- y ~ asymmetric(x1) +
#   sasymmetric(x2 + x3) +
#   lasymmetric(x4 + x5) +
#   deterministic(dummy1) + trend
# same_formula <- y ~ asymmetric(x1) +
#   sasymmetric(x2 + x3) +
#   lasymmetric(x4 + x5) +
#   deterministic(dummy1) + trend
# same_formula <- y ~ asym(x1) + sasym(x2 + x3) + lasym(x4 + x5) +
#   det(dummy1) + trend
# same_formula <- y ~ a(x1) + s(x2 + x3) + l(x4 + x5) + d(dummy1) + trend

## ----model-grid---------------------------------------------------------------
# Set model options
kardl_set(criterion = "BIC", different_asym_lag = TRUE, data = Seatbelts)
# Estimate model with grid mode
kardl_model <- kardl(
  data = Seatbelts, formula = my_formula,
  maxlag = 4, mode = "grid"
)
# View results
kardl_model

## ----model-grid-custom-summary------------------------------------------------
# Display model summary
summary(kardl_model)

## ----model-user-defined-------------------------------------------------------
kardl_model2 <- kardl(
  data = Seatbelts, my_formula,
  mode = c(2, 1, 1, 3, 0)
)
# View results
kardl_extract(kardl_model2, "opt_lag")

## ----model-user-defined-summary-----------------------------------------------
# Display model summary
summary(kardl_model2)

## ----model-all-vars-----------------------------------------------------------
kardl_set(data = Seatbelts)
kardl(formula = DriversKilled ~ . + deterministic(law), maxlag = 2)

## ----lag-criteria-------------------------------------------------------------
library(dplyr)
library(tidyr)
library(ggplot2)
# Convert lag_criteria to a data frame
lag_criteria <- as.data.frame(kardl_extract(kardl_model, "lag_criteria"))
lag_criteria <- lag_criteria |> mutate(across(c(AIC, BIC, HQ), as.numeric))

# Pivot to long format
lag_criteria_long <- lag_criteria |>
  select(-c(AICc, AdjR2)) |>
  pivot_longer(
    cols = c(AIC, BIC, HQ),
    names_to = "Criteria",
    values_to = "Value"
  )

# Find minimum values
min_values <- lag_criteria_long |>
  group_by(Criteria) |>
  slice_min(order_by = Value) |>
  ungroup()

# Plot
ggplot(
  lag_criteria_long,
  aes(x = lag, y = Value, color = Criteria, group = Criteria)
) +
  geom_line() +
  geom_point(
    data = min_values, aes(x = lag, y = Value),
    color = "red", size = 3, shape = 8
  ) +
  geom_text(
    data = min_values, aes(x = lag, y = Value, label = lag),
    vjust = 1.5, color = "black", size = 3.5
  ) +
  scale_x_discrete(
    breaks = lag_criteria$lag[seq(1, nrow(lag_criteria), by = 20)]
  ) +
  labs(
    title = "Lag Criteria Comparison",
    x = "Lag Configuration",
    y = "Criteria Value"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

## ----ecm-estimation-----------------------------------------------------------
ecm_model <- ecm(
  data = Seatbelts, formula = my_formula,
  maxlag = 4, mode = "grid_custom"
)
# View results
summary(ecm_model)

## ----long-run-----------------------------------------------------------------
# Long-run coefficients
my_long <- kardl_longrun(kardl_model)
my_long

## ----long-run-summary---------------------------------------------------------
# Summary of long-run coefficients
summary(my_long)

## ----asymmetry-test-----------------------------------------------------------
ast <- Seatbelts |>
  kardl(
    DriversKilled ~ PetrolPrice + drivers + asymmetric(PetrolPrice + drivers) +
      deterministic(law) + trend,
    mode = c(1, 2, 3, 0, 1),
    data = _
  ) |>
  symmetrytest()
ast

## ----asymmetry-test-summary---------------------------------------------------
# Summary of symmetry test
summary(ast)

## ----pssf---------------------------------------------------------------------
test_result <- kardl_model |> pssf(case = 3, signif_level = "0.05")
test_result

## ----pssf-summary-------------------------------------------------------------
summary(test_result)

## ----psst---------------------------------------------------------------------
test_result <- kardl_model |> psst(case = 3, signif_level = "0.05")
test_result

## ----psst-summary-------------------------------------------------------------
summary(test_result)

## ----narayan------------------------------------------------------------------
test_result <- kardl_model |> narayan(case = 3, signif_level = "0.05")
test_result

## ----narayan-summary----------------------------------------------------------
summary(test_result)

## ----dynamic-multipliers------------------------------------------------------
multipliers <- kardl_model |> mplier()
# View multipliers of the model
head(kardl_extract(multipliers, "multipliers"))
# View long-run multipliers
kardl_extract(multipliers, "omega")
# View short-run multipliers
head(kardl_extract(multipliers, "lambda"))

## ----plot-multipliers---------------------------------------------------------
plot(multipliers, variables = c("PetrolPrice", "drivers"))

## ----bootstrap-multipliers----------------------------------------------------
bootstrap_results <- kardl_model |>
  bootstrap(horizon = 12, replications = 10)
# View bootstrap summary
summary(bootstrap_results)

## ----plot-bootstrap-multipliers-----------------------------------------------
plot(bootstrap_results, variables = "drivers")

## ----asym-custom--------------------------------------------------------------
# Set custom prefixes and suffixes
kardl_reset()
kardl_set(asym_prefix = c("asyP_", "asyN_"), asym_suffix = c("_PP", "_NN"))
kardl_custom <- kardl(data = Seatbelts, my_formula)
kardl_custom

library("imf.data")
IFS <- load_datasets("IFS")

# install.packages("imf.data")
# library(imf.data)
# IFS <- load_datasets("IFS")
# # PCPI_IX             Prices, Consumer Price Index, All items, Index
# # AIP_IX              Economic Activity, Industrial Production, Index
# # ENDE_XDC_USD_RATE   Exchange Rates, Domestic Currency per U.S. Dollar,
#                       End of Period, Rate
#
trdata <- IFS$get_series(
    freq = "M", ref_area = "TR",
    indicator = c("PCPI_IX", "AIP_IX", "ENDE_XDC_USD_RATE"),
    start_period = "1985-01", end_period = "2024-02"
)
PeriodRow <- trdata[, 1]
trdata[, 1] <- NULL
colnames(trdata) <- c("ER", "CPI", "PPI")
trdata <- log(as.data.frame(lapply(trdata, as.numeric)))
rownames(trdata) <- PeriodRow
# # Inserting covid dummy variable
trdata <- cbind(trdata, covid = 0)
trdata[420:470, 4] <- 1
KARDL_model <- kardl(trdata,
    CPI ~ ER + PPI + asym(ER) + deterministic(covid) + trend,
    mode = c(1, 2, 2, 0)
)

#' Combine variable types and extract information from the model formula
#'
#' This function takes the user inputs, checks the model formula, and extracts information about the dependent variable, independent variables, asymmetric variables, and deterministic variables. It also checks if the variables exist in the data and prepares a list of extracted information for further processing.
#' @param inputs A list of user inputs, including the model formula and data.
#' @param checkInData A logical value indicating whether to check if the variables exist in
#' the data. Default is TRUE.
#' @return A list containing the original inputs, settings, and extracted information.
#' @noRd
combineVarTypes <-function(inputs,checkInData=TRUE){

  # checking the existence of model in gotten varaibales of the function
  if(is.null(inputs$formula)){
    #  stop("The model for the model not found! Please define the model like as: model=y~x+z")
    stop("The model is missing! Please define the model like as: model=y~x+z", call. = FALSE)
  }
  if(typeof(inputs$formula) != "language"){
    stop("The model should be a valid model without any \" or '. For y~x+z ", call. = FALSE)
  }
  extractedInfo<-list()
  parseFormula <-   parse_formula_vars(inputs$formula)
  extractedInfo[["noConstant"]] <- !parseFormula$intercept
  extractedInfo[["trend"]] <- "trend" %in% parseFormula$outside


  choices <- c("asymmetric","sasymmetric","lasymmetric","deterministic")
  names(parseFormula$inside) <- tolower( names(parseFormula$inside))
  matched <- vapply(
    names(parseFormula$inside),
    match.arg,
    character(1),
    choices = choices
  )
  allofInsides<-list()
  insideNames<-names(matched)
  for (variable in insideNames) {
    allofInsides[[matched[[variable]]]]<-parseFormula$inside[[variable]]
  }
  extractedInfo$ALvars <-unique(trimws(c(allofInsides$lasymmetric,allofInsides$asymmetric)))
  extractedInfo$ASvars<-unique(trimws(c(allofInsides$sasymmetric,allofInsides$asymmetric)))
  extractedInfo$deterministic <-unique(trimws(allofInsides$deterministic))

 extractedInfo$dependentVar <-parseFormula$response
  if(parseFormula$dot){
    completeVars<-colnames(inputs$data)
    completeVars<-completeVars[ ! completeVars %in% c("trend")]
    extractedInfo$independentVars<-setdiff(unique(completeVars),c(extractedInfo$dependentVar,extractedInfo$deterministic))
  }else{
    setdiff(unique(c(
    parseFormula$outside,
    unlist(parseFormula$inside, use.names = FALSE)
    )), c("trend",extractedInfo$deterministic) )-> extractedInfo$independentVars
    }



  extractedInfo$Allvars<-c( extractedInfo$dependentVar, extractedInfo$independentVars)

  # stop if any of Allvars is not in the data
  if(length(extractedInfo[["Allvars"]])==0){
    stop("No variable found in the model! Please check the model again.", call. = FALSE)
  }
  if(checkInData){
    for (x in extractedInfo[["Allvars"]]) {
      if(!(x %in% colnames(inputs$data))){
        stop("The variable: ", x, " not found in the data file's vars list!", call. = FALSE)
      }
    }
  }

  attr(extractedInfo, "source") <- "extractedInfo"
  attr(extractedInfo, "description") <-"This value was extractedInfo from user inputs."


  list( argsInfo=inputs,settings= kardl_get(c("AsymPrefix","AsymSuffix","LongCoef","ShortCoef")), extractedInfo=extractedInfo)
}

#' Verify user-defined lags for the model specification
#'
#' This function checks the user-defined lags provided in the model specification. It ensures that the lags are valid numeric values, non-decimal, and match the number of variables in the model. It also handles the naming of the lags based on the variable names and updates the maximum lag value in the specification.
#' @param spec A list containing the model specification, including user-defined lags and variable information.
#' @return The updated model specification with verified user-defined lags and maximum lag value.
#'
#' @srrstats {G2.0} The function checks for the validity of user-defined lags, ensuring that they are numeric, non-decimal, and match the number of variables in the model. It also provides descriptive error messages if any of the checks fail, guiding the user to correct their input.
#' @srrstats {G2.4a} Integer-valued lag counts are enforced; non-integer values produce an informative error before any lagged regressors are constructed.
#' @srrstats {G5.2a} Diagnostic messages identify the specific invalid lag vector encountered by the user.
#'
#' @noRd


verifyUserDefinedLags<-function(spec){

  if(typeof(spec$argsInfo$mode)!= "double"){
    stop("User-defined value should have valid vector. For example: c(1,0,1)", call. = FALSE)
  }
  if(!all(spec$argsInfo$mode==floor(spec$argsInfo$mode)) ){
    stop("User-defined should have valid numeric and non-decimal. Your pattern is ", paste(spec$argsInfo$mode, collapse = ","), ". For example: 1,0,1 ", call. = FALSE)
  }


  nlist<-c()
  j<-0
  for (i in seq_along(spec$extractedInfo$Allvars)) {
    if(spec$extractedInfo$Allvars[i] %in% spec$extractedInfo$ASvars){
      nlist[i+j]<-paste0(.kardl_Settings_env$AsymPrefix[1],spec$extractedInfo$Allvars[i],.kardl_Settings_env$AsymSuffix[1])
      j <- j+1
      nlist[i+j]<-paste0(.kardl_Settings_env$AsymPrefix[2],spec$extractedInfo$Allvars[i],.kardl_Settings_env$AsymSuffix[2])
    }else{
      nlist[i+j]<- spec$extractedInfo$Allvars[i]
    }
  }
  if(length(spec$argsInfo$mode)!=length(spec$extractedInfo$Allvars )+length(spec$extractedInfo$ASvars)){
    stop("User-defined should match with short-run variables. User defined lags vector must has exactly ",
         length(spec$extractedInfo$Allvars) + length(spec$extractedInfo$ASvars),
         " element. Your pattern is ", paste(spec$argsInfo$mode, collapse = ","),
         ". Please define by this order: ", paste(nlist, collapse = ","), call. = FALSE)
  }

  # If user defines by her order, the order should be redefined
  if (!is.null(names(spec$argsInfo$mode))) {
    yenisi <- c()
    for (i in seq_along(nlist)) {
      yenisi[nlist[i]] <- spec$argsInfo$mode[nlist[i]]
    }
    spec$argsInfo$mode <- yenisi
  }else{
    names(spec$argsInfo$mode) <- nlist
  }

  if(spec$argsInfo$mode[1]<1){
    stop("User-defined should start with a digit greater than zero. Your pattern is ", paste(spec$argsInfo$mode, collapse = ","), ". For example: 1,0,1 ", call. = FALSE)
  }
  if(all(spec$argsInfo$mode>=0)!=T){
    stop("User-defined should containt only positive values. Your pattern is ", paste(spec$argsInfo$mode, collapse = ","), ". For example: 1,0,1 ", call. = FALSE)
  }


  spec$argsInfo$maxlag<-max(spec$argsInfo$mode)

  spec
}

#' Check the validity of user inputs for the model specification
#'
#' This function performs various checks on the user inputs for the model specification. It ensures that the model formula is defined properly, the data is provided, the criterion is valid, and other parameters are of the correct type and format. If any of the checks fail, it raises an error with a descriptive message.
#' @param spec A list containing the model specification, including the model formula, data,
#' criterion, and other parameters.
#' @return The original model specification if all checks pass. If any check fails, an
#' error is raised with a descriptive message.
#'
#' @srrstats {G2.0} The function checks for the presence and validity of the model formula, data, criterion, and other parameters. It ensures that the model formula is defined properly as a language object, the data is provided, the criterion is either a function or one of the predefined criteria, and that other parameters are of the correct type (e.g., logical values for certain parameters). If any of these checks fail, it raises an error with a descriptive message to guide the user in correcting their input.
#' @srrstats {G2.1} Validates that `formula` is a language object and `data` is not NULL before extraction.
#' @srrstats {G2.3} The `criterion` argument is validated against the set of accepted string values `c("AIC","BIC","AICc","HQ")`; custom functions are also accepted.
#' @srrstats {G2.3a} Selection argument `criterion` is matched against predefined acceptable values.
#' @srrstats {G5.2a} Error messages identify the specific invalid input (e.g., wrong criterion, non-logical flag, malformed batch string).
#' @srrstats {G5.8a} Zero-length or malformed model inputs are rejected before estimation.
#'
#'
#' @noRd

CheckInputs<-function(spec){
  if(is.null(spec$argsInfo$formula)){
    stop("model is missing! Please define the model like as: model=y~x+z", call. = FALSE)
  }
  if(typeof(spec$argsInfo$formula)!="language"){
    stop("The model is not defined properly.", call. = FALSE)
  }
  if(is.null(spec$argsInfo$data)){
    stop("The data is missing! Please select related data. data=data", call. = FALSE)
  }
  # if(typeof(inputs$dataTimeSeriesStart) != "double" || ! is.numeric(inputs$dataTimeSeriesStart)){
  #   stop("dataTimeSeriesStart should be a valid  time series start point. For example c(2010,3) ")
  # }

  # if(typeof(inputs$sig) != "character" || !inputs$sig %in% c("0.01" , "0.05" , "0.10")){
  #   stop(paste0("Error: Significance level should be a valid significant level. The valid levels here are  0.01, 0.05, 0.10."))
  # }

  if(!is.function(spec$argsInfo$criterion) ){
    if(typeof(spec$argsInfo$criterion) != "character" && !spec$argsInfo$criterion %in% c("AIC","BIC","AICc","HQ")  ){
      stop("The entered criterion should be a function or one of the defined criteria here. The defined criteria are  AIC , BIC , AICc , HQ ", call. = FALSE)
    }
  }

  # if(typeof(spec$extractedInfo$trend)!= "logical"){
  #   stop(paste0("Error: The entered trend should be a logical value. TRUE/FALSE ."))
  # }
  if(typeof(spec$argsInfo$differentAsymLag)!= "logical"){
    stop("The entered DifferentAsymLag should be a logical value. TRUE/FALSE .", call. = FALSE)
  }
  if(!all(spec$argsInfo$mode %in% c("grid_custom","grid","quick") )){
    # The predefined option for user-defined is false. Then here, we check just if it is not false.

    spec<-verifyUserDefinedLags(spec) # store deined names

  }
  # if(typeof(inputs$makeCusumPlots)!= "logical"){
  #   stop(paste0("Error: The entered makeCusumPlots should be a logical value. TRUE/FALSE ."))
  # }

  if (!is.null(spec$argsInfo$batch) & !grepl("^\\d+/\\d+$", spec$argsInfo$batch)) {
    stop("Invalid batch format. Use 'x/y', where x is the batch number and y is the total number of batches.", call. = FALSE)
  }
  # Check whether are digit or not
  if(!grepl("^([1-9])[0-9]*$", spec$argsInfo$maxlag, perl = T)){
    stop("The entered maxlag should be a digit. You entered: ", spec$argsInfo$maxlag, call. = FALSE)
  }
  # if(!grepl("^([1-9])[0-9]*$", inputs$dataTimeSeriesFrequency, perl = T)){
  #   stop(paste0("Error: The entered dataTimeSeriesFrequency should be a digit."))
  # }

  spec # return inputs, If there are changes in inputs, it should be returned as a new formed inputs list.
}

#' Detect and prepare variables for the model specification
#'
#' This function detects the variables specified in the model formula, checks their existence in the data, and prepares lists of short-run and long-run variables based on the asymmetric variable specifications. It also determines the model type (LL, AS, SA, NN) based on the presence of asymmetric variables and calculates the number of lag rows needed for the model estimation.
#' @param spec A list containing the model specification, including extracted information about variables and user
#' inputs.
#' @return The updated model specification with detected variables, prepared variable lists, and model type
#' information. If any variable is not found in the data, an error is raised with a descriptive message.
#' @srrstats {G2.10} Variables are selected by name from the supplied data through formula parsing rather than by position-dependent assumptions.
#' @srrstats {G2.11} Standard data-frame inputs are processed by checking all formula-referenced columns against `colnames(data)`.
#' @srrstats {G5.8a} Variables not found in the data trigger descriptive errors before any estimation step.
#'
#'
#' @srrstats {G1.0} The package documentation and manuscript cite the ARDL bounds-testing and nonlinear ARDL literature on which the implemented estimators, asymmetry decomposition, bounds tests, and dynamic multipliers are based.
#' @srrstats {G1.1} The package documentation describes `kardl` as an R implementation and extension of ARDL and NARDL workflows, with emphasis on mixed symmetric and asymmetric regressors, flexible lag selection, and dynamic multiplier methods.
#' @srrstats {G1.2} The README, NEWS file, and package website describe the current development state, recent releases, and intended future maintenance of the package.
#' @srrstats {G1.3} Statistical terms such as ARDL, NARDL, ECM, bounds testing, short-run asymmetry, long-run asymmetry, and dynamic multipliers are defined in function documentation and vignettes.
#' @srrstats {G1.4} User-facing functions and S3 methods are documented with `roxygen2`, and the generated Rd files are kept under version control through the package documentation workflow.
#' @srrstats {G1.4a} Internal helper functions are documented with roxygen comments where clarification is needed and are hidden from the public help index with `@noRd` or marked as internal.
#' @srrstats {G1.5} Examples, tests, and vignettes include executable code using package data so that users can reproduce the estimation and multiplier outputs reported in the documentation.
#' @srrstats {G1.6} The related-software discussion compares the package with alternative ARDL and NARDL implementations in other software environments and explains the reproducibility advantages of the R implementation.
#' @srrstats {G2.0} Main modelling functions validate the expected scalar and vector inputs, including model formulae, lag choices, criteria, horizon values, and object classes.
#' @srrstats {G2.0a} Function documentation explains length expectations for important arguments such as formula input, lag structures, selected criteria, and dynamic multiplier horizons.
#' @srrstats {G2.1} The package checks that model input is supplied in the expected data and formula form and that S3 methods receive compatible `kardl` model objects.
#' @srrstats {G2.1a} Documentation describes the expected data structure for variables used in time-series regression and clarifies that model variables should be numeric where estimation requires numeric series.
#' @srrstats {G2.2} Model specifications restrict dependent-variable input to a single response variable and process independent variables through the formula parser.
#' @srrstats {G2.3} Character-valued options are limited to recognised choices through package option handlers and argument matching routines.
#' @srrstats {G2.3a} Selection arguments such as information criteria and asymmetry options are matched against predefined acceptable values.
#' @srrstats {G2.3b} Formula terms and asymmetry constructors are processed consistently so that documented model syntax is interpreted as intended.
#' @srrstats {G2.4} Data and formula-processing routines standardise model inputs before estimation so that subsequent estimation functions work with a consistent internal representation.
#' @srrstats {G2.4a} Integer-valued quantities such as lags and horizons are treated as discrete counts before they are used to construct lagged regressors or multiplier paths.
#' @srrstats {G2.4b} Numeric model variables are passed to estimation routines as numeric vectors or matrix columns suitable for ordinary least squares and post-estimation algebra.
#' @srrstats {G2.4c} Variable names and parsed formula terms are handled as character vectors when constructing lagged-variable names and asymmetric components.
#' @srrstats {G2.4d} The package does not require factor-valued model variables for ARDL estimation, and categorical effects should be supplied through suitable deterministic or dummy variables.
#' @srrstats {G2.4e} Factor inputs are not part of the core estimation algorithm; users are expected to provide numeric series or explicit dummy variables when categorical controls are required.
#' @srrstats {G2.5} The modelling interface is designed for numeric time-series regressors rather than ordered or unordered factor responses.
#' @srrstats {G2.6} One-dimensional variables selected through the formula are extracted and aligned through the model-preparation routines before lagged terms are generated.
#' @srrstats {G2.7} The modelling functions operate on standard tabular data supplied by users, including data frames containing the time-series variables referenced in the formula.
#' @srrstats {G2.8} Initial preprocessing converts the user formula and data into a common internal structure containing dependent variables, independent variables, lag information, and generated regressors.
#' @srrstats {G2.9} The package constructs generated variable names for lagged, differenced, and asymmetric components in a transparent way so users can inspect the resulting model object.
#' @srrstats {G2.10} Variables are selected by name from the supplied data through formula parsing rather than by position-dependent assumptions.
#' @srrstats {G2.11} The package tests standard data-frame inputs used in examples and vignettes and processes columns required by the model formula.
#' @srrstats {G2.12} The package is intended for numeric time-series columns and does not treat list columns as valid model variables for estimation.
#' @srrstats {G2.13} Missing observations introduced by lagging and differencing are handled during model-frame construction before estimation is performed.
#' @srrstats {G2.14} Missing data behaviour follows the documented estimation workflow, where observations unavailable after lag construction are excluded from the estimation sample.
#' @srrstats {G2.14a} Invalid model inputs that cannot be transformed into an estimable ARDL specification generate errors before post-estimation routines are run.
#' @srrstats {G2.14b} The estimation workflow removes structurally missing lagged observations created by differencing and lag construction.
#' @srrstats {G2.14c} The package does not impute missing time-series observations because imputation would change the dynamic structure of the fitted ARDL model.
#' @srrstats {G2.15} Estimation and post-estimation calculations are based on the cleaned model object rather than passing unprocessed missing values to lower-level numeric routines.
#' @srrstats {G2.16} Non-finite values are not treated as valid observations for the fitted regression sample and should be removed or corrected before estimation.
#' @srrstats {G3.0} Numerical comparisons in tests and post-estimation checks use tolerance-based expectations where exact floating-point equality would be inappropriate.
#' @srrstats {G3.1} Long-run multiplier inference uses the variance-covariance matrix of the fitted model object, allowing users to work with the covariance structure attached to the estimation result.
#' @srrstats {G3.1a} Multiplier documentation describes standard-error calculation from the fitted model covariance matrix and examples show how users can inspect the resulting estimates.
#' @srrstats {G5.0} Tests and examples use the package example data and fixed model specifications with known expected structure.
#' @srrstats {G5.1} The example data used in documentation and tests are included with the package so users can reproduce examples and test calculations.
#' @srrstats {G5.2} The test suite exercises representative successful calls and selected error conditions for model estimation, multiplier calculation, and S3 methods.
#' @srrstats {G5.2a} Diagnostic messages are written to identify the specific invalid input or unsupported object class encountered by the user.
#' @srrstats {G5.2b} Tests include error expectations for incompatible object classes and invalid post-estimation inputs.
#' @srrstats {G5.3} Tests verify that key returned objects, including fitted model components and multiplier matrices, have the expected dimensions, names, and classes.
#' @srrstats {G5.4} Correctness tests compare selected outputs against expected structural properties of ARDL and NARDL models, including lag information, long-run coefficients, and multiplier matrices.
#' @srrstats {G5.4a} Where exact external reference values are not available, tests use simple deterministic cases and expected object properties to verify implementation behaviour.
#' @srrstats {G5.4b} Validation work compares package results with established ARDL and NARDL software workflows where comparable outputs are available.
#' @srrstats {G5.4c} Published ARDL and NARDL formulae provide the basis for the implemented coefficient transformations and dynamic multiplier recursion.
#' @srrstats {G5.5} Tests involving simulated or random inputs set a random seed before execution.
#' @srrstats {G5.6} Parameter-recovery checks are implemented through deterministic model examples and comparisons of estimated quantities with expected algebraic relationships.
#' @srrstats {G5.6a} Numeric tests use tolerances where floating-point arithmetic or regression estimation makes exact equality inappropriate.
#' @srrstats {G5.6b} The core estimation functions are deterministic for a fixed data set and model specification, so repeated seeds are not required for standard tests.
#' @srrstats {G5.7} Tests examine behaviour across different model specifications, including linear, short-run asymmetric, long-run asymmetric, and fully asymmetric cases.
#' @srrstats {G5.8} Edge-condition tests check that invalid object classes and unsupported inputs produce clear errors rather than silent failures.
#' @srrstats {G5.8a} Zero-length or insufficient model inputs are rejected through the model-preparation and estimation checks.
#' @srrstats {G5.8b} Unsupported non-numeric model variables are not valid inputs for the regression calculations and are expected to fail before estimation.
#' @srrstats {G5.8c} Degenerate input cases are handled through the underlying regression and model-validation workflow.
#' @srrstats {G5.8d} Model specifications outside the estimable sample size or lag structure are rejected by estimation and lag-selection routines.
#' @srrstats {TS1.0} The package accepts time-ordered regression data and constructs lagged and differenced variables internally for ARDL and NARDL estimation.
#' @srrstats {TS1.1} Function documentation and vignettes describe the accepted input form for time-series variables used in `kardl` model formulae.
#' @srrstats {TS1.2} Model-preparation routines check that the supplied variables can be used to construct the lagged and differenced terms required by the specification.
#' @srrstats {TS1.3} The package uses a central preparation workflow to parse formulae, construct lagged variables, and return a uniform internal model data structure.
#' @srrstats {TS1.4} Observational order is preserved during lag and difference construction so that the dynamic structure corresponds to the input time ordering.
#' @srrstats {TS1.5} Lag construction assumes that rows are supplied in the intended chronological order, and examples document this ordering requirement.
#' @srrstats {TS1.6} Users are expected to sort data before estimation; violations of chronological ordering would change the constructed lags and therefore the estimated dynamic model.
#' @srrstats {TS1.8} The package works with ordered observations rather than calendar arithmetic, so period interpretation is determined by the frequency and ordering of the user-supplied data.
#' @srrstats {TS2.0} ARDL estimation assumes a regular ordered sequence after preprocessing; missing observations created by lagging are handled explicitly in the model sample.
#' @srrstats {TS2.1} Missing-data handling is documented as part of model preparation and estimation rather than as a separate imputation system.
#' @srrstats {TS2.1a} Inputs that cannot form an estimable model after missing-value handling generate errors through the estimation workflow.
#' @srrstats {TS2.1b} Structurally missing observations caused by lagging and differencing are removed from the fitted sample.
#' @srrstats {TS2.1c} The package does not impute missing time-series observations because that would impose additional assumptions not inherent in ARDL estimation.
#' @srrstats {TS2.2} The documentation discusses stationarity requirements in the context of ARDL bounds testing and the usual I(0) and I(1) integration-order framework.
#' @srrstats {TS2.3} Vignettes and function documentation explain that ARDL bounds testing is intended for variables integrated of order zero or one, not for I(2) processes.
#' @srrstats {TS2.4} Users are expected to assess integration order before applying bounds-test interpretations, and the documentation states these modelling assumptions.
#' @srrstats {TS2.4a} The package reports model diagnostics and test results to help users assess the fitted dynamic specification.
#' @srrstats {TS2.4b} Users can transform variables before modelling, including differencing, logs, or asymmetric partial sums when appropriate for the research design.
#' @srrstats {TS2.5} The package does not compute autocovariance matrices as primary outputs; lag order and model sample information are stored in the fitted object.
#' @srrstats {TS2.6} The package does not compute autocovariance matrices with physical units; time interpretation follows the user's ordered data.
#' @srrstats {TS4.0} Fitted models and post-estimation results are returned as classed objects with dedicated print, summary, plot, and extraction methods.
#' @srrstats {TS4.0a} The package returns model-specific objects rather than converting outputs back to the original data class.
#' @srrstats {TS4.0b} Return values use explicit classes such as `kardl_lm`, `kardl_mplier`, and related summary classes.
#' @srrstats {TS4.1} Units are not inferred or altered by the package; coefficient and multiplier interpretation follows the units of the user-supplied variables.
#' @srrstats {TS4.2} Documentation describes the list components and classes returned by fitted model, long-run, test, and multiplier functions.
#' @srrstats {TS4.3} Dynamic multiplier outputs include the horizon index so that multiplier paths can be interpreted over the modelled time steps.
#' @srrstats {TS4.4} The package does not automatically transform forecast data; any transformations applied by the user remain part of the model interpretation.
#' @srrstats {TS4.5} The package documents coefficient and multiplier interpretation rather than providing automatic back-transformation of user-transformed data.
#' @srrstats {TS4.5a} Users who transform variables before estimation are responsible for interpreting or back-transforming results according to their research design.
#' @srrstats {TS4.5b} Examples show estimation and multiplier calculation on the scale of the variables supplied to the model.
#' @srrstats {TS4.5c} Limitations of interpreting transformed variables are discussed through the model documentation and vignettes.
#' @srrstats {TS5.0} The package implements plot methods for classed outputs, including long-run results and dynamic multiplier objects.
#' @srrstats {TS5.1} Dynamic multiplier plots label the horizontal axis as the horizon or time-step index.
#' @srrstats {TS5.2} The horizon or time-step variable is plotted on the horizontal axis in dynamic multiplier visualisations.
#' @srrstats {TS5.3} Multiplier plots display the horizon index used in the model output, with interpretation tied to the data frequency supplied by the user.
#' @srrstats {TS5.5} Plot methods operate on computed model and multiplier outputs after missing-value handling in the fitted object.
#' @srrstats {TS5.7} Dynamic multiplier plots display the computed response path from the fitted model over the selected horizon.
#' @srrstats {TS5.8} Plot methods distinguish multiplier components by variable and shock direction where asymmetric effects are present.
#'
#'
#' @noRd
#'
detectVars <-function(spec){

  AllAsymVars<-unique(c(spec$extractedInfo$ALvars,spec$extractedInfo$ASvars))
  if(length(spec$extractedInfo$ASvars)>0){
    for (x in spec$extractedInfo$ASvars ){
      if(!(x %in% spec$extractedInfo$independentVars )){stop("Attention! The Short-run asymmetric variable: ", x, " not found in the main vars list!", call. = FALSE)}
    }
  }
  if(length(spec$extractedInfo$ALvars)>0){
    for (x in spec$extractedInfo$ALvars ){
      if(!(x %in% spec$extractedInfo$independentVars)){stop("Attention! The Long-run asymmetric variable: ", x, " not found in the main vars list!", call. = FALSE)}
    }
  }
  for (x in spec$extractedInfo$Allvars ){
    if(!(x %in% colnames(spec$argsInfo$data))){stop("Attention! The variable: ", x, " not found in the data file's vars list!", call. = FALSE)}
  }
  if(length(spec$extractedInfo$deterministic)>0){
    for (x in spec$extractedInfo$deterministic ){
      if((x %in% spec$extractedInfo$Allvars)){stop("Attention! The external variable: ", x, "  FOUND in the main vars list! The exegenious variables should be excluded from the main list", call. = FALSE)}
      if(!(x %in% colnames(spec$argsInfo$data))){stop("Attention! The variable: ", x, " not found in the data file's vars list!", call. = FALSE)}
    }
  }

  baslik<-c(spec$extractedInfo$dependentVar)
  baslik2<-c(spec$extractedInfo$dependentVar)
  for (x in  spec$extractedInfo$independentVars){
    if(x %in% spec$extractedInfo$ASvars){
      ## c(paste0(AsymPrefix[1],i,inputs$AsymSuffix[1])
      ##  baslik<-c(baslik,paste0(x,".NEG"));baslik<-c(baslik,paste0(x,".POS"));
      baslik<-c(baslik,paste0(.kardl_Settings_env$AsymPrefix[1],x,.kardl_Settings_env$AsymSuffix[1]))
      baslik<-c(baslik,paste0(.kardl_Settings_env$AsymPrefix[2],x,.kardl_Settings_env$AsymSuffix[2]))
      #   if(DifferentAsymLag) {} else baslik<-c(baslik,x)
    }else{
      baslik<-c(baslik,x)
    }
    if(x %in% spec$extractedInfo$ALvars){
      baslik2<-c(baslik2,paste0(.kardl_Settings_env$AsymPrefix[1],x,.kardl_Settings_env$AsymSuffix[1]))
      baslik2<-c(baslik2,paste0(.kardl_Settings_env$AsymPrefix[2],x,.kardl_Settings_env$AsymSuffix[2]))
    }else{
      baslik2<-c(baslik2,x)
    }
  }
  shortRunVars<-baslik
  longRunVars<-baslik2

  # independent variables short-run asymmetric excluded
  indepASexcluded<-spec$extractedInfo$independentVars[! spec$extractedInfo$independentVars %in% spec$extractedInfo$ASvars]
  # independent variables long-run asymmetric excluded
  indepALexcluded<-spec$extractedInfo$independentVars[! spec$extractedInfo$independentVars %in% spec$extractedInfo$ALvars]
  method<-""
  if(length( spec$extractedInfo$ALvars)>0 & length( spec$extractedInfo$ASvars)>0 ){  method<-"NN"}
  else if(length( spec$extractedInfo$ALvars)>0 ){method<-"SA"}
  else if(length( spec$extractedInfo$ASvars)>0 ){method<-"AS"}
  else {method<-"LL"}

  # calculating the column numbers for each row
  shortrunLength<-length(indepASexcluded)+length(spec$extractedInfo$ASvars)*(if(spec$argsInfo$differentAsymLag) 2 else 1)
  # calculating the row numbers for all of possibilities of lags
  lagRowsNumber <- (spec$argsInfo$maxlag^(shortrunLength+1))-(spec$argsInfo$maxlag^(shortrunLength))

  extractedInfo<-list(
    AllAsymVars=AllAsymVars,
    indepASexcluded=indepASexcluded,
    indepALexcluded=indepALexcluded,
    shortRunVars=shortRunVars,
    longRunVars=longRunVars,
    shortrunLength=shortrunLength,
    lagRowsNumber=lagRowsNumber,
    modelType=method
    # desc= "Make the varibles list and checking their existence in the data set"
  )



  spec$extractedInfo <- lmerge( spec$extractedInfo , extractedInfo)
  spec
}

#' Create new variables for the model specification
#'
#' This function creates new variables for the model specification based on the extracted information about asymmetric variables and the original data. It generates new variables for positive and negative changes of asymmetric variables, as well as lagged differences for short-run and long-run variables. It also adds a trend variable if specified in the model. The function updates the data in the model specification with the newly created variables and returns the updated specification.
#' @param spec A list containing the model specification, including extracted information about variables and the
#' original data.
#' @return The updated model specification with new variables created and added to the data. The
#' data in the model specification is updated with the new variables, and the original data is removed from the specification to avoid confusion. If any variable is not found in the data, an error is raised with a descriptive message.
#' @srrstats {G2.4b} Numeric model variables are converted to numeric vectors or matrix columns suitable for OLS before estimation.
#' @srrstats {G2.4c} Variable names and parsed formula terms are handled as character vectors when constructing lagged-variable names and asymmetric components.
#' @srrstats {G2.8} The original formula and data are converted into an internal structure containing the dependent variable, regressors, lag information, and generated variables.
#' @srrstats {G2.9} Generated variable names for lagged, differenced, and asymmetric components are constructed transparently so users can inspect the resulting model object.
#' @srrstats {TS1.4} Observational order is preserved during lag and difference construction.
#' @srrstats {TS2.1b} Structurally missing observations caused by lagging and differencing are removed from the fitted sample.
#' @srrstats {TS2.1c} The function does not impute missing time-series observations.
#' @noRd

CreateNewVars <-function( spec){
  # Create and adding asymmetric variables to the data
  tempData<-spec$argsInfo$data
  for (x in  spec$extractedInfo$AllAsymVars ){
    posName<-paste0(.kardl_Settings_env$AsymPrefix[1],x,.kardl_Settings_env$AsymSuffix[1]) ### posName<- paste0(x,".POS")
    negName<-paste0(.kardl_Settings_env$AsymPrefix[2],x,.kardl_Settings_env$AsymSuffix[2]) ### negName<- paste0(x,".NEG")
    if(! posName %in% colnames(tempData)){
      varNames<-colnames(tempData)
      gecici<-diff(tempData[,x])
      KARAMELpos<-KARAMELneg<-c(NA)
      data3<-cbind(tempData[,1],gecici,KARAMELpos,KARAMELneg)
      pos<-data3[,"KARAMELpos"]
      neg<-data3[,"KARAMELneg"]
      gecici<-data3[,"gecici"]
      for (i in seq_len(nrow(tempData))) {
        if(is.na(gecici[i])){
          pos[i]<-neg[i]<-NA
        }
        else{
          if(  gecici[i]>0){
            pos[i]<-gecici[i]+ifelse(is.na(pos[i-1]), 0, pos[i-1])
            neg[i]<-ifelse(is.na(neg[i-1]), 0, neg[i-1])

          }
          if(  gecici[i]<0){
            neg[i]<-gecici[i]+ifelse(is.na(neg[i-1]), 0, neg[i-1])
            pos[i]<-ifelse(is.na(pos[i-1]), 0, pos[i-1])
          }
          if(  gecici[i] == 0){
            pos[i]<-pos[i-1]
            neg[i]<-neg[i-1]
          }
        }
      }
      tempData<-cbind(tempData,pos,neg)
      colnames(tempData)<-c(varNames,posName,negName)

    }
  }



  varAdlari<-colnames(tempData)
  AsShortlar<-unlist(lapply(spec$extractedInfo$ASvars, function(i){c(paste0(.kardl_Settings_env$AsymPrefix[1],i,.kardl_Settings_env$AsymSuffix[1]),paste0(.kardl_Settings_env$AsymPrefix[2],i,.kardl_Settings_env$AsymSuffix[2]))})) ## AsShortlar<- as.vector(outer(spec$extractedInfo$ASvars,c("POS","NEG"), paste, sep="."))
  shortRunVars<-c(spec$extractedInfo$dependentVar,spec$extractedInfo$indepASexcluded,AsShortlar)

  for (x in shortRunVars ){
    fark<-diff(tempData[,x])
    m<-fark
    for(j in 0:spec$argsInfo$maxlag){
      newVarName<-replace_lag_var(.kardl_Settings_env$ShortCoef ,x,j) #paste0("L",j,".d.",x)
      if(! newVarName %in% colnames(tempData)){
        # y<-inputs$data[,x] # boyutunu alsın diye
        y<-c()
        y[1:(j+1)]<-NA
        y[(j+2):(length(tempData[,x]))]<-fark[1:(length(tempData[,x])-1-j)]
        y<-as.ts(y)
        tempData<-cbind(tempData,y)
        varAdlari<-c(varAdlari,newVarName)
        colnames(tempData)<-varAdlari
      }
    }
  }

  AsLonglar<-unlist(lapply(spec$extractedInfo$ALvars, function(i){c(paste0(.kardl_Settings_env$AsymPrefix[1],i,.kardl_Settings_env$AsymSuffix[1]),paste0(.kardl_Settings_env$AsymPrefix[2],i,.kardl_Settings_env$AsymSuffix[2]))}))## AsLonglar<- as.vector(outer(ALvars,c("POS","NEG"), paste, sep="."))
  longRunVars<-c(spec$extractedInfo$dependentVar,spec$extractedInfo$indepALexcluded,AsLonglar)

  for (x in longRunVars ){
    newVarName<-replace_lag_var(.kardl_Settings_env$LongCoef ,x,1) #paste0("L1.",x)
    if(! newVarName %in% colnames(tempData)){
      # y<-inputs$data[,x] # boyutunu alsın diye
      y<-c()
      y[1]<-NA
      y[2:length(tempData[,x])]<-tempData[,x][1:(length(tempData[,x])-1)]
      y<-as.ts(y)

      tempData<-cbind(tempData,y)
      varAdlari<-c(varAdlari,newVarName)
      colnames(tempData)<-varAdlari
    }
  }
  if(spec$extractedInfo$trend ){
    spec$extractedInfo$deterministic<-c(spec$extractedInfo$deterministic,"trend")
    if( ! "trend" %in% colnames(tempData)){
      trend1<-seq_len(nrow(tempData))
      tempData<-cbind(tempData,trend=c(trend1))
      varAdlari<-c(varAdlari,"trend")
      colnames(tempData)<-varAdlari
    }
  }
  spec$argsInfo$data<-NULL
  spec$extractedInfo$data<-tempData
  attr(spec$extractedInfo$data, "source") <- "spec$argsInfo$data"
  attr(spec$extractedInfo$data, "description") <-"This value was created by adding new variables to the original data set and removing the variables which are not used in the model."
  spec
}

#' Prepare the model specification for estimation
#'
#' This function prepares the model specification for estimation by combining variable types, checking user inputs, detecting variables, and creating new variables based on the model formula and data. It ensures that the inputs are defined properly, checks if the data is a time series, and updates the model specification with the necessary information for estimation.
#' @param inputs A list of user inputs, including the model formula and data.
#' @return The prepared model specification ready for estimation. If the inputs are not defined properly
#' or if the data is not a time series, an error is raised with a descriptive message.
#' @noRd

prepare<-function(inputs){
  # inputs<-convertListInputs(...)
  #estProcess <- list()
  # startTime<-Sys.time()

  spec<-combineVarTypes(inputs) #(lmerge(initial_,defaultVars()))
  #spec$timeinfo<- list(startTime=startTime)
  #spec<-lmerge(spec, estProcess) # in case of any changes in inputs during combineVarTypes function
  ####################################### BURDA
  if(is.null(spec)){
    stop("The inputs are not defined properly. Please check the inputs again.", call. = FALSE)
  }

  # here we will check whether the data is a time series or not.
  if(!is.data.frame(spec$argsInfo$data)){
    spec$argsInfo$data<-as.data.frame(spec$argsInfo$data) # convert to data frame
  }
  spec$argsInfo$data<-as.ts(spec$argsInfo$data[,c(spec$extractedInfo$Allvars[spec$extractedInfo$Allvars!="trend"],spec$extractedInfo$deterministic)]) # excluding trend. trend is going to be defined in the data later.
  spec<-CheckInputs(spec ) # It captures any changes during the checking process of the inputs. Especially user-defined nominations for each variable.
  spec<-detectVars(spec) # dependentVar independentVars  AllAsymVars   indepASexcluded   indepALexcluded method

  spec<-CreateNewVars( spec)
  spec
}

#' Create the short-run model formula based on the specified variables and lags
#'
#' This function generates the short-run model formula by combining the specified short-run variables, their corresponding lags, and any deterministic variables. It constructs the formula in a format suitable for estimation, ensuring that the variable names and lag structures are correctly represented based on the model specification.
#' @param shortRunVars A vector of short-run variable names to be included in the
#' model formula.
#' @param LagsList A list of lag values corresponding to each short-run variable,
#' indicating how many lags of each variable should be included in the model.
#' @param deterministic A vector of deterministic variable names to be included in the
#' model formula, if any.
#' @return A character string representing the short-run model formula, combining the specified variables,
#' their lags, and deterministic variables in the appropriate format for estimation.
#' @noRd
#'

makeShortrunMOdel<-function(shortRunVars,LagsList, deterministic){
  modd1<-c()
  for (j in seq_along(shortRunVars)){ ## to enhancing the performance it can be used shortRunVarsLength instead of length(shortRunVars) to avoiding recalcuting length at each loop.
    start<-as.numeric(j<2)
    # modd1<-c(modd1,lapply(start:LagsList[j], function(i,y){sprintf(paste0("L%d.d.",y),i)},y=spec$extractedInfo$shortRunVars[j]))
    # Using for loop has better performance than lapply
    for (k in start:LagsList[j]) {
      modd1<-c(modd1,replace_lag_var(.kardl_Settings_env$ShortCoef,shortRunVars[j],k) #paste0("L",k,".d.",shortRunVars[j])
      )
    }
  }
  paste(c(modd1,deterministic),collapse="+")
}

#' Create the long-run model formula based on the specified variables and lags
#'
#' This function generates the long-run model formula by combining the specified long-run variables, their corresponding lags, and any deterministic variables. It constructs the formula in a format suitable for estimation, ensuring that the variable names and lag structures are correctly represented based on the model specification. If a long-run part is already defined in the user inputs, it uses that instead of generating a new formula.
#' @param spec A list containing the model specification, including extracted information about variables and user
#' inputs. The function checks if a long-run part is already defined in the user inputs and uses it if available; otherwise, it generates the long-run model formula based on the specified long-run variables and their lags.
#' @return A character string representing the long-run model formula, combining the specified variables,
#' their lags, and deterministic variables in the appropriate format for estimation. If a long-run part is already defined in the user inputs, it returns that instead of generating a new formula.
#' @noRd
#'

makeLongrunMOdel<-function(spec){
  if(!is.null(spec$argsInfo$longRunPart  )){
    LS_longrun<-spec$argsInfo$longRunPart
  }else{
    modd<-lapply(spec$extractedInfo$longRunVars,function(i){ replace_lag_var(.kardl_Settings_env$LongCoef ,i,1) # paste0("L1.",i)
    } )
    LS_longrun<-paste(modd, collapse = "+")


  }
    if(spec$extractedInfo$noConstant){
      LS_longrun<-paste0(LS_longrun,"-1")
    }

  LS_dependent<- replace_lag_var(.kardl_Settings_env$ShortCoef,spec$extractedInfo$dependentVar,0) # paste0("L0.d.",spec$extractedInfo$dependentVar)
  list(LS_longrun=LS_longrun,
       LS_dependent=LS_dependent
  )
}

# =============================================================================
# Global settings environment
# =============================================================================

.kardl_Settings_env <- new.env(parent = emptyenv())

# Default values stored as a list (used for reset)
.kardl_Settings_env_default <- list(
  data             = NULL,
  formula          = NULL,
  maxlag           = 4L,
  mode             = "quick",
  criterion        = "AIC",
  differentAsymLag = TRUE,
  AsymPrefix       = c(),
  AsymSuffix       = c("_POS", "_NEG"),
  LongCoef         = "L{lag}.{varName}",
  ShortCoef        = "L{lag}.d.{varName}",
  batch            = "1/1"
)

# Initialize the environment with defaults
local({
  for (nm in names(.kardl_Settings_env_default)) {
    .kardl_Settings_env[[nm]] <- .kardl_Settings_env_default[[nm]]
  }
})

#' Set kardl Package Options
#'
#' This function allows users to set options for the kardl package. Users can specify named arguments to set options or call the function without arguments to retrieve all current settings.
#'
#' @param ... Named arguments corresponding to the options to be set. Valid option names include those defined in the kardl package settings.
#'
#' @return All current settings as a list after applying any updates from the provided named arguments invisibly.
#'
#'
#' @export
#' @examples
#' # Get default options
#' kardl_get("maxlag", "mode")
#' # Set options
#' kardl_set(maxlag = 5, mode = "grid")
#' # Get specific options
#' kardl_get("maxlag", "mode")
#'
#' # To have the updated settings available in the global environment, assign the output to a variable:
#' MySettings<- kardl_set(LongCoef = "LongRun_{varName}", ShortCoef = "ShortRun_{varName}")
#' # Now MySettings contains the updated settings, and the kardl package
#' # will use these settings for subsequent operations.
#' MySettings$LongCoef
#' MySettings$maxlag
#'
#' # Reset to defaults after demonstrating custom settings
#' kardl_reset()
#'
#' @return If no arguments are provided, returns all options as a list. If named arguments are provided, sets those options and returns the updated list.
#' @seealso  \code{\link{kardl_get}}, \code{\link{kardl_reset}}
#'
#' @srrstats {G2.3a} Package-level option names are validated against a predefined list of acceptable values; invalid names produce an informative error.
#' @srrstats {G2.4} Default argument values for `formula`, `data`, `maxlag`, `mode`, `criterion`, and `differentAsymLag` are stored centrally and merged with function arguments at call time.
#'



kardl_set <- function(...) {
  args <- list(...)

  # Set named arguments
  if (length(args) > 0 && !is.null(names(args))) {
    invalid <- setdiff(names(args), names(.kardl_Settings_env_default))
    if (length(invalid) > 0) {
      stop(sprintf("Invalid option(s): %s", toString(invalid)),
           call. = FALSE)
    }

    # Assign new values
    for (name in names(args)) {
      .kardl_Settings_env[[name]] <- args[[name]]
    }
  }

  # Return behavior

    return(invisible(as.list(.kardl_Settings_env)))

}



#' Get kardl Package Options
#'
#' This function retrieves the current settings of the kardl package. Users can specify option names to get their values or call the function without arguments to retrieve all current settings.
#'
#' @param ... Option names to retrieve. If no arguments are provided, all options will be returned.
#'
#' @export
#' @examples
#'
#' # Get all options
#' kardl_get()
#' # Get specific options
#' kardl_get("maxlag", "mode")
#'
#' # Note: In interactive use, avoid calling kardl_get() directly to prevent cluttering the console.
#'
#' a<-kardl_get()
#' a$AsymSuffix
#'
#' # To reset options and then get a default option:
#' mydefaults <- kardl_reset()
#' mydefaults$maxlag
#'
#' @return If no arguments are provided, returns all options as a list. If specific option names are provided, returns their values.
#'
#' @seealso \code{\link{kardl_set}}, \code{\link{kardl_reset}}
#'
#' @srrstats {G2.3a} Requested option names are validated against the predefined settings list; unknown names produce an informative error.
#'
kardl_get <- function(...) {
  names_requested <- c(...)

  # Return all settings if no arguments
  if (length(names_requested) == 0) {
    return(as.list(.kardl_Settings_env))
  }

  # Check for invalid names
  invalid <- setdiff(names_requested, names(.kardl_Settings_env_default))
  if (length(invalid) > 0) {
    stop(sprintf("Option(s) not found: %s", toString(invalid)),
         call. = FALSE)
  }

  # Extract requested values
  result <- lapply(names_requested, function(name) {
    .kardl_Settings_env[[name]]
  })
  names(result) <- names_requested

  # Return single value directly if only one requested
  if (length(result) == 1) {
    return(result[[1]])
  } else {
    return(result)
  }
}


#' Reset kardl Package Options to Default Values
#'
#' This function resets kardl package options to their default values.
#'
#' @param exclude Character vector of setting names that should not be reset.
#' These settings retain their current values. By default, all settings are reset.
#'
#' @return A list of the settings after reset, returned invisibly.
#' @export
#' @seealso \code{\link{kardl_set}}, \code{\link{kardl_get}}
#' @examples
#' kardl_set(criterion = "BIC", differentAsymLag = FALSE)
#'
#' # Reset all settings to defaults except "criterion"
#' kardl_reset(exclude = "criterion")
#'
#' # Get the current settings to verify the reset
#' print(kardl_get("criterion"))
#'
#' # This will show "BIC" since it was excluded from the reset,
#' #while other settings will be reset to their defaults.
#' print(kardl_get("differentAsymLag"))

kardl_reset <- function(exclude = NULL) {
  # Validate input
  if (!is.null(exclude) && !is.character(exclude)) {
    stop("`exclude` must be a character vector of setting names.")
  }

  # Get all setting names
  all_names <- names(.kardl_Settings_env_default)

  # Check validity of exclude
  if (!is.null(exclude)) {
    invalid <- setdiff(exclude, all_names)
    if (length(invalid) > 0) {
      stop("Invalid setting(s) in `exclude`: ", toString(invalid))
    }
  }

  to_reset <- setdiff(all_names, exclude)
  rm(list = to_reset, envir = .kardl_Settings_env)

  # Repopulate defaults only for those
  for (nm in to_reset) {
    .kardl_Settings_env[[nm]] <- .kardl_Settings_env_default[[nm]]
  }

  invisible(as.list(.kardl_Settings_env))
}

#' @srrstats {G5.2} Tests exercise successful calls and error conditions for
#' the kardl package settings system (`kardl_set()`, `kardl_get()`,
#' `kardl_reset()`).
#' @srrstats {G5.3} Checks that default settings match documented values
#' (maxlag = 4L, criterion = "AIC", differentAsymLag = TRUE).
#' @srrstats {G5.4a} Uses deterministic known defaults to verify the settings
#' round-trip through set -> get -> reset.
test_that("Settings can be set, retrieved, and reset", {
  kardl_reset()
  default_settings <- kardl_get()

  expect_identical(default_settings$maxlag, 4L)
  expect_identical(default_settings$criterion, "AIC")
  expect_true(default_settings$differentAsymLag)

  kardl_set(maxlag = 5, criterion = "BIC", differentAsymLag = FALSE)
  new_settings <- kardl_get()

  expect_identical(new_settings$maxlag, 5)
  expect_identical(new_settings$criterion, "BIC")
  expect_false(new_settings$differentAsymLag)

  kardl_reset()
  reset_settings <- kardl_get()

  expect_identical(reset_settings$maxlag, 4L)
  expect_identical(reset_settings$criterion, "AIC")
  expect_true(reset_settings$differentAsymLag)
})

# Create an environment to store settings
.kardl_Settings_env <- new.env(parent = emptyenv())

# Initialize settings in the environment
.kardl_Settings_env$data<-NULL
.kardl_Settings_env$formula<-NULL
.kardl_Settings_env$maxlag <- 4
.kardl_Settings_env$mode<-"quick"
.kardl_Settings_env$criterion <- "AIC"
.kardl_Settings_env$differentAsymLag <- TRUE
.kardl_Settings_env$AsymPrefix <- c()
.kardl_Settings_env$AsymSuffix <- c("_POS", "_NEG")
.kardl_Settings_env$LongCoef <- "L{lag}.{varName}" #  L{lag}.{varName}    L1.LRM
.kardl_Settings_env$ShortCoef <- "L{lag}.d.{varName}" # L{lag}.d.{varName}  L2.d.LRM
.kardl_Settings_env$batch <- "1/1"

# Store default settings as a list
.kardl_Settings_env_default <- as.list(.kardl_Settings_env)

#' Function to Set KARDL Package Options
#'
#' This function allows users to set options for the kardl package. Users can specify named arguments to set options or call the function without arguments to retrieve all current settings.
#'
#' @param . If provided and not `FALSE`, the function will return this value after setting the options. If not provided or set to `FALSE`, it will return the current settings.
#' @param ... Named arguments corresponding to the options to be set. Valid option names include those defined in the kardl package settings.
#'
#' @return If no arguments are provided, returns all options as a list. If named arguments are provided, sets those options and returns the updated list.
#'
#'
#' @export
#' @examples
#' # Set options
#' kardl_set(maxlag = 5, mode = "grid")
#' # Get all options
#' kardl_get()
#' # Get specific options
#' kardl_get("maxlag", "mode")
#'
#' # Note: In interactive use, avoid calling kardl_get() directly to prevent cluttering the console.
#'
#' \donttest{
#' kardl_get()
#' }
#'
#' # Example with magrittr pipe
#' library(magrittr)
#' # Set custom coefficient naming conventions
#'
#' MyFormula<-CPI~ER+PPI+asym(ER)+deterministic(covid)+trend
#' kardl_set(ShortCoef = "L___{lag}.d.{varName}", formula = MyFormula, data = imf_example_data)
#' imf_example_data %>%   kardl(MyFormula)
#'
#' kardl_reset()
#' kardl_get()
#'
#' imf_example_data %>%  kardl_set(LongCoef= "LK{lag}_{varName}",ShortCoef = "D{lag}.d.{varName}") %>%
#' kardl(MyFormula)
#' kardl_get(c("LongCoef","ShortCoef"))
#'
#' @return If no arguments are provided, returns all options as a list. If named arguments are provided, sets those options and returns the updated list.
#' @seealso  \code{\link{kardl_get}}, \code{\link{kardl_reset}}
#'
#'



kardl_set <- function(.=FALSE, ...) {
  args <- list(...)

  # Named arguments: set options
  if (!is.null(names(args))) {
    invalid <- setdiff(names(args), names(.kardl_Settings_env))
    if (length(invalid) > 0) {
      stop(sprintf("Invalid option(s): %s", paste(invalid, collapse = ", ")))
    }
    # Assign new values to the environment
    for (name in names(args)) {
      .kardl_Settings_env[[name]] <- args[[name]]
    }
  }

  # Return based on whether . exists and is non-NULL
  if (!missing(.) && !is.null(.) && !isFALSE(.)) {
    return(.)
  } else {
    return(invisible(as.list(.kardl_Settings_env)))
  }
}



#' Function to Get KARDL Package Options
#'
#' This function retrieves the current settings of the kardl package. Users can specify option names to get their values or call the function without arguments to retrieve all current settings.
#'
#' @param ... Option names to retrieve. If no arguments are provided, all options will be returned.
#'
#' @export
#' @examples
#'
#' # Get all options
#' kardl_get()
#' # Get specific options
#' kardl_get("maxlag", "mode")
#'
#' # Note: In interactive use, avoid calling kardl_get() directly to prevent cluttering the console.
#'
#' a<-kardl_get()
#' a$AsymSuffix
#'
#' @return If no arguments are provided, returns all options as a list. If specific option names are provided, returns their values.
#'
#' @seealso \code{\link{kardl_set}}, \code{\link{kardl_reset}}
#'
kardl_get <- function(...) {
  names_requested <- c(...)

  # No arguments: return all options as a list
  if (length(names_requested) == 0) {
    return(as.list(.kardl_Settings_env))
  }

  # Check for invalid option names
  invalid <- setdiff(names_requested, names(.kardl_Settings_env))
  if (length(invalid) > 0) {
    stop(sprintf("Option(s) not found: %s", paste(invalid, collapse = ", ")))
  }

  # Retrieve requested options
  result <- lapply(names_requested, function(name) .kardl_Settings_env[[name]])

  # Name the result list
  names(result) <- names_requested

  # If only one option is requested, return the value directly
  if (length(result) == 1) {
    return(result[[1]])
  } else {
    return(result)
  }
}


#' Function to Reset KARDL Package Options to Default Values
#'
#' This function resets all options in the kardl package to their default values.
#'
#' @param . If provided and not `FALSE`, the function will return this value after resetting the settings. If not provided or set to `FALSE`, it will return the current settings.
#'
#' @return If resetting options, returns the provided value (if any) or invisibly
#' returns the current settings as a list.
#' @export
#' @examples
#' # Set some options
#' kardl_set(criterion = "BIC", differentAsymLag = TRUE)
#'
#' # Reset to default options
#' kardl_get("criterion")  # Check current settings
#' kardl_reset()
#' kardl_get("criterion")  # Check settings after reset
#'
#' library(magrittr)
#'  MyFormula<-CPI~ER+PPI+asym(ER)+deterministic(covid)+trend
#' imf_example_data %>%
#'   kardl_set(LongCoef= "K1{lag}w1{varName}",differentAsymLag= FALSE ) %>%  kardl(MyFormula ) %>%
#'     kardl_reset()
#' kardl_get()
#'
#' imf_example_data %>%
#'   kardl_reset() %>%
#'     kardl_set(LongCoef= "K2{lag}w2{varName}",differentAsymLag=FALSE ) %>%  kardl(MyFormula)
#'
#' kardl_get(c("LongCoef","differentAsymLag","ShortCoef","batch"))
#'
#' @return If resetting options, returns the provided value (if any) or invisibly
#' returns the current settings as a list.
#' @seealso \code{\link{kardl_set}}, \code{\link{kardl_get}}
#'
kardl_reset <- function(.=FALSE) {
  # Reinitialize the environment with default settings
  for (name in names(.kardl_Settings_env_default)) {
    .kardl_Settings_env[[name]] <- .kardl_Settings_env_default[[name]]
  }
  # Return based on whether . exists and is non-NULL
  if (!missing(.) && !is.null(.) && !isFALSE(.)) {
    return(.)
  } else {
    return(invisible(as.list(.kardl_Settings_env)))
  }
}

#' Symmetry Test for Nonlinear kardl Models
#'
#' This function performs symmetry tests on non-linear KARDL models to assess whether the effects of positive and negative changes in independent variables are statistically different.
#'
#' This function evaluates whether the inclusion of a particular variable in the model follows a linear relationship or exhibits a non-linear pattern. By analyzing the behavior of the variable, the function helps to identify if the relationship between the variable and the outcome of interest adheres to a straight-line assumption or if it deviates, indicating a non-linear interaction. This distinction is important in model specification, as it ensures that the variable is appropriately represented, which can enhance the model's accuracy and predictive performance.
#'
#' @description
#' The symmetry test is a statistical procedure used to assess the presence of symmetry in the relationship between variables in a model. It is particularly useful in econometric analysis, where it helps to identify whether the effects of changes in one variable on another are symmetric or asymmetric. The test involves estimating a model that includes both positive and negative components of the variables and then performing a Wald test to determine if the coefficients of these components are significantly different from each other.
#' If the test indicates significant differences, it suggests that the relationship is asymmetric, meaning that the impact of increases and decreases in the variables differs.
#' This test returns results for both long-run and short-run variables in a KARDL model. Where applicable, it provides the Wald test statistics, p-values, degrees of freedom, sum of squares, and mean squares for each variable tested.
#' If the null hypothesis of symmetry is rejected, it indicates that the effects of positive and negative changes in the variable are significantly different, suggesting an asymmetric relationship.
#'
#' The non-linear model with one asymmetric variables is specified as follows:
#' \deqn{
#'  \Delta{y_{t}} = \psi + \eta_{0}y_{t - 1} + \eta^{+}_{1} x^{+}_{t - 1}+ \eta^{-}_{1} x^{-}_{t - 1} + \sum_{j = 1}^{p}{\gamma_{j}\Delta y_{t - j}} + \sum_{j = 0}^{q}{\beta^{+}_{j}\Delta x^{+}_{t - j}}  + \sum_{j = 0}^{m}{\beta^{-}_{j}\Delta x^{-}_{t - j}} + e_{t}
#' }
#'
#' This function performs the symmetry test both for long-run and short-run variables in a kardl model. It uses the \code{\link[nlWaldTest]{nlWaldtest}} function from the \pkg{nlWaldTest} package for long-run variables and the \code{\link[car]{linearHypothesis}} function from the \pkg{car} package for short-run variables.
#' The hypotheses for the long-run variables are:
#' \deqn{
#'     H_{0}: -\frac{\eta^{+}_{1}}{\eta_{0}} = -\frac{\eta^{-}_{1}}{\eta_{0}} }
#' \deqn{    H_{1}: -\frac{\eta^{+}_{1}}{\eta_{0}} \neq -\frac{\eta^{-}_{1}}{\eta_{0}}
#'  }
#'
#'  The hypotheses for the short-run variables are:
#'   \deqn{
#'   H_{0}: \sum_{j = 0}^{q}{\beta^{+}_{j}} = \sum_{j = 0}^{m}{\beta^{-}_{j}} }
#'   \deqn{H_{1}: \sum_{j = 0}^{q}{\beta^{+}_{j}} \neq \sum_{j = 0}^{m}{\beta^{-}_{j}}
#'   }
#'
#'
#'
#' @param kmodel The kardl obejct
#' @param vars A character vector specifying the names of the variables to be included in the symmetry test. If NULL (the default), all eligible variables will be tested. The variable names should match those used in the KARDL model, and they can be either long-run or short-run variables. If any specified variable is not found in the model, an error will be raised.
#' @param component A character string specifying which component(s) of the model to test for symmetry. The options are "both" (default), "shortrun", or "longrun". If "both" is selected, the function will perform symmetry tests for both long-run and short-run variables. If "shortrun" is selected, only short-run variables will be tested, and if "longrun" is selected, only long-run variables will be tested. Invalid values will result in an error.
#' @param type A character string specifying the type of test statistic to be used in the Wald tests. The options are "F" (default) or "Chisq". If "F" is selected, the function will perform an F-test, which is appropriate for smaller sample sizes. If "Chisq" is selected, the function will perform a chi-squared test, which is more suitable for larger sample sizes. Invalid values will result in an error.
#' @param ... Additional arguments to be passed to the underlying test functions, such as \code{\link[nlWaldTest]{nlWaldtest}} for long-run tests or \code{\link[car]{linearHypothesis}} for short-run tests. These arguments can include options for controlling the behavior of the tests, such as specifying the type of test statistic or adjusting for multiple comparisons.
#'
#' @return A list with class "kardl" containing the following components:
#' \itemize{
#' \item \code{Lwald:} A data frame containing the Wald test results for the long-run variables, including F-statistic, p-value, degrees of freedom, and residual degrees of freedom.
#' \item \code{Lhypotheses:} A list containing the null and alternative hypotheses for the long-run variables.
#' \item \code{Swald:} A data frame containing the Wald test results for the short-run variables, including F-statistic, p-value, degrees of freedom, residual degrees of freedom, and sum of squares.
#' \item \code{Shypotheses:} A list containing the null and alternative hypotheses for the short-run variables.
#' }
#' @srrstats {G1.0} This function cites the original source of the symmetry test for non-linear ARDL models, which is based on the work of Shin, Yu, and Greenwood-Nimmo (2014).
#' @srrstats {G1.1} The function implements the symmetry test for non-linear ARDL models as described in the original paper by Shin, Yu, and Greenwood-Nimmo (2014). It uses the appropriate test statistics and follows the correct procedures for conducting the tests.
#' @srrstats {G1.3} Statistical terms such as NULL hypothesis, alternative hypothesis, test statistic are defined in the context of the symmetry test for non-linear ARDL models.
#' @srrstats {G2.0} The function validates that `kmodel` is of class `kardl_lm` and rejects ECM objects that are not suitable for symmetry testing.
#' @srrstats {G2.3} The `component` argument is matched against `c("both", "shortrun", "longrun")` and `type` against `c("F", "Chisq")` using `match.arg()`.
#' @srrstats {G5.2b} Tests include error expectations for incompatible object classes and invalid post-estimation inputs.

#' @export
#' @import nlWaldTest
#' @importFrom car linearHypothesis
#' @references Shin, Y., Yu, B., & Greenwood-Nimmo, M. (2014). Modelling asymmetric cointegration and dynamic multipliers in a nonlinear ARDL framework. Festschrift in honor of Peter Schmidt: Econometric methods and applications, 281-314.
#' @seealso   \code{\link{kardl}}, \code{\link{pssf}}, \code{\link{psst}}, \code{\link{ecm}}, \code{\link{narayan}}
#' @examples
#'

#'  kardl_model<-kardl(imf_example_data,
#'                     CPI~Lasym(PPI+ER)+Sas(ER)+deterministic(covid)+trend)
#'  ast<- symmetrytest(kardl_model)
#'  ast
#'  # Detailed results of the test:
#'  summary(ast)
#'  # The null hypothesis of the test is that the model is symmetric, while the alternative
#'  # hypothesis is that the model is asymmetric. The test statistic and p-value are provided
#'  # in the output. If the p-value is less than a chosen significance level (e.g., 0.05),
#'  # we reject the null hypothesis and conclude that there is evidence of asymmetry in the model.
#'
#'  # The default significance level is 0.05, but you can specify a different level using the 'level'
#'  # argument in the summary function. For example, to use a significance level of 0.01,
#'  # you can use the following code:
#'  summary(ast, level = 0.01)
#'
#'  # To get symmetry test results in long-run, you can use the following code:
#'  ast$Lwald
#'
#'  # To get symmetry test results in short-run, you can use the following code:
#'  ast$Swald
#'
#'  # To get the null and alternative hypotheses of the test in long-run,
#'  # you can use the following code:
#'
#'  ast$Lhypotheses
#'
#'  # To get the null and alternative hypotheses of the test in short-run,
#'  # you can use the following code:
#'
#'  ast$Shypotheses
#'  # Alternatively, you can also use the symmetrytest function with the component
#'  # argument to specify whether you want to test for long-run or short-run symmetry.
#'  # For example, to test for long-run symmetry, you can use the following code:
#'  symmetrytest(kardl_model, component = "longrun")
#'  # To test for short-run symmetry, you can use the following code:
#'  symmetrytest(kardl_model, component = "shortrun")
#'
#'  # If you want to test for symmetry with respect to a specific variable,
#'  # you can use the vars argument in the symmetrytest function. For example,
#'  # to test for symmetry with respect to the PPI variable, you can use the following code:
#'  symmetrytest(kardl_model, vars = "PPI")
#'
#'  # To test for symmetry with respect to multiple variables, you can provide
#'  # a vector of variable names to the vars argument. For example, to test for
#'  # symmetry with respect to both PPI and ER, you can use the following code:
#'  symmetrytest(kardl_model, vars = c("PPI", "ER"))
#'
#'  # Finally, you can also specify the type of test statistic to be used in the
#'  # symmetry test. By default, the function uses the Wald test  F statistic,
#'  # but you can also choose to use the chi-squared test statistic.
#'  # For example, to use the chi-squared test statistic, you can use the following code:
#'  symmetrytest(kardl_model, type="Chisq")
#'
#'

symmetrytest <- function(kmodel,
                         vars = NULL,
                         component =  "both" ,
                         type =  "F" ,
                         ...) {

  component <- match.arg(tolower(component),choices = c("both", "shortrun", "longrun"))
  # to let users input "f" or "chisq" in any case format, we will standardize the input to "F" or "Chisq"
  newType<-paste0(toupper(substr(type, 1, 1)), tolower( substr(type, 2, nchar(type))))
  type <- match.arg(newType,choices= c("F", "Chisq"))

  if (!inherits(kmodel, "kardl_lm")) {
    stop("Not suitable input.", call. = FALSE)
  }

  if (kmodel$estInfo$type == "ecm") {
    stop("Symmetry test is not suitable for ECM models.", call. = FALSE)
  }

  coefNames <- names(kmodel$coefficients)
  kardlWald <- list()

  #--------------------------------------------------
  # Variable selection
  #--------------------------------------------------
  all_SR_vars <- kmodel$extractedInfo$ASvars
  all_LR_vars <- kmodel$extractedInfo$ALvars
  all_testable_vars <- unique(c(all_SR_vars, all_LR_vars))

  if (is.null(vars)) {
    selected_vars <- all_testable_vars
  } else {
    if (!is.character(vars)) {
      stop("'vars' must be NULL or a character vector.", call. = FALSE)
    }
    bad_vars <- setdiff(vars, all_testable_vars)
    if (length(bad_vars) > 0) {
      stop(
        "These variables are not available for symmetry testing: ",
        toString(bad_vars),
        call. = FALSE
      )
    }
    selected_vars <- vars
  }

  selected_SR_vars <- intersect(selected_vars, all_SR_vars)
  selected_LR_vars <- intersect(selected_vars, all_LR_vars)

  #--------------------------------------------------
  # Long-run symmetry
  #--------------------------------------------------
  if (component %in% c("both", "longrun") && length(selected_LR_vars) > 0) {
    LWaldList <- list()

    Lwald <- data.frame()
    Lhypotheses <- list(H0 = list(), H1 = list())

    LongDep <- replace_lag_var(
      .kardl_Settings_env$LongCoef,
      kmodel$extractedInfo$dependentVar,
      1
    )

    dependentID <- which(LongDep == coefNames)

    if (length(dependentID) == 0) {
      stop("Long-run dependent variable coefficient was not found.", call. = FALSE)
    }

    ms_res <- deviance(kmodel) / df.residual(kmodel)

    LongAs <- data.frame()

    for (x in selected_LR_vars) {
      POS_param <- replace_lag_var(
        .kardl_Settings_env$LongCoef,
        paste0(.kardl_Settings_env$AsymPrefix[1], x, .kardl_Settings_env$AsymSuffix[1]),
        1
      )
      NEG_param <- replace_lag_var(
        .kardl_Settings_env$LongCoef,
        paste0(.kardl_Settings_env$AsymPrefix[2], x, .kardl_Settings_env$AsymSuffix[2]),
        1
      )

      POS_index <- which(POS_param == coefNames)
      NEG_index <- which(NEG_param == coefNames)

      if (length(POS_index) == 0 || length(NEG_index) == 0) next

      LongAs <- rbind(
        LongAs,
        data.frame(
          varName = x,
          POS_param = POS_param,
          POS_index = POS_index,
          NEG_param = NEG_param,
          NEG_index = NEG_index,
          stringsAsFactors = FALSE
        )
      )
    }

    if (nrow(LongAs) > 0) {
      for (v in seq_len(nrow(LongAs))) {
        x <- LongAs[v, ]

        Ltest <- paste0(
          "-b[", x$POS_index, "]/b[", dependentID, "] = -b[", x$NEG_index, "]/b[", dependentID, "]"
        )

        Lhypotheses$H0[[x$varName]] <- paste0(
          "- Coef(", x$POS_param, ")/Coef(", LongDep, ") = - Coef(", x$NEG_param, ")/Coef(", LongDep, ")"
        )
        Lhypotheses$H1[[x$varName]] <- enc2utf8(paste0(
          "- Coef(", x$POS_param, ")/Coef(", LongDep, ") \u2260 - Coef(", x$NEG_param, ")/Coef(", LongDep, ")"
        ))



        if (identical(type, "F")) {
          # include ... in nlWaldtest
          newLW <- nlWaldtest(kmodel,texts = Ltest, df2 = T, ...)
          LWaldList[[x$varName]] <- newLW
          F_val <- unname(newLW$statistic[["F"]])
          df1 <- unname(newLW$parameter[[1]])

          SumSq_approx <- F_val * df1 * ms_res
          MeanSq_approx <- F_val * ms_res

          Lwald <- rbind(
            Lwald,
            data.frame(
              Df = df1,
              `Sum of Sq` = SumSq_approx,
              `Mean Sq` = MeanSq_approx,
              `F value` = F_val,
              `Pr(>F)` = newLW$p.value,
              row.names = x$varName,
              check.names = FALSE
            )
          )
        } else {
          newLW <- nlWaldtest(kmodel,texts = Ltest )
          LWaldList[[x$varName]] <- newLW
          chisq_val <- unname(newLW$statistic[[1]])
          df1 <- unname(newLW$parameter[[1]])

          Lwald <- rbind(
            Lwald,
            data.frame(
              Df = df1,
              Chisq = chisq_val,
              `Pr(>Chisq)` = newLW$p.value,
              row.names = x$varName,
              check.names = FALSE
            )
          )

        }
      }

      if (nrow(Lwald) > 0) {
        if (identical(type, "F")) {
          class(Lwald) <- c("anova", "data.frame")
          attr(Lwald, "note") <- "Long-run tests based on normalized coefficients."
        }else{
          class(Lwald) <- c("anova", "data.frame")
        }
        attr(Lwald, "heading") <- c(
          "Symmetry Test Results - Long-run:",
          "======================="
        )
        attr(Lwald, "type") <- "Symmetry Test"
        attr(Lwald, "method") <- paste("Wald Test", type)
        attr(Lwald, "signif.legend") <-
          "Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1"

        kardlWald$Lwald <- Lwald
        kardlWald$Lhypotheses <- Lhypotheses
      }
    }
  }

  #--------------------------------------------------
  # Short-run symmetry
  #--------------------------------------------------
  if (component %in% c("both", "shortrun") && length(selected_SR_vars) > 0) {
    SWaldList <- list()
    Swald <- data.frame()
    Shypotheses <- list(H0 = list(), H1 = list())

    for (v in selected_SR_vars) {
      coefPOS <- coefNEG <- NegHyp <- PosHyp <- character(0)

      for (i in 0:kmodel$argsInfo$maxlag) {
        posLagged <- replace_lag_var(
          .kardl_Settings_env$ShortCoef,
          paste0(.kardl_Settings_env$AsymPrefix[1], v, .kardl_Settings_env$AsymSuffix[1]),
          i
        )
        negLagged <- replace_lag_var(
          .kardl_Settings_env$ShortCoef,
          paste0(.kardl_Settings_env$AsymPrefix[2], v, .kardl_Settings_env$AsymSuffix[2]),
          i
        )

        if (posLagged %in% coefNames) {
          coefPOS <- c(coefPOS, posLagged)
          PosHyp <- c(PosHyp, paste0("Coef(", posLagged, ")"))
        }
        if (negLagged %in% coefNames) {
          coefNEG <- c(coefNEG, negLagged)
          NegHyp <- c(NegHyp, paste0("Coef(", negLagged, ")"))
        }
      }

      if (length(coefPOS) == 0 || length(coefNEG) == 0) next

      sw <- paste0(
        paste(coefPOS, collapse = " + "),
        " = ",
        paste(coefNEG, collapse = " + ")
      )

      Sh_pos_H0 <- paste(PosHyp, collapse = " + ")
      Sh_neg_H0 <- paste(NegHyp, collapse = " + ")

      Shypotheses$H0[[v]] <- paste0(Sh_pos_H0, " = ", Sh_neg_H0)
      Shypotheses$H1[[v]] <- enc2utf8(paste0(Sh_pos_H0, " \u2260 ", Sh_neg_H0))

      sw2 <- car::linearHypothesis(
        kmodel,
        hypothesis.matrix = sw,
        test = if (identical(type, "F")) "F" else "Chisq" , ...
      )
      SWaldList[[v]] <- sw2

      if (identical(type, "F")) {
        Swald <- rbind(
          Swald,
          data.frame(
            Df = sw2[2, 3],
            `Sum of Sq` = sw2[2, 4],
            `Mean Sq` = sw2[2, 4] / sw2[2, 3],
            `F value` = sw2[2, 5],
            `Pr(>F)` = sw2[2, 6],
            row.names = v,
            check.names = FALSE
          )
        )
      } else {
        Swald <- rbind(
          Swald,
          data.frame(
            Df = sw2[2, "Df"],
            `Sum of Sq` = sw2[2, "Sum of Sq"],
            Chisq = sw2[2, "Chisq"],
            `Pr(>Chisq)` = sw2[2, "Pr(>Chisq)"],
            row.names = v,
            check.names = FALSE
          )
        )
      }
    }

    if (nrow(Swald) > 0) {

      class(Swald) <- c("anova", "data.frame")
      attr(Swald, "signif.legend") <-
          "Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1"
      attr(Swald, "heading") <- c(
        "Symmetry Test Results - Short-run:",
        "======================="
      )
      attr(Swald, "type") <- "Symmetry Test"
      attr(Swald, "method") <- paste("Wald Test", type)

      kardlWald$Swald <- Swald
      kardlWald$Shypotheses <- Shypotheses
    }
  }

  #--------------------------------------------------
  # Final checks
  #--------------------------------------------------
  if (length(kardlWald) == 0) {
    stop("No valid symmetry test could be computed for the requested component and variables.",
         call. = FALSE)
  }

  kardlWald$LWaldList <- if (exists("LWaldList")) LWaldList else NULL
  kardlWald$SWaldList <- if (exists("SWaldList")) SWaldList else NULL
  kardlWald$component <- component
  kardlWald$vars <- selected_vars
  kardlWald$type <- type
  kardlWald$call <- match.call()

  class(kardlWald) <- c("kardl_symmetric", "list")
  kardlWald
}


#' Pesaran, Shin, and Smith Bounds F-Test
#'
#' This function performs the Pesaran, Shin, and Smith (PSS) F Bound test to assess the presence of a long-term relationship (cointegration) between variables in the context of an autoregressive distributed lag (ARDL) model. The PSS F Bound test examines the joint significance of lagged levels of the variables in the model. It provides critical values for both the upper and lower bounds, which help determine whether the variables are cointegrated. If the calculated F-statistic falls outside these bounds, it indicates the existence of a long-term equilibrium relationship. This test is particularly useful when the underlying data includes a mix of stationary and non-stationary variables.
#'
#' @param kmodel A fitted KARDL model object of class 'kardl_lm' created using the \code{\link{kardl}} function.
#' @param case Numeric or character. Specifies the case of the test to be used in the function.
#' Acceptable values are 1, 2, 3, 4, 5, and "auto". If "auto" is chosen, the function determines the case automatically based on the model's characteristics. Invalid values will result in an error.
#' \itemize{
#' \item \code{1}: No intercept and no trend
#' \item \code{2}: Restricted intercept and no trend
#' \item \code{3}: Unrestricted intercept and no trend
#' \item \code{4}: Unrestricted intercept and restricted trend
#' \item \code{5}: Unrestricted intercept and unrestricted trend
#' }
#'
#'
#' @param signif_level Character or numeric. Specifies the significance level to be used in the function.
#' Acceptable values are "auto", "0.10", "0.1", "0.05", "0.025", and "0.01".
#' If a numeric value is provided, it will be converted to a character string.
#'
#' When \code{"auto"} is selected, the function determines the significance level
#' sequentially, starting from the most stringent level (\code{"0.01"}) and proceeding
#' to \code{"0.025"}, \code{"0.05"}, and \code{"0.10"} until a suitable level is identified.
#' Invalid values will result in an error.
#'
#' @section Hypothesis testing:
#' The null hypothesis (H0) of the F Bound test is that there is no cointegration among the variables in the model. In other words, it tests whether the long-term relationship between the variables is statistically significant. If the calculated F-statistic exceeds the upper critical value, we reject the null hypothesis and conclude that there is cointegration. Conversely, if the F-statistic falls below the lower critical value, we fail to reject the null hypothesis, indicating no evidence of cointegration. If the F-statistic lies between the two critical values, the result is inconclusive.
#'
#'
#'  \deqn{
#'   \Delta {y}_t =  \psi  + \varphi t  + \eta _0   {y}_{t-1}  + \sum_{i=1}^{k} {  \eta _i   {x}_{i,t-1} }  +   \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }+ \sum_{i=1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
#' }
#'
#' \describe{
#'   \item{Cases 1, 3, 5:}{
#'      \deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k  = 0}
#'      \deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k} \neq 0}
#'   }
#'   \item{Case 2:}{
#'      \deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k =  \psi  = 0}
#'      \deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k}  \neq \psi \neq 0}
#'   }
#'
#'   \item{Case 4:}{
#'      \deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k = \varphi = 0}
#'      \deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k}  \neq \varphi \neq 0}
#'       }
#'   }
#'
#' @return A list with class "htest" containing the following components:
#' \itemize{
#' \item \code{statistic}: The calculated F-statistic for the test.
#' \item \code{caseTxt}: A character string describing the case used for the test, based on the specified case parameter.
#' \item \code{alternative}: A character string describing the alternative hypothesis of the test.
#' \item \code{sample.size}: The number of observations used in the test.
#' \item \code{varnames}: A character vector containing the names of the dependent variable and independent variables used in the test.
#' \item \code{k}: The number of independent variables (excluding the dependent variable) included in the test.
#' \item \code{sig}: The significance level used for the test, either specified by the user or determined automatically.
#' \item \code{notes}: A character vector containing any notes or warnings related to
#' the test, such as the suitability of the test for small sample sizes or any adjustments made to the case based on the model's characteristics.
#' }
#' @srrstats {G1.0} This function cites the original source of the Pesaran, Shin, and Smith (2001) bounds testing approach for cointegration analysis, which is a widely used method in econometrics for testing the existence of long-term relationships between variables in autoregressive distributed lag (ARDL) models. The function also provides guidance on the suitability of the test for small sample sizes and suggests alternative approaches when necessary.
#' @srrstats {G1.3}  Statistical terms such as NULL hypothesis, alternative hypothesis, test statistic are defined in the context of the Pesaran, Shin, and Smith (2001) bounds testing approach for cointegration analysis.
#' @srrstats {G2.0} Input argument `kmodel` is validated as a `kardl_lm` object; `case` and `signif_level` are validated against acceptable values before the test is performed.
#' @srrstats {G2.3} The `case` argument is validated against `c(1,2,3,4,5,"auto")`; `signif_level` against the set of accepted strings.
#' @srrstats {TS2.2} The documentation discusses stationarity requirements in the context of ARDL bounds testing and the I(0)/I(1) framework.
#' @srrstats {TS2.4} Users are expected to assess integration order before applying bounds-test interpretations; the documentation states these modelling assumptions.
#' @export
#' @importFrom car linearHypothesis
#' @seealso   \code{\link{psst}}  \code{\link{ecm}}  \code{\link{narayan}}
#' @references Pesaran, M. H., Shin, Y. and Smith, R. (2001), "Bounds Testing Approaches to the Analysis of Level Relationship", Journal of Applied Econometrics, 16(3), 289-326.
#'
#' @examples

#' kardl_model<-kardl(imf_example_data,
#'                    CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                    mode=c(1,2,3,0))
#' my_pssF<-pssf(kardl_model)
#' # Getting the results of the test.
#' my_pssF
#' # Getting details of the test.
#' my_summary<-summary(my_pssF)
#' my_summary
#'
#' # Getting the critical values of the test.
#' my_summary$crit_vals
#'
#'
#' # Using magrittr :
#'
#' @examplesIf requireNamespace("magrittr", quietly = TRUE)
#' library(magrittr)
#'    imf_example_data %>%
#'    kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                            mode=c(1,2,3,0)) %>% pssf()
#'
#'    # Getting details of the test results using magrittr:
#'    imf_example_data %>%
#'    kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                            mode=c(1,2,3,0)) %>%
#'    pssf() %>% summary()
#'
#'
#'
pssf<-function(kmodel,case=3,signif_level = "auto"){
  if(! case %in% c(1,2,3,4,5,"auto")){
    stop("Invalid 'case' specification. ", call. = FALSE)
  }
  if (is.numeric(signif_level)) {
    signif_level <- as.character(signif_level)
  }
  if (!signif_level %in% c( "auto","0.10","0.1","0.05","0.025", "0.01")) {
    stop("Invalid significance level. ",
         "Valid options are: 'auto', 0.10, 0.05, 0.025, or 0.01.", call. = FALSE )
  }
  if(inherits(kmodel, "kardl_lm")){
    if(kmodel$estInfo$type=="ecm"){
      stop("PSS F test is not applicable for ECM models. Please use 'psst()' function for residual-based tests.", call. = FALSE)
    }
  }else{
    stop("Input object must be of class 'kardl_lm'.",call. = FALSE)
  }

  notesArray<-c()
  if(kmodel$estInfo$TimeSpan<80){
    notesArray <- c(notesArray, paste0(
      "The Pesaran et al. (2001) bounds F-test is designed for large samples. ",
      "Your model uses only ", kmodel$estInfo$TimeSpan, " observations. ",
      "For greater accuracy with small samples, consider using the critical values from Narayan (2005) or use 'narayan()' function. ",
      "Results are provided here using the asymptotic Pesaran et al. (2001) bounds."
    ))
  }

  if(case == "auto"){
    case <- 3 # Default case is 3, which means unrestricted intercept and no trend
  }

  if(kmodel$extractedInfo$noConstant){
    if(case >1){
      notesArray <- c(notesArray,
                      "No constant term is included in the model. Case automatically adjusted to 1 (no intercept, no trend)."
      )
    }
    case<-1
  }else{
    if(case == 1){
      # if case is 1, it means that the model has a constant. If the constant is not included, it should be set to 2.
      if(! kmodel$extractedInfo$trend){
        case<-2
        notesArray <- c(notesArray,
                        "Constant included but no trend. Case automatically adjusted to 2 (restricted intercept, no trend).")
      }else{
        # if case is 1, it means that the model has a constant and no trend. If the trend is included, it should be set to 5.
        case<-5
        notesArray <- c(notesArray,
                        "Both constant and trend are included. Case automatically adjusted to 5 (unrestricted intercept and trend).")
      }
    }else if(case >3){
      # if case is greater than 3, it means that the model has a trend. If the trend is not included, it should be set to 3.
      if(! kmodel$extractedInfo$trend){
        case<-3
        notesArray <- c(notesArray,
                        "No trend is included. Case automatically adjusted to 3 (unrestricted intercept, no trend).")
      }
    }else if(case <4 && kmodel$extractedInfo$trend){
      # if case is less than 4, it means that the model does not have a trend. If the trend is included, it should be set to 4.
      case<-5
      notesArray <- c(notesArray,
                      "Trend detected in the model. Case automatically adjusted to 5 (unrestricted intercept and trend).")

    }
  }

  longrunNames<-replace_lag_var(.kardl_Settings_env$LongCoef ,kmodel$extractedInfo$longRunVars,1) # paste0("L1.",kmodel$extractedInfo$longRunVars)
  # check if case is 1,3,5
  if(case == 2){
    longrunNames<-c(longrunNames,"(Intercept)")
  }
  if(case == 4){
    longrunNames<-c(longrunNames,"trend")
  }


  kisit<-unique(c(paste(unlist(longrunNames),"=0")))
  Fmodel<-linearHypothesis(kmodel ,kisit,test="F")
  PF<-Fmodel$F[2]

  k<-length(kmodel$extractedInfo$longRunVars)-1
  if(k>10){
       k<-10
    notesArray <- c(notesArray,
                    "The number of long-run variables exceeds 10. Critical values are not available for more than 10 variables. Results are provided using the critical values for 10 variables, but interpret with caution."
    )
  }

  caseTxt <- switch(case,
                    "1"="I",
                    "2"="II",
                    "3"="III",
                    "4"="IV",
                    "5"="V"
  )

  structure(list(
    type="cointegration",
    caseTxt=caseTxt,
    case=case,
    statistic = c("F" = PF),
    method = "Pesaran-Shin-Smith (PSS) Bounds F-test for cointegration",
    alternative = "Cointegrating relationship exists",
    data.name = deparse(substitute(model)),
    sample.size=kmodel$estInfo$TimeSpan,
    varnames=longrunNames,
     k=k,
    sig=signif_level,
    notes=notesArray,
    test.func = "pssf"
  ), class = c("kardl_test","htest"))
}

#' Summary method for Pesaran, Shin, and Smith Bounds F-Test
#'
#' This function provides a summary of the results from the Pesaran, Shin, and Smith (PSS) Bounds F-test for cointegration. It checks the validity of the input object, extracts relevant information from the test results, and organizes it into a structured format for easy interpretation. The summary includes the test statistic, critical values based on the specified case and significance level, and any notes or warnings related to the test.
#' @param testObj An object of class "kardl_test" and "htest" that contains the results of the PSS Bounds F-test. The object should have been created using the \code{pssf()} function.
#' @param ... Additional arguments (not currently used).
#' @return A list containing the summary of the PSS Bounds F-test results, including
#' the test statistic, critical values, and any notes or warnings. The list is of class "kardl_test_summary".
#' @noRd

htestsummary.pssf<-function(testObj,...){
  # check if the testObj is of class "kardl_test" and "htest" and if the method is "PesaranF"
  if(!inherits(testObj, "kardl_test") || !inherits(testObj, "htest") || testObj$test.func != "pssf"){

    stop("Input object must be of class 'kardl_test' and 'htest' with test.func 'pssf'.", call. = FALSE)
  }

  notes<-testObj$notes
  case<-testObj$case
  signif_level<-testObj$sig
  PF<-testObj$statistic[["F"]]
  k<-testObj$k

  crVals <- switch(case,
                   # Case I: No intercept and no trend
                   c( 0,3.00,3.00,4.20,4.20,5.47,5.47,7.17,7.17,1,2.44,3.28,3.15,4.11,3.88,4.92,4.81,6.02,2,2.17,3.19,2.72,3.83,3.22,4.50,3.88,5.30,3,2.01,3.10,2.45,3.63,2.87,4.16,3.42,4.84,4,1.90,3.01,2.26,3.48,2.62,3.90,3.07,4.44,5,1.81,2.93,2.14,3.34,2.44,3.71,2.82,4.21,6,1.75,2.87,2.04,3.24,2.32,3.59,2.66,4.05,7,1.70,2.83,1.97,3.18,2.22,3.49,2.54,3.91,8,1.66,2.79,1.91,3.11,2.15,3.40,2.45,3.79,9,1.63,2.75,1.86,3.05,2.08,3.33,2.34,3.68,10,1.60,2.72,1.82,2.99,2.02,3.27,2.26,3.60),
                   # Case II: Restricted intercept and no trend
                   c(0,3.80,3.80,4.60,4.60,5.39,5.39,6.44,6.44,1,3.02,3.51,3.62,4.16,4.18,4.79,4.94,5.58,2,2.63,3.35,3.10,3.87,3.55,4.38,4.13,5.00,3,2.37,3.20,2.79,3.67,3.15,4.08,3.65,4.66,4,2.20,3.09,2.56,3.49,2.88,3.87,3.29,4.37,5,2.08,3.00,2.39,3.38,2.70,3.73,3.06,4.15,6,1.99,2.94,2.27,3.28,2.55,3.61,2.88,3.99,7,1.92,2.89,2.17,3.21,2.43,3.51,2.73,3.90,8,1.85,2.85,2.11,3.15,2.33,3.42,2.62,3.77,9,1.80,2.80,2.04,3.08,2.24,3.35,2.50,3.68,10,1.76,2.77,1.98,3.04,2.18,3.28,2.41,3.61),
                   # Case III: Unrestricted intercept and no trend
                   c(0,6.58,6.58,8.21,8.21,9.80,9.80,11.79,11.79,1,4.04,4.78,4.94,5.73,5.77,6.68,6.84,7.84,2,3.17,4.14,3.79,4.85,4.41,5.52,5.15,6.36,3,2.72,3.77,3.23,4.35,3.69,4.89,4.29,5.61,4,2.45,3.52,2.86,4.01,3.25,4.49,3.74,5.06,5,2.26,3.35,2.62,3.79,2.96,4.18,3.41,4.68,6,2.12,3.23,2.45,3.61,2.75,3.99,3.15,4.43,7,2.03,3.13,2.32,3.50,2.60,3.84,2.96,4.26,8,1.95,3.06,2.22,3.39,2.48,3.70,2.79,4.10,9,1.88,2.99,2.14,3.30,2.37,3.60,2.65,3.97,10,1.83,2.94,2.06,3.24,2.28,3.50,2.54,3.86),
                   # Case IV: Unrestricted intercept and restricted trend
                   c(0,5.37,5.37,6.29,6.29,7.14,7.14,8.26,8.26,1,4.05,4.49,4.68,5.15,5.30,5.83,6.10,6.73,2,3.38,4.02,3.88,4.61,4.37,5.16,4.99,5.85,3,2.97,3.74,3.38,4.23,3.80,4.68,4.30,5.23,4,2.68,3.53,3.05,3.97,3.40,4.36,3.81,4.92,5,2.49,3.38,2.81,3.76,3.11,4.13,3.50,4.63,6,2.33,3.25,2.63,3.62,2.90,3.94,3.27,4.39,7,2.22,3.17,2.50,3.50,2.76,3.81,3.07,4.23,8,2.13,3.09,2.38,3.41,2.62,3.70,2.93,4.06,9,2.05,3.02,2.30,3.33,2.52,3.60,2.79,3.93,10,1.98,2.97,2.21,3.25,2.42,3.52,2.68,3.84),
                   # Case V: Unrestricted intercept and unrestricted trend
                   c(0,9.81,9.81,11.64,11.64,13.36,13.36,15.73,15.73,1,5.59,6.26,6.56,7.30,7.46,8.27,8.74,9.63,2,4.19,5.06,4.87,5.85,5.49,6.59,6.34,7.52,3,3.47,4.45,4.01,5.07,4.52,5.62,5.17,6.36,4,3.03,4.06,3.47,4.57,3.89,5.07,4.40,5.72,5,2.75,3.79,3.12,4.25,3.47,4.67,3.93,5.23,6,2.53,3.59,2.87,4.00,3.19,4.38,3.60,4.90,7,2.38,3.45,2.69,3.83,2.98,4.16,3.34,4.63,8,2.26,3.34,2.55,3.68,2.82,4.02,3.15,4.43,9,2.16,3.24,2.43,3.56,2.67,3.87,2.97,4.24,10,2.07,3.16,2.33,3.46,2.56,3.76,2.84,4.10)
  )



  MatName <- c("k","0.10L","0.10U","0.05L","0.05U","0.025L","0.025U","0.01L","0.01U")
  PSSCrVals<-t(matrix(crVals,9,11))




  colnames(PSSCrVals)<-MatName


  bu<-data.frame( L = PSSCrVals[k+1,c(2,4,6,8)],
                  U = PSSCrVals[k+1,c(3,5,7,9)]
  )
  rownames(bu)<-c("0.10","0.05","0.025","0.01")

  if(signif_level == "auto"){
    numericDec<-1

    if(PF>=bu[4,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 1% level)"
      sig<-"0.01"
    }else if(PF>=bu[3,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 2.5% level)"
      sig<-"0.025"
    }else if(PF>=bu[2,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 5% level)"
      sig<-"0.05"
    }else if(PF>=bu[1,"U"]){
      decision<-"Reject H0 \u2192 Weak evidence of cointegration (at 10% level)"
      sig<-"0.10"
    }else{
      sig<-""
      if(PF>=bu[1,"L"]){
        decision<-"Inconclusive"
        numericDec <- 0
      }else{
        decision<-"Fail to reject H0 \u2192 No Cointegration"
        numericDec <- -1
      }
    }
  }else{
    sig<-signif_level
    buRow<-switch (signif_level,"0.1"=1, "0.10"=1,"0.05"=2,"0.025"=3,"0.01"=4)
    if(PF>=bu[buRow,"U"]){
      if(buRow==1){
        decision<-"Reject H0 \u2192 Weak evidence of cointegration (at 10% level)"
      }else {
        decision<-paste0("Reject H0 \u2192 Cointegration (at ",switch (buRow,"1"="10%","2"="5%","3"="2.5%","4"="0.01")," level)")
      }
      numericDec <- 1
    }else if(PF>=bu[buRow,"L"]){
      decision<-"Inconclusive"
      numericDec<-0
    }else{
      decision<-"Fail to reject H0 \u2192 No Cointegration"
      numericDec <- -1
    }
  }


  caseTxt <- switch(case,
                    "1"="I",
                    "2"="II",
                    "3"="III",
                    "4"="IV",
                    "5"="V"
  )


  structure(list(
    type="cointegration",
    case=caseTxt,
    statistic = c("F" = PF),
    method = "Pesaran-Shin-Smith (PSS) Bounds F-test for cointegration",
    crit_vals   = bu,
    k=k,
    varnames=testObj$varnames,
    decision=decision,
    numericDec=numericDec, # -1: fail to reject, 0: inconclusive, 1: reject
    siglvl=sig,
    notes=notes
  ), class = "summary_htest")
}

#' Summary method for cointegration tests
#'
#' This function provides a summary of the results from various cointegration tests, including the Pesaran, Shin, and Smith (PSS) Bounds F-test and the Narayan test. It checks the validity of the input object, extracts relevant information from the test results, and organizes it into a structured format for easy interpretation. The summary includes the test statistic, critical values based on the specified case and significance level, and any notes or warnings related to the test.
#' @param inputs An object of class "kardl_test" and "htest
#' that contains the results of a cointegration test. The object should have been created using either the \code{pssf()}, \code{pssf()} or \code{narayan()} function.
#' @param ... Additional arguments (not currently used).
#' @return A list containing the summary of the cointegration test results, including
#' the test statistic, critical values, and any notes or warnings. The list is of
#' class "summary_htest".
#' @noRd

htestsummary<-function(inputs,...){

  switch(inputs$test.func,
         "pssf"=htestsummary.pssf(inputs,...),
         "narayan"=htestsummary.narayan(inputs,...),
         "psst"=htestsummary.psst(inputs,...),
         stop("No summary method available for this test type.", call. = FALSE)
  )

}

#' Narayan Bounds Test
#'
#' This function performs the Narayan test, which is designed to assess cointegration using critical values specifically tailored for small sample sizes. Unlike traditional cointegration tests that may rely on asymptotic distributions, the Narayan test adjusts for the limitations of small samples, providing more accurate results in such contexts. This makes the test particularly useful for studies with fewer observations, as it accounts for sample size constraints when determining the presence of a long-term equilibrium relationship between variables.
#'
#' @param kmodel The kardl obejct
#' @param case Numeric or character. Specifies the case of the test to be used in the function.
#' Acceptable values are 1, 2, 3, 4, 5, and "auto". If "auto" is chosen, the function determines the case automatically based on the model's characteristics. Invalid values will result in an error.
#'
#' \itemize{
#' \item \code{1}: No intercept and no trend. This case is not supported by the Narayan test.
#' \item \code{2}: Restricted intercept and no trend.
#' \item \code{3}: Unrestricted intercept and no trend.
#' \item \code{4}: Unrestricted intercept and restricted trend.
#' \item \code{5}: Unrestricted intercept and unrestricted trend.
#' }
#'
#'
#' @param signif_level Character or numeric. Specifies the significance level to be used in the function.
#' Acceptable values are "auto", "0.10", "0.1", "0.05", "0.025", and "0.01".
#' If a numeric value is provided, it will be converted to a character string.
#'
#' When \code{"auto"} is selected, the function determines the significance level
#' sequentially, starting from the most stringent level (\code{"0.01"}) and proceeding
#' to \code{"0.025"}, \code{"0.05"}, and \code{"0.10"} until a suitable level is identified.
#' Invalid values will result in an error.
#'
#' @inheritSection pssf Hypothesis testing
#' @inherit pssf return
#' @importFrom car linearHypothesis
#'
#' @srrstats {G1.0} This function cites the original source of the Narayan (2005) bounds testing approach for cointegration analysis in small samples, which provides critical values specifically designed for such contexts.
#' @srrstats {G1.3}  Statistical terms such as NULL hypothesis, alternative hypothesis, test statistic are defined in the context of the Narayan (2005) bounds testing approach for cointegration analysis in small samples.
#' @srrstats {G2.0} Input argument `kmodel` is validated as a `kardl_lm` object; `case` and `signif_level` are validated against acceptable values before the test is performed.
#' @srrstats {G2.3} The `case` argument is validated against `c(2,3,4,5,"auto")`; `signif_level` against the set of accepted strings.
#' @srrstats {TS2.3} The documentation notes that the Narayan test is designed for small samples (n < 100) and suggests `pssf()` for larger samples.
#' @references Narayan, P. K. (2005). The saving and investment nexus for China: evidence from cointegration tests. Applied economics, 37(17), 1979-1990.
#' @export
#' @seealso \code{\link{pssf}}  \code{\link{psst}}  \code{\link{ecm}}
#' @examples

#' kardl_model<-kardl(imf_example_data,
#'                    CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                    mode=c(1,2,3,0))
#' my_test<-narayan(kardl_model)
#' # Getting the results of the test.
#' my_test
#' # Getting details of the test.
#' my_summary<-summary(my_test)
#' my_summary
#'
#' # Getting the critical values of the test.
#' my_summary$crit_vals
#'
#'
#'
#'
#' # Using magrittr :
#'
#' @examplesIf requireNamespace("magrittr", quietly = TRUE)
#' library(magrittr)
#'   imf_example_data %>%
#'   kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                         mode=c(1,2,3,0)) %>% narayan()
#'
#'   # Getting details of the test results using magrittr:
#'    imf_example_data %>%
#'    kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                            mode=c(1,2,3,0)) %>%
#'    narayan() %>% summary()
#'
#'
#'
narayan<-function(kmodel,case=3,signif_level = "auto"){
  if(! case %in% c(2,3,4,5,"auto")){
    stop("Invalid 'case' specification. ",
         "Please choose from: 2, 3, 4, 5, or 'auto'.",call. = FALSE)
  }
  if (is.numeric(signif_level)) {
    signif_level <- as.character(signif_level)
  }
  if (!signif_level %in% c( "auto","0.10","0.1","0.05","0.01")) {
    stop("Invalid significance level. ",
         "Valid options are: 'auto', 0.10, 0.05, or 0.01.",call. = FALSE)
  }
  if(inherits(kmodel, "kardl_lm")){
    if(kmodel$estInfo$type=="ecm"){
      stop("Narayan F test is not applicable for ECM models. Please use 'psst()' function for residual-based tests.", call. = FALSE)
    }
  }else{
    stop("Input object must be of class 'kardl_lm'.",call. = FALSE)
  }


  notesArray<-c()
  if(kmodel$estInfo$TimeSpan>=100){
    notesArray <- c(notesArray, paste0(
      "The Narayan F-test is designed for small samples. ",
      "Your model uses only ", kmodel$estInfo$TimeSpan, " observations. ",
      "For greater accuracy with large samples, consider pssf() function. "
    ))
  }

  if(case == "auto"){
    case <- 3 # Default case is 5, which means unrestricted intercept and unrestricted trend
  }

  if(kmodel$extractedInfo$noConstant){
    notesArray <- c(notesArray, "No constant term is included in the model. The Narayan test does not support models without a constant. Please include a constant term.")

  }

  if(case == 1){
    # if case is 1, it means that the model has a constant. If the constant is not included, it should be set to 2.
    if(! kmodel$extractedInfo$trend){
      case<-2
      notesArray <- c(notesArray, "No trend is used in the model. The case was set to 2.")
    }else{
      # if case is 1, it means that the model has a constant and no trend. If the trend is included, it should be set to 5.
      case<-5
      notesArray <- c(notesArray,
                      "Both constant and trend are included. Case automatically adjusted to 5 (unrestricted intercept and trend).")
    }
  } else  if(case >3){
    # if case is greater than 3, it means that the model has a trend. If the trend is not included, it should be set to 3.
    if(! kmodel$extractedInfo$trend){
      case<-3
      notesArray <- c(notesArray,
                      "No trend is included. Case automatically adjusted to 3 (unrestricted intercept, no trend).")
    }
  } else   if(case <4 && kmodel$extractedInfo$trend){
    # if case is less than 4, it means that the model does not have a trend. If the trend is included, it should be set to 4.
    case<-5
    notesArray <- c(notesArray,
                    "Trend detected in the model. Case automatically adjusted to 5 (unrestricted intercept and trend).")

  }





  longrunNames<-replace_lag_var(.kardl_Settings_env$LongCoef ,kmodel$extractedInfo$longRunVars,1) # paste0("L1.",kmodel$extractedInfo$longRunVars)
  # check if case is 1,3,5
  if(case == 2){
    longrunNames<-c(longrunNames,"(Intercept)")
  }
  if(case == 4){
    longrunNames<-c(longrunNames,"trend")
  }


  # Null Hypothesis (H0)
  kisit<-c(paste(unlist(longrunNames),"=0"))

  Fmodel<-linearHypothesis(kmodel ,kisit,test="F")
  PF<-Fmodel$F[2]
 sayi<-length(kmodel$extractedInfo$longRunVars)-1


  structure(list(
    type="cointegration",
    case=case,
    statistic = c("F" = PF),
    method = "Narayan F Test for Cointegration",
    alternative = "Cointegrating relationship exists",
    data.name = deparse(substitute(model)),
    sample.size=kmodel$estInfo$TimeSpan,
    varnames=longrunNames,
   k=sayi,
   n=kmodel$estInfo$end,
   sig=signif_level,
    notes=notesArray,
 test.func  = "narayan"
  ), class = c("kardl_test","htest"))
}

#' Summary method for Narayan Bounds F-Test
#'
#' This function provides a summary of the results from the Narayan Bounds F-test for cointegration. It checks the validity of the input object, extracts relevant information from the test results, and organizes it into a structured format for easy interpretation. The summary includes the test statistic, critical values based on the specified case and significance level, and any notes or warnings related to the test.
#' @param testObj An object of class "kardl_test" and "htest" that contains
#' the results of the Narayan Bounds F-test. The object should have been created using the \code{narayan()} function.
#' @param ... Additional arguments (not currently used).
#' @return A list containing the summary of the Narayan Bounds F-test results, including
#' the test statistic, critical values, and any notes or warnings. The list is of
#' class "summary_htest".
#' @noRd

htestsummary.narayan<-function(testObj,...){
  # check if the testObj is of class "kardl_test" and "htest" and if the method is "PesaranF"
  if(!inherits(testObj, "kardl_test") || !inherits(testObj, "htest") || testObj$test.func != "narayan"){

    stop("Input object must be of class 'kardl_test' and 'htest' with test.func 'pssf'.", call. = FALSE)
  }

  notes<-testObj$notes
  case<-testObj$case
  signif_level<-testObj$sig
  PF<-testObj$statistic[["F"]]
  k<-testObj$k
  crVals <- switch(case,
                   # Case I: No intercept and no trend
                   c(),
                   # Case II: Restricted intercept and no trend
                   c(7.595,7.595,6.027,6.76,5.155,6.265,4.614,5.966,4.28,5.84,4.134,5.761,3.976,5.691,3.864,5.694,7.35,7.35,5.763,6.48,4.948,6.028,4.428,5.816,4.093,5.532,3.9,5.419,3.713,5.326,3.599,5.23,7.22,7.22,5.593,6.333,4.77,5.855,4.31,5.544,3.967,5.455,3.657,5.256,3.505,5.121,3.402,5.031,7.265,7.265,5.607,6.193,4.8,5.725,4.27,5.412,3.892,5.173,3.674,5.019,3.54,4.931,3.383,4.832,7.065,7.065,5.503,6.24,4.695,5.758,4.188,5.328,3.845,5.15,3.593,4.981,3.424,4.88,3.282,4.73,6.965,6.965,5.377,6.047,4.61,5.563,4.118,5.2,3.738,4.947,3.543,4.839,3.33,4.708,3.194,4.562,6.96,6.96,5.383,6.033,4.558,5.59,4.068,5.25,3.71,4.965,3.451,4.764,3.293,4.615,3.129,4.507,6.825,6.825,5.35,6.017,4.538,5.475,4.056,5.158,3.725,4.94,3.43,4.721,3.225,4.571,3.092,4.478,6.74,6.74,5.157,5.957,4.398,5.463,3.916,5.088,3.608,4.86,3.373,4.717,3.18,4.596,3.034,4.426,6.915,6.915,5.26,5.957,4.458,5.41,4.048,5.092,3.687,4.842,3.427,4.62,3.219,4.526,3.057,4.413,6.695,6.695,5.157,5.917,4.358,5.393,3.908,5.004,3.602,4.787,3.351,4.587,3.173,4.485,3.021,4.35,5.07,5.07,4.09,4.663,3.538,4.428,3.272,4.306,3.058,4.223,2.91,4.193,2.794,4.148,2.73,4.163,4.945,4.945,3.957,4.53,3.478,4.335,3.164,4.194,2.947,4.088,2.804,4.013,2.685,3.96,2.597,3.907,4.96,4.96,3.937,4.523,3.435,4.26,3.1,4.088,2.893,4,2.734,3.92,2.618,3.863,2.523,3.829,4.895,4.895,3.877,4.46,3.368,4.203,3.078,4.022,2.85,3.905,2.694,3.829,2.591,3.766,2.504,3.723,4.815,4.815,3.86,4.44,3.368,4.178,3.048,4.002,2.823,3.872,2.67,3.781,2.55,3.708,2.457,3.65,4.795,4.795,3.79,4.393,3.303,4.1,2.982,3.942,2.763,3.813,2.617,3.743,2.49,3.658,2.414,3.608,4.78,4.78,3.803,4.363,3.288,4.07,2.962,3.91,2.743,3.792,2.589,3.683,2.456,3.598,2.373,3.54,4.78,4.78,3.787,4.343,3.285,4.07,2.976,3.896,2.75,3.755,2.596,3.677,2.473,3.583,2.373,3.519,4.75,4.75,3.78,4.327,3.243,4.043,2.924,3.86,2.725,3.718,2.564,3.65,2.451,3.559,2.351,3.498,4.76,4.76,3.777,4.32,3.253,4.065,2.946,3.862,2.725,3.718,2.574,3.641,2.449,3.55,2.36,3.478,4.725,4.725,3.74,4.303,3.235,4.053,2.92,3.838,2.688,3.698,2.55,3.606,2.431,3.518,2.336,3.458,4.025,4.025,3.303,3.797,2.915,3.695,2.676,3.586,2.525,3.56,2.407,3.517,2.334,3.515,2.277,3.498,3.98,3.98,3.223,3.757,2.845,3.623,2.618,3.532,2.46,3.46,2.331,3.417,2.254,3.388,2.196,3.37,3.955,3.955,3.21,3.73,2.835,3.585,2.592,3.454,2.427,3.395,2.306,3.353,2.218,3.314,2.152,3.296,3.95,3.95,3.19,3.73,2.788,3.54,2.56,3.428,2.402,3.345,2.276,3.297,2.188,3.254,2.131,3.223,3.935,3.935,3.177,3.653,2.788,3.513,2.538,3.398,2.372,3.32,2.259,3.264,2.17,3.22,2.099,3.181,3.9,3.9,3.143,3.67,2.748,3.495,2.508,3.356,2.345,3.28,2.226,3.241,2.139,3.204,2.069,3.148,3.88,3.88,3.127,3.65,2.738,3.465,2.496,3.346,2.323,3.273,2.204,3.21,2.114,3.153,2.044,3.104,3.88,3.88,3.143,3.623,2.74,3.455,2.492,3.35,2.335,3.252,2.209,3.201,2.12,3.145,2.043,3.094,3.875,3.875,3.12,3.623,2.73,3.445,2.482,3.31,2.32,3.232,2.193,3.161,2.1,3.121,2.024,3.079,3.895,3.895,3.133,3.597,2.725,3.455,2.482,3.334,2.313,3.228,2.196,3.166,2.103,3.111,2.023,3.068,3.87,3.87,3.113,3.61,2.713,3.453,2.474,3.312,2.303,3.22,2.303,3.154,2.088,3.103,2.017,3.052)
                   ,
                   # Case III: Unrestricted intercept and no trend
                   c(13.68,13.68,8.17,9.285,6.183,7.873,5.333,7.063,4.768,6.67,4.537,6.37,4.27,6.211,4.104,6.151,13.29,13.29,7.87,8.96,6.14,7.607,5.198,6.845,4.59,6.368,4.257,6.04,4.016,5.797,3.841,5.686,13.07,13.07,7.625,8.825,5.893,7.337,5.018,6.61,4.428,6.25,4.045,5.898,3.8,5.643,3.644,5.464,12.93,12.93,7.74,8.65,5.92,7.197,4.983,6.423,4.394,5.914,4.03,5.598,3.79,5.411,3.595,5.225,12.73,12.73,7.56,8.685,5.817,7.303,4.865,6.36,4.306,5.874,3.955,5.583,3.656,5.331,3.498,5.149,12.7,12.7,7.435,8.46,5.707,6.977,4.828,6.195,4.244,5.726,3.928,5.408,3.636,5.169,3.424,4.989,12.49,12.49,7.4,8.51,5.697,6.987,4.748,6.188,4.176,5.676,3.783,5.338,3.531,5.081,3.346,4.895,12.4,12.4,7.32,8.435,5.583,6.853,4.69,6.143,4.188,5.694,3.783,5.3,3.501,5.051,3.31,4.871,12.24,12.24,7.17,8.405,5.487,6.88,4.635,6.055,4.098,5.57,3.747,5.285,3.436,5.044,3.261,4.821,12.54,12.54,7.225,8.3,5.513,6.86,4.725,6.08,4.168,5.548,3.772,5.213,3.496,4.966,3.266,4.801,12.12,12.12,7.095,8.26,5.407,6.783,4.568,5.96,4.096,5.512,3.725,5.163,3.457,4.943,3.233,4.76,8.77,8.77,5.395,6.35,4.267,5.473,3.71,5.018,3.354,4.774,3.125,4.608,2.97,4.499,2.875,4.445,8.64,8.64,5.29,6.175,4.183,5.333,3.615,4.913,3.276,4.63,3.037,4.443,2.864,4.324,2.753,4.209,8.57,8.57,5.26,6.16,4.133,5.26,3.548,4.803,3.202,4.544,2.962,4.338,2.797,4.211,2.676,4.13,8.59,8.59,5.235,6.135,4.083,5.207,3.535,4.733,3.178,4.45,2.922,4.268,2.764,4.123,2.643,4.004,8.51,8.51,5.22,6.07,4.07,5.19,3.5,4.7,3.136,4.416,2.9,4.218,2.726,4.057,2.593,3.941,8.39,8.39,5.125,6.045,3.987,5.09,3.408,4.623,3.068,4.334,2.848,4.16,2.676,3.999,2.556,3.904,8.46,8.46,5.125,6,4,5.057,3.415,4.615,3.062,4.314,2.817,4.097,2.643,3.939,2.513,3.823,8.49,8.49,5.13,5.98,4.01,5.08,3.435,4.583,3.068,4.274,2.835,4.09,2.647,3.921,2.525,3.808,8.37,8.37,5.055,5.915,3.947,5.02,3.37,4.545,3.022,4.256,2.788,4.073,2.629,3.906,2.494,3.786,8.42,8.42,5.14,5.92,3.983,5.06,3.408,4.55,3.042,4.244,2.802,4.065,2.637,3.9,2.503,3.768,8.4,8.4,5.06,5.93,3.94,5.043,3.363,4.515,3.01,4.216,2.787,4.015,2.627,3.864,2.476,3.746,6.84,6.84,4.29,5.08,3.437,4.47,3.008,4.15,2.752,3.994,2.578,3.858,2.457,3.797,2.384,3.728,6.81,6.81,4.225,5.05,3.393,4.41,2.958,4.1,2.696,3.898,2.508,3.763,2.387,3.671,2.3,3.606,6.76,6.76,4.235,5,3.373,4.377,2.933,4.02,2.66,3.838,2.483,3.708,2.353,3.599,2.26,3.534,6.76,6.76,4.225,5.02,3.33,4.347,2.893,3.983,2.638,3.772,2.458,3.647,2.327,3.541,2.238,3.461,6.74,6.74,4.19,4.94,3.333,4.313,2.873,3.973,2.614,3.746,2.435,3.6,2.309,3.507,2.205,3.421,6.7,6.7,4.155,4.925,3.28,4.273,2.843,3.92,2.578,3.71,2.393,3.583,2.27,3.486,2.181,3.398,6.7,6.7,4.145,4.95,3.27,4.26,2.838,3.923,2.568,3.712,2.385,3.565,2.253,3.436,2.155,3.353,6.74,6.74,4.175,4.93,3.3,4.25,2.843,3.923,2.574,3.682,2.397,3.543,2.256,3.43,2.156,3.334,6.67,6.67,4.125,4.88,3.25,4.237,2.818,3.88,2.552,3.648,2.363,3.51,2.233,3.407,2.138,3.325,6.72,6.72,4.15,4.885,3.277,4.243,2.838,3.898,2.558,3.654,2.38,3.515,2.244,3.397,2.134,3.313,6.72,6.72,4.135,4.895,3.26,4.247,2.823,3.885,2.548,3.644,2.355,3.5,2.236,3.381,2.129,3.289)
                   ,
                   # Case IV: Unrestricted intercept and restricted trend
                   c(10.2,10.2,7.593,8.35,6.428,7.505,5.666,6.988,5.205,6.64,4.85,6.473,4.689,6.358,4.49,6.328,9.975,9.975,7.477,8.213,6.328,7.408,5.654,6.926,5.147,6.617,4.849,6.511,4.629,5.698,4.489,5.064,9.575,9.575,7.207,7.86,5.98,6.973,5.258,6.526,4.763,6.2,4.427,5.837,4.154,5.699,3.971,5.486,9.555,9.555,7.133,7.82,5.878,6.87,5.15,6.28,4.628,5.865,4.251,5.596,3.998,5.463,3.829,5.313,9.32,9.32,7.017,7.727,5.805,6.79,5.05,6.182,4.557,5.793,4.214,5.52,3.983,5.345,3.762,5.172,9.3,9.3,6.893,7.537,5.678,6.578,4.99,6.018,4.455,5.615,4.111,5.329,3.87,5.171,3.643,5.021,9.245,9.245,6.78,7.377,5.62,6.503,4.928,5.95,4.412,5.545,4.013,5.269,3.775,5.086,3.584,4.922,8.96,8.96,6.707,7.36,5.545,6.453,4.848,5.842,4.347,5.552,4.02,5.263,3.758,5.04,3.557,4.902,8.89,8.89,6.577,7.313,5.448,6.435,4.76,5.798,4.293,5.46,3.966,5.234,3.72,5.004,3.509,4.808,8.905,8.905,6.613,7.253,5.505,6.298,4.808,5.786,4.3,5.377,3.984,5.153,3.728,4.954,3.511,4.789,6.695,6.695,5.157,5.917,4.358,5.393,3.908,5.004,3.602,4.787,3.351,4.587,3.173,4.485,3.021,4.35,7.04,7.04,5.377,5.963,4.535,5.415,4.048,5.09,3.715,4.878,3.504,4.743,3.326,4.653,3.194,4.604,6.9,6.9,5.233,5.777,4.433,5.245,3.936,4.918,3.578,4.668,3.353,4.5,3.174,4.383,3.057,4.319,6.87,6.87,5.18,5.733,4.36,5.138,3.85,4.782,3.512,4.587,3.257,4.431,3.07,4.309,2.933,4.224,6.75,6.75,5.13,5.68,4.335,5.078,3.822,4.714,3.47,4.47,3.211,4.309,3.025,4.198,2.899,4.087,6.685,6.685,5.043,5.607,4.225,5.03,3.73,4.666,3.383,4.432,3.149,4.293,2.975,4.143,2.832,4.012,6.66,6.66,5.013,5.547,4.183,4.955,3.692,4.582,3.358,4.365,3.131,4.206,2.946,4.065,2.791,3.95,6.63,6.63,4.98,5.527,4.18,4.938,3.684,4.584,3.323,4.333,3.086,4.154,2.9,3.999,2.756,3.892,6.55,6.55,4.95,5.467,4.123,4.903,3.626,4.538,3.3,4.28,3.063,4.123,2.88,3.978,2.73,3.879,6.53,6.53,4.93,5.457,4.1,4.9,3.6,4.512,3.272,4.272,3.043,4.1,2.86,3.951,2.711,3.842,6.58,6.58,4.937,5.443,4.12,4.855,3.624,4.488,3.298,4.26,3.054,4.079,2.874,3.914,2.718,3.807,4.725,4.725,3.74,4.303,3.235,4.053,2.92,3.838,2.688,3.698,2.55,3.606,2.431,3.518,2.336,3.458,5.785,5.785,4.427,4.957,3.77,4.535,3.378,4.274,3.097,4.118,2.907,4.01,2.781,3.941,2.681,3.887,5.69,5.69,4.38,4.867,3.698,4.42,3.29,4.176,3.035,3.997,2.831,3.879,2.685,3.785,2.578,3.71,5.68,5.68,4.343,4.823,3.663,4.378,3.264,4.094,2.985,3.918,2.781,3.813,2.634,3.719,2.517,3.65,5.625,5.625,4.3,4.78,3.625,4.33,3.226,4.054,2.95,3.862,2.75,3.739,2.606,3.644,2.484,3.57,5.57,5.57,4.23,4.74,3.573,4.288,3.174,4.004,2.905,3.822,2.703,3.697,2.55,3.609,2.44,3.523,5.57,5.57,4.23,4.73,3.553,4.238,3.132,3.956,2.868,3.782,2.674,3.659,2.538,3.56,2.42,3.481,5.555,5.555,4.203,4.693,3.54,4.235,3.13,3.968,2.852,3.773,2.653,3.637,2.51,3.519,2.392,3.444,5.51,5.51,4.187,4.66,3.535,4.208,3.122,3.942,2.848,3.743,2.647,3.603,2.499,3.49,2.379,3.406,5.53,5.53,4.173,4.647,3.505,4.198,3.098,3.92,2.832,3.738,2.631,3.589,2.485,3.473,2.363,3.394,5.53,5.53,4.193,4.647,3.505,4.213,3.11,3.9,2.832,3.717,2.636,3.579,2.486,3.469,2.372,3.37,3.87,3.87,3.113,3.61,2.713,3.453,2.474,3.312,2.303,3.22,2.18,3.154,2.088,3.103,2.017,3.052)
                   ,
                   # Case V: Unrestricted intercept and unrestricted trend
                   c(18.56,18.56,10.605,11.65,7.977,9.413,6.643,8.313,5.856,7.578,5.347,7.242,5.046,6.93,4.779,6.821,18.02,18.02,10.365,11.295,7.643,9.063,6.38,7.73,5.604,7.172,5.095,6.77,4.704,6.537,4.459,6.206,17.91,17.91,10.15,11.23,7.527,8.803,6.238,7.74,5.376,7.092,4.885,6.55,4.527,6.263,4.31,5.965,17.5,17.5,9.89,10.965,7.317,8.72,6.053,7.458,5.224,6.696,4.715,6.293,4.364,6.006,4.109,5.785,17.53,17.53,9.895,10.965,7.337,8.643,5.995,7.335,5.184,6.684,4.672,6.232,4.31,5.881,4.055,5.64,17.48,17.48,9.8,10.675,7.227,8.34,5.955,7.225,5.108,6.494,4.608,5.977,4.23,5.713,3.955,5.474,17.02,17.02,9.585,10.42,7.057,8.243,5.835,7.108,5.066,6.394,4.505,5.92,4.117,5.597,3.87,5.338,16.85,16.85,9.475,10.515,7.013,8.23,5.795,7.053,4.974,6.378,4.482,5.923,4.111,5.586,3.835,5.339,16.66,16.66,9.37,10.32,6.873,8.163,5.663,6.953,4.922,6.328,4.428,5.898,4.07,5.534,3.774,5.248,16.61,16.61,9.325,10.325,6.93,8.027,5.698,6.97,4.932,6.224,4.393,5.788,4.06,5.459,3.768,5.229,16.6,16.6,9.17,10.24,6.73,8.053,5.62,6.908,4.89,6.164,4.375,5.703,4,5.397,3.728,5.16,12.74,12.74,7.36,8.265,5.55,6.747,4.683,5.98,4.154,5.54,3.818,5.253,3.576,5.065,3.394,4.939,12.58,12.58,7.21,8.055,5.457,6.57,4.568,5.795,4.036,5.304,3.673,5.002,3.426,4.79,3.251,4.64,12.51,12.51,7.135,7.98,5.387,6.437,4.51,5.643,3.958,5.226,3.577,4.923,3.327,4.7,3.121,4.564,12.4,12.4,7.08,7.91,5.36,6.373,4.45,5.56,3.89,5.104,3.532,4.8,3.267,4.584,3.091,4.413,12.17,12.17,6.985,7.86,5.247,6.303,4.368,5.545,3.834,5.064,3.48,4.782,3.229,4.536,3.039,4.339,12.17,12.17,6.93,7.785,5.19,6.223,4.313,5.425,3.794,4.986,3.442,4.69,3.197,4.46,2.989,4.271,12.2,12.2,6.905,7.735,5.19,6.2,4.298,5.445,3.772,4.956,3.407,4.632,3.137,4.393,2.956,4.23,11.96,11.96,6.89,7.66,5.137,6.173,4.268,5.415,3.732,4.92,3.372,4.613,3.137,4.363,2.924,4.206,12,12,6.86,7.645,5.11,6.19,4.235,5.363,3.72,4.904,3.368,4.59,3.107,4.343,2.913,4.168,12.08,12.08,6.88,7.675,5.14,6.153,4.253,5.333,3.724,4.88,3.382,4.567,3.111,4.31,2.915,4.143,12.06,12.06,6.82,7.67,5.067,6.103,4.203,5.32,3.678,4.84,3.335,4.535,3.077,4.284,2.885,4.111,10.34,10.34,6.01,6.78,4.577,5.6,3.868,4.965,3.43,4.624,3.157,4.412,2.977,4.26,2.843,4.16,10.24,10.24,5.95,6.68,4.517,5.48,3.8,4.888,3.374,4.512,3.087,4.277,2.879,4.114,2.729,3.985,10.16,10.16,5.915,6.63,4.477,5.42,3.76,4.795,3.334,4.438,3.032,4.213,2.831,4.04,2.668,3.92,10.15,10.15,5.88,6.64,4.437,5.377,3.74,4.78,3.298,4.378,3.012,4.147,2.796,3.97,2.635,3.838,10.02,10.02,5.78,6.54,4.38,5.35,3.673,4.715,3.24,4.35,2.95,4.11,2.75,3.944,2.59,3.789,10.11,10.11,5.8,6.515,4.37,5.303,3.64,4.67,3.21,4.294,2.927,4.068,2.724,3.893,2.573,3.76,10.03,10.03,5.765,6.5,4.35,5.283,3.645,4.678,3.2,4.31,2.912,4.047,2.709,3.856,2.551,3.716,9.97,9.97,5.755,6.47,4.353,5.257,3.638,4.643,3.196,4.262,2.897,4.022,2.69,3.83,2.531,3.685,10.02,10.02,5.765,6.455,4.33,5.243,3.615,4.635,3.182,4.258,2.893,4.008,2.683,3.807,2.519,3.669,10.03,10.03,5.765,6.47,4.323,5.273,3.618,4.63,3.182,4.248,2.89,3.993,2.681,3.8,2.53,3.648,9.96,9.96,5.725,6.45,4.307,5.223,3.588,4.605,3.16,4.23,2.867,3.975,2.657,3.776,2.504,3.631)
  )

  bas<-c("L0" ,"U0", "L1", "U1", "L2", "U2", "L3", "U3", "L4", "U4", "L5", "U5", "L6", "U6", "L7", "U7")
  sat<-seq(30,80,by=5)




  obsnumber<-round((testObj$n-25)/5)
  if(obsnumber>11){
    obsnumber<-11
    notes<-c(notes,"The number of observations exceeds the maximum limit for the critical values table. Using the critical values for 80 observations.")

  }

  if(k>7){
    k<-7
    notes<-c(notes,"The number of regressors exceeds the maximum limit for the critical values table. Using the critical values for 7 regressors.")
  }


  out.01<-matrix(crVals[1:176],11,16,byrow = T)
  rownames(out.01)<-sat
  colnames(out.01)<-bas
  out.05<-matrix(crVals[177: 352],11,16,byrow = T)
  rownames(out.05)<-sat
  colnames(out.05)<-bas
  out.10<-matrix(crVals[353: 528],11,16,byrow = T)
  rownames(out.10)<-sat
  colnames(out.10)<-bas

  sutun1<-k*2+1
  sutun2<-k*2+2

  bu<-data.frame( L = c(out.10[obsnumber,sutun1],
                        out.05[obsnumber,sutun1],
                        out.01[obsnumber,sutun1]),
                  U = c(out.10[obsnumber,sutun2],
                        out.05[obsnumber,sutun2],
                        out.01[obsnumber,sutun2])
  )

  rownames(bu)<-c("0.10","0.05","0.01")


  if(signif_level == "auto"){
    numericDec<-1

    if(PF>=bu[3,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 1% level)"
      sig<-"0.01"
    }else if(PF>=bu[2,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 5% level)"
      sig<-"0.05"
    }else if(PF>=bu[1,"U"]){
      decision<-"Reject H0 \u2192 Weak evidence of cointegration (at 10% level)"
      sig<-"0.10"
    }else{
      sig<-""
      if(PF>=bu[1,"L"]){
        decision<-"Inconclusive"
        numericDec <- 0
      }else{
        decision<-"Fail to reject H0 \u2192 No Cointegration"
        numericDec <- -1
      }
    }
  }else{
    sig<-signif_level
    buRow<-switch (signif_level,"0.1"=1, "0.10"=1,"0.05"=2,"0.01"=3)
    if(PF>=bu[buRow,"U"]){
      if(buRow==1){
        decision<-"Reject H0 \u2192 Weak evidence of cointegration (at 10% level)"
      }else {
        decision<-paste0("Reject H0 \u2192 Cointegration (at ",switch (buRow,"1"="10%","2"="5%","3"="0.01")," level)")
      }
      numericDec <- 1
    }else if(PF>=bu[buRow,"L"]){
      decision<-"Inconclusive"
      numericDec<-0
    }else{
      decision<-"Fail to reject H0 \u2192 No Cointegration"
      numericDec <- -1
    }
  }
  caseTxt <- switch(case,
                    "1"="I",
                    "2"="II",
                    "3"="III",
                    "4"="IV",
                    "5"="V"
  )


  structure(list(
    type="cointegration",
    case=caseTxt,
    statistic = c("F" = PF),
    method = testObj$method,
    crit_vals   = bu,
    k=k,
    varnames=testObj$varnames,
    decision=decision,
    numericDec=numericDec, # -1: fail to reject, 0: inconclusive, 1: reject
    siglvl=sig,
    notes=notes
  ), class = "summary_htest")


}


#' Pesaran, Shin, and Smith t Bounds Test
#'
#' This function performs the Pesaran t Bound test
#'
#' This function performs the Pesaran, Shin, and Smith (PSS) t Bound test, which is used to detect the existence of a long-term relationship (cointegration) between variables in an autoregressive distributed lag (ARDL) model. The t Bound test specifically focuses on the significance of the coefficient of the lagged dependent variable, helping to assess whether the variable reverts to its long-term equilibrium after short-term deviations. The test provides critical values for both upper and lower bounds. If the t-statistic falls within the appropriate range, it confirms the presence of cointegration. This test is particularly useful when working with datasets containing both stationary and non-stationary variables.
#' @inheritParams pssf
#'

#' @section Hypothesis testing:
#' The PSS t Bound test evaluates the null hypothesis that the long-run coefficients of the model are equal to zero against the alternative hypothesis that at least one of them is non-zero. The test is conducted under different cases, depending on the model specification.
#'
#'
#'  \deqn{
#'   \Delta {y}_t =  \psi  + \varphi t  + \eta _0   {y}_{t-1}  + \sum_{i=1}^{k} {  \eta _i   {x}_{i,t-1} }  +   \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }+ \sum_{i=1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
#' }
#'
#'   \deqn{\mathbf{H_{0}:} \eta_0   = 0}
#'   \deqn{\mathbf{H_{1}:} \eta_{0}  \neq 0}
#'
#'
#' @return The function returns an object of class "htest" containing the following components:
#' \describe{
#' \item{statistic}{The calculated t-statistic for the test.}
#' \item{method}{A description of the test performed.}
#' \item{data.name}{The name of the data used in the test.}
#' \item{k}{The number of independent variables in the model.}
#' \item{notes}{Any notes or warnings related to the test results, such as
#' sample size considerations or adjustments made to the case based on model characteristics.}
#' \item{sig}{The significance level used for the test, either specified by the user or determined automatically.}
#' \item{alternative}{The alternative hypothesis being tested.}
#' \item{case}{The case used for the test, either specified by the user or determined automatically based on the model's characteristics.}
#' }
#'
#' @srrstats {G1.0} This function cite the original source of the test, which is Pesaran, M. H., Shin, Y. and Smith, R. (2001), "Bounds Testing Approaches to the Analysis of Level Relationship", Journal of Applied Econometrics, 16(3), 289-326.
#' @srrstats {G1.3}  Statistical terms such as NULL hypothesis, alternative hypothesis, test statistic are defined in the documentation.
#' @srrstats {G2.0} Input argument `kmodel` is validated as a `kardl_lm` object; `case` and `signif_level` are validated before the test is performed.
#' @srrstats {G2.3} The `case` argument is validated against `c(1,2,3,4,5,"auto")`; `signif_level` against the set of accepted character strings.
#' @srrstats {TS2.2} The PSS t-test evaluates the error-correction coefficient, which is relevant to the cointegration and stationarity assumptions of the ARDL framework.
#'
#'
#' @export
#' @seealso \code{\link{pssf}}   \code{\link{ecm}}  \code{\link{narayan}}
#' @importFrom stats vcov
#' @importFrom lmtest coeftest
#' @references Pesaran, M. H., Shin, Y. and Smith, R. (2001), "Bounds Testing Approaches to the Analysis of Level Relationship", Journal of Applied Econometrics, 16(3), 289-326.
#' @examples
#'

#' kardl_model<-kardl(imf_example_data,
#'                    CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                    mode=c(1,2,3,0))
#' my_test<-psst(kardl_model)
#' # Getting the results of the test.
#' my_test
#' # Getting details of the test.
#' my_summary<-summary(my_test)
#' my_summary
#'
#' # Getting the critical values of the test.
#' my_summary$crit_vals
#'
#'
#'
#'
#' # Using magrittr :
#'
#' @examplesIf requireNamespace("magrittr", quietly = TRUE)
#' library(magrittr)
#'      imf_example_data %>%
#'      kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                            mode=c(1,2,3,0))    %>%
#'                            psst()
#'
#'    # Getting details of the test results using magrittr:
#'    imf_example_data %>%
#'    kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                            mode=c(1,2,3,0)) %>%
#'    psst() %>% summary()
#'
#'
#'

psst<-function(kmodel,case=3,signif_level = "auto"){
  notesArray<-c()
  if(! inherits(kmodel, "kardl_lm"))  {
    stop("Input object must be of class 'kardl_lm'.",call. = FALSE)
  }
  if(! case %in% c(1,2,3,4,5,"auto")){
    stop("Invalid 'case' specification. ",
         "Please choose from: 1, 2, 3, 4, 5, or 'auto'.",call. = FALSE)
  }
  if (is.numeric(signif_level)) {
    signif_level <- as.character(signif_level)
  }
  if (!signif_level %in% c( "auto","0.10","0.1","0.05","0.025", "0.01")) {
    stop("Invalid significance level. ",
         "Valid options are: 'auto', 0.10, 0.05, 0.025, or 0.01.",call. = FALSE)
  }

  if(case == "auto"){
    case <- 3 # Default case is 3, which means unrestricted intercept and no trend
  }

  if(kmodel$extractedInfo$noConstant){
    if(case >1){
      notesArray <- c(notesArray,
                      "No constant term is included in the model. Case automatically adjusted to 1 (no intercept, no trend)."
      )
    }
    case<-1
  }else if(kmodel$extractedInfo$trend){
    if(case <4){
      notesArray <- c(notesArray,
                      "Trend detected in the model. Case automatically adjusted to 5 (unrestricted intercept and trend).")
    }
    case<-5

  }else{
    if(case != 3){
      # model has not trend but have constant
      notesArray <- c(notesArray,
                      "No trend is included. Case automatically adjusted to 3 (unrestricted intercept, no trend).")
    }
    case<-3
  }


  vcov_matrix <- stats::vcov(kmodel)
  CoefTest<-lmtest::coeftest(kmodel, vcov = vcov_matrix)
  # null_values <- 0

  if(kmodel$estInfo$type=="ecm"){
    testVarName<-"EcmRes"
    # names(null_values) <- "EcmRes"
  }else{
    testVarName<-replace_lag_var(.kardl_Settings_env$LongCoef ,kmodel$extractedInfo$dependentVar,1)
    # names(null_values) <- replace_lag_var(.kardl_Settings_env$LongCoef ,kmodel$extractedInfo$dependentVar,1)
  }

  Pt <- CoefTest[testVarName ,3]
  k <-length(kmodel$extractedInfo$longRunVars)-1

  structure(list(
    type="cointegration",
    case=case,
    statistic = c("t" = Pt),
    method = "Pesaran-Shin-Smith (PSS) Bounds t-test for cointegration",
    alternative = "Cointegrating relationship exists",
    data.name = deparse(substitute(model)),
    sample.size=kmodel$estInfo$TimeSpan,
    varnames=testVarName,
    k=k,
    sig=signif_level,
    notes=notesArray,
    test.func="psst"
  ), class = c("kardl_test","htest") )

}

#'
#' Summary method for Pesaran, Shin, and Smith t Bounds Test
#'
#' This function provides a summary of the results from the Pesaran, Shin, and Smith (PSS) t Bounds test for cointegration. It includes the test statistic, critical values, and the decision regarding the null hypothesis of no cointegration.
#'
#' @param testObj An object of class "kardl_test" and "htest" resulting from the \code{\link{psst}} function.
#' @param ... Additional arguments (not used).
#' @return An object of class "summary_htest" containing the test results, including the test statistic, critical values, decision, and any relevant notes.
#' @noRd
htestsummary.psst<-function(testObj,...){
  # check if the testObj is of class "kardl_test" and "htest" and if the method is "PesaranF"
  if(!inherits(testObj, "kardl_test") || !inherits(testObj, "htest") || testObj$test.func != "psst"){

    stop("Input object must be of class 'kardl_test' and 'htest' with test.func 'pssf'.", call. = FALSE)
  }
  notes<-testObj$notes
  case<-testObj$case
  signif_level<-testObj$sig
  Pt<-testObj$statistic[["t"]]
  k<-testObj$k

  if(case == 1){
    crVals <-c(-1.62, -1.62, -1.95, -1.95, -2.24, -2.24, -2.58, -2.58,
               -1.62, -2.28, -1.95, -2.60, -2.24, -2.90, -2.58, -3.22,
               -1.62, -2.68, -1.95, -3.02, -2.24, -3.31, -2.58, -3.66,
               -1.62, -3.00, -1.95, -3.33, -2.24, -3.64, -2.58, -3.97,
               -1.62, -3.26, -1.95, -3.60, -2.24, -3.89, -2.58, -4.23,
               -1.62, -3.49, -1.95, -3.83, -2.24, -4.12, -2.58, -4.44,
               -1.62, -3.70, -1.95, -4.04, -2.24, -4.34, -2.58, -4.67,
               -1.62, -3.90, -1.95, -4.23, -2.24, -4.54, -2.58, -4.88,
               -1.62, -4.09, -1.95, -4.43, -2.24, -4.72, -2.58, -5.07,
               -1.62, -4.26, -1.95, -4.61, -2.24, -4.89, -2.58, -5.25,
               -1.62, -4.42, -1.95, -4.76, -2.24, -5.06, -2.58, -5.44)
  }else if(case %in% c(2,3)){
    crVals <- c(-2.57, -2.57, -2.86, -2.86, -3.13, -3.13, -3.43, -3.43,-2.57, -2.91, -2.86, -3.22, -3.13, -3.50, -3.43, -3.82,-2.57, -3.21, -2.86, -3.53, -3.13, -3.80, -3.43, -4.10,-2.57, -3.46, -2.86, -3.78, -3.13, -4.05, -3.43, -4.37,-2.57, -3.66, -2.86, -3.99, -3.13, -4.26, -3.43, -4.60,-2.57, -3.86, -2.86, -4.19, -3.13, -4.46, -3.43, -4.79,-2.57, -4.04, -2.86, -4.38, -3.13, -4.66, -3.43, -4.99,-2.57, -4.23, -2.86, -4.57, -3.13, -4.85, -3.43, -5.19,-2.57, -4.40, -2.86, -4.72, -3.13, -5.02, -3.43, -5.37,-2.57, -4.56, -2.86, -4.88, -3.13, -5.18, -3.42, -5.54,-2.57, -4.69, -2.86, -5.03, -3.13, -5.34, -3.43, -5.68
    )
    case <- 3 # Default case is 3, which means unrestricted intercept and no trend
  }else if(case %in% c(4,5)){
    crVals <-c(-3.13, -3.13, -3.41, -3.41, -3.65, -3.66, -3.96, -3.97,-3.13, -3.40, -3.41, -3.69, -3.65, -3.96, -3.96, -4.26,-3.13, -3.63, -3.41, -3.95, -3.65, -4.20, -3.96, -4.53,-3.13, -3.84, -3.41, -4.16, -3.65, -4.42, -3.96, -4.73,-3.13, -4.04, -3.41, -4.36, -3.65, -4.62, -3.96, -4.96,-3.13, -4.21, -3.41, -4.52, -3.65, -4.79, -3.96, -5.13,-3.13, -4.37, -3.41, -4.69, -3.65, -4.96, -3.96, -5.31,-3.13, -4.53, -3.41, -4.85, -3.65, -5.14, -3.96, -5.49,-3.13, -4.68, -3.41, -5.01, -3.65, -5.30, -3.96, -5.65,-3.13, -4.82, -3.41, -5.15, -3.65, -5.44, -3.96, -5.79,-3.13, -4.96, -3.41, -5.29, -3.65, -5.59, -3.96, -5.94
    )
    case <- 5 # Default case is 5, which means unrestricted intercept and trend
  }

  #  crVals <- switch(case,

  PSSCrVals<-t(matrix(crVals,8,11))
  PSSCrVals<-cbind(0:10,PSSCrVals)
  MatName <- c("k","0.10L","0.10U","0.05L","0.05U","0.025L","0.025U","0.01L","0.01U")
  colnames(PSSCrVals)<-MatName

  if(k>10){
    k<-10
    notes<-c(notes,"The number of regressors exceeds the maximum limit for the critical values table. Using the critical values for 10 regressors.")
  }
  bu<-data.frame( L = PSSCrVals[k+1,c(2,4,6,8)],
                  U = PSSCrVals[k+1,c(3,5,7,9)]
  )
  rownames(bu)<-c("0.10","0.05","0.025","0.01")

  # Pt<- CoefTest_tidy[3]

  if(signif_level == "auto"){
    numericDec<-1

    if(Pt>=bu[4,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 1% level)"
      sig<-"0.01"
    }else if(Pt>=bu[3,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 2.5% level)"
      sig<-"0.025"
    }else if(Pt>=bu[2,"U"]){
      decision<-"Reject H0 \u2192 Cointegration (at 5% level)"
      sig<-"0.05"
    }else if(Pt>=bu[1,"U"]){
      decision<-"Reject H0 \u2192 Weak evidence of cointegration (at 10% level)"
      sig<-"0.10"
    }else{
      sig<-""
      if(Pt>=bu[1,"L"]){
        decision<-"Inconclusive"
        numericDec <- 0
      }else{
        decision<-"Fail to reject H0 \u2192 No Cointegration"
        numericDec <- -1
      }
    }
  }else{
    sig<-signif_level
    buRow<-switch (signif_level,"0.1"=1, "0.10"=1,"0.05"=2,"0.025"=3,"0.01"=4)
    if(Pt>=bu[buRow,"U"]){
      if(buRow==1){
        decision<-"Reject H0 \u2192 Weak evidence of cointegration (at 10% level)"
      }else {
        decision<-paste0("Reject H0 \u2192 Cointegration (at ",switch (buRow,"1"="10%","2"="5%","3"="2.5%","4"="0.01")," level)")
      }
      numericDec<-1
    }else if(Pt>=bu[buRow,"L"]){
      decision<-"Inconclusive"
      numericDec<-0
    }else{
      decision<-"Fail to reject H0 \u2192 No Cointegration"
      numericDec<- -1
    }
  }

  caseTxt <- switch(case,
                    "1"="I",
                    "2"="II",
                    "3"="III",
                    "4"="IV",
                    "5"="V"
  )
  structure(list(
    type="cointegration",
    case=caseTxt,
    statistic = c("t" = Pt),
    method = testObj$method,
    crit_vals   = bu,
    k=k,
    varnames=testObj$varnames,
    decision=decision,
    numericDec=numericDec, # -1: fail to reject, 0: inconclusive, 1: reject
    siglvl=sig,
    notes=notes
  ), class = "summary_htest")
}

#' @srrstats {G5.2} Tests exercise successful calls and error conditions for
#' print, summary, and plot S3 methods across all kardl object classes.
#' @srrstats {G5.3} Tests verify expected output strings, class labels, and
#' component values for summary and print methods.


local_models <- local({
  kardl_reset()
  linear <- kardl(imf_example_data, CPI ~ ER + PPI + trend,
                  mode = c(1, 1, 1))
  asym <- kardl(imf_example_data, CPI ~ ER + PPI + asymmetric(ER) + trend,
                mode = c(1, 1, 1, 1))
  list(linear = linear, asym = asym)
})


#' @srrstats {G5.3} Checks that `print.kardl_lm()` outputs the lag summary
#' line for both linear and asymmetric model objects.
test_that("print.kardl_lm produces expected output", {
  expect_output(print(local_models$linear), "Optimal lags")
  expect_output(print(local_models$asym),   "Optimal lags")
})


  #' @srrstats {G5.3} Checks that `kardl_longrun()` returns a `kardl_longrun`
  #' object with a populated summary table containing estimation type and
  #' coefficient labels.
test_that("summary_kardl_longrun works (class is kardl_longrun)", {
  lr <- kardl_longrun(local_models$linear)
  # The actual class assigned in longrun.R is "kardl_longrun"
  expect_s3_class(lr, "kardl_longrun")

  # summary dispatches correctly via summary.kardl_longrun
  sm <- summary(lr)
  expect_s3_class(sm, "summary_kardl_longrun")
  expect_false(is.null(sm$coefficients))

  expect_output(print(sm), "Estimation type")
  expect_output(print(sm), "Coefficients")
})

#' @srrstats {G5.3} Checks that print and summary for `kardl_symmetric`
#' produce the correct class and output strings for the F-test variant.
test_that("print and summary of kardl_symmetric (type F) work", {
  st <- symmetrytest(local_models$asym)

  # print.kardl_symmetric
  expect_output(print(st))

  # summary.kardl_symmetric + print.summary.kardl_symmetric
  sm <- summary(st)
  expect_s3_class(sm, "summary.kardl_symmetric")
  expect_output(print(sm), "Long-run symmetry tests")
})

#' @srrstats {G5.7} Tests that the `level` argument is accepted and does not
#' alter the returned class.
test_that("summary.kardl_symmetric with a custom level works", {
  st <- symmetrytest(local_models$asym)
  sm_strict <- summary(st, level = 0.01)
  expect_s3_class(sm_strict, "summary.kardl_symmetric")
})

#' @srrstats {G5.7} Tests chi-squared variant of the symmetry test summary.
test_that("print and summary of kardl_symmetric (type Chisq) work", {
  st_chi <- symmetrytest(local_models$asym, type = "Chisq")
  expect_output(print(st_chi))

  sm_chi <- summary(st_chi)
  expect_s3_class(sm_chi, "summary.kardl_symmetric")
  expect_output(print(sm_chi), "Long-run symmetry tests")
})

#' @srrstats {G5.7} Confirms that short-run-only symmetry test results
#' print correctly with the expected heading.
test_that("print.kardl_symmetric handles short-run-only results", {
  kardl_reset()
  sasym_model <- kardl(imf_example_data, CPI ~ PPI + sasymmetric(ER),
                       mode = c(1, 1, 1, 1))
  st_sr <- symmetrytest(sasym_model, component = "shortrun")
  expect_output(print(st_sr))

  sm_sr <- summary(st_sr)
  expect_output(print(sm_sr), "Short-run symmetry tests")
})

#' @srrstats {G5.3} Checks that `summary()` on a `pssf()` result returns
#' a `summary_htest` object containing critical values.
test_that("summary.kardl_test dispatches correctly for pssf", {
  pf <- pssf(local_models$linear)
  sm <- summary(pf)
  expect_s3_class(sm, "summary_htest")
  expect_output(print(sm), "Critical Values")
})

#' @srrstats {G5.3} Checks that `summary()` on a `psst()` result returns
#' a `summary_htest` object containing critical values.
test_that("summary.kardl_test dispatches correctly for psst", {
  pt <- psst(local_models$linear)
  sm <- summary(pt)
  expect_s3_class(sm, "summary_htest")
  expect_output(print(sm), "Critical Values")
})

#' @srrstats {G5.3} Checks that `summary()` on a `narayan()` result returns
#' a `summary_htest` object containing critical values.
test_that("summary.kardl_test dispatches correctly for narayan", {
  nr <- narayan(local_models$linear)
  sm <- summary(nr)
  expect_s3_class(sm, "summary_htest")
  expect_output(print(sm), "Critical Values")
})

#' @srrstats {G5.3} Checks print output headings and summary class for
#' `kardl_mplier` objects.
#' @srrstats {TS4.3} Confirms the horizon value is echoed in the print output.
test_that("print and summary for kardl_mplier work", {
  mp <- mplier(local_models$asym, horizon = 10)

  expect_output(print(mp), "kardl Dynamic Multiplier Object")
  expect_output(print(mp), "Horizon: 10")

  sm <- summary(mp)
  expect_s3_class(sm, "summary.kardl_mplier")
  expect_output(print(sm), "Summary of Dynamic Multipliers")
  expect_output(print(sm), "Horizon: 10")
})

#' @srrstats {G5.3} Checks print output and summary class for `kardl_boot`.
#' @srrstats {G5.5} Bootstrap replications use the random residual resampling
#' method; this test confirms that results are returned correctly.
test_that("print and summary for kardl_boot work", {
  kardl_reset()
  asym_only <- kardl(imf_example_data, CPI ~ asymmetric(ER),
                     mode = c(1, 1, 1))
  bt <- bootstrap(asym_only, horizon = 10, replications = 5, level = 90)

  expect_s3_class(bt, "kardl_boot")
  expect_output(print(bt), "kardl Bootstrap Results")
  expect_output(print(bt), "Confidence level: 90")

  sm <- summary(bt)
  expect_s3_class(sm, "summary.kardl_boot")
  expect_output(print(sm), "Summary of Dynamic Multipliers")
})

#' @srrstats {TS5.0} Confirms that `plot.kardl_mplier()` runs without error
#' for both default (all variables) and single-variable calls.
#' @srrstats {TS5.8} Verifies that variable-specific plots can be requested.
test_that("plot.kardl_mplier runs without error", {
  mp <- mplier(local_models$asym, horizon = 5)
  expect_no_error(plot(mp))
  expect_no_error(plot(mp, variables = "ER"))
})

#' @srrstats {G5.2a} Checks that requesting an unknown variable in `plot()`
#' produces a warning rather than a silent failure.
test_that("plot.kardl_mplier warns for unknown variable", {
  mp <- mplier(local_models$asym, horizon = 5)
  expect_warning(plot(mp, variables = "NOTAVAR"), "not exits")
})

#' @srrstats {TS5.5} Confirms that plot methods operate on computed model
#' outputs without error.
test_that("plot on kardl_longrun runs without error", {
  # Note: class is "kardl_longrun", so plot.lm is dispatched (not plot.kardl_long_run)
  lr <- kardl_longrun(local_models$linear)
  expect_no_error(plot(lr, which = 1))
})

#' @srrstats {TS4.3} Verifies that the horizon index is stored in the result.
#' @srrstats {G5.2} The test suite exercises representative successful calls
#' and selected error conditions for post-estimation tests, multipliers, and
#' bootstrap confidence intervals.
#' @srrstats {G5.3} Tests verify expected classes, dimensions, and component
#' values for all post-estimation objects.
#' @srrstats {G5.4} Correctness tests compare selected outputs against
#' expected structural properties of ARDL and NARDL models.
test_that("Post-estimation tests run correctly", {
  kardl_reset()
  formula <- CPI ~ ER + PPI + trend

  model <- kardl(data = imf_example_data, formula = formula, mode = "quick", maxlag = 2)

  #' Test kardl_longrun
  lr <- kardl_longrun(model)
  expect_s3_class(lr, c("kardl_long_run","lm"))
  expect_gt(length(lr$coef), 0)

  #' Test symmetrytest
  symm_model <- kardl(data = imf_example_data, formula = CPI ~ ER + PPI + asymmetric(ER) + trend, mode = "quick", maxlag = 2)
  st <- symmetrytest(symm_model)
  expect_s3_class(st, c("kardl_symmetric","list"))
  expect_s3_class(st$Lwald, c("anova","data.frame"))
  expect_identical(st$Lwald$Df, 1L)

  #' Test pssf
  pf <- pssf(model, case = 3, signif_level = "auto")
  expect_s3_class(pf,c("kardl_test", "htest" ))
  expect_identical(pf$type, "cointegration")
  expect_false(is.null(pf$statistic))

  #' Test psst
  pt <- psst(model, case = 3, signif_level = "auto")
  expect_s3_class(pt,c("kardl_test", "htest" ))
  expect_identical(pt$type, "cointegration")
  expect_false(is.null(pt$statistic))

  #' Test narayan
  nr <- narayan(model, case = 3, signif_level = "auto")
  expect_s3_class(nr,c("kardl_test", "htest" ))
  expect_identical(nr$type, "cointegration")
  expect_false(is.null(nr$statistic))

  #' Test mplier
  mp <- mplier(symm_model,horizon = 38)
  expect_s3_class(mp, "kardl_mplier" )
  expect_identical(mp$horizon, 38)
  expect_false(is.null(mp$mpsi))

  #' Test ecm
  ec <- ecm(data = imf_example_data, formula = formula, mode = "quick", case = 3, signif_level = "auto")
  expect_s3_class(ec, c("kardl_lm", "lm"))
})

#' @srrstats {G5.7} Tests symmetry test when restricted to the long-run
#' component, confirming Swald is NULL and Lwald is populated.
test_that("symmetrytest with component = 'longrun' only works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + asymmetric(ER) + trend,
                 mode = c(1, 1, 1, 1))
  st <- symmetrytest(model, component = "longrun")
  expect_s3_class(st, "kardl_symmetric")
  expect_false(is.null(st$Lwald))
  expect_null(st$Swald)
})
#' @srrstats {G5.7} Tests symmetry test when restricted to the short-run
#' component, confirming Lwald is NULL and Swald is populated.
test_that("symmetrytest with component = 'shortrun' only works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ PPI + sasymmetric(ER),
                 mode = c(1, 1, 1, 1))
  st <- symmetrytest(model, component = "shortrun")
  expect_s3_class(st, "kardl_symmetric")
  expect_null(st$Lwald)
  expect_false(is.null(st$Swald))
})
#' @srrstats {G5.7} Tests that chi-squared test type produces a Pr(>Chisq)
#' column in the Lwald table.
test_that("symmetrytest with type = 'Chisq' works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + asymmetric(ER) + trend,
                 mode = c(1, 1, 1, 1))
  st <- symmetrytest(model, type = "Chisq")
  expect_s3_class(st, "kardl_symmetric")
  expect_identical(st$type, "Chisq")
  expect_true("Pr(>Chisq)" %in% colnames(st$Lwald))
})
#' @srrstats {G5.7} Confirms that when `vars` is restricted to a single
#' variable, only that variable appears in the Lwald row names.
test_that("symmetrytest with specific vars argument works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + asymmetric(ER) + trend,
                 mode = c(1, 1, 1, 1))
  st <- symmetrytest(model, vars = "ER")
  expect_s3_class(st, "kardl_symmetric")
  expect_identical(rownames(st$Lwald), "ER")
})
#' @srrstats {G5.2b} Checks that a variable name not in the model produces
#' an informative error.
#' @srrstats {G5.8} Verifies edge-condition handling for invalid variable selection.
test_that("symmetrytest errors on bad vars", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + asymmetric(ER) + trend,
                 mode = c(1, 1, 1, 1))
  expect_error(symmetrytest(model, vars = "NOTAVAR"))
})
#' @srrstats {G5.2b} Confirms that passing an ECM object to `symmetrytest()`
#' triggers an informative error.
test_that("symmetrytest errors on ECM model", {

  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_error(symmetrytest(ec), "ECM")
})

#' @srrstats {G5.2b} Checks that a plain `lm` object is rejected with an error.
test_that("symmetrytest errors on non-kardl_lm input", {
  expect_error(symmetrytest(lm(mpg ~ wt, data = mtcars)))
})

#' @srrstats {G5.7} Tests `pssf()` with an explicitly specified significance
#' level, confirming both test and summary objects are returned correctly.
test_that("pssf with specific signif_level works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  pf <- pssf(model, signif_level = "0.05")
  expect_s3_class(pf, "kardl_test")
  sm <- summary(pf)
  expect_s3_class(sm, "summary_htest")
})

#' @srrstats {G5.7} Tests that a numeric significance level is accepted and
#' coerced to a character value internally.
test_that("pssf with numeric signif_level works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  pf <- pssf(model, signif_level = 0.05)
  expect_s3_class(pf, "kardl_test")
})

#' @srrstats {G5.7} Tests `pssf()` under the unrestricted-intercept-and-trend
#' case specification.
test_that("pssf with case = 5 (trend model) works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  pf <- pssf(model, case = 5)
  expect_s3_class(pf, "kardl_test")
})

#' @srrstats {G5.2b} Checks that passing an ECM object to `pssf()` produces
#' an informative error.
test_that("pssf errors on ECM model", {
  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_error(pssf(ec))
})

#' @srrstats {G5.8} Checks that an out-of-range case value is rejected.
test_that("pssf errors on invalid case", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_error(pssf(model, case = 99))
})

#' @srrstats {G5.8} Checks that an unsupported significance level string
#' is rejected before the test is performed.
test_that("pssf errors on invalid signif_level", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_error(pssf(model, signif_level = "0.03"))
})

#' @srrstats {G5.7} Tests the PSS t-test on an ECM object where the
#' error-correction residual is the test variable.
test_that("psst with ECM model works (uses EcmRes as test var)", {
  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  pt <- psst(ec)
  expect_s3_class(pt, "kardl_test")
  sm <- summary(pt)
  expect_s3_class(sm, "summary_htest")
})

#' @srrstats {G5.4} Checks that the case auto-adjustment logic correctly
#' detects a trend term and elevates the case to 5.
test_that("psst with trend model auto-adjusts to case 5", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  pt <- psst(model, case = 3)
  expect_s3_class(pt, "kardl_test")
  expect_identical(pt$case, 5)  #' auto-adjusted because model has trend
})

#' @srrstats {G5.8} Verifies that an invalid case value is rejected.
test_that("psst errors on invalid case", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_error(psst(model, case = 99))
})

#' @srrstats {G5.7} Tests the Narayan small-sample bounds test under case 5.
test_that("narayan with case = 5 works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  nr <- narayan(model, case = 5)
  expect_s3_class(nr, "kardl_test")
})

#' @srrstats {G5.7} Tests `narayan()` with an explicit 1% significance level
#' and confirms that `summary()` dispatches correctly.
test_that("narayan with specific signif_level works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI, mode = c(1, 1, 1))
  nr <- narayan(model, signif_level = "0.01")
  expect_s3_class(nr, "kardl_test")
  sm <- summary(nr)
  expect_s3_class(sm, "summary_htest")
})

#' @srrstats {G5.2b} Checks that passing an ECM object to `narayan()` is rejected.
test_that("narayan errors on ECM model", {
  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_error(narayan(ec))
})

#' @srrstats {G5.8} Verifies that case = 1 (unsupported by Narayan) is rejected.
test_that("narayan errors on invalid case", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI, mode = c(1, 1, 1))
  expect_error(narayan(model, case = 1))
})

#' @srrstats {G5.7} Tests that the `minProb` threshold filters coefficients
#' and returns a `kardl_mplier` object with the correct number of rows.
#' @srrstats {TS4.3} Verifies the horizon index dimension of the returned mpsi matrix.
test_that("mplier with minProb > 0 works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI + asymmetric(ER) + trend,
                 mode = c(1, 1, 1, 1))
  mp <- mplier(model, horizon = 10, minProb = 0.1)
  expect_s3_class(mp, "kardl_mplier")
  expect_identical(nrow(mp$mpsi), 11L)
})

#' @srrstats {G5.2b} Checks that a plain `lm` object is rejected by `mplier()`.
test_that("mplier errors on non-kardl_lm input", {
  expect_error(mplier(lm(mpg ~ wt, data = mtcars)))
})

#' @srrstats {G5.7} Tests `mplier()` on a long-run-asymmetric (SA) model.
test_that("mplier with lasymmetric model works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ lasymmetric(ER), mode = c(1, 1))
  mp <- mplier(model, horizon = 10)
  expect_s3_class(mp, "kardl_mplier")
})

#' @srrstats {G5.7} Tests `mplier()` on a short-run-asymmetric (AS) model.
test_that("mplier with sasymmetric model works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ PPI + sasymmetric(ER),
                 mode = c(1, 1, 1, 1))
  mp <- mplier(model, horizon = 10)
  expect_s3_class(mp, "kardl_mplier")
})



#' @srrstats {G5.5} The bootstrap resampling uses a random seed approach
#' (set externally before test runs under `testthat`).
#' @srrstats {G5.3} Checks that the returned `kardl_boot` object has the
#' correct class, level, horizon, and a non-NULL `mpsi` component.
test_that("bootstrap with asymmetric model works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ asymmetric(ER),
                 mode = c(1, 1, 1))
  bt <- bootstrap(model, horizon = 10, replications = 5, level = 95)
  expect_s3_class(bt, "kardl_boot")
  expect_identical(bt$level, 95)
  expect_identical(bt$horizon, 10)
  expect_false(is.null(bt$mpsi))
})

#' @srrstats {G5.2b} Confirms that a plain `lm` object is rejected by `bootstrap()`.
test_that("bootstrap errors on non-kardl_lm input", {
  expect_error(bootstrap(lm(mpg ~ wt, data = mtcars)))
})

#' @srrstats {G5.7} Tests that `bootstrap()` runs on a linear (non-asymmetric)
#' model without attempting to compute confidence interval columns.
test_that("bootstrap with linear model (no CI columns) still works", {
  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI, mode = c(1, 1, 1))
  bt <- bootstrap(model, horizon = 5, replications = 3)
  expect_s3_class(bt, "kardl_boot")
})

#' Print and summary methods for kardl objects
#' @description
#' These methods provide custom print and summary outputs for various kardl object classes, including `
#' kardl_mplier`, `kardl_boot`, `kardl_longrun`, `kardl_symmetric`, and `kardl_test`. The print methods display key information about the object, while the summary methods provide detailed summaries of the results, including statistical tests and decisions.
#'
#'
#' @export
#' @method print kardl_mplier
#' @noRd
print.kardl_mplier <- function(x, ...) {
  cat("kardl Dynamic Multiplier Object\n")
  cat("Horizon:", x$horizon, "\n")
  print(x$mpsi, ...)
  invisible(x)
}


#' @export
#' @method summary kardl_mplier
#' @noRd
summary.kardl_mplier <- function(object, ...) {

  out <- list(
    horizon = object$horizon,
    summary=summary(object$mpsi)
  )
  class(out) <- "summary.kardl_mplier"
  out
}

#' @export
#' @method print summary.kardl_mplier
#' @noRd
print.summary.kardl_mplier <- function(x, ...) {
  cat("Summary of Dynamic Multipliers\n")
  cat("Horizon:", x$horizon, "\n\n")
  print( x$summary, ...)
  invisible(x)
}

#' @export
#' @method print kardl_boot
#' @noRd
print.kardl_boot <- function(x, ...) {
  cat("kardl Bootstrap Results\n")
  cat("Confidence level:", x$level, "%\n")
  cat("Horizon:", x$horizon, "\n")
  print(x$mpsi, ...)
  invisible(x)
}

#' @export
#' @method summary kardl_boot
#' @noRd
summary.kardl_boot <- function(object, ...) {

  out <- list(
    horizon = object$horizon,
    summary=summary(object$mpsi)
  )
  class(out) <- "summary.kardl_boot"
  out
}

#' @export
#' @method print summary.kardl_boot
#' @noRd
print.summary.kardl_boot <- function(x, ...) {
  cat("Summary of Dynamic Multipliers\n")
  cat("Horizon:", x$horizon, "\n\n")
  print( x$summary, ...)
  invisible(x)
}

#' @export
#' @method plot kardl_mplier
#' @srrstats {TS5.0} A plot method is implemented for `kardl_mplier` objects returned by `mplier()`.
#' @srrstats {TS5.1} The horizontal axis of multiplier plots is labelled as the horizon or time-step index.
#' @srrstats {TS5.2} The horizon variable is plotted on the horizontal axis in dynamic multiplier visualisations.
#' @srrstats {TS5.3} Multiplier plots display the horizon index used in the model output.
#' @srrstats {TS5.8} Multiplier plots distinguish positive and negative shock components by variable where asymmetric effects are present.
#' @noRd
plot.kardl_mplier <- function(x, variables ="all" , ...) {
  mpsi <- x$mpsi
  plots<-list()

    if(all(variables == "all")){
      for (variable in  x$vars$independentVars) {
        plots[[variable]] <- mplierggplot(mpsi, x$vars,variable)
      }
    }else{
      for (variable in variables) {
        if(! variable %in% x$vars$independentVars){
          warning(variable, " is not exits among independent variables!", call.=F)
        }else{
          plots[[variable]] <- mplierggplot(mpsi, x$vars,variable)
        }
      }
    }
  for (p in plots) {
    print(p)
  }

}

#' @import stats
#' @export
#' @method  summary kardl_longrun
#' @srrstats {G3.1} Long-run multiplier standard errors returned in the summary are computed via the delta method using the fitted model's variance-covariance matrix.
#' @srrstats {TS4.2} The summary documents coefficients, standard errors, t-values, and p-values for each long-run multiplier.
#' @noRd
summary.kardl_longrun <- function(object, ...) {
  objcoef <- object$original_model$coefficients

  vcov_matrix <- stats::vcov( object$original_model)

  my_dep <- object$depvar
  my_indep <- object$indepvars

  multipliers_coef <- object$coefficients

  A <- - 1 / objcoef[my_dep]

  lr_var <- vapply(names(multipliers_coef), function(i) {
    B <- objcoef[i] / (objcoef[my_dep]^2)
    (A^2) * vcov_matrix[i, i] +
      2 * A * B * vcov_matrix[i, my_dep] +
      (B^2) * vcov_matrix[my_dep, my_dep]
  }, numeric(1))

  multipliers_coef_se <- sqrt(as.vector(lr_var))
  tvals <- multipliers_coef / multipliers_coef_se
  pvals <- 2 * stats::pt(-abs(tvals), df = stats::df.residual(object$original_model))

  coef_table <- cbind(
    Estimate = multipliers_coef,
    `Std. Error` = multipliers_coef_se,
    `t value` = tvals,
    `Pr(>|t|)` = pvals
  )

  ans <- list(
    call = object$call,
    coefficients = coef_table,
    sigma = sqrt(sum(object$residuals^2) / object$df.residual),
    df = c(object$rank, object$df.residual, length(object$coefficients)),
    r.squared = NA_real_,
    adj.r.squared = NA_real_,
    fstatistic = NA,
    cov.unscaled = NA,
    note = attr(object, "note"),
    estimation_type = attr(object, "estimation_type")
  )

  class(ans) <- c("summary_kardl_longrun", "summary.lm")
  ans
}

#' @export
#' @method print summary_kardl_longrun
#' @noRd
print.summary_kardl_longrun <- function(x, ...) {
  cat("\nCall:\n", paste(deparse(x$call), collapse = "\n"), "\n", sep = "")

  cat("\nEstimation type:\n")
  cat(x$estimation_type, "\n")

  cat("\nCoefficients:\n")
  printCoefmat(x$coefficients, ...)

  cat("\nNote:\n")
  cat(x$note, "\n")

  invisible(x)
}

#' @export
#' @method print kardl_symmetric
#' @noRd
print.kardl_symmetric <- function(x, ...) {
  #print(x$allWald)
  # cat("\n==============================\n")
  cat("\n")
  if (!is.null(x$Lwald) && nrow(x$Lwald) > 0) {
    #    cat("Long-run tests only:\n")
    print(x$Lwald, ...)
    cat("\n")
  }

  if (!is.null(x$Swald) && nrow(x$Swald) > 0) {
    # cat("Short-run tests only:\n")
    print(x$Swald, ...)
    cat("\n")
  }
  invisible(x)
}

#' @export
#' @method summary kardl_symmetric
#' @noRd
summary.kardl_symmetric <- function(object,level=0.05 ,...) {
  decision <- list()
  pValType<-ifelse(object$type == "Chisq","Pr(>Chisq)", "Pr(>F)")
   if(!is.null(object$Lwald)){
     decision$long_run <-list()
       for(v in rownames(object$Lwald)){
         decision$long_run[[v]]<- ifelse( object$Lwald[v,pValType] < level,  paste0("Reject H0 at ",level*100,"% level. Indicating long-run asymmetry for variable ",v,"."),
                                          paste0("Fail to Reject H0 at ",level*100,"% level. Indicating long-run symmetry for variable ",v,"."))
       }
     }

   if(!is.null(object$Swald)){
     decision$short_run <-list()
     for(v in rownames(object$Swald)){
       decision$short_run[[v]]<- ifelse( object$Swald[v,pValType] < level,  paste0("Reject H0 at ",level*100,"% level. Indicating short-run asymmetry for variable ",v,"."),
                                        paste0("Fail to Reject H0 at ",level*100,"% level. Indicating short-run symmetry for variable ",v,"."))
     }
   }
  out <- list(
    Lwald = object$Lwald,
    Swald = object$Swald,
    Lhypotheses = object$Lhypotheses,
    Shypotheses = object$Shypotheses,
    type = object$type,
    decision = decision
  )
  class(out) <- "summary.kardl_symmetric"
  out
}

#' @export
#' @method print summary.kardl_symmetric
#' @noRd
print.summary.kardl_symmetric <- function(x, ...){
  if(x$type == "Chisq"){
    pValType <- "Pr(>Chisq)"
    Valtype <- "Chi-squared"
    testLable <- "Chisq"
  } else if(x$type == "F"){
    pValType <- "Pr(>F)"
    Valtype <- "F"
    testLable <- "F value"
  }

  if (!is.null(x$Lwald) && nrow(x$Lwald) > 0) {
    cat("Long-run symmetry tests:\n\n")
    for(v in rownames(x$Lwald)){
      cat("Test for variable: ",v,"\n")
      cat(Valtype, " statistic: ",
          format(x$Lwald[v, testLable], digits = getOption("digits")),
          ", p-value: ",
          format(x$Lwald[v, pValType], digits = getOption("digits")),
          "\n", sep = "")
      cat("Test Decision: ", x$decision$long_run[[v]], "\n")
      cat("Hypotheses:\n")
      cat(paste0("H0: ", x$Lhypotheses$H0[[v]], "\n"))
      cat(paste0("H1: ",enc2utf8( x$Lhypotheses$H1[[v]]), "\n\n"))
      # write F value and p-value

    }

  }
  if(!is.null(x$Swald) && !is.null(x$Lwald)){  cat("\n_____________________________\n") }
  if (!is.null(x$Swald) && nrow(x$Swald) > 0) {
    cat("Short-run symmetry tests:\n\n")
    for(v in rownames(x$Swald)){
      cat("Test for variable: ",v,"\n")

      cat(Valtype," statistic: ",
          format(x$Swald[v,testLable], digits = getOption("digits")),
          ", p-value: ",
          format(x$Swald[v,pValType], digits = getOption("digits")),
          "\n")
      cat("Test Decision: ", x$decision$short_run[[v]], "\n")
      cat("Hypotheses:\n")
      cat(paste0("H0: ", x$Shypotheses$H0[[v]], "\n"))
      cat(paste0("H1: ", enc2utf8(x$Shypotheses$H1[[v]]), "\n\n"))
      # write F value and p-value

    }
  }
  invisible(x)

}


#' @export
#' @method summary kardl_test
#' @noRd
summary.kardl_test <- function(object, ...){
    htestsummary(object,...)

}

#' @export
#' @method print summary_htest
#' @noRd
print.summary_htest <- function(x, ...){
  cat(x$method, "\n")
  cat(names(x$statistic), " = ", format(unname(x$statistic), digits = getOption("digits")),"\n")
  cat("k = ", x$k, "\n")



  cat("\nHypotheses:\n")
  cat( paste0("H0: Coef(", paste(x$varnames,collapse = ") = Coef(" ),") = 0"), "\n")
  cat(paste0("H1: Coef(", paste(x$varnames,collapse = ") \u2260 Coef(" ),")\u2260",  " 0"), "\n")
   # add x$decision
  cat("\nTest Decision: ", x$decision, "\n")
  cat("\nCritical Values (Case ",x$case,"):\n")
  print(x$crit_vals, ...)
  if (!is.null(x$notes)) {
    cat("\nNotes:\n")
    for (note in x$notes) {
      cat("   \u2022 ", note, "\n", sep = "")
    }
    cat("\n")
  }
  invisible(x)
}

#' @export
#' @method print kardl_long_run
#' @noRd
print.kardl_long_run<- function(x, ...) {
  cat("Long-run multiplier estimate\n")
  cat("=================================\n")

  NextMethod()  # continues with normal coefficient printing
  cat("Note:",attr(x, "note"), "\n\n")
  invisible(x)
}

#' @export
#' @method plot kardl_long_run
#' @noRd
plot.kardl_long_run <- function(x,  ...) {
  cat("WARNING: Residual diagnostic plots are meaningless for long-run estimators\n")
  if (interactive()) {
    ans <- readline("Show plots anyway? (y/n): ")
    if (tolower(ans) != "y") return(invisible(NULL))
  }
  NextMethod("plot")
}


#' @export
#' @method print kardl_lm
#' @noRd
print.kardl_lm<- function(x, ...) {
  cat("Optimal lags for each variable (",x$argsInfo$criterion,"):\n")
  cat(toString(sprintf("%s: %d", names(x$lagInfo$OptLag), x$lagInfo$OptLag)),"\n")
  NextMethod()  # continues with normal coefficient printing
  if (!is.null(x$notes)) {
    cat("\nNotes:\n")
    for (note in x$notes) {
      cat("   \u2022 ", note, "\n", sep = "")
    }
    cat("\n")
  }
  invisible(x)
}

#' Compute Long-Run Multipliers from a kardl Model
#'
#' This function calculates the long-run parameters of a KARDL model estimated using the \code{kardl} function. The long-run parameters are calculated by dividing the negative of the coefficients of the independent variables by the coefficient of the dependent variable. If an intercept is included in the model, it is also standardized by dividing it by the negative of the long-run parameter of the dependent variable.
#'
#' The function also calculates the standard errors of the long-run multipliers using the delta method, which accounts for the covariance between the coefficients. The fitted values and residuals of the long-run model are calculated based on the original data and the long-run multipliers.
#'
#' The function returns an object of class \code{kardl_long_run}, which contains the long-run multipliers, their standard errors, t-statistics, p-values, fitted values, residuals, and other relevant information for further analysis and diagnostics.
#'
#' Note that the fitted values and residuals from the long-run model are not centered (i.e., they do not have a mean of zero) by design, which means that diagnostic plots and residual-based tests may not be valid for this model. The primary focus of this function is on the estimation of the long-run multipliers and their associated statistics.
#'
#' The long-run multipliers are calculated using the formula:
#' \deqn{LRM_i = -\frac{\eta_i}{\eta_0}}.
#'
#' t-values and p-values are calculated using the standard errors obtained from the delta method, which accounts for the covariance between the coefficients.
#' Delta method formula for standard errors of long-run multipliers:
#' \deqn{SE(LR_i) = \sqrt{(A^2) \cdot Var(\eta_i) + 2 \cdot A \cdot B \cdot Cov(\eta_i, \eta_0) + (B^2) \cdot Var(\eta_0)}} where \deqn{A = \frac{\partial LRM_i}{\partial \eta_i} = -\frac{1}{\eta_0}} and \deqn{B = \frac{\partial LRM_i}{\partial \eta_0} = \frac{\eta_i}{\eta_0^2}} .
#' Hence, \eqn{\eta_i} is the coefficient of the independent variable and \eqn{\eta_0} is the coefficient of the dependent variable in the original KARDL model.
#'
#'
#'
#' @param model An object of class \code{kardl} estimated using the \code{kardl} function.
#'
#'
#' @return An object of class \code{kardl_long_run}, which is a list containing:
#' \itemize{
#'    \item \code{coefficients}: A named vector of long-run multipliers.
#'    \item \code{residuals}: A vector of residuals from the long-run model.
#'    \item \code{effects}: A vector of effects from the long-run model.
#'    \item \code{rank}: The rank of the long-run model.
#'    \item \code{fitted.values}: A vector of fitted values from the long-run model.
#'    \item \code{assign}: A vector indicating the assignment of coefficients to terms in the long-run model.
#'    \item \code{qr}: The QR decomposition of the design matrix of the long-run model.
#'    \item \code{df.residual}: The degrees of freedom of the residuals of the long-run model.
#'    \item \code{xlevels}: A list of factor levels used in the long-run model.
#'    \item \code{call}: The matched call used to create the long-run model.
#'    \item \code{terms}: The terms object of the long-run model.
#'    \item \code{model}: The data frame used in the long-run model.
#' }
#' @export
#' @import stats
#'
#'
#' @srrstats {G1.3}  Statistical terms such as "long-run multipliers", "standard errors", "t-statistics", and "p-values" are clearly defined and used consistently throughout the documentation and examples.
#' @srrstats {G2.0} The function validates that `model` is a `kardl_lm` object before extracting coefficients and covariance matrices.
#' @srrstats {G3.1} Long-run multiplier standard errors are computed using the delta method applied to the fitted model variance-covariance matrix.
#' @srrstats {G3.1a} Documentation describes the delta-method formula for standard errors and the partial derivatives A and B used in the calculation.
#' @srrstats {TS4.2} The return value is documented: `coefficients`, `residuals`, `fitted.values`, `df.residual`, `call`, `terms`, and `model` are all described.
#' @srrstats {TS4.0b} The return value uses the explicit class `kardl_longrun`.
#' @importFrom msm deltamethod
#' @seealso \code{\link{kardl}}, \code{\link{pssf}}, \code{\link{psst}}
#' @examples

#' kardl_model<-kardl(imf_example_data,
#'                    CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
#'                    mode=c(1,2,3,0))
#' long<-kardl_longrun(kardl_model)
#'
#' # Calculate the long-run multipliers
#' long
#' # Details of the long-run multipliers
#' summary(long)
#'
#'
#' # Using magrittr
#'
#' @examplesIf requireNamespace("magrittr", quietly = TRUE)
#' library(magrittr)
#'      imf_example_data %>%
#'      kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend, mode=c(1,2,3,0)) %>%
#'      kardl_longrun() %>% summary()
#'

kardl_longrun <- function(model) {
  objcoef <- model$coefficients

  myvars <- replace_lag_var(
    .kardl_Settings_env$LongCoef,
    model$extractedInfo$longRunVars,
    1
  )

  my_dep <- myvars[1]
  my_indep <- myvars[-1]
  my_assign <- rep(1, length(my_indep))

  multipliers_coef <- -objcoef[my_indep] / objcoef[my_dep]

  mydata <- model$model[, myvars, drop = FALSE]
  mydata <- as.data.frame(mydata)

  indepMatrix <- as.matrix(mydata[, !names(mydata) %in% my_dep, drop = FALSE])
  fitted_values <- indepMatrix %*% as.vector(multipliers_coef)
  residuals_values <- as.numeric(mydata[, my_dep] - fitted_values)

  tol <- 1e-7
  qr_X <- qr(indepMatrix, tol = tol, LAPACK = FALSE)
  qr_X$tol <- tol

  qty <- qr.qty(qr_X, mydata[, my_dep])
  eff <- numeric(length(mydata[, my_dep]))
  eff[qr_X$pivot[1:qr_X$rank]] <- qty[1:qr_X$rank]
  eff[-seq_len(ncol(qr_X$qr))] <- qty[-(1:qr_X$rank)]
  names(eff) <- names(multipliers_coef)

  my_terms <- as.formula(
    paste(my_dep, "~", paste(my_indep, collapse = " + "), "-1")
  )
  proper_terms <- terms(my_terms, data = mydata)

  lm_obj <- list(
    original_model = model,
    coefficients = multipliers_coef,
    residuals = residuals_values,
    effects = eff,
    rank = length(multipliers_coef),
    fitted.values = as.vector(fitted_values),
    assign = my_assign,
    qr = qr_X,
    df.residual = length(residuals_values) - length(multipliers_coef),
    xlevels = model$xlevels,
    call = match.call(),
    terms = proper_terms,
    model = mydata,
    depvar = my_dep,
    indepvars = my_indep
  )

  class(lm_obj) <- c("kardl_longrun", "lm")

  attr(lm_obj, "estimation_type") <- "Long-run multipliers"
  attr(lm_obj, "note") <- paste0(
    "Coefficients, standard errors, t-statistics and p-values are reliably estimated.\n",
    "Fitted values and residuals are NOT centered (E(u) \u2260 0 by design) \u2192 ",
    "diagnostic plots and residual-based tests are invalid."
  )

  lm_obj
}

#' Merge Lists with Priority to the First Argument
#'
#' The first list of values takes precedence. When both lists have items with the same names, the values from the first list will be applied. In merging the two lists, priority is given to the left list, so if there are overlapping items, the corresponding value from the left list will be used in the merged result.
#' @param first The first list
#' @param second The second list
#' @param ... Additional lists to merge
#'
#' @return A merged list with unique names, prioritizing values from the first list in case of name conflicts.
#' @export
#' @seealso  \code{\link[base]{append}}
#' @details
#' The \code{lmerge()} function is designed to merge multiple lists while giving precedence to the values in the first list.
#' This function is particularly useful when you want to combine settings or parameters from multiple sources while ensuring that the primary source (the first list) takes priority over others.
#'
#' For right merge, a user can simply swap the order of the lists to give priority to the second list. For example, \code{lmerge(b, a)} will prioritize values from list \code{b} over those in list \code{a}.
#'
#' @examples
#'
#' a<-list("a"="first a","b"="second a","c"=list("w"=12,"k"=c(1,3,6)))
#' b<-list("a"="first b","b"="second b","d"=14,"e"=45)
#' myMerged<- lmerge(a,b)
#' print(unlist(myMerged))
#'
#' # for right merge
#' myMerged<- lmerge(b,a)
#' print(unlist(myMerged))
#'
#' # for more than two lists
#' myMerged<- lmerge(a,b,c("v1"=11,22,3,"v5"=5))
#' print(unlist(myMerged))
#'
#' # for more than two lists with nested lists
#' m2<-list("m1"="kk2","m1.2.3"=list("m1.1.1"=333,"m.1.4"=918,"m.1.5"=982,"m.1.6"=981,"m.1.7"=928))
#' m3<-list("m1"="kk23","m2.3"=2233,"m1.2.4"=list("m1.1.1"=333444,"m.1.5"=982,"m.1.6"=91,"m.1.7"=928))
#' a<-c(32,34,542,"k"=35)
#' b<-c(65,"k"=34)
#'
#' h1<-lmerge(a, m2)
#' print(unlist(h1))
#'
#' h2<-lmerge(a,b,m2,m3,list("m1.1"=4))
#' print(unlist(h2))
#'

lmerge<-function(first,second,...){
  a<-first
  b<-second
  otherArgs<-list(...)
  for (v in seq_along(a)) {
    n<-names(a[v])
    if(isFALSE( is.list( b[n]))){
      if(nzchar(n)){
        no <- which(names(b) == n)
        if(length(no)>0){
          b <- b[-no]
        }
      }
    }else{
      b[n]<-NULL
    }

  }
  o<-c(a,b)
  if(length(otherArgs)>0){
    for (q in seq_along(otherArgs)) {
      b2<-otherArgs[q][[1]]
      for (v in seq_along(o)) {
        n<-names(o[v])
        if(isFALSE( is.list( b2[n]))){
          if(nzchar(n)){
            no <- which(names(b2) == n)
            if(length(no)>0)
              b2 <- b2[-no]
          }
        }else{
          b2[n]<-NULL
        }
      }
      o<- c(o,b2)
    }
  }
  o
}


#' Parse Formula Variables
#'
#' The \code{parseFormula()} function analyzes a given formula to identify and extract variables that match specified patterns. It is particularly useful for isolating variables enclosed within certain functions or constructs in the formula, such as \code{asym()}, \code{det()}, or any user-defined patterns.
#'
#'
#' @param formula The initial formula for the model, typically specified using R's formula syntax (e.g.,  \code{y ~ x + f(x1 + x2)}).
#'
#'
#' @return A list containing:
#' \itemize{
#' \item \code{response}: The response variable(s) extracted from the formula.
#' \item \code{intercept}: A logical value indicating whether the formula includes an intercept (default is TRUE).
#' \item \code{dot}: A logical value indicating whether the formula includes a dot (.) representing all other variables (default is FALSE).
#' \item \code{outside}: A vector of variables that are outside any specified patterns. It includes the variables that are not detected within the specified patterns in the formula.
#' \item \code{inside}: A list where each element corresponds to a detected pattern (e.g., function name) and contains the variables found inside that pattern. For example, if the formula includes \code{asym(x1 + x2)}, the \code{inside} list will have an element named "asym" containing the variables "x1" and "x2". This allows for easy identification of variables that are part of specific constructs in the formula.
#' }
#' @export
#'
#' @seealso \code{\link[stats]{formula}} and \code{\link[base]{gregexpr}}
#'
#' @srrstats {G2.4c} Variable names and parsed formula terms are handled as character vectors when constructing variable lists from the formula.
#' @srrstats {G2.6} One-dimensional variables selected through the formula are extracted and aligned through this routine before lagged terms are generated.
#' @srrstats {G2.3b} Formula terms and asymmetry constructors are processed consistently so that documented model syntax is interpreted as intended.
#'
#' @examples
#'
#' # Parse formulas containing various collection types like ()
#' formula_ <- y ~ x +det(s -gg- d) + asymS(d2 -rr+ s)-mm(y1+y2+y3)+asym(k1+k2+k3)+trend-huseyin
#' # Extract variables
#' parse_formula_vars(formula_)
#'
#'

parse_formula_vars <- function(formula) {
  if (is.character(formula)) {
    formula <- as.formula(formula)
  }

  response <- all.vars(formula[[2]])
  rhs <- formula[[3]]

  res0 <- list(
    response  = response,
    intercept = TRUE,
    dot       = FALSE,
    outside   = character(0),
    inside    = list()
  )

  parse_rhs <- function(expr, current_fun = NULL, res) {

    # Symbol
    if (is.symbol(expr)) {
      v <- as.character(expr)

      if (v == ".") {
        res$dot <- TRUE
        return(res)
      }

      if (is.null(current_fun)) {
        res$outside <- unique(c(res$outside, v))
      } else {
        res$inside[[current_fun]] <-
          unique(c(res$inside[[current_fun]], v))
      }
      return(res)
    }

    # Numeric constants (0 or 1)
    if (is.numeric(expr)) {
      if (expr == 0) res$intercept <- FALSE
      return(res)
    }

    # Call
    if (is.call(expr)) {
      fname <- as.character(expr[[1]])

      # +
      if (fname == "+") {
        for (i in 2:length(expr)) {
          res <- parse_rhs(expr[[i]], current_fun, res)
        }
        return(res)
      }

      # -
      if (fname == "-") {
        res <- parse_rhs(expr[[2]], current_fun, res)
        if (length(expr) == 3) {
          if (is.numeric(expr[[3]]) && expr[[3]] == 1) {
            res$intercept <- FALSE
          } else {
            res <- parse_rhs(expr[[3]], current_fun, res)
          }
        }
        return(res)
      }

      # Function calls (myu(), hjj(), etc.)
      if (!(fname %in% c("*", "/", "^", ":"))) {
        if (is.null(res$inside[[fname]])) {
          res$inside[[fname]] <- character(0)
        }
        for (i in 2:length(expr)) {
          res <- parse_rhs(expr[[i]], fname, res)
        }
        return(res)
      }
    }

    res
  }

  parse_rhs(rhs, NULL, res0)
}


#' Batch Control
#'
#' The \code{BatchControl()} function is designed to manage the execution of tasks in batches. It calculates the starting and ending row indices for a given batch based on the total number of tasks and the specified batch format. The function supports both single batch execution (where all tasks are processed at once) and multiple batch execution (where tasks are divided into specified batches).
#'
#' @param spec A list containing the necessary information for batch control, including:
#' \itemize{
#' \item \code{argsInfo$batch}: A string specifying the batch format.
#' \item \code{extractedInfo$lagRowsNumber}: The total number of tasks (rows) to be processed.
#' }
#' @return A list containing:
#' \itemize{
#' \item \code{startRow}: The starting row index for the current batch.
#' \item \code{endRow}: The ending row index for the current batch.
#' \item \code{batch_size}: The number of tasks in the current batch.
#' }
#' @noRd
#'


BatchControl<-function(spec){
  if ( spec$argsInfo$batch =="1/1") {
    startRow<-1
    endRow<-spec$extractedInfo$lagRowsNumber # Default: all tasks if batch is NULL
    batch_size <- spec$extractedInfo$lagRowsNumber
  } else
  {
    if (!grepl("^\\d+/\\d+$", spec$argsInfo$batch)) {
      stop("Invalid batch format. Use 'x/y', where x is the batch number and y is the total number of batches.",call. = FALSE)
    }
    # Extract batch number and total batches
    batch_parts <- as.numeric(strsplit(spec$argsInfo$batch, "/", fixed = TRUE)[[1]])
    current_batch <- batch_parts[1]
    total_batches <- batch_parts[2]

    # Validate batch numbers
    if (current_batch < 1 || current_batch > total_batches) {
      warning("Batch number must be between 1 and the total number of batches. Current batch changed to 1.",call.=F)
      current_batch<-1
    }

    # Calculate the range of tasks for the current batch
    batch_size <- ceiling(spec$extractedInfo$lagRowsNumber / total_batches)
    startRow <- (current_batch - 1) * batch_size + 1
    endRow <- min(current_batch * batch_size, spec$extractedInfo$lagRowsNumber)
    if(startRow>spec$extractedInfo$lagRowsNumber){
      startRow<-endRow<-0
    }
    #task_range <- seq(start_task, end_task)


  }
  list(startRow=startRow,
       endRow=endRow,batch_size=batch_size
  )

}



#' Display Progress Bar
#'
#' The \code{progressBar()} function provides a visual representation of the progress of a task in the console. It calculates the percentage of completion based on the current and total values and displays a progress bar accordingly. The function also allows for additional strings to be displayed alongside the progress bar for more context.
#' @param current The current progress value (e.g., number of tasks completed).
#' @param total The total value representing the completion point (e.g., total number of
#' tasks).
#' @param additionalStrings Optional additional strings to display alongside the progress bar for more context.
#' @param verbose A logical value indicating whether to display the progress bar (default is TRUE
#' to show progress).
#' @param use_message A logical value indicating whether to use the \code{message()}
#' function for output (CRAN-friendly but no overwrite) instead of \code{cat()} (which overwrites in the console). Default is FALSE, meaning \code{cat()} will be used for an overwriting progress bar.
#' @return Invisibly returns NULL. The primary purpose of this function is to display the
#' progress bar in the console, and it does not return any meaningful value.
#' @noRd
#'

progressBar<-function(current, total,additionalStrings="",
                      verbose = TRUE, use_message = FALSE){
  if (!interactive()) return(invisible()) # Skip in non-interactive environments

  persentage<-floor(current/total*100)
  totalLines<-50
  #displayArr<-c("\\","|","/","-","=")
  displayArr<-c(" ","_"," ")
  if(current==total){
    animate<-displayArr[3]
  }else{
    animate<-displayArr[(current %% 2)+1]
  }

  theFirstTimes<-floor(persentage*totalLines/100)
  theSecondTimes<-totalLines-theFirstTimes # floor((100-persentage)*totalLines/100)
  theFirstPart<-strrep("#", theFirstTimes)
  theSecondPart<-strrep(" ", theSecondTimes)
  txt <- sprintf("%% %s [ %s%s%s ] %s/%s %s",
                 persentage, theFirstPart, animate, theSecondPart,
                 current, total, additionalStrings)
  if (verbose) {
    if (use_message) {
      message(txt)   # CRAN-friendly but no overwrite
    } else {
      cat("\r", txt) # Overwrite in console
      flush.console()
    }
  }
 # cat("\r",paste0("% ",persentage, " [ ", theFirstPart,animate,theSecondPart, "] ",current,"/",total," ",additionalStrings))
}

#' Replace Lag Variable in String
#'
#' The \code{replace_lag_var()} function is designed to replace placeholders for variable names and lag values in a given string. It takes a string with placeholders, a variable name (or vector of variable names), and a new lag value, and returns the string with the placeholders replaced accordingly. This function is particularly useful for dynamically generating strings that involve lagged variables in time series analysis or similar contexts.
#' @param string The input string containing placeholders for variable names and lag values. The placeholders should be in the format \code{varName} for variable names and \code{lag} for lag values.
#' @param varName A character vector of variable names to replace the \code{var
#' Name} placeholder in the input string. If a single variable name is provided, it will replace the placeholder once. If a vector of variable names is provided, the function will return a vector of strings with each variable name replacing the placeholder in the input string.
#' @param new_lag A character or numeric value to replace the \code{lag
#' } placeholder in the input string. This value will be used to indicate the lag value in the resulting string(s).
#' @return A string or a vector of strings with the \code{varName}
#' and \code{lag} placeholders replaced by the provided variable names and lag value. If a single variable name is provided, the function returns a single string. If a vector of variable names is provided, the function returns a vector of strings, each with the corresponding variable name replaced in the input string.
#' @noRd

replace_lag_var <- function(string, varName, new_lag) {
  # Ensure varName is a character vector
  varName <- as.character(varName)

  # If varName is a single value, return a single string
  if (length(varName) == 1) {
    result <- gsub("{varName}", varName, gsub("{lag}", new_lag, string, fixed = TRUE), fixed = TRUE)
    return(result)
  }

  # If varName is a vector, return a vector of replaced strings
  result <- vapply(varName, function(var) {
    gsub("{varName}", var, gsub("{lag}", new_lag, string, fixed = TRUE), fixed = TRUE)
  }, character(1))

  return(result)
}

#' @srrstats {G5.2} Tests exercise successful calls and error conditions for
#' utility functions including `parse_formula_vars()`, `lmerge()`, and
#' `BatchControl()`.
#' @srrstats {G5.3} Tests verify expected types, values, and list structure
#' for utility function outputs.
#' @srrstats {G5.6} Results of `modelCriterion()` with a function argument
#' match the base-R AIC/BIC exactly, confirming correctness.
#' @srrstats {G2.6} `parse_formula_vars()` correctly extracts variables by
#' name and group from complex formula expressions.
test_that("model Criterion functions work correctly", {

  mylm<- lm(mpg ~ wt + hp, data = mtcars)

   expect_identical(modelCriterion(mylm,AIC), stats::AIC(mylm))
   expect_identical(modelCriterion(mylm,BIC), stats::BIC(mylm))

  #' Not strictly necessary to test formulas, but good to know `parse_formula_vars` runs
  formula <- y ~ x + z + asymmetric(z) + Lasymmetric(x1+x2) + Sasymmetric(x3+x4+x5) + deterministic(d1) + trend
  vars <- parse_formula_vars(formula)

  expect_type(vars, "list")
  expect_true("asymmetric" %in% names(vars$inside))
  expect_true("Sasymmetric" %in% names(vars$inside))
  expect_true("x4" %in% vars$inside$Sasymmetric)
  expect_false("x2" %in% vars$inside$Sasymmetric)
  expect_true(vars$intercept)
  expect_false(vars$dot)
})

#' @srrstats {G5.4a} Tests simple deterministic cases to verify formula
#' parsing behaviour with the dot operator and no-intercept syntax.
test_that("parse_formula_vars handles dot and intercept removal", {
  vars_dot <- parse_formula_vars(y ~ .)
  expect_true(vars_dot$dot)

  vars_noint <- parse_formula_vars(y ~ x - 1)
  expect_false(vars_noint$intercept)

  vars_noint2 <- parse_formula_vars(y ~ x + 0)
  expect_false(vars_noint2$intercept)
})

#' @srrstats {G5.4a} Confirms that a character string formula is coerced
#' and parsed correctly.
test_that("parse_formula_vars handles character input", {
  vars <- parse_formula_vars("y ~ x + z")
  expect_type(vars, "list")
  expect_true("x" %in% vars$outside)
})

#' @srrstats {G5.8c} Tests degenerate/unusual formula syntax (variable
#' subtraction) to ensure robust parsing.
test_that("parse_formula_vars handles subtraction of a variable", {
  vars <- parse_formula_vars(y ~ x + z - w)
  expect_true("w" %in% vars$outside)
})


#' @srrstats {G5.4a} Verifies correct priority logic: keys in the first list
#' win over the same keys in the second list.
test_that("lmerge prioritises first list", {
  a <- list(x = 1, y = 2)
  b <- list(x = 99, z = 3)
  result <- lmerge(a, b)
  expect_identical(result$x, 1)   #' a wins
  expect_identical(result$z, 3)   #' only in b
  expect_identical(result$y, 2)   #' only in a
})

#' @srrstats {G5.4a} Tests three-way merge with priority to the leftmost list.
test_that("lmerge works with more than two lists", {
  a <- list(x = 1)
  b <- list(y = 2)
  cc <- list(z = 3, x = 100)
  result <- lmerge(a, b, cc)
  expect_identical(result$x, 1)   #' a wins over cc
  expect_identical(result$z, 3)
})

#' @srrstats {G5.4a} Verifies lmerge behaviour with named atomic vectors
#' in addition to lists.
test_that("lmerge handles named vectors", {
  a <- c(k = 1)
  b <- c(k = 99, m = 5)
  result <- lmerge(a, b)
  expect_identical(result[["k"]], 1)
  expect_identical(result[["m"]], 5)
})


#' @srrstats {G5.4a} Tests the single-batch case where all lag combinations
#' are processed in one run.
test_that("BatchControl returns full range for '1/1'", {
  kardl_reset()
  spec <- kardl(imf_example_data, CPI ~ ER + PPI,
                mode = "grid_custom", maxlag = 2)
  #' Reconstruct a minimal spec-like list for BatchControl
  fake_spec <- list(
    argsInfo = list(batch = "1/1"),
    extractedInfo = list(lagRowsNumber = 100)
  )
  res <- BatchControl(fake_spec)
  expect_identical(res$startRow, 1)
  expect_identical(res$endRow, 100)
})

#' @srrstats {G5.4a} Tests that the second of four equal batches returns the
#' correct start and end row indices.
test_that("BatchControl splits correctly for '2/4'", {
  fake_spec <- list(
    argsInfo = list(batch = "2/4"),
    extractedInfo = list(lagRowsNumber = 100)
  )
  res <- BatchControl(fake_spec)
  expect_identical(res$startRow, 26)
  expect_identical(res$endRow, 50)
})

#' @srrstats {G5.2a} Checks that an out-of-range batch number triggers a
#' warning with an informative message.
test_that("BatchControl warns and defaults to batch 1 when batch number out of range", {
  fake_spec <- list(
    argsInfo = list(batch = "10/4"),
    extractedInfo = list(lagRowsNumber = 100)
  )
  expect_warning(BatchControl(fake_spec), "between 1 and")
})

#' @srrstats {G5.2a} Checks that a malformed batch string produces an
#' informative error before any processing occurs.
#' @srrstats {G5.8a} Confirms that invalid inputs are rejected early.
test_that("BatchControl errors on malformed batch string", {
  fake_spec <- list(
    argsInfo = list(batch = "bad_format"),
    extractedInfo = list(lagRowsNumber = 100)
  )
  expect_error(BatchControl(fake_spec), "Invalid batch format")
})

#' Estimate ARDL and NARDL Models with Automatic Lag Selection
#'
#' This function estimates an Autoregressive Distributed Lag (ARDL) or Nonlinear ARDL (NARDL) model based on the provided data and model formula.
#' It allows for flexible specification of variables, including deterministic terms, asymmetric variables, and trend components.
#' The function also supports automatic lag selection using various information criteria.
#'
#' @details
#' The general formula for the long-run model is specified as follows:
#' \deqn{
#' \begin{aligned}
#'  \Delta {y}_t =  c  +  \eta _0   {y}_{t-1} + \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }   +\sum_{i=1}^{m} {( \eta ^{+}_i   {x}^{+}_{i,t-1 } + \eta ^{-}_i   {x}^{-}_{i,t-1 } ) }    +  \sum_{i=m+1}^{k} {\eta _i   {x}_{i,t-1 } }
#' +  \newline  \sum_{i=1}^{m} {\sum_{j=0}^{q^+_i} { \beta^+_{ij}   \Delta {x}^+_{i,t-j} } } + \sum_{i=1}^{m} {\sum_{j=0}^{q^-_i} { \beta^-_{ij}   \Delta {x}^-_{i,t-j} } }  + \sum_{i=m+1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
#' \end{aligned}
#'  }
#'
#' Where:
#' \itemize{
#' \item \eqn{y_t} is the dependent variable at time t.
#' \item \eqn{c} is the constant term.
#' \item \eqn{\eta_0} is the coefficient of the lagged dependent variable.
#' \item \eqn{\gamma_j} are the coefficients of the lagged differences of the dependent variable.
#' \item \eqn{\eta^+_i} and \eqn{\eta^-_i} are the coefficients of the positive and negative decompositions of the independent variables, respectively.
#' \item \eqn{\eta_i} are the coefficients of the independent variables that do not have asymmetric decompositions.
#' \item \eqn{\beta^+_{ij}} and \eqn{\beta^-_{ij}} are the coefficients of the lagged differences of the positive and negative decompositions of the independent variables, respectively.
#' \item \eqn{\beta_{ij}} are the coefficients of the lagged differences of the independent variables that do not have asymmetric decompositions.
#' \item \eqn{e_t} is the error term at time t.
#' \item \eqn{p} is the maximum lag length for the dependent variable.
#' \item \eqn{q^+_i} and \eqn{q^-_i} are the maximum lag lengths for the positive and negative decompositions of the independent variables, respectively.
#' \item \eqn{q_i} is the maximum lag length for the independent variables that do not have asymmetric decompositions.
#' \item \eqn{m} is the number of independent variables with asymmetric decompositions.
#' \item \eqn{k} is the total number of independent variables.
#' \item \eqn{\Delta} denotes the first difference operator.
#' \item \eqn{x^+_{i,t}} and \eqn{x^-_{i,t}} represent the positive and negative decompositions of the independent variable \eqn{x_i} at time t, respectively.
#' }
#'
#' @section Notation of reported coefficients:
#'
#' In the reported coefficients, the prefix \code{L} denotes lagged variables,
#' where the accompanying number indicates the lag order, and \code{.d.} denotes
#' first differences. Accordingly, \code{L1.PPI} represents the first lag of the
#' level of \code{PPI} (long-run component), while \code{L3.d.PPI} denotes the
#' third lag of the first-differenced \code{PPI} (short-run component).
#'
#' In addition, the suffixes \code{_POS} and \code{_NEG} indicate the positive
#' and negative partial sum components of a variable, respectively. This notation
#' is used by default and remains valid unless modified through the
#' \code{kardl_set()} function.
#'
#'
#' @param data The data of analysis
#' @param formula A formula specifying the long-run model equation. This formula defines the relationships
#'        between the dependent variable and explanatory variables, including options for deterministic terms,
#'        asymmetric variables, and a trend component.
#'
#'        Example formula:
#'        \code{y ~ x + z + Asymmetric(z) + Lasymmetric(x2 + x3) + Sasymmetric(x3 + x4) + deterministic(dummy1 + dummy2) + trend}
#'
#'
#'
#' \strong{\emph{Details}}
#'
#' The formula allows flexible specification of variables and their roles:
#'
#' \itemize{
#' \item \strong{Deterministic variables:}
#' Deterministic regressors (e.g., dummy variables) can be included using
#' \code{deterministic()}. Multiple deterministic variables may be supplied
#' using \code{+}, for example \code{deterministic(dummy1 + dummy2)}.
#' These variables are treated as fixed components and are not associated
#' with short-run or long-run dynamics.
#' \item \strong{Asymmetric variables:}
#' Asymmetric decompositions can be specified for short-run and/or
#' long-run dynamics:
#' \itemize{
#' \item \strong{Sasymmetric}: Specifies short-run asymmetric variables.
#' For example, \code{Sasymmetric(x1 + x2)} applies short-run
#' asymmetric decomposition to \code{x1} and \code{x2}.
#' \item \strong{Lasymmetric}: Specifies long-run asymmetric variables.
#' For example, \code{Lasymmetric(x1 + x3)} applies long-run
#' asymmetric decomposition to \code{x1} and \code{x3}.
#' \item \strong{Asymmetric}: Specifies variables that enter both
#' short-run and long-run asymmetric decompositions.
#' For example, \code{Asymmetric(x1 + x3)} applies asymmetric
#' decomposition in both dynamics.
#' }
#' }
#'
#' A \strong{trend} term may be included to capture deterministic linear
#' time trends by simply adding \code{trend} to the formula.
#'
#' The formula also supports the use of \code{.} to represent all available
#' regressors in the supplied data (excluding the dependent variable),
#' following standard R formula conventions.
#'
#' All of the operators \code{Deterministic()}, \code{Sasymmetric()},
#' \code{Lasymmetric()}, and \code{Asymmetric()} follow the same usage
#' rules:
#'
#' \itemize{
#' \item They can be freely combined within a single formula, for example:
#'   \preformatted{
#'   y ~ . +
#'     Asymmetric(z) +
#'     Lasymmetric(x2 + x3) +
#'     Sasymmetric(x3 + x4) +
#'     deterministic(dummy1 + dummy2) +
#'     trend
#'   }
#'
#' \item They must not be nested within one another. Valid usage:
#'   \code{y ~ x + deterministic(dummy) + Asymmetric(z)}.
#'   Invalid usage (to be avoided):
#'   \code{y ~ x + deterministic(Asymmetric(z))} or
#'   \code{y ~ x + Asymmetric(deterministic(dummy))}.
#'
#' \item Where applicable, arguments are validated internally using
#'   \code{match.arg()}. Consequently, abbreviated inputs are accepted
#'   provided they uniquely identify a valid option. For example, if
#'   \code{"asymmetric"} is an admissible value, specifying \code{"a"}
#'   is sufficient. For clarity and reproducibility, however, full
#'   argument names are recommended.
#' }
#'
#' These components may therefore be combined flexibly to construct
#' a specification tailored to the empirical analysis.
#'
#'
#' @param maxlag An integer specifying the maximum number of lags to be considered for the model.
#'        The default value is \code{4}. This parameter sets an upper limit on the lag length during
#'        the model estimation process.
#'
#'
#' \strong{\emph{details}}
#'
#'
#' The \code{maxlag} parameter is crucial for defining the maximum lag length that the model
#' will evaluate when selecting the optimal lag structure based on the specified \code{criterion}.
#' It controls the computational effort and helps prevent overfitting by restricting the search
#' space for lag selection.
#' \itemize{
#' \item  If the data has a short time horizon or is prone to overfitting, consider reducing \code{maxlag}.
#' \item  If the data is expected to have long-term dependencies, increasing \code{maxlag} may be necessary to capture the relevant dynamics.
#' }
#'
#' Setting an appropriate value for \code{maxlag} depends on the nature of your dataset and the
#' context of the analysis:
#' \itemize{
#' \item For small datasets or quick tests, use smaller values (e.g., \code{maxlag = 2}).
#' \item For datasets with more observations or longer-term patterns, larger values (e.g., \code{maxlag = 8})  may be appropriate, though this increases computational time.
#' }
#'
#'
#' \strong{\emph{examples}}
#'
#' Using the default maximum lag (4)
#'
#' \code{kardl(data, MyFormula, maxlag = 4)}
#'
#' Reducing the maximum lag to 2 for faster computation
#'
#' \code{kardl(data, MyFormula, maxlag = 2)}
#'
#' Increasing the maximum lag to 8 for datasets with longer dependencies
#'
#' \code{kardl(data, MyFormula, maxlag = 8)}
#'
#' @param mode Specifies the mode of estimation and output control. This parameter determines how
#'        the function handles lag estimation and what kind of feedback or control is provided during
#'        the process. The available options are:
#'
#' \itemize{
#' \item \strong{"quick"} (default):
#'         Displays progress and messages in the console while the function estimates the optimal lag values.
#'         This mode is suitable for interactive use or for users who want to monitor the estimation process
#'         in real-time. It provides detailed feedback for debugging or observation but may use additional
#'         resources due to verbose output.
#'
#' \item \strong{"grid"} :
#'         Displays progress and messages in the console while the function estimates the optimal lag values.
#'         This mode is suitable for interactive use or for users who want to monitor the estimation process
#'         in real-time. It provides detailed feedback for debugging or observation but may use additional
#'         resources due to verbose output.
#'
#' \item \strong{"grid_custom"}:
#'         Suppresses most or all console output, prioritizing faster execution and reduced resource usage
#'         on PCs or servers. This mode is recommended for high-performance scenarios, batch processing,
#'         or when the estimation process does not require user monitoring. Suitable for large-scale or
#'         repeated runs where output is unnecessary.
#'
#' \item \strong{User-defined vector}:
#' A numeric vector of lag values specified by the user, allowing full customization of the lag
#' structure used in model estimation. When a user-defined vector is provided (e.g., `c(1, 2, 4, 5)`),
#' the function skips the lag optimization process and directly uses the specified lags.
#'
#' Users can define lag values directly as a numeric vector. For example:
#' \code{mode = c(1, 2, 4, 5)} assigns lags of 1, 2, 4, and 5 to variables in the specified order.
#' Alternatively, lag values can be assigned to variables by name for clarity and control. For example:
#' \code{mode = c(CPI = 2, ER_POS = 3, ER_NEG = 1, PPI = 3)} assigns lags to variables explicitly.
#' Ensure that the lags are correctly designated by verifying the result using
#' \code{kardl_model$properLag} after estimation.
#'
#' \strong{\emph{Attention!}}
#' A function-based criterion or user-defined function can be specified
#' for model selection, but this is only supported for \code{mode = "grid_custom"}
#' and \code{mode = "quick"}. The \code{mode = "grid"} option is restricted to
#' predefined criteria (e.g., AIC or BIC). For more information on available criteria,
#' see the \code{\link{modelCriterion}} function documentation.
#' \itemize{
#' \item When using a numeric vector, ensure the order of lag values matches the variables in your formula.
#' \item If using named vectors, double-check the variable names to avoid mismatches or unintended results.
#' \item This mode bypasses the automatic lag optimization and assumes the user-defined lags are correct.
#' }
#' }
#'
#'
#' @param criterion A string specifying the information criterion to be used for selecting the optimal lag structure.
#'       The available options are:
#' \itemize{
#' \item \strong{"AIC"}: Akaike Information Criterion (default). This criterion balances model fit and complexity,
#'       favoring models that explain the data well with fewer parameters.
#' \item \strong{"BIC"}: Bayesian Information Criterion. This criterion imposes a stronger penalty for model complexity
#'       than AIC, making it more conservative in selecting models with fewer parameters.
#' \item \strong{"AICc"}: Corrected Akaike Information Criterion. This is an adjusted version of AIC that accounts for small sample sizes,
#'       making it more suitable when the number of observations is limited relative to the number of parameters.
#' \item \strong{"HQ"}: Hannan-Quinn Information Criterion. This criterion provides a compromise between AIC and BIC,
#'       favoring models that balance fit and complexity without being overly conservative.
#' }
#'       The criterion can be specified as a string (e.g., \code{"AIC"}) or as a user-defined function that takes a fitted model object.
#'       Please visit the \code{\link{modelCriterion}} function documentation for more details on using custom criteria.
#'
#' @param differentAsymLag A logical value indicating whether to allow different lag lengths for positive and negative decompositions.
#' @param batch A string specifying the batch processing configuration in the format "current_batch/total_batches".
#' If a user utilize grid or grid_custom mode and want to split the lag search into multiple batches, this parameter can be used to define the current batch and the total number of batches.
#'       For example, "2/5" indicates that the current batch is the second out of a total of five batches.
#'       The default value is "1/1", meaning that the entire lag search is performed in a single batch.
#'
#' @param ... Additional arguments that can be passed to the function. These arguments can be used to specify other settings or parameters that are not explicitly defined in the main arguments.
#'
#'@return An object of class \code{kardl_lm} containing the estimated ARDL or NARDL model.
#' The object includes the following components:
#' \describe{
#' \item{argsInfo}{A list of input arguments used for the estimation. It includes the data, formula, maxlag, mode, criterion, differentAsymLag, and batch settings.}
#' \item{extractedInfo}{A list containing extracted information from the input data and formula, such as variable names, deterministic terms, asymmetric variables, and the prepared dataset for estimation.}
#' \item{timeInfo}{A list containing timing information for the estimation process, including start time, end time, and total duration.}
#' \item{lagInfo}{A list containing lag selection information, including the optimal lag configuration and criteria values for different lag combinations.}
#' \item{estInfo}{A list containing estimation details, such as the type of model, estimation method, model formula, number of parameters (k), number of observations (n), start and end points of the fitted values, and total time span.}
#' \item{model}{The fitted linear model object of class \code{lm} representing the estimated ARDL or NARDL model.}
#' }
#'
#'
#' @seealso  \code{\link{ecm}}, \code{\link{kardl_set}}, \code{\link{kardl_get}}, \code{\link{kardl_reset}}, \code{\link{modelCriterion}}
#'
#' @srrstats {G1.1} The package documentation describes `kardl` as an R implementation and extension of ARDL and NARDL workflows, with emphasis on mixed symmetric and asymmetric regressors, flexible lag selection, and dynamic multiplier methods.
#' @srrstats {G2.0} Main input arguments including `formula`, `data`, `maxlag`, `mode`, `criterion`, and `differentAsymLag` are validated before estimation proceeds.
#' @srrstats {G2.0a} The `formula` argument expects a single language object specifying the long-run ARDL equation; `maxlag` must be a positive integer; `mode` accepts predefined strings or a numeric lag vector.
#' @srrstats {G2.1} The function checks that `formula` is a language object and that `data` contains all variables referenced in the formula before estimation.
#' @srrstats {G2.3} Character-valued arguments such as `mode` and `criterion` are matched against predefined acceptable values via `match.arg()` internally.
#' @srrstats {G2.3a} The `criterion` argument is matched against `c("AIC", "BIC", "AICc", "HQ")`; the `mode` argument against `c("quick", "grid", "grid_custom")`.
#' @srrstats {G2.4} Formula parsing and data alignment are performed before estimation so all subsequent routines operate on a consistent internal representation.
#' @srrstats {G2.4a} The `maxlag` argument is treated as an integer count before constructing lagged regressors.
#' @srrstats {G2.13} Missing observations introduced by lagging and differencing are handled during model-frame construction before estimation.
#' @srrstats {G2.14} Observations unavailable after lag construction are excluded from the estimation sample.
#' @srrstats {TS1.0} The function accepts time-ordered data and constructs lagged and differenced variables internally for ARDL and NARDL estimation.
#' @srrstats {TS1.3} A central preparation workflow parses the formula, constructs lagged variables, and returns a uniform internal model data structure.
#' @srrstats {TS4.0} The fitted model is returned as an object of class `kardl_lm` with dedicated `print`, `summary`, and accessor methods.
#' @srrstats {TS4.0b} The return value uses the explicit class `kardl_lm`.


#' @export
#'
#' @import lmtest stats
#' @importFrom utils  methods  capture.output tail flush.console
#'
#' @examples
#'
#'
#' # Sample article: THE DYNAMICS OF EXCHANGE RATE PASS-THROUGH TO DOMESTIC PRICES IN TURKEY
#'
#' kardl_set(formula =CPI~ER+PPI+Asymmetr(ER)+deterministic(covid)+trend ,
#'           data=imf_example_data,
#'           maxlag=2
#' ) # setting the default values of the kardl function
#'
#'
#' # using the grid_custom mode with batch processing
#'
#' kardl_model_grid<-kardl( mode = "grid_custom",batch = "2/3",criterion = "BIC")
#' kardl_model_grid
#'
#' kardl_model2<-kardl(mode = c( 2    ,  1    ,  1   ,   3 ))
#'
#' # Getting the results
#' kardl_model2
#'
#' # Getting the summary of the results
#' summary(kardl_model2)
#'
#' # using '.' in the formula means that all variables in the data will be used
#'
#' fit_bic <- kardl(formula=CPI~.+deterministic(covid))
#' fit_bic
#'
#' # Setting max lag instead of default value [4]
#' kardl(imf_example_data,
#'       CPI~ER+PPI+Lasymmetric(ER),
#'       maxlag = 3, mode = "grid_custom")
#'
#' # Using another criterion for finding the best lag
#' kardl_set(criterion = "HQ") # setting the criterion to HQ
#' kardl( mode = "grid_custom")
#'
#' # using default values of lags
#' kardl( mode=c(1,2,3,0))
#'
#' # For using different lag values for negative and positive decompositions of non-linear variables
#' # setting the same lags for positive and negative decompositions.
#'
#' same<-kardl(formula=CPI~Asymmetric(ER),maxlag=2, mode = "grid_custom",differentAsymLag = FALSE)
#' dif<-kardl(formula=CPI~Sasymmetric(ER),maxlag=2, mode = "grid_custom",differentAsymLag = TRUE)
#'
#' same$lagInfo$OptLag
#' dif$lagInfo$OptLag
#'
#' # Optional: use magrittr if available
#' @examplesIf requireNamespace("magrittr", quietly = TRUE)
#' library(magrittr)
#'   kardl_model_pipe <-  imf_example_data %>%
#'     kardl(mode = "grid_custom")
#'
#'   kardl_model_pipe
#'


kardl <- function(data = NULL, formula = NULL,
                  maxlag  = NULL,
                  mode    = NULL,
                  criterion = NULL,
                  differentAsymLag = NULL,
                  batch = NULL,
                  ...
){
  argsInfo <- list(data=data, formula=formula, maxlag=maxlag, mode=mode,
                   criterion=criterion,differentAsymLag=differentAsymLag,batch=batch)
  otherargsInfo <- list(...)
  kardlVars <- lmerge(argsInfo,otherargsInfo)
  myNames <- names(kardlVars)
  for (i in seq_along(kardlVars)) {
    name <- myNames[i]
    if(!nzchar(name)) {
      kardlVars[[i]] <- NULL
      next
    }
    if (is.null(kardlVars[[name]])) {
      kardlVars[[name]] <- kardl_get(name)
      if (is.null(kardlVars[[name]])) {
        stop("No ", name, " provided. Please supply `", name, "` or set it with kardl_set(", name, " = ...).",
             call. = FALSE)
      }
      attr(kardlVars[[name]], "source") <- "kardl_set"
    }else{
      attr(kardlVars[[name]], "source") <- "argument"
    }
  }
  spec<-prepare(kardlVars)
  # inputs<<-kardlVars
  makemodel(spec,...)
}


#' Estimate a Restricted ECM Model
#'
#' The `ecm` function estimates a restricted Error Correction Model (ECM) based on the provided data and model specification. This function is designed to test for cointegration using the PSS t Bound test, which assesses the presence of a long-term equilibrium relationship between the dependent variable and the independent variables in the model.
#'
#' @inheritParams kardl
#' @inheritSection kardl Notation of reported coefficients
#' @section Hypothesis testing:
#'
#'
#' The null and alternative hypotheses for the restricted ECM test are as follows:
#'
#'   \deqn{\mathbf{H_{0}:} \theta  =   0}
#'   \deqn{\mathbf{H_{1}:} \theta  \neq 0}
#'
#'   The null hypothesis (\eqn{H_{0}}) states that there is no cointegration in the model, meaning that the long-run relationship between the variables is not significant. The alternative hypothesis (\eqn{H_{1}}) suggests that there is cointegration, indicating a significant long-term relationship between the variables.
#'
#'   The test statistic is calculated as the t-statistic of the coefficient of the error correction term (\eqn{\theta}) in the ECM model. If the absolute value of the t-statistic exceeds the critical value from the PSS t Bound table, we reject the null hypothesis in favor of the alternative hypothesis, indicating that cointegration is present.
#'
#'   The cases for the restricted ECM Bound test are defined as follows:
#'
#'
#'
#' \itemize{
#' \item \code{case 1}: No constant, no trend.
#'
#'   This case is used when the model does not include a constant term or a trend term. It is suitable for models where the variables are stationary and do not exhibit any long-term trends.
#'
#'   The model is specified as follows:
#'
#'   \deqn{
#' \begin{aligned}
#' \Delta y_t =  \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1}  - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
#' \end{aligned}
#' }
#'
#' \item \code{case 2}: Restricted constant, no trend.
#'
#'   This case is used when the model includes a constant term but no trend term. It is suitable for models where the variables exhibit a long-term relationship but do not have a trend component.
#'   The model is specified as follows:
#'   \deqn{
#'   \begin{aligned}
#'   \Delta y_t &= \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1} - \alpha_0 - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
#'   \end{aligned}
#'   }
#'
#' \item \code{case 3}: Unrestricted constant, no trend.
#'
#'   This case is used when the model includes an unrestricted constant term but no trend term. It is suitable for models where the variables exhibit a long-term relationship with a constant but do not have a trend component.
#'
#'   The model is specified as follows:
#'
#'   \deqn{
#'   \begin{aligned}
#'   \Delta y_t &= \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1} - \alpha_0 - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
#'   \end{aligned}
#'     }
#' \item \code{case 4}: Unrestricted Constant, restricted trend.
#'
#'   This case is used when the model includes an unrestricted constant term and a restricted trend term. It is suitable for models where the variables exhibit a long-term relationship with a constant and a trend component.
#'
#'   The model is specified as follows:
#'
#'   \deqn{
#'   \begin{aligned}
#'   \Delta y_t &= \phi + \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1} - \pi (t-1) - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
#'   \end{aligned}
#'    }
#' \item \code{case 5}: Unrestricted constant, unrestricted trend.
#' }
#'

#'
#'  The Error Correction Model (ECM) is specified as follows:
#' \deqn{
#' \begin{aligned}
#' \Delta y_t &= \phi + \varphi t +  \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1}  - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
#' \end{aligned}
#' }
#'

#'
#' @return A list containing the results of the restricted ECM test, including:
#' \itemize{
#' \item \code{ecm}: The estimated ECM model objects including:
#' \itemize{
#' \item \code{longrunEQ}: The estimated long-run model equation object.
#' \item \code{shortrunEQ}: The estimated short-run model equation
#' \item \code{ecmL}: The estimated long-run model object.
#' }
#' \item \strong{argsInfo}: A list of input arguments used for the estimation. It includes the data, formula, maxlag, mode, criterion, differentAsymLag, and batch settings.
#' \item \strong{extractedInfo}: A list containing extracted information from the input data and formula, such as variable names, deterministic terms, asymmetric variables, and the prepared dataset for estimation.
#' \item \strong{timeInfo}: A list containing timing information for the estimation process, including start time, end time, and total duration.
#' \item \strong{lagInfo}: A list containing lag selection information, including the optimal lag configuration and criteria values for different lag combinations.
#' \item \strong{estInfo}: A list containing estimation details, such as the type of model, estimation method, model formula, number of parameters (k), number of observations (n), start and end points of the fitted values, and total time span.
#' \item \strong{model}: The fitted linear model object of class \code{lm} representing the estimated ARDL or NARDL model.
#' }
#'
#' @srrstats {G2.0} Input arguments `formula`, `data`, `maxlag`, `mode`, `criterion`, `differentAsymLag`, and `batch` are validated before the ECM is estimated.
#' @srrstats {G2.1} The function checks that `formula` is a valid language object and that `data` contains all model variables before proceeding.
#' @srrstats {TS2.0} ECM estimation assumes a regular ordered sequence after preprocessing; structurally missing observations created by lagging are handled explicitly.
#' @srrstats {TS2.1} Missing-data handling is part of the model-preparation step, not a separate imputation system.
#' @srrstats {TS4.0} The fitted ECM is returned as an object of class `kardl_lm` with dedicated `print` and `summary` methods.
#'
#'

#' @export
#' @seealso \code{\link{kardl}} \code{\link{pssf}}  \code{\link{psst}}   \code{\link{ecm}}  \code{\link{narayan}}
#'
#'
#' @examples
#'
#' # Sample article: THE DYNAMICS OF EXCHANGE RATE PASS-THROUGH TO DOMESTIC PRICES IN TURKEY
#' kardl_set(
#'   formula = CPI ~ ER + PPI + asym(ER) + deterministic(covid) + trend,
#'   data = imf_example_data,
#'   maxlag = 3
#' )
#'
#' # Using the grid mode with batch processing to decrease execution time
#' ecm_model_grid <- ecm(mode = "grid")
#' ecm_model_grid
#'
#' # Checking the cointegration test results using Pesaran t test
#' psst(ecm_model_grid)
#'
#' # Getting the details of psst result
#' summary(psst(ecm_model_grid))
#'
#' # Using the grid_custom mode for faster execution without console output
#' ecm_model <- ecm(imf_example_data, mode = "grid_custom", criterion = "HQ", batch = "2/3")
#' ecm_model
#'
#' # Estimating the model with user-defined lag values
#' ecm_model2 <- ecm(mode = c(2, 1, 1, 3))
#'
#' # Getting the results
#' ecm_model2
#'
#' # Getting the summary of the results
#' summary(ecm_model2)
#'
#' # Alternative specification
#' summary(ecm(imf_example_data, CPI ~ PPI + asym(ER) + trend, case = 4))
#'
#' # For increasing the performance of finding the most fitted lag vector
#' ecm(mode = "grid_custom")
#'
#' # Setting max lag instead of default value [4]
#' ecm(maxlag = 2, mode = "grid_custom")
#'
#' # Using another criterion for finding the best lag
#' ecm(criterion = "HQ", mode = "grid_custom")
#'
#' # For using different lag values for positive and negative decompositions
#' # Setting the same lags for positive and negative decompositions
#' kardl_set(differentAsymLag = FALSE)
#'
#' diffAsymLags <- ecm(mode = "grid_custom")
#' diffAsymLags$lagInfo$OptLag
#'
#' # Setting different lags for positive and negative decompositions
#' sameAsymLags <- ecm(differentAsymLag = TRUE, mode = "grid_custom")
#' sameAsymLags$lagInfo$OptLag
#'
#' # Setting the prefixes and suffixes for nonlinear variables
#' kardl_reset()
#' kardl_set(AsymPrefix = c("asyP_", "asyN_"), AsymSuffix = c("_PP", "_NN"))
#' customizedNames <- ecm(imf_example_data, CPI ~ ER + PPI + asym(ER))
#' customizedNames
#'

ecm<-function(data = NULL, formula = NULL,
              maxlag  = NULL,
              mode    = NULL,
              criterion = NULL,
              differentAsymLag = NULL,
              batch = NULL,
              ...  ){

  notesArray<-c()
  get_proper <-kardl(data, formula =formula,  maxlag = maxlag, mode =mode,criterion = criterion,
               differentAsymLag = differentAsymLag,   batch = batch,list(...))

EcmData<-get_proper$model

longrunEQ<- paste0(replace_lag_var(.kardl_Settings_env$LongCoef ,get_proper$extractedInfo$dependentVar,1)  ," ~ " , paste( replace_lag_var(.kardl_Settings_env$LongCoef ,get_proper$extractedInfo$longRunVars[-1],1) ,collapse = " + "))
longrunEQ<- as.formula(longrunEQ)

  ecmL<-lm(longrunEQ,EcmData)
  ecmL$call <- longrunEQ

EcmData$EcmRes<-residuals(ecmL)

f <- get_proper$call
to_remove <- all.vars(longrunEQ)
tt <- terms(f)
rhs_terms <- attr(tt, "term.labels")
new_rhs <- setdiff(rhs_terms, to_remove)
shortrunEQ <- reformulate(new_rhs, response = as.character(f[[2]]))
if (attr(tt, "intercept") == 0) {
  shortrunEQ <- update(shortrunEQ, . ~ . - 1)
}

shortrunEQ<-update(shortrunEQ, . ~ EcmRes+ . )
ecmS <-lm( shortrunEQ, EcmData)



  ecmList<-list(ecm=list(
    longrunEQ=longrunEQ,
    shortrunEQ=shortrunEQ,
    ecmL=ecmL
  ),
  argsInfo=get_proper$argsInfo,
  extractedInfo=get_proper$extractedInfo,
  timeInfo=get_proper$timeInfo,
  estInfo=get_proper$estInfo,
  lagInfo=get_proper$lagInfo
  )

  ecmList$estInfo$type<-"ecm"
  ecmOutput<-lmerge(ecmList,ecmS)
  if(coef(ecmS)[["EcmRes"]]>=0){
    notesArray <- c(notesArray, "The coefficient of the error correction term (EcmRes) is non-negative. This may indicate a lack of long-run equilibrium adjustment.")
  }
  if(coef(ecmS)[["EcmRes"]] < -1){
    notesArray <- c(notesArray, "The coefficient of the error correction term (EcmRes) is less than -1. This may suggest over-adjustment or instability in the long-run relationship.")
  }

  ecmOutput$notes<-notesArray
  class(ecmOutput) <- c("kardl_lm","lm")
  return(ecmOutput)

}


#' Model Selection Criteria
#'
#' Computes a model selection criterion (AIC, BIC, AICc, or HQ) or applies a user-defined function
#' to evaluate a statistical model.
#'
#'
#' @param estModel An object containing the fitted model. The object should include at least:
#' \itemize{
#' \item \code{estModel$model} - the actual fitted model object (e.g., from \code{lm}, \code{glm}).
#' \item \code{k} - the number of estimated parameters.
#' \item \code{n} - the sample size.
#' }
#' @param cr A character string specifying the criterion to compute.
#'           Options are \code{"AIC"}, \code{"BIC"}, \code{"AICc"}, and \code{"HQ"}. Alternatively,
#'           a user-defined function can be provided. See details below for more information on using custom criteria.
#' @param ... Additional arguments passed to the user-defined criterion function if \code{cr} is a function.
#'
#' @return A numeric value representing the selected criterion, normalized by the sample size if one of the predefined options is used.
#'
#' @details
#' This function returns model selection criteria used to compare the quality of different models.
#' All criteria are defined such that \strong{lower values indicate better models} (i.e., the goal is minimization).
#'
#' If you wish to compare models using a maximization approach (e.g., log-likelihood),
#' you can multiply the result by \code{-1}.
#'
#' Note: The predefined string options (e.g., \code{"AIC"}) are \strong{not} the same as the built-in R functions \code{AIC()} or \code{BIC()}.
#' In particular, the values returned by this function are adjusted by dividing by the sample size \code{n}
#' (i.e., normalized AIC/BIC), which makes it more comparable across datasets of different sizes.
#'
#' The function returns:
#' \itemize{
#' \item \strong{"AIC"}:  \eqn{ \frac{2k - 2\ell}{n} } Akaike Information Criterion divided by \code{n}.
#' \item \strong{"BIC"}:  \eqn{ \frac{\log(n) \cdot k - 2\ell}{n} } Bayesian Information Criterion divided by \code{n}.
#' \item \strong{"AICc"}: \eqn{ \frac{2k(k+1)}{n - k - 1} + \frac{2k - 2\ell}{n} } Corrected Akaike Information Criterion divided by \code{n}.
#' \item \strong{"HQ"}: \eqn{ \frac{2 \log(\log(n)) \cdot k - 2\ell}{n} } Hannan-Quinn Criterion divided by \code{n}.
#' }
#'
#' where:
#' \itemize{
#' \item \eqn{k} is the number of parameters,
#' \item \eqn{n} is the sample size,
#' \item \eqn{\ell} is the log-likelihood of the model.
#' }
#'
#' If \code{cr} is a function, it is called with the fitted model and any additional arguments passed through \code{...}.
#' @seealso \code{\link{kardl}}
#'
#' @srrstats {G1.6} This function provides criteria for model selection, including AIC, BIC, AICc, and HQ, which can be used in R based model selection processes too.
#' @srrstats {G2.3} The `cr` argument is matched against `c("AIC", "BIC", "AICc", "HQ")` via `match.arg()` when a string is supplied.
#' @srrstats {G3.0} Numerical comparisons in criterion calculations use tolerance-based floating-point arithmetic where appropriate.
#'
#'
#' @examples
#'
#' # Example usage of modelCriterion function with a simple linear model
#' mylm<- lm(mpg ~ wt + hp, data = mtcars)
#' modelCriterion(mylm, AIC )
#' modelCriterion(mylm, "AIC" )
#'
#'  # Example usage of modelCriterion function with a kardl model
#'  kardl_model <- kardl(imf_example_data,
#'                       CPI ~ ER + PPI + asym(ER) + deterministic(covid) + trend,
#'                       mode = c(1, 2, 3, 0))
#'
#'  # Using AIC as the kardl package's built-in criterion function which is different
#'  # from the base R AIC function.
#'  modelCriterion(kardl_model, "AIC")
#'
#'  # Using the base R AIC function directly on the fitted model object
#'  modelCriterion(kardl_model, AIC)
#'  # Using the base R AIC function outside of modelCriterion to compute AIC for the fitted model
#'  AIC(kardl_model)
#'
#'  # Using BIC as the criterion for the kardl model which is different from the base R BIC function.
#'  modelCriterion(kardl_model, "BIC")
#'
#'  # Using a custom criterion function that divides AIC by the sample size
#'  my_cr_fun <- function(mod, ...) { AIC(mod) / length(mod$model[[1]]) }
#'  modelCriterion(kardl_model, my_cr_fun)
#'
#' @export
modelCriterion<-function(estModel,cr,...){
  if (is.function(cr)) {
    do.call(cr,list(estModel, ...))
  } else {
    myCr<- match.arg( cr ,c("AIC","BIC","AICc","HQ"))
    k<-length(estModel$coefficients) #k
    n<-length(estModel$residuals)
    llh<-logLik(estModel)
    val<-switch (myCr,"AIC"=((2*k-2*llh)/n),"BIC"=((log(n)*k-2*llh)/n ),"AICc"=(((2 * k * (k + 1))/(n - k - 1))+(2*k-2*llh)/n),"HQ"=((2*log(log(n))*k-2*llh)/n) )
    as.numeric( val)
  }
}


#'  Model Estimation Dispatcher
#'
#' This function serves as a dispatcher for estimating the ARDL or NARDL model based on the specified mode in the input arguments. It determines the appropriate estimation method to use (e.g., "quick", "grid_custom", "grid", or user-defined) and calls the corresponding S3 method for model estimation.
#' @param spec A list containing the prepared specifications for model estimation, including extracted information and arguments.
#' @param ... Additional arguments that can be passed to the specific estimation methods.
#' @return The estimated model object based on the specified mode.
#' @noRd

makemodel<-function(spec,...){
  myMethod<-"quick"
  # if(is.vector(inputs$mode))
  if(is.vector(spec$argsInfo$mode[1]) && is.numeric(spec$argsInfo$mode) && isFALSE(isFALSE(spec$argsInfo$mode)) )  {
    class(myMethod)<- "user"
  }else{
    class(myMethod)<- match.arg(tolower(spec$argsInfo$mode) ,c("quick","grid_custom","grid"))
  }
  UseMethod("makemodel",myMethod)
}



#' Quick Model Estimation Method
#'
#' This function implements the "quick" estimation method for ARDL or NARDL models.
#' It performs a stepwise search to find the optimal lag structure based on the specified
#' criterion. The function iteratively tests different lag combinations and evaluates the
#'  model using the provided criterion until it converges to the best lag configuration.
#' @param spec A list containing the prepared specifications for model estimation,
#' including extracted information and arguments.
#' @param ... Additional arguments that can be passed to the function.
#' @return An object of class \code{kardl_lm} containing the estimated
#' ARDL or NARDL model based on the "quick" estimation method.
#' @noRd
#'
#' @export
makemodel.quick<-function(spec , ...  ){#model,data,inputs){
  start_time <- Sys.time()
  # inputs<-prepare(lmerge(list( model=model,data=data),inputs))

  preModel<-makeLongrunMOdel(spec) #retruns LS_longrun   LS_dependent


  #fmodel<-paste0(LS_dependent,"~",paste(LS_longrun,makeShortrunMOdel(spec$extractedInfo$shortRunVars,LagsList,deterministic),sep="+"))

  Xlength<-length( spec$extractedInfo$shortRunVars)

  failed_checks<-matrix(NA,1,Xlength)[-1,]
  colnames(failed_checks)<- spec$extractedInfo$shortRunVars

  toporders<-matrix(0,1,Xlength+1)[-1,]
  colnames(toporders)<- c(spec$extractedInfo$shortRunVars,"criterion_value")

  #internal func
  myEst<-function(order){

    # function(shortRunVars,LagsList,deterministic,LS_dependent,LS_longrun,thisData)
      theResults<-lm(as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,order,spec$extractedInfo$deterministic),sep="+")) ) ,spec$extractedInfo$data)

    # theResults<-makeEstimation(spec$extractedInfo$shortRunVars,
    #                            order,spec$extractedInfo$deterministic,preModel$LS_dependent,preModel$LS_longrun,spec$extractedInfo$data)
    modelCriterion(theResults,spec$argsInfo$criterion)
  }

  for (i in 1:spec$argsInfo$maxlag) { # for each parse
    ardl_converge <- FALSE
    order1 <- rep(i, Xlength)
    Base_cr <- myEst(order1)
    toporders<-rbind(toporders,c(order1 ,Base_cr))
    while (!ardl_converge ) {
      for(j in 1:(Xlength-1)) {
        order2_back <- order1
        order2_forth <- order1
        if (order1[j + 1] == 0) {
          # order2_back is the order1
          order2_forth[j + 1] <- order1[j + 1] + 1
        } else {
          if (order1[j + 1] == spec$argsInfo$maxlag) {
            order2_back[j + 1] <- order1[j + 1] - 1
          }else{
            order2_back[j + 1] <- order1[j + 1] - 1
            order2_forth[j + 1] <- order1[j + 1] + 1
          }
        }
        model0 <- myEst(order2_back)
        model1 <- myEst(order2_forth)
        minCr<-min(model0,model1)
        if(Base_cr>minCr){
          if( model0 >  model1){
            order1<-order2_forth
          }else{
            order1<- order2_back
          }
          toporders<-rbind(toporders,c(order1 ,minCr))
          Base_cr<-minCr
          ardl_converge <- FALSE
        }else{
          if(any(apply(failed_checks, 1, function(row) all(row == order1)))){
            ardl_converge <- TRUE
          }else {
            failed_checks<-rbind(failed_checks,order1)
          }
        }
      }
    }
  }

   minValue<-toporders[  which.min(toporders[, "criterion_value"]), , drop = FALSE]

  best_order <- minValue[,-ncol(minValue)]
  MyFormula<-as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,best_order,spec$extractedInfo$deterministic),sep="+")) )
  theResults<-lm(MyFormula ,spec$extractedInfo$data)
  theResults$call <- MyFormula

  # model_tidy <- summary(theResults$formula)$coefficients %>%   as_tibble(rownames = "term")
  # colnames(model_tidy) <- c("term", "estimate", "std.error", "statistic", "p.value")

  end_time <- Sys.time()
  spec$timeInfo<-list(
    start_time=start_time,
    end_time=end_time,
    span = difftime(end_time, start_time, units = "secs")
  )
  spec$lagInfo<-list(
    OptLag=best_order,
    LagCriteria=toporders
  )
  fittedVars<-fitted(theResults)
  k<-length(theResults$coefficients)
  n<-length(theResults$residuals)
  spec$estInfo<- list(type="kardlmodel",
                      method = "quick",
                      ModelFormula = theResults$call,
                      k=k ,
                      n=n,
                      start=as.numeric(names(fittedVars[1])),
                      end=as.numeric(names(tail(fittedVars,n=1))),
                      TimeSpan=n+spec$argsInfo$maxlag+1
  )


  Karamelikli<-lmerge(spec,theResults)
  class(Karamelikli) <- c("kardl_lm","lm")
  Karamelikli
}

#' User-Defined Lag Estimation Method
#'
#' This function implements the estimation method for ARDL or NARDL models when the
#' user provides a specific lag structure. It estimates the model using the provided
#' lag configuration and evaluates it based on the specified criterion. The function
#' returns the estimated model object based on the user-defined lag structure.
#'
#' @param spec A list containing the prepared specifications for model estimation,
#'  including extracted information and arguments.
#' @param ... Additional arguments that can be passed to the function.
#' @return An object of class \code{kardl_lm} and \code{lm} containing the
#' estimated ARDL or NARDL model based on the user-defined lag structure.
#' @noRd
#' @export
#'
makemodel.user <-function(spec, ...  ){#model,data,inputs){
  # spec$argsInfo$mode<-"user"
  start_time <- Sys.time()

  # inputs<-prepare(lmerge(list( model=model,data=data),inputs))

  preModel<-makeLongrunMOdel(spec) #retruns LS_longrun   LS_dependent

  MyFormula<- as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,spec$argsInfo$mode,spec$extractedInfo$deterministic),sep="+")) )
  theResults<- lm(MyFormula ,spec$extractedInfo$data)
  theResults$call <- MyFormula

    #makeEstimation(spec$extractedInfo$shortRunVars,LagsList=spec$argsInfo$mode,spec$extractedInfo$deterministic,preModel$LS_dependent,preModel$LS_longrun,spec$extractedInfo$data)


  # model_tidy <- summary(theResults$formula)$coefficients %>%   as_tibble(rownames = "term")
  # colnames(model_tidy) <- c("term", "estimate", "std.error", "statistic", "p.value")
  OptLag<-spec$argsInfo$mode
  attr( OptLag,"source")<- NULL
  attr( OptLag,"description")<- NULL

  end_time <- Sys.time()

  spec$timeInfo<-list(
    start_time=start_time,
    end_time=end_time,
    span = difftime(end_time, start_time, units = "secs")
  )
  spec$lagInfo<-list(
    OptLag=OptLag
  )
  fittedVars<-fitted(theResults)
  k<-length(theResults$coefficients) #k
  n<-length(theResults$residuals)
  spec$estInfo<- list(
    type="kardlmodel",
    method = "user",
    ModelFormula = theResults$call,
    k=k ,
    n=n,
    start=as.numeric(names(fittedVars[1])),
    end=as.numeric(names(tail(fittedVars,n=1))),
    TimeSpan=n+max(OptLag)+1
  )


  Karamelikli<-lmerge(spec,theResults)
  class(Karamelikli) <- c("kardl_lm","lm")
  Karamelikli
}

#' Grid Search Estimation Method
#'
#' This function implements the "grid_custom" estimation method for ARDL or NARDL models. It performs a grid search over possible lag combinations to find the optimal lag structure based on the specified criterion. The function iteratively tests different lag combinations within the defined grid and evaluates the model using the provided criterion until it identifies the best lag configuration.
#' @param spec A list containing the prepared specifications for model estimation, including extracted information and arguments.
#' @param ... Additional arguments that can be passed to the function.
#' @return An object of class \code{kardl_lm} and \code
#' {lm} containing the estimated ARDL or NARDL model based on the "grid_custom" estimation method.
#' @noRd
#' @export
makemodel.grid_custom<-function(spec, ...  ){ #model ,  data,inputs  ){
  # inputs$mode<-"grid_custom"
  start_time <- Sys.time()
  # inputs<-prepare(lmerge(list( model=model,data=data),inputs))

  # inputs<-prepare(model=model,data=data  ,inputs)
  # cat("\r",paste0("Note: Performance mode will only have output once all estimations are finished.  Starting of estimation of the model: ",inputs$formulaName))

  preModel<-makeLongrunMOdel(spec) #retruns LS_longrun   LS_dependent
  batch<-BatchControl(spec)
  spec$extractedInfo  <- lmerge(batch,spec$extractedInfo)

  OrderForShortRun <- rep(list((spec$argsInfo$maxlag-1):0),spec$extractedInfo$shortrunLength) # I am reversing orders due to alerting about the insufficiency of the degree of freedom.
  OrderForInd<-list((spec$argsInfo$maxlag-1):1)
  GeneralOrder<-append(OrderForShortRun,OrderForInd) # p<-append(m,b)
  LagQueue<-rev(expand.grid(GeneralOrder))  # q<-expand.grid(p)

  if(length(spec$extractedInfo$ASvars)>0 && ! spec$argsInfo$differentAsymLag) {
    i<-1 # satrt from the second var
    for (x in  spec$extractedInfo$independentVars){
      i<-i+1
      if((x %in% spec$extractedInfo$ASvars)){
        LagQueue<-cbind(LagQueue[,1:i],LagQueue[,i:ncol(LagQueue)])
        i<-i+1
      }
    }
  }
  colnames(LagQueue)<-spec$extractedInfo$shortRunVars
  ## Note: This part will be updated to reduce RAM requirement to keep huge list. It will be cut by the batch function's return.
  Mincr<-1000000 # max value for be compared with criteria.
  OptRow<-0 # The row with contain the most proper lag orders

  for(i in batch$startRow:batch$endRow)  {

    MyFormula<- as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,unlist(LagQueue[i,]),spec$extractedInfo$deterministic),sep="+")) )
    theResults<- lm(MyFormula ,spec$extractedInfo$data)
    cr<-modelCriterion(theResults, spec$argsInfo$criterion, ...)
   if(cr<Mincr){
      Mincr<-cr
      OptRow<-i
    }
  }

  endLag  <-paste(LagQueue[batch$endRow,],collapse = ",")
  startLag<-paste(LagQueue[batch$startRow,],collapse = ",")
  properRow<-1

  finalLags<-data.frame(c( paste(LagQueue[OptRow,],collapse = ","),Mincr)  )
  if(!is.function(spec$argsInfo$criterion)){
    colnames(finalLags)<-spec$argsInfo$criterion
  }
  rownames(finalLags)<-c("lag","value")
  Mincr<-NULL

  MyFormula<- as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,unlist(LagQueue[OptRow,]),spec$extractedInfo$deterministic),sep="+")) )
  theResults<- lm(MyFormula ,spec$extractedInfo$data)
  theResults$call <- MyFormula
  fittedVars<-fitted(theResults)
  properLag <- unlist(LagQueue[OptRow,])
  MaxLag<-max( properLag)
  end_time <- Sys.time()




  end_time <- Sys.time()
  spec$timeInfo<-list(
    start_time=start_time,
    end_time=end_time,
    span = difftime(end_time, start_time, units = "secs")
  )

  fittedVars<-fitted(theResults)
  k<-length(theResults$coefficients) #k
  n<-length(theResults$residuals)
  spec$estInfo<- list(
    type="kardlmodel",
    method = "grid_custom",
    ModelFormula = theResults$call,
    k=k ,
    n=n,
    start=as.numeric(names(fittedVars[1])),
    end=as.numeric(names(tail(fittedVars,n=1))),
    TimeSpan=n+MaxLag+1
  )
  spec$lagInfo<-list(
    OptLag= properLag,
    allCrLaga = finalLags,
    properRow=OptRow,
    LagsFrom=startLag
  )

  Karamelikli<-lmerge(spec,theResults)
  class(Karamelikli) <- c("kardl_lm","lm")
  Karamelikli
}

#' Grid Search Estimation Method with Console Output
#'
#' This function implements the "grid" estimation method for ARDL or NARDL models. It performs a grid search over possible lag combinations to find the optimal lag structure based on the specified criterion. The function iteratively tests different lag combinations within the defined grid and evaluates the model using the provided criterion until it identifies the best lag configuration. Additionally, this method provides console output during the estimation process to indicate progress and the current lag combination being evaluated.
#' @param spec A list containing the prepared specifications for model estimation, including extracted information and
#' arguments.
#' @param ... Additional arguments that can be passed to the function.
#' @return An object of class \code{kardl_lm} and \code
#' {lm} containing the estimated ARDL or NARDL model based on the "grid" estimation method with console output.
#' @noRd
#'
#' @export
makemodel.grid<-function(spec , ... ){#model ,  data,inputs  ){ # makemodel.default
  # inputs$mode<-"grid"

  start_time <- Sys.time()
  # inputs<-prepare(lmerge(list( model=model,data=data),inputs))

  # inputs<-prepare(model=model,data=data  ,inputs)
  #cat("\r",paste0("Appearance mode will have outputs during estimations. Starting of estimation of the model: ",inputs$formulaName))

  preModel<-makeLongrunMOdel(spec) #retruns LS_longrun   LS_dependent
  batch<-BatchControl(spec)

  LagCriteria<-matrix(NA,spec$extractedInfo$lagRowsNumber  ,5) # rows: lag , dependent,LongrunTerms, Shortrun terms, AIC, BIC ,SC,HQ
  colnames(LagCriteria) <- c("lag" , "AIC", "BIC" ,"AICc","HQ")
  bir<-rep(rep(c(1:(spec$argsInfo$maxlag-1)), each = (spec$argsInfo$maxlag)^(spec$extractedInfo$shortrunLength)),time=1)
  for (i in 1:spec$extractedInfo$shortrunLength) {
    r<-rep(rep(c(0:(spec$argsInfo$maxlag-1)), each = (spec$argsInfo$maxlag)^(spec$extractedInfo$shortrunLength-i)),time=(spec$argsInfo$maxlag^i -spec$argsInfo$maxlag^(i-1)) )
    if(length(spec$extractedInfo$ASvars)>0 && (spec$extractedInfo$independentVars[i] %in% spec$extractedInfo$ASvars)){
      if(! spec$argsInfo$differentAsymLag) {
        bir<-cbind(bir,r)
      }
    }
    bir<-cbind(bir,r)
  }
  colnames(bir)<-spec$extractedInfo$shortRunVars
  LagMatrix <-bir[rev(seq_len(nrow(bir))),] # I am reversing orders due to alerting about the insufficiency of the degree of freedom.
  colNum<-ncol(LagMatrix)
  for (i in seq_len(nrow(LagMatrix))){
    LagCriteria[i,1] <- paste(LagMatrix[i,],collapse=",")
  }
  for(i in batch$startRow:batch$endRow)  {
    MyFormula <- as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,unlist(LagMatrix[i,]),spec$extractedInfo$deterministic),sep="+")) )
    theResults <- lm(MyFormula ,spec$extractedInfo$data)
    theResults$call <- MyFormula
    # theResults<- makeEstimation(spec$extractedInfo$shortRunVars,LagsList=unlist(LagMatrix[i,]),spec$extractedInfo$deterministic,preModel$LS_dependent,preModel$LS_longrun,spec$extractedInfo$data)


    k<-length(theResults$coefficients) #k
    n<-length(theResults$residuals)  # T
    llh<-logLik(theResults)
    aic<- (2*k-2*llh)/n
    bic<- (log(n)*k-2*llh)/n
    hq<- (2*log(log(n))*k-2*llh)/n
    LagCriteria[i,2]<-aic
    LagCriteria[i,3]<-bic #BIC(est)
    LagCriteria[i,4]<- aic + ((2 * k * (k + 1))/(n - k - 1)) # AICc(est)
    LagCriteria[i,5]<-hq #   -2 * as.numeric(llh) + 2 * k * log(log(n))
    progressBar(i,batch$endRow,as.character(LagCriteria[i,1]))
  }
  cat("\n")
  endLag  <- LagMatrix[batch$endRow ,]# paste(LagMatrix[batch$startRow,],collapse = ",")
  startLag<- LagMatrix[batch$startRow ,]#paste(LagMatrix[batch$endRow,],collapse = ",")
  aicRow<-which(LagCriteria[,2]==min(as.numeric(LagCriteria[,2]), na.rm=T), arr.ind=T)[1]
  bicRow<-which(LagCriteria[,3]==min(as.numeric(LagCriteria[,3]), na.rm=T), arr.ind=T)[1]
  aiccRow<-which(LagCriteria[,4]==min(as.numeric(LagCriteria[,4]), na.rm=T), arr.ind=T)[1]
  hqRow<-which(LagCriteria[,5]==min(as.numeric(LagCriteria[,5]), na.rm=T), arr.ind=T)[1]
  finalLags<-data.frame("AIC"=c(LagCriteria[aicRow,1],LagCriteria[aicRow,2]),
                        "BIC"=c(LagCriteria[bicRow,1],LagCriteria[bicRow,3])   ,
                        "AICc"=c(LagCriteria[aiccRow,1],LagCriteria[aiccRow,4]),
                        "HQ"=c(LagCriteria[hqRow,1],LagCriteria[hqRow,5])
  )

  rownames(finalLags)<-c("lag","value")
  properRow<-switch (spec$argsInfo$criterion,"AIC"=aicRow,"BIC"=bicRow ,"AICc"=aiccRow,"HQ"=hqRow )
  properLag <-LagMatrix[properRow,]
  MyFormula <- as.formula(paste0(preModel$LS_dependent,"~",paste(preModel$LS_longrun,
                      makeShortrunMOdel(spec$extractedInfo$shortRunVars,unlist(properLag),spec$extractedInfo$deterministic),sep="+")) )
  theResults <- lm(MyFormula ,spec$extractedInfo$data)
  theResults$call <- MyFormula

  # theResults<- makeEstimation(spec$extractedInfo$shortRunVars,LagsList=unlist(properLag),spec$extractedInfo$deterministic,preModel$LS_dependent,preModel$LS_longrun,spec$extractedInfo$data)
  k<-length(theResults$coefficients) #k
  n<-length(theResults$residuals)  # T
  fittedVars<-fitted(theResults)
  # model_tidy <- summary(theResults$formula)$coefficients %>%   as_tibble(rownames = "term")
  # colnames(model_tidy) <- c("term", "estimate", "std.error", "statistic", "p.value")


  MaxLag<-max(properLag)
  end_time<- Sys.time()
  #cat("\r The collapsed time is: ", secondsToPeriod(as.numeric(difftime(end_time, start_time, units = "secs"))))

  end_time <- Sys.time()
  spec$timeInfo<-list(
    start_time=start_time,
    end_time=end_time,
    span = difftime(end_time, start_time, units = "secs")
  )

  fittedVars<-fitted(theResults)
  spec$estInfo<- list(
    type="kardlmodel",
    method = "grid",
    ModelFormula = theResults$call,
    k=k ,
    n=n,
    start=as.numeric(names(fittedVars[1])),
    end=as.numeric(names(tail(fittedVars,n=1))),
    TimeSpan=n+MaxLag+1
  )
  spec$lagInfo<-list(
    OptLag= LagMatrix[properRow,],
    allCrLaga = finalLags,
    properRow=properRow,
    Criterion=spec$argsInfo$criterion,
    LagsFrom=startLag,
    LagsTo=endLag,
    LagCriteria=as.data.frame( LagCriteria),
    LagMatrix=LagMatrix
  )

  Karamelikli<-lmerge(spec,theResults)
  class(Karamelikli) <- c("kardl_lm","lm")
  Karamelikli
}

#' @srrstats {G5.2} The test suite exercises representative successful calls and
#' selected error conditions for model estimation, multiplier calculation, and
#' S3 methods.
#' @srrstats {G5.3} Tests verify that returned objects have the expected class,
#' @srrstats {G5.4} Verifies correct structural properties of the fitted object
#' for the baseline linear ARDL specification.
#' dimensions, names, and component values.

test_that("kardl model estimates correctly in quick mode", {
  kardl_reset()
  formula <- CPI ~ ER + PPI + asymmetric(ER) + trend

  model <- kardl(data = imf_example_data, formula = formula, mode = "quick", maxlag = 2)

  expect_s3_class(model, "kardl_lm")
  expect_false(is.null(model))
  expect_s3_class(model, "lm")
  expect_identical(model$estInfo$type, "kardlmodel")
})
#' @srrstats {G5.3} Checks that user-defined lag mode produces an object with
#' a non-NULL `lagInfo$OptLag` component.
#' @srrstats {G5.6} Confirms the lag vector supplied by the user is applied
#' and recoverable from the fitted object.
test_that("kardl model estimates correctly with custom lags", {

  kardl_reset()
  formula <- CPI ~ ER + PPI + asymmetric(ER) + deterministic(covid) + trend

  model <- kardl(data = imf_example_data, formula = formula, mode = c(2, 1, 1,  0))

  expect_s3_class(model, "kardl_lm")
  expect_false(is.null(model$lagInfo$OptLag))
})
#' @srrstats {G5.3} Checks returned class and `estInfo$method` for grid_custom mode.
#' @srrstats {G5.7} Tests behaviour across a different lag-search algorithm (grid_custom).
test_that("kardl with grid_custom mode works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI, mode = "grid_custom", maxlag = 2)

  expect_s3_class(model, "kardl_lm")
  expect_identical(model$estInfo$method, "grid_custom")
})
#' @srrstats {G5.3} Verifies OptLag is populated when BIC is used as the criterion.
#' @srrstats {G5.7} Tests model estimation with an alternative information criterion.
test_that("kardl with grid_custom mode and BIC criterion works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI, mode = "grid_custom",
                 maxlag = 2, criterion = "BIC")

  expect_s3_class(model, "kardl_lm")
  expect_false(is.null(model$lagInfo$OptLag))
})
#' @srrstats {G5.3} Checks returned class, method, and LagCriteria for grid mode.
#' @srrstats {G5.7} Tests the full grid-search estimation path.
test_that("kardl with grid mode works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI, mode = "grid", maxlag = 2)

  expect_s3_class(model, "kardl_lm")
  expect_identical(model$estInfo$method, "grid")
  expect_false(is.null(model$lagInfo$LagCriteria))
})
#' @srrstats {G5.4} Verifies that the dot (.) formula expansion produces a
#' valid estimable model.
test_that("kardl with dot formula works", {

  kardl_reset()
  #' Dot expands to ER, PPI, covid; covid goes to deterministic -> 3 shortRunVars
  model <- kardl(imf_example_data, CPI ~ . + deterministic(covid),
                 mode = c(1, 1, 1))
  expect_s3_class(model, "kardl_lm")
})
#' @srrstats {G5.3} Checks that `ALvars` is populated and `ASvars` is empty
#' for a long-run-asymmetric-only specification.
#' @srrstats {G5.7} Tests model estimation across the SA (long-run asymmetric) configuration.
test_that("kardl with lasymmetric produces correct structure", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ lasymmetric(ER), mode = c(1, 1))
  expect_s3_class(model, "kardl_lm")
  expect_gt(length(model$extractedInfo$ALvars), 0)
  expect_length(model$extractedInfo$ASvars, 0)
})
#' @srrstats {G5.3} Checks that `ASvars` is populated and `ALvars` is empty
#' for a short-run-asymmetric-only specification.
#' @srrstats {G5.7} Tests model estimation across the AS (short-run asymmetric) configuration.
test_that("kardl with sasymmetric produces correct structure", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ PPI + sasymmetric(ER),
                 mode = c(1, 1, 1, 1))
  expect_s3_class(model, "kardl_lm")
  expect_gt(length(model$extractedInfo$ASvars), 0)
  expect_length(model$extractedInfo$ALvars, 0)
})
#' @srrstats {G5.3} Verifies that batch-mode estimation returns a valid
#' `kardl_lm` object.
test_that("kardl grid_custom with batch processing works", {

  kardl_reset()
  model <- kardl(imf_example_data, CPI ~ ER + PPI,
                 mode = "grid_custom", maxlag = 2, batch = "1/2")
  expect_s3_class(model, "kardl_lm")
})
#' @srrstats {G5.2a} Checks that a missing formula without a fallback in
#' `kardl_set()` produces an informative error.
#' @srrstats {G5.8a} Verifies that missing required input is rejected before
#' any estimation occurs.
test_that("kardl fails informatively when formula missing and not in kardl_set", {

  kardl_reset()
  expect_error(kardl(data = imf_example_data, mode = c(1, 1, 1)))
})
#' @srrstats {G5.2} Tests all four predefined string criterion options.
#' @srrstats {G5.3} Verifies that each criterion returns a numeric scalar.
test_that("modelCriterion works with all built-in criteria", {

  kardl_reset()
  mylm <- lm(mpg ~ wt + hp, data = mtcars)

  aic  <- modelCriterion(mylm, "AIC")
  bic  <- modelCriterion(mylm, "BIC")
  aicc <- modelCriterion(mylm, "AICc")
  hq   <- modelCriterion(mylm, "HQ")

  expect_type(aic, "double")
  expect_type(bic, "double")
  expect_type(aicc, "double")
  expect_type(hq, "double")
})
#' @srrstats {G5.2} Tests the user-defined function path of `modelCriterion()`.
#' @srrstats {G5.6} Result matches the base-R AIC function, confirming
#' correctness of the function-dispatch branch.
test_that("modelCriterion works with a user-defined function", {

  mylm <- lm(mpg ~ wt + hp, data = mtcars)
  my_fn <- function(mod, ...) stats::AIC(mod)
  result <- modelCriterion(mylm, my_fn)
  expect_identical(result, stats::AIC(mylm))
})
#' @srrstats {G5.3} Checks returned class and `estInfo$type` for ECM estimation.
#' @srrstats {TS2.0} Confirms the ECM estimation path handles lagged and
#' differenced variables correctly.
test_that("ecm model estimates correctly", {

  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  expect_s3_class(ec, "kardl_lm")
  expect_identical(ec$estInfo$type, "ecm")
})
#' @srrstats {G5.3} Verifies that `summary()` on an ECM object returns
#' a coefficient table.
test_that("ecm summary works", {

  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI + trend, mode = c(1, 1, 1))
  sm <- summary(ec)
  expect_false(is.null(sm$coefficients))
})
#' Just check that model is returned; note checking is secondary
#' @srrstats {G5.3} Confirms that a valid `kardl_lm` object is returned
#' even when the ECM coefficient triggers a note.
test_that("ecm model with notes when ECM coeff is non-negative or < -1", {

  kardl_reset()
  ec <- ecm(imf_example_data, CPI ~ ER + PPI, mode = c(1, 1, 1))
  expect_s3_class(ec, "kardl_lm")
})

library("imf.data")
IFS <- load_datasets("IFS")

#' install.packages("imf.data")
#' library(imf.data)
#' IFS <- load_datasets("IFS")
#' # PCPI_IX             Prices, Consumer Price Index, All items, Index
#' # AIP_IX              Economic Activity, Industrial Production, Index
#' # ENDE_XDC_USD_RATE   Exchange Rates, Domestic Currency per U.S. Dollar, End of Period, Rate
#' 
trdata<-IFS$get_series(freq = "M", ref_area = "TR", indicator = c("PCPI_IX","AIP_IX","ENDE_XDC_USD_RATE"),start_period = "1985-01",end_period = "2024-02")
 PeriodRow<-trdata[,1]
trdata[,1]<-NULL
colnames(trdata)<-c("ER","CPI","PPI")
trdata<-log(as.data.frame(lapply(trdata, function(x) as.numeric(x))))
rownames(trdata)<-PeriodRow
#' # Inserting covid dummy variable
trdata<-cbind(trdata,covid=0)
trdata[420:470,4]<-1
KARDL_model <-KARDL(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,trdata,userdefined=c(1,2,2,0))

#' IMF Example Data
#'
#' This is an example data set used for testing purposes. It contains monthly data on exchange rates, consumer price index, producer price index, and a dummy variable for the COVID-19 pandemic for Turkey from January 1985 to February 2024.
#'
#'
#' @details
#' These data obtained from \pkg{imf.data} package.
#' The sample data is not updated and obtained by following codes:
#'

#' \code{install.packages("imf.data")}\cr
#' \code{library("imf.data")}\cr
#' \code{IFS <- load_datasets("IFS")}
#'
#' \itemize{
#' \item \strong{PCPI_IX}             Prices, Consumer Price Index, All items, Index\cr
#' \item \strong{AIP_IX}              Economic Activity, Industrial Production, Index\cr
#' \item \strong{ENDE_XDC_USD_RATE}   Exchange Rates, Domestic Currency per U.S. Dollar, End of Period, Rate}
#'
#' \code{  trdata<-IFS$get_series(freq = "M", ref_area = "TR", indicator = c("PCPI_IX","AIP_IX","ENDE_XDC_USD_RATE"),start_period = "1985-01",end_period = "2024-02")}\cr
#' \code{  PeriodRow<-trdata[,1]}\cr
#' \code{  trdata[,1]<-NULL}\cr
#' \code{  colnames(trdata)<-c("ER","CPI","PPI")}\cr
#' \code{  trdata<-log(as.data.frame(lapply(trdata, function(x) as.numeric(x))))}\cr
#' \code{  rownames(trdata)<-PeriodRow}
#'
#' \strong{Inserting covid dummy variable}
#'
#' \code{  trdata<-cbind(trdata,covid=0)}\cr
#' \code{  trdata[420:470,4]<-1}
#'
#' @format A data frame with 470 rows and 4 variables:
#' \describe{
#'   \item{ER}{Numeric. Exchange rate of Turkey.}
#'   \item{CPI}{Numeric. CPI of Turkey.}
#'   \item{PPI}{Numeric. PPI of Turkey.}
#'   \item{covid}{Integer.Covid19 dummy variable.}
#' }
#'
#' @seealso \code{\link[imf.data]{load_datasets}}
#'
#' @examples
#' data(imf_example_data)
#' head(imf_example_data)
"imf_example_data"

#' Compute Dynamic Multipliers for kardl Models
#'
#'
#' Computes cumulative dynamic multipliers based on a model estimated using the
#' \code{kardl} framework. The function supports different configurations of
#' linearity and asymmetry in both short-run and long-run dynamics.
#'
#' The asymmetry structure is determined internally:
#' \itemize{
#'   \item Variables in \code{extractedInfo$ASvars} are treated as asymmetric in the short run.
#'   \item Variables in \code{extractedInfo$ALvars} are treated as asymmetric in the long run.
#' }
#'
#' This allows four possible configurations:
#' \itemize{
#'   \item \strong{LL}: Linear in both short-run and long-run
#'   \item \strong{NN}: Asymmetric in both short-run and long-run
#'   \item \strong{SA}: Short-run linear, long-run asymmetric
#'   \item \strong{AS}: Short-run asymmetric, long-run linear
#' }
#'
#' When a component is linear, the same coefficient path is used for both positive
#' and negative changes. When asymmetric, separate positive and negative effects
#' are computed.
#'
#' @param kmodel An object of class \code{kardl_lm} produced by the \code{kardl} function.
#' @param horizon Integer. Number of periods ahead for which dynamic multipliers are computed.
#' @param minProb Numeric. Minimum p-value threshold for including coefficients in the calculation. Coefficients with p-values above this threshold will be set to zero. Default is \code{0} (no threshold).
#' This parameter allows users to control the inclusion of coefficients in the calculation based on their statistical significance. Setting a threshold can help focus the analysis on more relevant variables, but it may also exclude potentially important effects if set too stringently.
#'
#' @return A list of class \code{kardl_mplier} containing:
#' \itemize{
#'   \item \strong{mpsi}: Matrix of cumulative dynamic multipliers.
#'   \item \strong{omega}: Vector of omega coefficients (persistence structure).
#'   \item \strong{lambda}: Matrix of short-run dynamic coefficients.
#'   \item \strong{horizon}: Forecast horizon used.
#'   \item \strong{vars}: Extracted model variable structure.
#' }
#'
#' @details
#'
#' The \code{mplier} function computes dynamic multipliers based on the coefficients and lag structure of a model estimated using the \code{kardl} package. The function extracts necessary
#' information from the model, such as coefficients, lag structure, and variable names, to compute the dynamic multipliers. It calculates the short-run coefficients, Lambda values, and omega values based on the model's parameters and lag structure. The output includes a matrix of dynamic multipliers (mpsi), which can be used for further analysis or visualization.
#' The dynamic multipliers provide insight into how changes in the independent variables affect the dependent variable over time, allowing for a deeper understanding of the relationships captured by the model. The function also allows users to set a minimum p-value threshold for including coefficients in the calculation, providing flexibility in focusing on statistically significant effects.
#'
#' The function constructs dynamic multipliers based on the recursive relationship:
#'
#' \deqn{
#' \psi_{h}^{+} = \sum_{i=0}^{h} \frac{\partial y_{t+i}}{\partial x_{t}^{+}}, \quad
#' \psi_{h}^{-} = \sum_{i=0}^{h} \frac{\partial y_{t+i}}{\partial x_{t}^{-}}
#' }
#'
#' where \eqn{\psi_h^{+}} and \eqn{\psi_h^{-}} represent cumulative responses to
#' positive and negative shocks.
#'
#' The recursion is defined as:
#'
#' \deqn{
#' \psi_h = \lambda_h + \sum_{j=1}^{p} \omega_j \psi_{h-j}
#' }
#'
#' where \eqn{\lambda_h} captures short-run effects and \eqn{\omega_j} reflects
#' persistence through lagged dependent variables.
#'
#' When asymmetry is present, positive and negative shocks are propagated separately.
#' Otherwise, the same dynamic path is used.
#'
#' @seealso \code{\link{bootstrap}}
#'
#' @import stats msm
#'
#'
#' @srrstats {G1.3}  Statistical terms such as "dynamic multipliers" are clearly defined and used consistently throughout the documentation and examples.
#' @srrstats {G2.0} The function validates that `kmodel` is of class `kardl_lm` before extracting lag and coefficient information.
#' @srrstats {G3.1} Long-run multiplier inference uses the variance-covariance matrix of the fitted model object.
#' @srrstats {G3.1a} Documentation describes the recursive formula used to compute cumulative multipliers and the role of the `omega` persistence structure.
#' @srrstats {TS4.2} The return value documents the `mpsi`, `omega`, `lambda`, `horizon`, and `vars` components with their meanings.
#' @srrstats {TS4.3} Dynamic multiplier outputs include the horizon index so that multiplier paths can be interpreted over modelled time steps.
#' @srrstats {TS5.0} A `plot` method is implemented for `kardl_mplier` objects.
#' @srrstats {TS5.7} The plot method displays the computed response path from the fitted model over the selected horizon.
#' @srrstats {TS5.8} Plot methods distinguish multiplier components by variable and shock direction where asymmetric effects are present.
#'
#'
#'
#'
#' @export
#'
#' @examples
#'
#' # This example demonstrates how to use the mplier function to calculate dynamic multipliers
#' # from a model estimated using the kardl package. The example includes fitting a model with
#' # the kardl function, calculating the multipliers, and visualizing the results using both
#' # base R plotting and ggplot2.
#'
#'  # Calculating dynamic multipliers for a linear model in short and long run (NN)
#'
#'  kardl_model<-kardl(imf_example_data, CPI~ER )
#'  m<-mplier(kardl_model,40)
#'  head(m$mpsi)
#'  plot(m)
#'
#'  # Calculating dynamic multipliers for a model with
#'  # Short-run linear, long-run asymmetric (SA)
#'  kardl_model<-kardl(imf_example_data, CPI~lasym(ER) )
#'  m<-mplier(kardl_model,40)
#'  head(m$mpsi)
#'  plot(m)
#'
#'  # Calculating dynamic multipliers for a model with
#'  # Short-run asymmetric, long-run linear (AS)
#'  kardl_model<-kardl(imf_example_data, CPI~sasym(ER) )
#'  m<-mplier(kardl_model,40)
#'  head(m$mpsi)
#'  plot(m)
#'
#'  # Calculating dynamic multipliers for a model with
#'  # asymmetric effects in both short and long run (NN)
#'  kardl_model<-kardl(imf_example_data, CPI~asym(ER) )
#'  m<-mplier(kardl_model,40)
#'  head(m$mpsi)
#'  plot(m)
#'
#'  # The mpsi matrix contains the cumulative dynamic multipliers for each variable and time horizon.
#'  # The omega vector contains the persistence structure of the model,
#'  # while the lambda matrix contains the short-run dynamic coefficients.
#'  # You can inspect these components to understand the dynamics captured by the model.
#'  kardl_model<-kardl(imf_example_data, CPI~PPI+asym(ER) )
#'  m<-mplier(kardl_model,40)
#'  head(m$mpsi)
#'  head(m$omega)
#'  head(m$lambda)
#'
#'  # For plotting specific variables, you can specify them in the plot function. For example,
#'  # to plot the multipliers for the variable "PPI":
#'
#'  plot(m, variable = "PPI")
#'
#'
mplier<-function(kmodel,horizon=80,minProb=0){

  if(! inherits(kmodel, "kardl_lm")){

    stop("Input object must be of class 'kardl_lm'.",call. = FALSE)
  }
  var_names <- names(coefficients(kmodel))
  properLag <- kmodel$lagInfo$OptLag
  h <- horizon + 1
  vars <- kmodel$extractedInfo

  depvar <- replace_lag_var(.kardl_Settings_env$LongCoef, vars$dependentVar, 1)
  a_index <- which(var_names == depvar)

  gamma__lags <- properLag[[vars$dependentVar]]
  gamma__index <- which(var_names %in% unlist(lapply(1:gamma__lags, function(i) {
    replace_lag_var(.kardl_Settings_env$ShortCoef, vars$dependentVar, i)
  })))

  model_params <- kmodel$coefficients

  if (minProb > 0) {
    tempModel <- summary(kmodel)$coefficients
    for (v in seq_len(nrow(tempModel))) {
      if (tempModel[v, 4] > minProb) {
        model_params[v] <- 0
      }
    }
  }

  my_theta <- model_params[a_index]
  gamma_ <- model_params[gamma__index]
  p <- length(gamma__index)

  omega_ <- numeric(p + 1)
  omega_[1] <- 1 + model_params[a_index] + gamma_[1]
  if (p >= 2) {
    for (i in 2:p) {
      omega_[i] <- gamma_[i] - gamma_[i - 1]
    }
  }
  omega_[p + 1] <- -gamma_[p]

  ColNum <- length(vars$independentVars)

  ColName <- ColName_F <- c()
  for (x in vars$independentVars) {
    posName <- paste0(.kardl_Settings_env$AsymPrefix[1], x, .kardl_Settings_env$AsymSuffix[1])
    negName <- paste0(.kardl_Settings_env$AsymPrefix[2], x, .kardl_Settings_env$AsymSuffix[2])
    diff_name <- paste0(x, "_dif")
    ColName_F <- c(ColName_F, posName, negName, diff_name)
    ColName <- c(ColName, posName, negName)
  }

  shortRun_coefs <- matrix(0, nrow = h, ncol = ColNum * 2)
  lambda_mtx <- matrix(0, nrow = h, ncol = ColNum * 2)
  mpsi_mtx <- matrix(0, nrow = h, ncol = ColNum * 2)
  colnames(shortRun_coefs) <- colnames(lambda_mtx) <- colnames(mpsi_mtx) <- ColName

  for (v in seq_len(ColNum)) {

    x <- vars$independentVars[v]

    posName <- paste0(.kardl_Settings_env$AsymPrefix[1], x, .kardl_Settings_env$AsymSuffix[1])
    negName <- paste0(.kardl_Settings_env$AsymPrefix[2], x, .kardl_Settings_env$AsymSuffix[2])

    # decide short-run / long-run asymmetry
    sr_asym <- x %in% vars$ASvars
    lr_asym <- x %in% vars$ALvars

    ## names used in long-run part
    lr_pos <- if (lr_asym) posName else x
    lr_neg <- if (lr_asym) negName else x

    ## names used in short-run part
    sr_pos <- if (sr_asym) posName else x
    sr_neg <- if (sr_asym) negName else x

    ## long-run coefficient indices
    mtx_index_POS <- which(var_names == replace_lag_var(.kardl_Settings_env$LongCoef, lr_pos, 1))
    mtx_index_NEG <- which(var_names == replace_lag_var(.kardl_Settings_env$LongCoef, lr_neg, 1))

    ## short-run lag indices
    mtx_beta_index_p <- which(var_names %in% unlist(lapply(0:properLag[[sr_pos]], function(i) {
      replace_lag_var(.kardl_Settings_env$ShortCoef, sr_pos, i)
    })))

    mtx_beta_index_n <- which(var_names %in% unlist(lapply(0:properLag[[sr_neg]], function(i) {
      replace_lag_var(.kardl_Settings_env$ShortCoef, sr_neg, i)
    })))

    mtx_beta_POS <- model_params[mtx_beta_index_p]
    mtx_beta_NEG <- model_params[mtx_beta_index_n]

    tempColNo <- (v - 1) * 2 + 1

    shortRun_coefs[seq_along(mtx_beta_POS), tempColNo] <- mtx_beta_POS
    shortRun_coefs[seq_along(mtx_beta_NEG), tempColNo + 1] <- mtx_beta_NEG

    lambda_mtx[1, tempColNo] <- shortRun_coefs[1, tempColNo]
    lambda_mtx[1, tempColNo + 1] <- shortRun_coefs[1, tempColNo + 1]

    lambda_mtx[2, tempColNo] <-
      model_params[mtx_index_POS] - shortRun_coefs[1, tempColNo] + shortRun_coefs[2, tempColNo]

    lambda_mtx[2, tempColNo + 1] <-
      model_params[mtx_index_NEG] - shortRun_coefs[1, tempColNo + 1] + shortRun_coefs[2, tempColNo + 1]

    my_beta_lags_p <- properLag[[sr_pos]]
    if (my_beta_lags_p > 1) {
      for (i in 2:length(mtx_beta_POS)) {
        lambda_mtx[i + 1, tempColNo] <- shortRun_coefs[i + 1, tempColNo] - shortRun_coefs[i, tempColNo]
      }
    }
    if (my_beta_lags_p > 0) {
      lambda_mtx[my_beta_lags_p + 2, tempColNo] <- -shortRun_coefs[my_beta_lags_p + 1, tempColNo]
    }

    ## if short-run is linear, copy lambda from POS to NEG
    if (!sr_asym) {
      lambda_mtx[, tempColNo + 1] <- lambda_mtx[, tempColNo]
    } else {
      my_beta_lags_n <- properLag[[sr_neg]]
      if (my_beta_lags_n > 1) {
        for (i in 2:length(mtx_beta_NEG)) {
          lambda_mtx[i + 1, tempColNo + 1] <- shortRun_coefs[i + 1, tempColNo + 1] - shortRun_coefs[i, tempColNo + 1]
        }
      }
      if (my_beta_lags_n > 0) {
        lambda_mtx[my_beta_lags_n + 2, tempColNo + 1] <- -shortRun_coefs[my_beta_lags_n + 1, tempColNo + 1]
      }
    }

    ## if long-run is linear, force same long-run effect in lambda[2]
    if (!lr_asym) {
      lambda_mtx[2, tempColNo + 1] <- lambda_mtx[2, tempColNo]
    }
  }

  mpsi_mtx[1, ] <- lambda_mtx[1, ]
  p2 <- p + 1

  for (v in 2:h) {
    n_say <- ifelse(v > p2, p2, v - 1)
    g_say <- ifelse(v - p2 <= 0, 1, v - p2)
    n_omega <- omega_[1:n_say]
    mpsi_mtx[v, ] <- lambda_mtx[v, ] + n_omega %*% mpsi_mtx[(v - 1):g_say, ]
  }

  mpsi_mtx2 <- apply(mpsi_mtx, 2, cumsum)

  mpsi <- matrix(0, nrow = h, ncol = ColNum * 3)
  for (v in seq_len(ColNum)) {
    colNo2 <- v * 2 - 1
    colNo3 <- v * 3 - 2
    mpsi[, colNo3] <- mpsi_mtx2[, colNo2]
    mpsi[, colNo3 + 1] <- -mpsi_mtx2[, colNo2 + 1]
    mpsi[, colNo3 + 2] <- mpsi_mtx2[, colNo2] - mpsi_mtx2[, colNo2 + 1]
  }

  mpsi <- cbind(0:horizon, mpsi)
  colnames(mpsi) <- c("h", ColName_F)

  structure(
    list(
      call = match.call(),
      mpsi = mpsi,
      omega = omega_,
      lambda = lambda_mtx,
      horizon = horizon,
      vars = vars
    ),
    class = "kardl_mplier"
  )
}

#' Bootstrap Confidence Intervals for Dynamic Multipliers
#'
#' This function computes bootstrap confidence intervals (CI) for dynamic multipliers
#' of a specified variable in a model estimated using the \code{kardl} package. The bootstrap method
#' generates resampled datasets to estimate the variability of the dynamic multipliers,
#' providing upper and lower bounds for the confidence interval.
#'
#' @param kmodel The model produced by the \code{\link{kardl}} function. This is the model object
#'        from which the dynamic multipliers are calculated.
#' @param horizon An integer specifying the horizon over which dynamic multipliers will be computed.
#'        The horizon defines the time frame for the analysis (e.g., 40 periods).
#' @param replications An integer indicating the number of bootstrap replications to perform.
#'        Higher values increase accuracy but also computational time. Default is \code{100}.
#' @param level A numeric value specifying the confidence level for the intervals (e.g., 95 for 95% confidence).
#'        Default is \code{90}.
#' @param minProb A numeric value specifying the minimum p-value threshold for including coefficients in the bootstrap.
#'       Coefficients with p-values above this threshold will be set to zero in the bootstrap samples. Default is \code{0} (no threshold).
#'       This parameter allows users to control the inclusion of coefficients in the bootstrap process based on their statistical significance. Setting a threshold can help focus the analysis on more relevant variables, but it may also exclude potentially important effects if set too stringently.
#'@param seed An optional integer to set the random seed for reproducibility of the bootstrap results. If not provided, the bootstrap will use the current random state.
#'
#' @seealso
#' \code{\link{mplier}} for calculating dynamic multipliers
#'
#' @return A list containing the following elements:
#' \itemize{
#' \item \strong{mpsi}: A data frame containing the dynamic multiplier estimates along with their upper and lower confidence intervals for each variable and time horizon.
#' \item \strong{level}: The confidence level used for the intervals (e.g 95).
#' \item \strong{horizon}: The horizon over which the multipliers were computed (e.g., 40).
#' \item \strong{vars}: A list of variable information extracted from the model, including dependent variable, independent variables, asymmetric variables, and deterministic terms.
#' \item \strong{replications}: The number of bootstrap replications performed.
#' \item \strong{type}: A character string indicating the type of analysis, in this case "bootstrap".
#' }
#'
#'
#' @details
#'
#' The \code{mpsi} component of the output contains the dynamic multiplier estimates along with their upper
#' and lower confidence intervals. These values are provided for each variable and at each time horizon.
#'
#' @srrstats {G5.5} Tests involving random resampling in the bootstrap set a random seed before the replication loop.
#' @srrstats {G5.6} Parameter-recovery properties of the bootstrap are assessed through deterministic examples in the test suite.
#' @srrstats {G3.1} Bootstrap standard errors for dynamic multipliers are derived from the covariance structure of the fitted model's residuals.
#' @srrstats {TS4.2} The bootstrap return value documents the `mpsi`, `level`, `horizon`, `vars`, and `replications` components.
#' @srrstats {G5.9} The documentation includes examples demonstrating the use of the bootstrap function, including how to specify the number of replications and how to interpret the output.
#'
#'
#' @import ggplot2
#' @export
#'
#' @examples
#'
#'   # Example usage of the bootstrap function
#'
#'
#'  # Fit a model using kardl
#'  kardl_model <- kardl(imf_example_data,
#'                       CPI ~ ER + PPI + asy(ER) +
#'                        det(covid) + trend,
#'                       mode = c(1, 2, 3, 0))
#'
#'  # Perform bootstrap with specific variables for plotting
#'  boot <-
#'    bootstrap(kardl_model,   replications=5, seed = 123L)
#'  # The boot object will include all plots for the specified variables
#'  # Displaying the boot object provides an overview of its components
#'  names(boot)
#'
#'
#'  # Inspect the first few rows of the dynamic multiplier estimates
#'   head(boot$mpsi)
#'
#'
#'   summary(boot)
#'
#'  # Retrieve plots generated during the bootstrap process
#'  # Accessing all plots
#'   plot(boot)
#'
#'  # Accessing the plot for a specific variable by its name
#'   plot(boot, variable = "PPI")
#'  plot(boot, variable = "ER")
#'
#'
#'
#' @examplesIf requireNamespace("magrittr", quietly = TRUE)
#' library(magrittr)
#'
#'    imf_example_data %>% kardl( CPI ~ PPI + asym(ER) +trend, maxlag=2) %>%
#'    bootstrap(replications=5) %>% plot(variable = "ER")
#'
#'
bootstrap<-function(kmodel,
                    horizon=80,
                    replications=100,
                    level=95,
                    minProb=0,
                    seed = NULL){

  if(! inherits(kmodel, "kardl_lm")){

    stop("Input object must be of class 'kardl_lm'.",call. = FALSE)
  }
  if (!is.null(seed)) {
    set.seed(seed)
  }

  My_mp<-mplier(kmodel,horizon, minProb)
  mpsi<-as.data.frame(My_mp[["mpsi"]])
  vars<-kmodel$extractedInfo

  if(length( vars$ASvars)>0){
    b0 <- coef(kmodel )
    tmin <- kmodel$estInfo$start
    tmax <-OBS <- kmodel$estInfo$end
    OptLag<-kmodel$lagInfo$OptLag
    nardlres<-residuals(kmodel)
    NewDiff<-matrix(NA,nrow = 1,ncol = length(vars$ASvars)+1)
    colnames(NewDiff)<-c("h",  vars$ASvars )
    phi <-  My_mp$omega
    phi.length<-length(phi)

    p<-OptLag[[vars$dependentVar]]+1

    q<-max(OptLag[names(OptLag)!= vars$dependentVar])+2
    # q<-max(OptLag[-1])+2
    theta<-My_mp$lambda[1:q,]


    # excluding symmetric independent vars
    symPosNames<- unlist(lapply(vars$indepASexcluded ,function(x){paste0(.kardl_Settings_env$AsymPrefix[1],x,.kardl_Settings_env$AsymSuffix[1])}) )
    symNegNames<- unlist(lapply(vars$indepASexcluded ,function(x){paste0(.kardl_Settings_env$AsymPrefix[2],x,.kardl_Settings_env$AsymSuffix[2])}) )

     if(!is.null(symPosNames)){
      theta<-theta[, colnames(theta) != symPosNames]
      colnames(theta)[colnames(theta) %in% symNegNames]<-vars$indepASexcluded
    }

    ## names of the indep. variables that matter for multipliers
    inddColnames <- colnames(theta)

    ## build the column list; add deterministic terms only if they exist
    reqCols <- c(vars$dependentVar,
                 inddColnames,
                 if (!is.null(vars$deterministic)) vars$deterministic)

    vars$data <- as.data.frame(vars$data)

    ## --- NEW: make sure every requested column exists --------------------
    missingCols <- setdiff(reqCols, colnames(vars$data))
    if (length(missingCols)) {
      vars$data[ , missingCols] <- 0          # fill with zeros
    }
    ## --------------------------------------------------------------------

    ## subset the data safely
    my_newData <- as.data.frame(vars$data)[ , reqCols, drop = FALSE]


    # Addin dif suffix to asymmetric short-run var names


    # Replace leading NA values with 0 for each column
    for(col in colnames(my_newData)) {
      first_non_na_index <- which(!is.na(my_newData[,col]))[1] # Find the index of first non-NA value
      if (!is.na(first_non_na_index) && first_non_na_index>1) { # Check if non-NA value exists
        my_newData[1:(first_non_na_index - 1), col] <- 0 # Replace leading NA values with 0
      }
    }



    newFormula <- kmodel$argsInfo$formula
    newDepData<- my_newData[,vars$dependentVar] #my_newData[,1]

    for (r in 1:replications) {
        for (t in tmin:tmax) {
          newy1<-sum(phi*newDepData[(t-1):(t-phi.length)]) # calculating dependent
          newy2<- sum(theta*my_newData[(t-0):(t-q+1),inddColnames]) #calculating independent vars
          newy3<-sum(my_newData[t,vars$deterministic]*b0[vars$deterministic]) #calculating deterministic vars
          newy <- newy1+ newy2+newy3+ b0[[1]] # adding constant
          newDepData[t]  <-   newy  +   nardlres[sample(seq_len(NROW(nardlres)), 1)] #adding random residual
        }
        if(r==1){
          BSnewdata<-cbind(newDepData,vars$data) # adding to original data
          colnames(BSnewdata)<-c("BStmp",colnames(vars$data))
          tempFormula<-BStmp~x+ff
          newFormula[[2]]<-tempFormula[[2]]
          names(OptLag)[1]<-"BStmp"
          OptLag<-as.double(OptLag)
        }else{
          BSnewdata[,"BStmp"]<-newDepData
        }

        My_model<-kardl(BSnewdata,newFormula,mode=OptLag)
        My_mpNew<- mplier(My_model,horizon ,minProb)
        NewDiff<-rbind(NewDiff, cbind(horizon=0:horizon, val=My_mpNew$mpsi[,paste0( vars$ASvars,"_dif")]))
    }

    NewDiff<-as.data.frame( NewDiff[-1,])
    rownames(NewDiff)<-seq_len(nrow(NewDiff))

    lowerZ <- 0.5 * (1 - level / 100)
    rZ<-replications * lowerZ
    lowerIndex1 <- ceiling(rZ)
    lowerIndex2 <- ifelse(lowerIndex1 - rZ != 0,lowerIndex1,rZ + 1)

    medianZ<-0.5
    rZ<-replications * medianZ
    medianIndex1 <- ceiling(rZ)
    medianIndex2 <- ifelse(medianIndex1 - rZ != 0,medianIndex1,rZ + 1)

    upperZ<-1-lowerZ
    rZ<-replications * upperZ
    upperIndex1 <- ceiling(rZ)
    upperIndex2 <- ifelse(upperIndex1 - rZ != 0,upperIndex1,rZ + 1)

    for (variable in vars$ASvars) {
      lowerCI<-medianCI<-upperCI<-c()
      for (jh in 0:horizon) {
        nardlbs_sub <- NewDiff[NewDiff$h==jh,]
        nardlbs_sub <- nardlbs_sub[order(nardlbs_sub[,variable]), variable]
        # nardlbs_sub <- unlist( nardlbs_sub[2])
        lowerCI[jh+1]<-(nardlbs_sub[lowerIndex1]+nardlbs_sub[lowerIndex2])/2
        medianCI[jh+1]<-(nardlbs_sub[medianIndex1]+nardlbs_sub[medianIndex2])/2
        upperCI[jh+1]<-(nardlbs_sub[upperIndex1]+nardlbs_sub[upperIndex2])/2
      }
      dynNames<-colnames(mpsi)
      mpsi<-cbind(mpsi,upperCI,lowerCI)
      colnames(mpsi)<-c(dynNames,paste0(variable,c("_CI_upper","_CI_lower")))

    }
  }


  my_output<-  list(
    mpsi=mpsi,
    level=level,
    horizon=horizon,
    vars=vars,
    replications=replications,
    type="bootstrap"
  )
  class(my_output)<-c("kardl_boot","kardl_mplier")
  my_output

}

#' Plot Dynamic Multipliers with ggplot2
#'
#' This function creates a ggplot visualization of the dynamic multipliers for a specified variable from a model estimated using the \code{kardl} package. The plot includes lines for positive and negative changes, as well as confidence intervals if available. The function allows for customization of colors, line types, and legend formatting to enhance the interpretability of the results.
#' @param mpsi A data frame containing the dynamic multipliers and confidence intervals for the specified variable. This data frame is typically generated by the \code{bootstrap} function.
#' @param vars A list of variable information extracted from the model, including dependent variable,
#' independent variables, asymmetric variables, and deterministic terms. This information is used to label the plot appropriately.
#' @param varName A character string specifying the name of the variable for which the dynamic
#' multipliers are being plotted. This variable should be one of the independent variables in the model, and it is used to construct the plot labels and legend.
#' @return A ggplot object visualizing the dynamic multipliers for the specified variable,
#' including lines for positive and negative changes, as well as confidence intervals if available. The plot is formatted with appropriate labels, colors, and legend to enhance interpretability.
#' @import ggplot2
#' @noRd

mplierggplot<-function(mpsi,vars, varName  ){
  x_p <- paste0(.kardl_Settings_env$AsymPrefix[1], varName, .kardl_Settings_env$AsymSuffix[1]) # paste0(varName,".POS")
  x_n <- paste0(.kardl_Settings_env$AsymPrefix[2], varName, .kardl_Settings_env$AsymSuffix[2]) # paste0(varName,".NEG")

  x_diff<-paste0(varName,"_dif")
  upperCI<-paste0(varName,"_CI_upper")
  lowerCI<-paste0(varName,"_CI_lower")

  p<- ggplot(mpsi, aes(x=.data$h)) +
    geom_line(aes(y = .data[[x_p]], color = "Positive")) +
    geom_line(aes(y = .data[[x_n]], color = "Negative"))
    # geom_line(aes(y=!!rlang::sym(x_p),color="Positive")) +
    # geom_line(aes(y=!!rlang::sym(x_n),color="Negative"))


  myBreaks<- c("Difference", "Positive", "Negative")
  labels <- c("Difference" = "Difference",
    "Positive" = "Positive Change",
    "Negative" = "Negative Change")
  values <- c( # Specify colors for each legend item
    "Difference" = alpha("black",1),
    "Positive" =  alpha("blue",1),
    "Negative" =  alpha("red",1) )
  color<-c("black","blue", "red")
  fill<-c("transparent", "transparent","transparent")
  linetype <-c(2,1,1) # c("dashed", "solid","solid","solid") ,
  size<-c(8,8,8)  # Increases the width of the legend key

  if(upperCI  %in% colnames(mpsi) ){ # !is.null(mpsi[[upperCI]])){

    # p <- p+    geom_line(aes(y=!!rlang::sym(x_diff),color="Difference"),linetype = "dashed") +
    #   geom_ribbon(aes(ymin = !!rlang::sym(lowerCI), ymax = !!rlang::sym(upperCI),color="CI") ,fill=  "blue"  ,   alpha = 0.2 )
     p <- p+
       # geom_line(aes(y = .data[[x_diff]], color = "Difference"), linetype = "dashed") +
      geom_ribbon(aes(ymin = .data[[lowerCI]], ymax = .data[[upperCI]], color = "CI"), fill = "blue", alpha = 0.2)

    myBreaks<- c(myBreaks, "CI")
    labels["Difference"]<- "Asymmetry"
    labels <- c( labels  , "CI" = "Confidence Interval")
    values <- c(  values  ,       "CI" =  alpha("blue",0.2) )
    color<-c(color,"transparent")
    fill<-c(fill,"blue")
    linetype <-c(linetype,0)
    size<-c( size,8)
  }

  p <- p+    geom_line(aes(y = .data[[x_diff]], color = "Difference"), linetype = "dashed") +
    scale_colour_manual(name="",
                                  breaks =myBreaks,  # Include "Confidence Interval" in breaks
                                  labels = labels,  # Include label for "Confidence Interval"
                                  values = values


  ) +  # Use the same color for "Confidence Interval" as the fill color
    #scale_fill_manual(name="",values=  "#84CA72"                   ,labels="CI")+
    scale_x_continuous(limits = c(0,nrow(mpsi)-1),  expand = c(0, 0))+
    labs(x = "time",y="") + ggtitle(paste0("Cumulative effect of ",varName," on ",vars$dependentVar))+
    guides( color = guide_legend(
      override.aes = list(
        color = color,
        fill  = fill,
        linetype =linetype, # c("dashed", "solid","solid","solid") ,

        size=size  # Increases the width of the legend key
      ) )
    )+  theme_bw()  +
    theme( legend.key.width = unit(0, "cm") ,
           legend.position = "bottom"  ,
           legend.margin = margin(t = 0, unit='cm')

    )

  p
}

#' @srrstats {G5.9} The \code{bootstrap()} function tests for expected stochastic behaviour: confidence intervals are stable under both trivial noise perturbations (G5.9a) and different random seeds (G5.9b), as verified in \code{tests/testthat/test-bootstrap-noise.R}.
#' @srrstats {G5.9a} Bootstrap confidence intervals are stable under trivial noise perturbations: adding machine-epsilon noise to the data should not meaningfully change point estimates or confidence interval bounds.
#' @srrstats {G5.9b} Bootstrap results are stable across different random seeds: point estimates should be identical, and confidence interval widths should not differ by more than 10% of the signal range.

test_that("G5.9b: bootstrap results are stable across different seeds", {
  fit_base <- kardl(imf_example_data, CPI ~ asymmetric(ER),
                    mode = c(1, 1, 1))
  b1 <- bootstrap(fit_base, horizon = 10, replications = 10, seed = 1L)
  b2 <- bootstrap(fit_base, horizon = 10, replications = 10, seed = 99L)

  # Point estimates come from mplier() which is deterministic — must be identical
  expect_equal(b1$mpsi[["ER_dif"]], b2$mpsi[["ER_dif"]])

  # CI width range-to-range: variation between seeds should be small
  # relative to the range of CI widths themselves
  ci_width_1   <- b1$mpsi[["ER_CI_upper"]] - b1$mpsi[["ER_CI_lower"]]
  ci_width_2   <- b2$mpsi[["ER_CI_upper"]] - b2$mpsi[["ER_CI_lower"]]

  diff_of_widths  <- abs(ci_width_1 - ci_width_2)          # seed-to-seed diff
  range_of_widths <- diff(range(c(ci_width_1, ci_width_2))) # combined range

  expect_true(
    max(diff_of_widths) < 0.25 * max(range_of_widths, 1e-8),
    label = paste0(
      "CI widths differ too much between seeds. ",
      "Max seed-to-seed diff: ", round(max(diff_of_widths), 6),
      ", 25% of CI width range: ", round(0.25 * range_of_widths, 6)
    )
  )
})

#' @srrstats {G5.9} The \code{bootstrap()} function tests for expected stochastic behaviour: confidence intervals are stable under both trivial noise perturbations (G5.9a) and different random seeds (G5.9b), as verified in \code{tests/testthat/test-bootstrap-noise.R}.
#' @srrstats {G5.9a} Bootstrap confidence intervals are stable under trivial noise perturbations: adding machine-epsilon noise to the data should not meaningfully change point estimates or confidence interval bounds.
#' @srrstats {G5.9b} Bootstrap results are stable across different random seeds: point estimates should be identical, and confidence interval widths should not differ by more than 10% of the signal range.

test_that("G5.9b: same seed yields identical bootstrap output", {
  fit_base <- kardl(imf_example_data, CPI ~ asymmetric(ER),
                    mode = c(1, 1, 1))
  b_a <- bootstrap(fit_base, horizon = 10, replications = 10, seed = 42L)
  b_b <- bootstrap(fit_base, horizon = 10, replications = 10, seed = 42L)

  expect_equal(b_a$mpsi, b_b$mpsi)
})

#' @srrstats {G5.9a} Bootstrap confidence intervals are stable under trivial noise perturbations: adding machine-epsilon noise to the data should not meaningfully change point estimates
#' or confidence interval bounds.

test_that("G5.9a: adding machine-epsilon noise to data does not change results", {

  fit_base <- kardl(imf_example_data, CPI ~ asymmetric(ER), mode = c(1, 1, 1))

  data_noisy <- imf_example_data
  numeric_cols <- sapply(data_noisy, is.numeric)

  # Scale noise relative to each column's magnitude, not absolute eps.
  # This ensures the perturbation stays at machine-epsilon *relative* scale
  # even after cumulative summation inside kardl's asymmetric decomposition.
  col_scales <- sapply(data_noisy[, numeric_cols, drop = FALSE],
                       function(x) mean(abs(x), na.rm = TRUE))
  col_scales <- pmax(col_scales, 1)   # floor at 1 to avoid zero scale

  noise_matrix <- matrix(
    rnorm(sum(numeric_cols) * nrow(data_noisy), mean = 0, sd = .Machine$double.eps),
    nrow = nrow(data_noisy)
  )
  noise_matrix <- sweep(noise_matrix, 2, col_scales, `*`)

  data_noisy[, numeric_cols] <- data_noisy[, numeric_cols] + noise_matrix

  fit_noisy <- kardl(data_noisy, CPI ~ asymmetric(ER), mode = c(1, 1, 1))

  b_clean <- bootstrap(fit_base,  horizon = 10, replications = 10, seed = 7L)
  b_noisy <- bootstrap(fit_noisy, horizon = 10, replications = 10, seed = 7L)

  # Multiplier point estimates should be virtually identical
  expect_equal(
    b_clean$mpsi[["ER_dif"]],
    b_noisy$mpsi[["ER_dif"]],
    tolerance = 1e-4,
    label = "ER_dif multipliers changed meaningfully under trivial noise"
  )

  # CI bounds should also be negligibly different
  expect_equal(
    b_clean$mpsi[["ER_CI_upper"]],
    b_noisy$mpsi[["ER_CI_upper"]],
    tolerance = 1e-3,
    label = "Upper CI changed meaningfully under trivial noise"
  )
})

#' srr_stats
#'
#' Package-specific compliance statements for the srr statistical software standards.
#'
#' @srrstatsVerbose TRUE
#'
#' @srrstats {G1.0} The package documentation and manuscript cite the ARDL bounds-testing and nonlinear ARDL literature on which the implemented estimators, asymmetry decomposition, bounds tests, and dynamic multipliers are based.
#' @srrstats {G1.1} Statistical Software should document whether the algorithm(s) it implements are: the first implementation of a novel algorithm; or the first implementation within R of an algorithm which has previously been implemented in other languages or contexts; or an improvement on other implementations of similar algorithms in R.
#'
#' The **kardl** package implements several methodological innovations and extensions
#' that are not available in other R packages for ARDL/NARDL modelling:
#'
#' * **differentAsymLag** option: Allows different lag orders for the positive and negative
#'   partial sum decompositions in NARDL models. This provides greater flexibility
#'   compared to standard symmetric lag restrictions in existing packages.
#'
#' * **Narayan test** (`narayan()`): Small-sample cointegration test optimised for
#'   cases 2–5, following Narayan (2005). While the test statistic itself exists in
#'   the literature, its dedicated, user-friendly implementation with automatic
#'   critical value handling for small samples is unique within the R ecosystem
#'   for ARDL/NARDL workflows.
#'
#' * **Symmetry test** (`symmetrytest()`): Dedicated test for long-run and short-run
#'   symmetry in NARDL models.
#'
#' * **Bootstrap confidence intervals for dynamic multipliers** (`bootstrap()` and
#'   associated `plot.kardl_mplier()` methods): Provides robust uncertainty quantification
#'   around dynamic multipliers through resampling. While bootstrap methods exist in
#'   the broader econometric literature, this integrated implementation within a
#'   complete ARDL/NARDL workflow (including asymmetric multipliers) represents
#'   a novel and practical contribution in R.
#'
#' * Additional unique features include flexible formula parsing, multiple model
#'   selection criteria (AIC, BIC, AICc, HQ, and user-defined), and comprehensive
#'   post-estimation tools tailored specifically for asymmetric ARDL analysis.
#'
#' These extensions go beyond the standard Shin et al. (2014) NARDL framework
#' and improve upon existing R implementations (e.g., `nardl`, `ardl.nardl`,
#' `dynamac`) by offering greater customisation and additional diagnostic/testing
#' capabilities.
#'
#' **Prior Art / Comparable Implementations**
#' - Shin, Y., Yu, B., & Greenwood-Nimmo, M. (2014). Modelling Asymmetric Cointegration
#'   and Dynamic Multipliers in a Nonlinear ARDL Framework.
#' - Narayan, P. K. (2005). The saving and investment nexus for China: evidence from
#'   cointegration tests.
#' - Other R packages: `nardl`, `ardl.nardl`, `dynamac`, and more recent packages
#'   such as `ardlverse` and `bootCT` (which focus on bootstrap bounds testing
#'   rather than multiplier uncertainty in NARDL).
#'
#' Users are encouraged to compare results with these packages where overlapping
#' functionality exists.
#' @srrstats {G1.2} The README, NEWS file, CONTRIBUTING.md, and package website describe the current development state, recent releases, and intended future maintenance of the package.
#' @srrstats {G1.3} Statistical terms such as ARDL, NARDL, ECM, bounds testing, short-run asymmetry, long-run asymmetry, and dynamic multipliers are defined in function documentation and vignettes.
#' @srrstats {G1.4} User-facing functions and S3 methods are documented with `roxygen2`, and the generated Rd files are kept under version control through the package documentation workflow.
#' @srrstats {G1.4a} Internal helper functions are documented with roxygen comments where clarification is needed and are hidden from the public help index with `@noRd` or marked as internal.
#' @srrstats {G1.5} Examples, tests, and vignettes include executable code using package data so that users can reproduce the estimation and multiplier outputs reported in the documentation.
#' @srrstats {G1.6} The related-software discussion compares the package with alternative ARDL and NARDL implementations in other software environments and explains the reproducibility advantages of the R implementation.
#' @srrstats {G2.0} Main modelling functions validate the expected scalar and vector inputs, including model formulae, lag choices, criteria, horizon values, and object classes.
#' @srrstats {G2.0a} Function documentation explains length expectations for important arguments such as formula input, lag structures, selected criteria, and dynamic multiplier horizons.
#' @srrstats {G2.1} The package checks that model input is supplied in the expected data and formula form and that S3 methods receive compatible `kardl` model objects.
#' @srrstats {G2.1a} Documentation describes the expected data structure for variables used in time-series regression and clarifies that model variables should be numeric where estimation requires numeric series.
#' @srrstats {G2.2} Model specifications restrict dependent-variable input to a single response variable and process independent variables through the formula parser.
#' @srrstats {G2.3} Character-valued options are limited to recognised choices through package option handlers and argument matching routines.
#' @srrstats {G2.3a} Selection arguments such as information criteria and asymmetry options are matched against predefined acceptable values.
#' @srrstats {G2.3b} Formula terms and asymmetry constructors are processed consistently so that documented model syntax is interpreted as intended.
#' @srrstats {G2.4} Data and formula-processing routines standardise model inputs before estimation so that subsequent estimation functions work with a consistent internal representation.
#' @srrstats {G2.4a} Integer-valued quantities such as lags and horizons are treated as discrete counts before they are used to construct lagged regressors or multiplier paths.
#' @srrstats {G2.4b} Numeric model variables are passed to estimation routines as numeric vectors or matrix columns suitable for ordinary least squares and post-estimation algebra.
#' @srrstats {G2.4c} Variable names and parsed formula terms are handled as character vectors when constructing lagged-variable names and asymmetric components.
#' @srrstats {G2.4d} The package does not require factor-valued model variables for ARDL estimation, and categorical effects should be supplied through suitable deterministic or dummy variables.
#' @srrstats {G2.4e} Factor inputs are not part of the core estimation algorithm; users are expected to provide numeric series or explicit dummy variables when categorical controls are required.
#' @srrstats {G2.5} The modelling interface is designed for numeric time-series regressors rather than ordered or unordered factor responses.
#' @srrstats {G2.6} One-dimensional variables selected through the formula are extracted and aligned through the model-preparation routines before lagged terms are generated.
#' @srrstats {G2.7} The modelling functions operate on standard tabular data supplied by users, including data frames containing the time-series variables referenced in the formula.
#' @srrstats {G2.8} Initial preprocessing converts the user formula and data into a common internal structure containing dependent variables, independent variables, lag information, and generated regressors.
#' @srrstats {G2.9} The package constructs generated variable names for lagged, differenced, and asymmetric components in a transparent way so users can inspect the resulting model object.
#' @srrstats {G2.10} Variables are selected by name from the supplied data through formula parsing rather than by position-dependent assumptions.
#' @srrstats {G2.11} The package tests standard data-frame inputs used in examples and vignettes and processes columns required by the model formula.
#' @srrstats {G2.12} The package is intended for numeric time-series columns and does not treat list columns as valid model variables for estimation.
#' @srrstats {G2.13} Missing observations introduced by lagging and differencing are handled during model-frame construction before estimation is performed.
#' @srrstats {G2.14} Missing data behaviour follows the documented estimation workflow, where observations unavailable after lag construction are excluded from the estimation sample.
#' @srrstats {G2.14a} Invalid model inputs that cannot be transformed into an estimable ARDL specification generate errors before post-estimation routines are run.
#' @srrstats {G2.14b} The estimation workflow removes structurally missing lagged observations created by differencing and lag construction.
#' @srrstats {G2.14c} The package does not impute missing time-series observations because imputation would change the dynamic structure of the fitted ARDL model.
#' @srrstats {G2.15} Estimation and post-estimation calculations are based on the cleaned model object rather than passing unprocessed missing values to lower-level numeric routines.
#' @srrstats {G2.16} Non-finite values are not treated as valid observations for the fitted regression sample and should be removed or corrected before estimation.
#' @srrstats {G3.0} Numerical comparisons in tests and post-estimation checks use tolerance-based expectations where exact floating-point equality would be inappropriate.
#' @srrstats {G3.1} Long-run multiplier inference uses the variance-covariance matrix of the fitted model object, allowing users to work with the covariance structure attached to the estimation result.
#' @srrstats {G3.1a} Multiplier documentation describes standard-error calculation from the fitted model covariance matrix and examples show how users can inspect the resulting estimates.
#' @srrstats {G5.0} Tests and examples use the package example data and fixed model specifications with known expected structure.
#' @srrstats {G5.1} The example data used in documentation and tests are included with the package so users can reproduce examples and test calculations.
#' @srrstats {G5.2} The test suite exercises representative successful calls and selected error conditions for model estimation, multiplier calculation, and S3 methods.
#' @srrstats {G5.2a} Diagnostic messages are written to identify the specific invalid input or unsupported object class encountered by the user.
#' @srrstats {G5.2b} Tests include error expectations for incompatible object classes and invalid post-estimation inputs.
#' @srrstats {G5.3} Tests verify that key returned objects, including fitted model components and multiplier matrices, have the expected dimensions, names, and classes.
#' @srrstats {G5.4} Correctness tests compare selected outputs against expected structural properties of ARDL and NARDL models, including lag information, long-run coefficients, and multiplier matrices.
#' @srrstats {G5.4a} Where exact external reference values are not available, tests use simple deterministic cases and expected object properties to verify implementation behaviour.
#' @srrstats {G5.4b} Validation work compares package results with established ARDL and NARDL software workflows where comparable outputs are available.
#' @srrstats {G5.4c} Published ARDL and NARDL formulae provide the basis for the implemented coefficient transformations and dynamic multiplier recursion.
#' @srrstats {G5.5} Tests involving simulated or random inputs set a random seed before execution.
#' @srrstats {G5.6} Parameter-recovery checks are implemented through deterministic model examples and comparisons of estimated quantities with expected algebraic relationships.
#' @srrstats {G5.6a} Numeric tests use tolerances where floating-point arithmetic or regression estimation makes exact equality inappropriate.
#' @srrstats {G5.6b} The core estimation functions are deterministic for a fixed data set and model specification, so repeated seeds are not required for standard tests.
#' @srrstats {G5.7} Tests examine behaviour across different model specifications, including linear, short-run asymmetric, long-run asymmetric, and fully asymmetric cases.
#' @srrstats {G5.8} Edge-condition tests check that invalid object classes and unsupported inputs produce clear errors rather than silent failures.
#' @srrstats {G5.8a} Zero-length or insufficient model inputs are rejected through the model-preparation and estimation checks.
#' @srrstats {G5.8b} Unsupported non-numeric model variables are not valid inputs for the regression calculations and are expected to fail before estimation.
#' @srrstats {G5.8c} Degenerate input cases are handled through the underlying regression and model-validation workflow.
#' @srrstats {G5.8d} Model specifications outside the estimable sample size or lag structure are rejected by estimation and lag-selection routines.
#' @srrstats {G5.9} The \code{bootstrap()} function tests for expected stochastic behaviour: confidence intervals are stable under both trivial noise perturbations (G5.9a) and different random seeds (G5.9b), as verified in \code{tests/testthat/test-bootstrap-noise.R}.
#' @srrstats {G5.9a} Adding noise at the scale of \code{.Machine$double.eps} to input data does not meaningfully change the dynamic multiplier estimates or bootstrap confidence intervals produced by \code{bootstrap()}.
#' @srrstats {G5.9b} The \code{bootstrap()} function accepts a \code{seed} argument passed to \code{set.seed()} before resampling; running under different random seeds does not meaningfully change results, and the same seed reproduces identical output, as tested in \code{tests/testthat/test-bootstrap-noise.R}.
#' @srrstats {TS1.0} The package accepts time-ordered regression data and constructs lagged and differenced variables internally for ARDL and NARDL estimation.
#' @srrstats {TS1.1} Function documentation and vignettes describe the accepted input form for time-series variables used in `kardl` model formulae.
#' @srrstats {TS1.2} Model-preparation routines check that the supplied variables can be used to construct the lagged and differenced terms required by the specification.
#' @srrstats {TS1.3} The package uses a central preparation workflow to parse formulae, construct lagged variables, and return a uniform internal model data structure.
#' @srrstats {TS1.4} Observational order is preserved during lag and difference construction so that the dynamic structure corresponds to the input time ordering.
#' @srrstats {TS1.5} Lag construction assumes that rows are supplied in the intended chronological order, and examples document this ordering requirement.
#' @srrstats {TS1.6} Users are expected to sort data before estimation; violations of chronological ordering would change the constructed lags and therefore the estimated dynamic model.
#' @srrstats {TS1.8} The package works with ordered observations rather than calendar arithmetic, so period interpretation is determined by the frequency and ordering of the user-supplied data.
#' @srrstats {TS2.0} ARDL estimation assumes a regular ordered sequence after preprocessing; missing observations created by lagging are handled explicitly in the model sample.
#' @srrstats {TS2.1} Missing-data handling is documented as part of model preparation and estimation rather than as a separate imputation system.
#' @srrstats {TS2.1a} Inputs that cannot form an estimable model after missing-value handling generate errors through the estimation workflow.
#' @srrstats {TS2.1b} Structurally missing observations caused by lagging and differencing are removed from the fitted sample.
#' @srrstats {TS2.1c} The package does not impute missing time-series observations because that would impose additional assumptions not inherent in ARDL estimation.
#' @srrstats {TS2.2} The documentation discusses stationarity requirements in the context of ARDL bounds testing and the usual I(0) and I(1) integration-order framework.
#' @srrstats {TS2.3} Vignettes and function documentation explain that ARDL bounds testing is intended for variables integrated of order zero or one, not for I(2) processes.
#' @srrstats {TS2.4} Users are expected to assess integration order before applying bounds-test interpretations, and the documentation states these modelling assumptions.
#' @srrstats {TS2.4a} The package reports model diagnostics and test results to help users assess the fitted dynamic specification.
#' @srrstats {TS2.4b} Users can transform variables before modelling, including differencing, logs, or asymmetric partial sums when appropriate for the research design.
#' @srrstats {TS2.5} The package does not compute autocovariance matrices as primary outputs; lag order and model sample information are stored in the fitted object.
#' @srrstats {TS2.6} The package does not compute autocovariance matrices with physical units; time interpretation follows the user's ordered data.
#' @srrstats {TS4.0} Fitted models and post-estimation results are returned as classed objects with dedicated print, summary, plot, and extraction methods.
#' @srrstats {TS4.0a} The package returns model-specific objects rather than converting outputs back to the original data class.
#' @srrstats {TS4.0b} Return values use explicit classes such as `kardl_lm`, `kardl_mplier`, and related summary classes.
#' @srrstats {TS4.1} Units are not inferred or altered by the package; coefficient and multiplier interpretation follows the units of the user-supplied variables.
#' @srrstats {TS4.2} Documentation describes the list components and classes returned by fitted model, long-run, test, and multiplier functions.
#' @srrstats {TS4.3} Dynamic multiplier outputs include the horizon index so that multiplier paths can be interpreted over the modelled time steps.
#' @srrstats {TS4.4} The package does not automatically transform forecast data; any transformations applied by the user remain part of the model interpretation.
#' @srrstats {TS4.5} The package documents coefficient and multiplier interpretation rather than providing automatic back-transformation of user-transformed data.
#' @srrstats {TS4.5a} Users who transform variables before estimation are responsible for interpreting or back-transforming results according to their research design.
#' @srrstats {TS4.5b} Examples show estimation and multiplier calculation on the scale of the variables supplied to the model.
#' @srrstats {TS4.5c} Limitations of interpreting transformed variables are discussed through the model documentation and vignettes.
#' @srrstats {TS5.0} The package implements plot methods for classed outputs, including long-run results and dynamic multiplier objects.
#' @srrstats {TS5.1} Dynamic multiplier plots label the horizontal axis as the horizon or time-step index.
#' @srrstats {TS5.2} The horizon or time-step variable is plotted on the horizontal axis in dynamic multiplier visualisations.
#' @srrstats {TS5.3} Multiplier plots display the horizon index used in the model output, with interpretation tied to the data frequency supplied by the user.
#' @srrstats {TS5.5} Plot methods operate on computed model and multiplier outputs after missing-value handling in the fitted object.
#' @srrstats {TS5.7} Dynamic multiplier plots display the computed response path from the fitted model over the selected horizon.
#' @srrstats {TS5.8} Plot methods distinguish multiplier components by variable and shock direction where asymmetric effects are present.
#' @noRd
NULL

#' NA_standards
#'
#' Non-applicable srr standards for kardl.
#'
#' @srrstatsNA {G4.0} The package does not write statistical outputs to local files; users receive R objects that can be saved with standard R tools if needed.
#' @srrstatsNA {G5.10} The package test suite does not require long-running extended tests beyond the standard `testthat` workflow.
#' @srrstatsNA {G5.11} The package tests use included data and do not require external large data downloads.
#' @srrstatsNA {G5.11a} The package does not download external test data, so failed downloads are not part of the testing workflow.
#' @srrstatsNA {G5.12} No special platform, memory, runtime, or manual-inspection requirements are needed for extended tests.
#' @srrstatsNA {TS1.7} The package does not use `units` objects; time-series interpretation is based on ordered observations and user-supplied variable scales.
#' @srrstatsNA {TS3.0} The package estimates ARDL and NARDL models and computes dynamic multipliers; it does not implement out-of-sample forecast intervals.
#' @srrstatsNA {TS3.1} Forecast-error widening tests are not applicable because the package does not produce forecast distributions.
#' @srrstatsNA {TS3.2} Forecast-error drivers are not documented because forecasting is outside the package scope.
#' @srrstatsNA {TS3.3} Forecast trimming is not applicable because the package does not generate forecasts with error margins.
#' @srrstatsNA {TS3.3a} The package does not provide forecast-error examples or trimming procedures.
#' @srrstatsNA {TS3.3b} The package does not include explicit forecast trimming because it does not implement forecasting.
#' @srrstatsNA {TS4.6} The package does not return forecast distributions or prediction intervals.
#' @srrstatsNA {TS4.6a} Distribution objects are not applicable because the package does not implement probabilistic forecasting.
#' @srrstatsNA {TS4.6b} Forecast means and forecast standard errors are not returned because forecasting is outside the package scope.
#' @srrstatsNA {TS4.6c} General forecast-error measures are not returned because the package is focused on estimation and post-estimation multipliers.
#' @srrstatsNA {TS4.7} Distinguishing fitted and forecast values is not applicable because the package does not return forecast objects.
#' @srrstatsNA {TS4.7a} Forecast-only outputs are not produced by the package.
#' @srrstatsNA {TS4.7b} Separate model and forecast output lists are not produced by the package.
#' @srrstatsNA {TS4.7c} Combined observed and forecast data objects are not produced by the package.
#' @srrstatsNA {TS5.4} The package does not implement frequency-domain or spectral visualisation, so frequency-axis conventions are not applicable.
#' @srrstatsNA {TS5.6} Forecast distributional limits are not plotted because the package does not implement forecast plots.
#' @noRd
NULL

---
title: "Introduction to kardl"
author: "Huseyin karamelikli and Huseyin Utku Demir"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
 %\VignetteIndexEntry{Introduction to kardl}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---



```{r setup, include=FALSE}
knitr::opts_chunk$set(
  echo = TRUE, message = FALSE, warning = FALSE, fig.width = 16,
  fig.height = 8,
  out.width = "100%",
  dpi = 150
)
```

## Introduction

The `kardl` package is an R tool for estimating symmetric and asymmetric Autoregressive Distributed Lag (ARDL) and Nonlinear ARDL (NARDL) models, designed for econometricians and researchers analyzing cointegration and dynamic relationships in time series data. It offers flexible model specifications, allowing users to include deterministic variables, asymmetric effects for short- and long-run dynamics, and trend components. The package supports customizable lag structures, model selection criteria (AIC, BIC, AICc, HQ), and parallel processing for computational efficiency. Key features include:

- **Flexible Formula Specification**: Use `asymmetric()`, `lasymmetric()`, and `sasymmetric()` to model asymmetric effects in short- and long-run dynamics, and `deterministic()` for dummy variables.
- **Lag Optimization**: Choose between automatic lag selection (`"quick"`, `"grid"`, `"grid_custom"`) or user-defined lags.
- **Dynamic Analysis**: Compute long-run coefficients, perform cointegration tests (PSS F, PSS t, Narayan), and ECM estimation.


This vignette demonstrates how to use the `kardl()` function to estimate an asymmetric ARDL model, perform diagnostic tests, and visualize results, using economic data from Turkey.

```{r srr-tags, eval = FALSE, echo = FALSE}
#' srr tags for the kardl package
#'
#' This package follows structured statistical reporting rules using the
#' \code{srrstats} framework. The standards governing implementation and
#' documentation are summarized below.
#'
#' @srrstats {G1.0} The package documentation and accompanying materials cite
#'           the ARDL bounds-testing and nonlinear ARDL literature on which the
#'           implemented estimators, asymmetry decomposition, bounds tests, and
#'           dynamic multipliers are based.
#' @srrstats {G1.1} The package is described as an implementation and extension
#'           of ARDL and NARDL workflows, supporting mixed symmetric and
#'           asymmetric regressors, flexible lag selection, and dynamic
#'           multiplier methods.
#' @srrstats {G1.2} The README, NEWS file, and package website describe the
#'           development status, recent changes, and future maintenance plans
#'           of the package.
#' @srrstats {G1.3} Key statistical concepts such as ARDL, NARDL, ECM, bounds
#'           testing, short-run asymmetry, long-run asymmetry, and dynamic
#'           multipliers are defined in function documentation and vignettes.
#' @srrstats {G1.4} All user-facing functions and S3 methods are documented
#'           using \code{roxygen2}, and the generated Rd files are maintained
#'           as part of the package documentation workflow.
#' @srrstats {G1.4a} Internal helper functions are documented where necessary
#'           and are excluded from the public help index using \code{@noRd}
#'           or marked as internal.
```


## Installation


`kardl` in R can easily be installed from its CRAN repository:

```{r install-cran, eval=FALSE}
install.packages("kardl")
library(kardl)
```

Alternatively, you can use the `devtools` package to load directly from GitHub:

```{r install, eval=FALSE}
# Install required packages
install.packages(c(
  "stats", "msm", "lmtest", "nlWaldTest", "car", "strucchange",
  "utils", "ggplot2"
))
# Install kardl from GitHub
install.packages("devtools")
devtools::install_github("karamelikli/kardl")
```

Load the package:

```{r load}
library(kardl)
```

## Unique Features and Methodological Contributions

The `kardl` package implements several methodological extensions and improvements for ARDL/NARDL modelling that go beyond standard implementations available in R and other software:

- **different_asym_lag** option: Enables different lag orders for the positive and negative partial sum decompositions in NARDL models. This provides greater flexibility than the common symmetric lag restriction used in most existing packages.
- **Narayan cointegration test** (`narayan()`): A dedicated small-sample bounds test (Narayan, 2005) with automatic handling of critical values for cases II–V. While the test exists in the literature, its seamless integration into a full ARDL/NARDL workflow is unique in R.
- **Symmetry tests** (`symmetrytest()`): Comprehensive Wald tests for both short-run and long-run symmetry in NARDL models.
- **Bootstrap confidence intervals for dynamic multipliers**: Robust uncertainty quantification around short-run, long-run, and impact multipliers through resampling methods, fully integrated with asymmetric multiplier estimation.
- Additional advanced capabilities include highly flexible formula parsing for mixed symmetric/asymmetric regressors, multiple model selection criteria (AIC, BIC, AICc, HQ), parallel processing support, and extensive post-estimation tools tailored for asymmetric analysis.

These features make `kardl` particularly suitable for researchers needing fine-grained control over asymmetric dynamics and small-sample inference.


## Estimating an asymmetric ARDL Model

This example estimates an asymmetric ARDL model to analyze the impact of petrol prices and driving patterns on road fatalities in the UK, using the built-in `Seatbelts` dataset with variables for DriversKilled, PetrolPrice, drivers, kms, and a seatbelt law dummy variable.

### Step 1: Data Preparation

The `Seatbelts` dataset contains monthly data on road casualties in Great Britain from 1969 to 1984. We convert it to a data frame for analysis.

Note: The `Seatbelts` dataset is a built-in R dataset included in the `datasets` package.
The data can be accessed directly by converting the time series object to a data frame.


### Step 2: Define the Model Formula

We define the model formula using R's formula syntax, incorporating asymmetric effects and deterministic variables. We use `asymmetric()` for variables with both short- and long-run asymmetry, `lasymmetric()` for long-run asymmetry, `sasymmetric()` for short-run asymmetry, and `deterministic()` for fixed dummy variables. The `trend` term includes a linear time trend in the model.

```{r data-prepare}
# Define the model formula
my_formula <- DriversKilled ~ PetrolPrice + drivers +
  asymmetric(PetrolPrice + drivers) + deterministic(law) + trend
```
Indeed, the formula syntax is flexible, allowing for various combinations of asymmetric and deterministic variables. The following variations of the formula are equivalent and will yield the same model specification:

```{r eval=FALSE}
same_formula <- y ~ asymmetric(x1) +
  sasymmetric(x2 + x3) +
  lasymmetric(x4 + x5) +
  deterministic(dummy1) + trend
same_formula <- y ~ asymmetric(x1) +
  sasymmetric(x2 + x3) +
  lasymmetric(x4 + x5) +
  deterministic(dummy1) + trend
same_formula <- y ~ asym(x1) + sasym(x2 + x3) + lasym(x4 + x5) +
  det(dummy1) + trend
same_formula <- y ~ a(x1) + s(x2 + x3) + l(x4 + x5) + d(dummy1) + trend
```




### Step 3: Model Estimation

We estimate the ARDL model using different `mode` settings to demonstrate flexibility in lag selection. The `kardl()` function supports various modes: `"grid"`, `"grid_custom"`, `"quick"`, or a user-defined lag vector.

#### Using `mode = "grid"`
The `"grid"` mode evaluates all lag combinations up to `maxlag` and provides console feedback.

```{r model-grid}
# Set model options
kardl_set(criterion = "BIC", different_asym_lag = TRUE, data = Seatbelts)
# Estimate model with grid mode
kardl_model <- kardl(
  data = Seatbelts, formula = my_formula,
  maxlag = 4, mode = "grid"
)
# View results
kardl_model
```
Summary of the model provides detailed information about the estimated coefficients, standard errors, t-values, and significance levels.

```{r model-grid-custom-summary}
# Display model summary
summary(kardl_model)
```


#### Using User-Defined Lags

Specify custom lags to bypass automatic lag selection:

```{r model-user-defined}
kardl_model2 <- kardl(
  data = Seatbelts, my_formula,
  mode = c(2, 1, 1, 3, 0)
)
# View results
kardl_extract(kardl_model2, "opt_lag")
```
```{r model-user-defined-summary}
# Display model summary
summary(kardl_model2)
```



#### Using All Variables
Use the `.` operator to include all variables except the dependent variable:

```{r model-all-vars}
kardl_set(data = Seatbelts)
kardl(formula = DriversKilled ~ . + deterministic(law), maxlag = 2)
```

#### Visualizing Lag Criteria

The `lag_criteria` component contains lag combinations and their criterion values. We visualize these to compare model selection criteria (AIC, BIC, HQ).

```{r lag-criteria}
library(dplyr)
library(tidyr)
library(ggplot2)
# Convert lag_criteria to a data frame
lag_criteria <- as.data.frame(kardl_extract(kardl_model, "lag_criteria"))
lag_criteria <- lag_criteria |> mutate(across(c(AIC, BIC, HQ), as.numeric))

# Pivot to long format
lag_criteria_long <- lag_criteria |>
  select(-c(AICc, AdjR2)) |>
  pivot_longer(
    cols = c(AIC, BIC, HQ),
    names_to = "Criteria",
    values_to = "Value"
  )

# Find minimum values
min_values <- lag_criteria_long |>
  group_by(Criteria) |>
  slice_min(order_by = Value) |>
  ungroup()

# Plot
ggplot(
  lag_criteria_long,
  aes(x = lag, y = Value, color = Criteria, group = Criteria)
) +
  geom_line() +
  geom_point(
    data = min_values, aes(x = lag, y = Value),
    color = "red", size = 3, shape = 8
  ) +
  geom_text(
    data = min_values, aes(x = lag, y = Value, label = lag),
    vjust = 1.5, color = "black", size = 3.5
  ) +
  scale_x_discrete(
    breaks = lag_criteria$lag[seq(1, nrow(lag_criteria), by = 20)]
  ) +
  labs(
    title = "Lag Criteria Comparison",
    x = "Lag Configuration",
    y = "Criteria Value"
  ) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))
```

#### Error Correction Model (ECM) Estimation

The `ecm()` function estimates a Restricted ECM for cointegration testing. We specify the same formula and lag structure as in the ARDL model.
```{r ecm-estimation}
ecm_model <- ecm(
  data = Seatbelts, formula = my_formula,
  maxlag = 4, mode = "grid_custom"
)
# View results
summary(ecm_model)
```


### Step 4: Long-Run Coefficients

We calculate long-run coefficients using `kardl_longrun()`, which standardizes coefficients by dividing them by the negative of the dependent variable’s long-run parameter.

```{r long-run}
# Long-run coefficients
my_long <- kardl_longrun(kardl_model)
my_long
```

The `summary()` function provides detailed information about the long-run coefficients, including standard errors, t-values, and significance levels.

```{r long-run-summary}
# Summary of long-run coefficients
summary(my_long)
```


### Step 5: Symmetry Test

The `symmetrytest()` function performs Wald tests to assess short- and long-run asymmetry in the model.

```{r asymmetry-test}
ast <- Seatbelts |>
  kardl(
    DriversKilled ~ PetrolPrice + drivers + asymmetric(PetrolPrice + drivers) +
      deterministic(law) + trend,
    mode = c(1, 2, 3, 0, 1),
    data = _
  ) |>
  symmetrytest()
ast
```

Summary of the symmetry test provides detailed results for both long-run and short-run asymmetry tests, including F-values, p-values, hypotheses, and test decisions.

```{r asymmetry-test-summary}
# Summary of symmetry test
summary(ast)
```



### Step 6: Cointegration Tests

We perform cointegration tests to assess long-term relationships using `pssf()`, `psst()`, and `narayan()`.

#### PSS F Bound Test

The `pssf()` function tests for cointegration using the Pesaran, Shin, and Smith F Bound test.

```{r pssf}
test_result <- kardl_model |> pssf(case = 3, signif_level = "0.05")
test_result
```

Summary of the PSS F Bound test provides detailed information about the test statistic, critical values, hypotheses, and decision regarding cointegration.

```{r pssf-summary}
summary(test_result)
```

#### PSS t Bound Test

The `psst()` function tests the significance of the lagged dependent variable’s coefficient.

```{r psst}
test_result <- kardl_model |> psst(case = 3, signif_level = "0.05")
test_result
```
Summary of the PSS t Bound test provides detailed information about the test statistic, critical values, hypotheses, and decision regarding cointegration.
```{r psst-summary}
summary(test_result)
```

#### Narayan Test

The `narayan()` function is tailored for small sample sizes. It tests for cointegration using critical values optimized for small samples.

```{r narayan}
test_result <- kardl_model |> narayan(case = 3, signif_level = "0.05")
test_result
```
Summary of the Narayan test provides detailed information about the test statistic, critical values, hypotheses, and decision regarding cointegration.

```{r narayan-summary}
summary(test_result)
```



### Step 7: Dynamic Multipliers

The `mplier()` function calculates dynamic multipliers for the model, showing how changes in independent variables affect the dependent variable over time.
```{r dynamic-multipliers}
multipliers <- kardl_model |> mplier()
# View multipliers of the model
head(kardl_extract(multipliers, "multipliers"))
# View long-run multipliers
kardl_extract(multipliers, "omega")
# View short-run multipliers
head(kardl_extract(multipliers, "lambda"))
```

Plotting dynamic multipliers for specific variables can be done using the `plot()` function, which visualizes the response of the dependent variable to changes in independent variables over time.
```{r plot-multipliers}
plot(multipliers, variables = c("PetrolPrice", "drivers"))
```

 
 To handle a large number of variables, you can specify a subset of variables to plot or use `variables = "all"` to visualize all dynamic multipliers.
 
 Bootstrap confidence intervals for dynamic multipliers can be calculated using the `bootstrap()` function, which provides robust estimates of uncertainty around the multipliers.
```{r bootstrap-multipliers}
bootstrap_results <- kardl_model |>
  bootstrap(horizon = 12, replications = 10)
# View bootstrap summary
summary(bootstrap_results)
```

Visualize bootstrap results for specific variables to understand the variability and confidence intervals of the dynamic multipliers.

```{r plot-bootstrap-multipliers}
plot(bootstrap_results, variables = "drivers")
```
 


### Step 8: Customizing asymmetric Variables

We demonstrate how to customize prefixes and suffixes for asymmetric variables using `kardl_set()`.

```{r asym-custom}
# Set custom prefixes and suffixes
kardl_reset()
kardl_set(asym_prefix = c("asyP_", "asyN_"), asym_suffix = c("_PP", "_NN"))
kardl_custom <- kardl(data = Seatbelts, my_formula)
kardl_custom
```

## Key Functions and Parameters

- **`kardl(data, model, maxlag, mode, ...)`**:
  - `data`: A time series dataset (e.g., a data frame with DriversKilled, PetrolPrice, drivers).
  - `formula`: A formula specifying the long-run equation, e.g., `y ~ x + z + asymmetric(z) + lasymmetric(x2 + x3) + sasymmetric(x3 + x4) + deterministic(dummy1 + dummy2) + trend`. Supports:
    - `asymmetric()`: asymmetric effects for both short- and long-run dynamics.
    - `lasymmetric()`: Long-run asymmetric variables.
    - `sasymmetric()`: Short-run asymmetric variables.
    - `deterministic()`: Fixed dummy variables.
    - `trend`: Linear time trend.
  - `maxlag`: Maximum number of lags (default: 4). Use smaller values (e.g., 2) for small datasets, larger values (e.g., 8) for long-term dependencies.
  - `mode`: Estimation mode:
    - `"quick"`: Verbose output for interactive use.
    - `"grid"`: Verbose output with lag optimization.
    - `"grid_custom"`: Silent, efficient execution.
    - User-defined vector (e.g., `c(1, 2, 4, 5)` or `c(DriversKilled = 2, PetrolPrice_POS = 3, PetrolPrice_NEG = 1, drivers = 3)`).
  - Returns a list with components: `inputs`, `finalModel`, `start_time`, `end_time`, `properLag`, `time_span`, `opt_lag`, `lag_criteria`, `type` ("kardlmodel").

- **`kardl_set(...)`**: Configures options like `criterion` (AIC, BIC, AICc, HQ), `different_asym_lag`, `asym_prefix`, `Sasymuffix`, `short_coef`, and `long_coef`. Use `kardl_get()` to retrieve settings and `kardl_reset()` to restore defaults.

- **`kardl_longrun(model)`**: Calculates standardized long-run coefficients, returning `type` ("kardl_longrun"), `coef`, `delta_se`, `results`, and `starsDesc`.

- **`symmetrytest(model)`**: Performs Wald tests for short- and long-run asymmetry, returning `Lhypotheses`, `Lwald`, `Shypotheses`, `Swald`, and `type` ("symmetrytest").

- **`pssf(model, case, signif_level)`**: Performs the Pesaran, Shin, and Smith F Bound test for cointegration, supporting cases 1–5 and significance levels ("auto", 0.01, 0.025, 0.05, 0.1, 0.10).

- **`psst(model, case, signif_level)`**: Performs the PSS t Bound test, focusing on the lagged dependent variable’s coefficient.

- **`narayan(model, case, signif_level)`**: Conducts the Narayan test for cointegration, optimized for small samples (cases 2–5).

- **`ecm(data, model, maxlag, mode,  ...)`**: Conducts the Restricted ECM test for cointegration, with similar parameters to `kardl()` and case/significance level options.

For detailed documentation, use `?kardl`, `?kardl_set`, `?kardl_longrun`, `?symmetrytest`, `?pssf`, `?psst`, `?narayan`, or `?ecm`.

## Conclusion

The `kardl` package is a versatile tool for econometric analysis, offering robust support for symmetric and asymmetric ARDL/NARDL modeling, cointegration tests, stability diagnostics, and heteroskedasticity checks. Its flexible formula specification, lag optimization, and support for parallel processing make it ideal for studying complex economic relationships. For more information, visit [https://github.com/karamelikli/kardl](https://github.com/karamelikli/kardl) or contact the authors at [hakperest@gmail.com](mailto:hakperest@gmail.com).

---

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{parse_formula_vars}
\alias{parse_formula_vars}
\title{Parse Formula Variables}
\usage{
parse_formula_vars(formula)
}
\arguments{
\item{formula}{The initial formula for the model, typically specified using R's formula syntax (e.g.,  \code{y ~ x + f(x1 + x2)}).}
}
\value{
A list containing:
\itemize{
\item \code{response}: The response variable(s) extracted from the formula.
\item \code{intercept}: A logical value indicating whether the formula includes an intercept (default is TRUE).
\item \code{dot}: A logical value indicating whether the formula includes a dot (.) representing all other variables (default is FALSE).
\item \code{outside}: A vector of variables that are outside any specified patterns. It includes the variables that are not detected within the specified patterns in the formula.
\item \code{inside}: A list where each element corresponds to a detected pattern (e.g., function name) and contains the variables found inside that pattern. For example, if the formula includes \code{asym(x1 + x2)}, the \code{inside} list will have an element named "asym" containing the variables "x1" and "x2". This allows for easy identification of variables that are part of specific constructs in the formula.
}
}
\description{
The \code{parseFormula()} function analyzes a given formula to identify and extract variables that match specified patterns. It is particularly useful for isolating variables enclosed within certain functions or constructs in the formula, such as \code{asym()}, \code{det()}, or any user-defined patterns.
}
\examples{

# Parse formulas containing various collection types like ()
formula_ <- y ~ x +det(s -gg- d) + asymS(d2 -rr+ s)-mm(y1+y2+y3)+asym(k1+k2+k3)+trend-huseyin
# Extract variables
parse_formula_vars(formula_)


}
\seealso{
\code{\link[stats]{formula}} and \code{\link[base]{gregexpr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{lmerge}
\alias{lmerge}
\title{Merge Lists with Priority to the First Argument}
\usage{
lmerge(first, second, ...)
}
\arguments{
\item{first}{The first list}

\item{second}{The second list}

\item{...}{Additional lists to merge}
}
\value{
A merged list with unique names, prioritizing values from the first list in case of name conflicts.
}
\description{
The first list of values takes precedence. When both lists have items with the same names, the values from the first list will be applied. In merging the two lists, priority is given to the left list, so if there are overlapping items, the corresponding value from the left list will be used in the merged result.
}
\details{
The \code{lmerge()} function is designed to merge multiple lists while giving precedence to the values in the first list.
This function is particularly useful when you want to combine settings or parameters from multiple sources while ensuring that the primary source (the first list) takes priority over others.

For right merge, a user can simply swap the order of the lists to give priority to the second list. For example, \code{lmerge(b, a)} will prioritize values from list \code{b} over those in list \code{a}.
}
\examples{

a<-list("a"="first a","b"="second a","c"=list("w"=12,"k"=c(1,3,6)))
b<-list("a"="first b","b"="second b","d"=14,"e"=45)
myMerged<- lmerge(a,b)
print(unlist(myMerged))

# for right merge
myMerged<- lmerge(b,a)
print(unlist(myMerged))

# for more than two lists
myMerged<- lmerge(a,b,c("v1"=11,22,3,"v5"=5))
print(unlist(myMerged))

# for more than two lists with nested lists
m2<-list("m1"="kk2","m1.2.3"=list("m1.1.1"=333,"m.1.4"=918,"m.1.5"=982,"m.1.6"=981,"m.1.7"=928))
m3<-list("m1"="kk23","m2.3"=2233,"m1.2.4"=list("m1.1.1"=333444,"m.1.5"=982,"m.1.6"=91,"m.1.7"=928))
a<-c(32,34,542,"k"=35)
b<-c(65,"k"=34)

h1<-lmerge(a, m2)
print(unlist(h1))

h2<-lmerge(a,b,m2,m3,list("m1.1"=4))
print(unlist(h2))

}
\seealso{
\code{\link[base]{append}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longrun.R
\name{kardl_longrun}
\alias{kardl_longrun}
\title{Compute Long-Run Multipliers from a kardl Model}
\usage{
kardl_longrun(model)
}
\arguments{
\item{model}{An object of class \code{kardl} estimated using the \code{kardl} function.}
}
\value{
An object of class \code{kardl_long_run}, which is a list containing:
\itemize{
\item \code{coefficients}: A named vector of long-run multipliers.
\item \code{residuals}: A vector of residuals from the long-run model.
\item \code{effects}: A vector of effects from the long-run model.
\item \code{rank}: The rank of the long-run model.
\item \code{fitted.values}: A vector of fitted values from the long-run model.
\item \code{assign}: A vector indicating the assignment of coefficients to terms in the long-run model.
\item \code{qr}: The QR decomposition of the design matrix of the long-run model.
\item \code{df.residual}: The degrees of freedom of the residuals of the long-run model.
\item \code{xlevels}: A list of factor levels used in the long-run model.
\item \code{call}: The matched call used to create the long-run model.
\item \code{terms}: The terms object of the long-run model.
\item \code{model}: The data frame used in the long-run model.
}
}
\description{
This function calculates the long-run parameters of a KARDL model estimated using the \code{kardl} function. The long-run parameters are calculated by dividing the negative of the coefficients of the independent variables by the coefficient of the dependent variable. If an intercept is included in the model, it is also standardized by dividing it by the negative of the long-run parameter of the dependent variable.
}
\details{
The function also calculates the standard errors of the long-run multipliers using the delta method, which accounts for the covariance between the coefficients. The fitted values and residuals of the long-run model are calculated based on the original data and the long-run multipliers.

The function returns an object of class \code{kardl_long_run}, which contains the long-run multipliers, their standard errors, t-statistics, p-values, fitted values, residuals, and other relevant information for further analysis and diagnostics.

Note that the fitted values and residuals from the long-run model are not centered (i.e., they do not have a mean of zero) by design, which means that diagnostic plots and residual-based tests may not be valid for this model. The primary focus of this function is on the estimation of the long-run multipliers and their associated statistics.

The long-run multipliers are calculated using the formula:
\deqn{LRM_i = -\frac{\eta_i}{\eta_0}}.

t-values and p-values are calculated using the standard errors obtained from the delta method, which accounts for the covariance between the coefficients.
Delta method formula for standard errors of long-run multipliers:
\deqn{SE(LR_i) = \sqrt{(A^2) \cdot Var(\eta_i) + 2 \cdot A \cdot B \cdot Cov(\eta_i, \eta_0) + (B^2) \cdot Var(\eta_0)}} where \deqn{A = \frac{\partial LRM_i}{\partial \eta_i} = -\frac{1}{\eta_0}} and \deqn{B = \frac{\partial LRM_i}{\partial \eta_0} = \frac{\eta_i}{\eta_0^2}} .
Hence, \eqn{\eta_i} is the coefficient of the independent variable and \eqn{\eta_0} is the coefficient of the dependent variable in the original KARDL model.
}
\examples{
kardl_model<-kardl(imf_example_data,
                   CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                   mode=c(1,2,3,0))
long<-kardl_longrun(kardl_model)

# Calculate the long-run multipliers
long
# Details of the long-run multipliers
summary(long)


# Using magrittr

\dontshow{if (requireNamespace("magrittr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(magrittr)
     imf_example_data \%>\%
     kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend, mode=c(1,2,3,0)) \%>\%
     kardl_longrun() \%>\% summary()
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{kardl}}, \code{\link{pssf}}, \code{\link{psst}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multipliers.R
\name{bootstrap}
\alias{bootstrap}
\title{Bootstrap Confidence Intervals for Dynamic Multipliers}
\usage{
bootstrap(
  kmodel,
  horizon = 80,
  replications = 100,
  level = 95,
  minProb = 0,
  seed = NULL
)
}
\arguments{
\item{kmodel}{The model produced by the \code{\link{kardl}} function. This is the model object
from which the dynamic multipliers are calculated.}

\item{horizon}{An integer specifying the horizon over which dynamic multipliers will be computed.
The horizon defines the time frame for the analysis (e.g., 40 periods).}

\item{replications}{An integer indicating the number of bootstrap replications to perform.
Higher values increase accuracy but also computational time. Default is \code{100}.}

\item{level}{A numeric value specifying the confidence level for the intervals (e.g., 95 for 95\% confidence).
Default is \code{90}.}

\item{minProb}{A numeric value specifying the minimum p-value threshold for including coefficients in the bootstrap.
Coefficients with p-values above this threshold will be set to zero in the bootstrap samples. Default is \code{0} (no threshold).
This parameter allows users to control the inclusion of coefficients in the bootstrap process based on their statistical significance. Setting a threshold can help focus the analysis on more relevant variables, but it may also exclude potentially important effects if set too stringently.}

\item{seed}{An optional integer to set the random seed for reproducibility of the bootstrap results. If not provided, the bootstrap will use the current random state.}
}
\value{
A list containing the following elements:
\itemize{
\item \strong{mpsi}: A data frame containing the dynamic multiplier estimates along with their upper and lower confidence intervals for each variable and time horizon.
\item \strong{level}: The confidence level used for the intervals (e.g 95).
\item \strong{horizon}: The horizon over which the multipliers were computed (e.g., 40).
\item \strong{vars}: A list of variable information extracted from the model, including dependent variable, independent variables, asymmetric variables, and deterministic terms.
\item \strong{replications}: The number of bootstrap replications performed.
\item \strong{type}: A character string indicating the type of analysis, in this case "bootstrap".
}
}
\description{
This function computes bootstrap confidence intervals (CI) for dynamic multipliers
of a specified variable in a model estimated using the \code{kardl} package. The bootstrap method
generates resampled datasets to estimate the variability of the dynamic multipliers,
providing upper and lower bounds for the confidence interval.
}
\details{
The \code{mpsi} component of the output contains the dynamic multiplier estimates along with their upper
and lower confidence intervals. These values are provided for each variable and at each time horizon.
}
\examples{

  # Example usage of the bootstrap function


 # Fit a model using kardl
 kardl_model <- kardl(imf_example_data,
                      CPI ~ ER + PPI + asy(ER) +
                       det(covid) + trend,
                      mode = c(1, 2, 3, 0))

 # Perform bootstrap with specific variables for plotting
 boot <-
   bootstrap(kardl_model,   replications=5, seed = 123L)
 # The boot object will include all plots for the specified variables
 # Displaying the boot object provides an overview of its components
 names(boot)


 # Inspect the first few rows of the dynamic multiplier estimates
  head(boot$mpsi)


  summary(boot)

 # Retrieve plots generated during the bootstrap process
 # Accessing all plots
  plot(boot)

 # Accessing the plot for a specific variable by its name
  plot(boot, variable = "PPI")
 plot(boot, variable = "ER")



\dontshow{if (requireNamespace("magrittr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(magrittr)

   imf_example_data \%>\% kardl( CPI ~ PPI + asym(ER) +trend, maxlag=2) \%>\%
   bootstrap(replications=5) \%>\% plot(variable = "ER")

\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{mplier}} for calculating dynamic multipliers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tests.R
\name{symmetrytest}
\alias{symmetrytest}
\title{Symmetry Test for Nonlinear kardl Models}
\usage{
symmetrytest(kmodel, vars = NULL, component = "both", type = "F", ...)
}
\arguments{
\item{kmodel}{The kardl obejct}

\item{vars}{A character vector specifying the names of the variables to be included in the symmetry test. If NULL (the default), all eligible variables will be tested. The variable names should match those used in the KARDL model, and they can be either long-run or short-run variables. If any specified variable is not found in the model, an error will be raised.}

\item{component}{A character string specifying which component(s) of the model to test for symmetry. The options are "both" (default), "shortrun", or "longrun". If "both" is selected, the function will perform symmetry tests for both long-run and short-run variables. If "shortrun" is selected, only short-run variables will be tested, and if "longrun" is selected, only long-run variables will be tested. Invalid values will result in an error.}

\item{type}{A character string specifying the type of test statistic to be used in the Wald tests. The options are "F" (default) or "Chisq". If "F" is selected, the function will perform an F-test, which is appropriate for smaller sample sizes. If "Chisq" is selected, the function will perform a chi-squared test, which is more suitable for larger sample sizes. Invalid values will result in an error.}

\item{...}{Additional arguments to be passed to the underlying test functions, such as \code{\link[nlWaldTest]{nlWaldtest}} for long-run tests or \code{\link[car]{linearHypothesis}} for short-run tests. These arguments can include options for controlling the behavior of the tests, such as specifying the type of test statistic or adjusting for multiple comparisons.}
}
\value{
A list with class "kardl" containing the following components:
\itemize{
\item \code{Lwald:} A data frame containing the Wald test results for the long-run variables, including F-statistic, p-value, degrees of freedom, and residual degrees of freedom.
\item \code{Lhypotheses:} A list containing the null and alternative hypotheses for the long-run variables.
\item \code{Swald:} A data frame containing the Wald test results for the short-run variables, including F-statistic, p-value, degrees of freedom, residual degrees of freedom, and sum of squares.
\item \code{Shypotheses:} A list containing the null and alternative hypotheses for the short-run variables.
}
}
\description{
The symmetry test is a statistical procedure used to assess the presence of symmetry in the relationship between variables in a model. It is particularly useful in econometric analysis, where it helps to identify whether the effects of changes in one variable on another are symmetric or asymmetric. The test involves estimating a model that includes both positive and negative components of the variables and then performing a Wald test to determine if the coefficients of these components are significantly different from each other.
If the test indicates significant differences, it suggests that the relationship is asymmetric, meaning that the impact of increases and decreases in the variables differs.
This test returns results for both long-run and short-run variables in a KARDL model. Where applicable, it provides the Wald test statistics, p-values, degrees of freedom, sum of squares, and mean squares for each variable tested.
If the null hypothesis of symmetry is rejected, it indicates that the effects of positive and negative changes in the variable are significantly different, suggesting an asymmetric relationship.

The non-linear model with one asymmetric variables is specified as follows:
\deqn{
 \Delta{y_{t}} = \psi + \eta_{0}y_{t - 1} + \eta^{+}_{1} x^{+}_{t - 1}+ \eta^{-}_{1} x^{-}_{t - 1} + \sum_{j = 1}^{p}{\gamma_{j}\Delta y_{t - j}} + \sum_{j = 0}^{q}{\beta^{+}_{j}\Delta x^{+}_{t - j}}  + \sum_{j = 0}^{m}{\beta^{-}_{j}\Delta x^{-}_{t - j}} + e_{t}
}

This function performs the symmetry test both for long-run and short-run variables in a kardl model. It uses the \code{\link[nlWaldTest]{nlWaldtest}} function from the \pkg{nlWaldTest} package for long-run variables and the \code{\link[car]{linearHypothesis}} function from the \pkg{car} package for short-run variables.
The hypotheses for the long-run variables are:
\deqn{
    H_{0}: -\frac{\eta^{+}_{1}}{\eta_{0}} = -\frac{\eta^{-}_{1}}{\eta_{0}} }
\deqn{    H_{1}: -\frac{\eta^{+}_{1}}{\eta_{0}} \neq -\frac{\eta^{-}_{1}}{\eta_{0}}
 }

The hypotheses for the short-run variables are:
\deqn{
  H_{0}: \sum_{j = 0}^{q}{\beta^{+}_{j}} = \sum_{j = 0}^{m}{\beta^{-}_{j}} }
\deqn{H_{1}: \sum_{j = 0}^{q}{\beta^{+}_{j}} \neq \sum_{j = 0}^{m}{\beta^{-}_{j}}
  }
}
\details{
This function performs symmetry tests on non-linear KARDL models to assess whether the effects of positive and negative changes in independent variables are statistically different.

This function evaluates whether the inclusion of a particular variable in the model follows a linear relationship or exhibits a non-linear pattern. By analyzing the behavior of the variable, the function helps to identify if the relationship between the variable and the outcome of interest adheres to a straight-line assumption or if it deviates, indicating a non-linear interaction. This distinction is important in model specification, as it ensures that the variable is appropriately represented, which can enhance the model's accuracy and predictive performance.
}
\examples{

 kardl_model<-kardl(imf_example_data,
                    CPI~Lasym(PPI+ER)+Sas(ER)+deterministic(covid)+trend)
 ast<- symmetrytest(kardl_model)
 ast
 # Detailed results of the test:
 summary(ast)
 # The null hypothesis of the test is that the model is symmetric, while the alternative
 # hypothesis is that the model is asymmetric. The test statistic and p-value are provided
 # in the output. If the p-value is less than a chosen significance level (e.g., 0.05),
 # we reject the null hypothesis and conclude that there is evidence of asymmetry in the model.

 # The default significance level is 0.05, but you can specify a different level using the 'level'
 # argument in the summary function. For example, to use a significance level of 0.01,
 # you can use the following code:
 summary(ast, level = 0.01)

 # To get symmetry test results in long-run, you can use the following code:
 ast$Lwald

 # To get symmetry test results in short-run, you can use the following code:
 ast$Swald

 # To get the null and alternative hypotheses of the test in long-run,
 # you can use the following code:

 ast$Lhypotheses

 # To get the null and alternative hypotheses of the test in short-run,
 # you can use the following code:

 ast$Shypotheses
 # Alternatively, you can also use the symmetrytest function with the component
 # argument to specify whether you want to test for long-run or short-run symmetry.
 # For example, to test for long-run symmetry, you can use the following code:
 symmetrytest(kardl_model, component = "longrun")
 # To test for short-run symmetry, you can use the following code:
 symmetrytest(kardl_model, component = "shortrun")

 # If you want to test for symmetry with respect to a specific variable,
 # you can use the vars argument in the symmetrytest function. For example,
 # to test for symmetry with respect to the PPI variable, you can use the following code:
 symmetrytest(kardl_model, vars = "PPI")

 # To test for symmetry with respect to multiple variables, you can provide
 # a vector of variable names to the vars argument. For example, to test for
 # symmetry with respect to both PPI and ER, you can use the following code:
 symmetrytest(kardl_model, vars = c("PPI", "ER"))

 # Finally, you can also specify the type of test statistic to be used in the
 # symmetry test. By default, the function uses the Wald test  F statistic,
 # but you can also choose to use the chi-squared test statistic.
 # For example, to use the chi-squared test statistic, you can use the following code:
 symmetrytest(kardl_model, type="Chisq")


}
\references{
Shin, Y., Yu, B., & Greenwood-Nimmo, M. (2014). Modelling asymmetric cointegration and dynamic multipliers in a nonlinear ARDL framework. Festschrift in honor of Peter Schmidt: Econometric methods and applications, 281-314.
}
\seealso{
\code{\link{kardl}}, \code{\link{pssf}}, \code{\link{psst}}, \code{\link{ecm}}, \code{\link{narayan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{kardl_reset}
\alias{kardl_reset}
\title{Reset kardl Package Options to Default Values}
\usage{
kardl_reset(exclude = NULL)
}
\arguments{
\item{exclude}{Character vector of setting names that should not be reset.
These settings retain their current values. By default, all settings are reset.}
}
\value{
A list of the settings after reset, returned invisibly.
}
\description{
This function resets kardl package options to their default values.
}
\examples{
kardl_set(criterion = "BIC", differentAsymLag = FALSE)

# Reset all settings to defaults except "criterion"
kardl_reset(exclude = "criterion")

# Get the current settings to verify the reset
print(kardl_get("criterion"))

# This will show "BIC" since it was excluded from the reset,
#while other settings will be reset to their defaults.
print(kardl_get("differentAsymLag"))
}
\seealso{
\code{\link{kardl_set}}, \code{\link{kardl_get}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tests.R
\name{narayan}
\alias{narayan}
\title{Narayan Bounds Test}
\usage{
narayan(kmodel, case = 3, signif_level = "auto")
}
\arguments{
\item{kmodel}{The kardl obejct}

\item{case}{Numeric or character. Specifies the case of the test to be used in the function.
Acceptable values are 1, 2, 3, 4, 5, and "auto". If "auto" is chosen, the function determines the case automatically based on the model's characteristics. Invalid values will result in an error.

\itemize{
\item \code{1}: No intercept and no trend. This case is not supported by the Narayan test.
\item \code{2}: Restricted intercept and no trend.
\item \code{3}: Unrestricted intercept and no trend.
\item \code{4}: Unrestricted intercept and restricted trend.
\item \code{5}: Unrestricted intercept and unrestricted trend.
}}

\item{signif_level}{Character or numeric. Specifies the significance level to be used in the function.
Acceptable values are "auto", "0.10", "0.1", "0.05", "0.025", and "0.01".
If a numeric value is provided, it will be converted to a character string.

When \code{"auto"} is selected, the function determines the significance level
sequentially, starting from the most stringent level (\code{"0.01"}) and proceeding
to \code{"0.025"}, \code{"0.05"}, and \code{"0.10"} until a suitable level is identified.
Invalid values will result in an error.}
}
\value{
A list with class "htest" containing the following components:
\itemize{
\item \code{statistic}: The calculated F-statistic for the test.
\item \code{caseTxt}: A character string describing the case used for the test, based on the specified case parameter.
\item \code{alternative}: A character string describing the alternative hypothesis of the test.
\item \code{sample.size}: The number of observations used in the test.
\item \code{varnames}: A character vector containing the names of the dependent variable and independent variables used in the test.
\item \code{k}: The number of independent variables (excluding the dependent variable) included in the test.
\item \code{sig}: The significance level used for the test, either specified by the user or determined automatically.
\item \code{notes}: A character vector containing any notes or warnings related to
the test, such as the suitability of the test for small sample sizes or any adjustments made to the case based on the model's characteristics.
}
}
\description{
This function performs the Narayan test, which is designed to assess cointegration using critical values specifically tailored for small sample sizes. Unlike traditional cointegration tests that may rely on asymptotic distributions, the Narayan test adjusts for the limitations of small samples, providing more accurate results in such contexts. This makes the test particularly useful for studies with fewer observations, as it accounts for sample size constraints when determining the presence of a long-term equilibrium relationship between variables.
}
\section{Hypothesis testing}{

The null hypothesis (H0) of the F Bound test is that there is no cointegration among the variables in the model. In other words, it tests whether the long-term relationship between the variables is statistically significant. If the calculated F-statistic exceeds the upper critical value, we reject the null hypothesis and conclude that there is cointegration. Conversely, if the F-statistic falls below the lower critical value, we fail to reject the null hypothesis, indicating no evidence of cointegration. If the F-statistic lies between the two critical values, the result is inconclusive.

\deqn{
  \Delta {y}_t =  \psi  + \varphi t  + \eta _0   {y}_{t-1}  + \sum_{i=1}^{k} {  \eta _i   {x}_{i,t-1} }  +   \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }+ \sum_{i=1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
}

\describe{
\item{Cases 1, 3, 5:}{
\deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k  = 0}
\deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k} \neq 0}
}
\item{Case 2:}{
\deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k =  \psi  = 0}
\deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k}  \neq \psi \neq 0}
}

\item{Case 4:}{
\deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k = \varphi = 0}
\deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k}  \neq \varphi \neq 0}
}
}
}

\examples{
kardl_model<-kardl(imf_example_data,
                   CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                   mode=c(1,2,3,0))
my_test<-narayan(kardl_model)
# Getting the results of the test.
my_test
# Getting details of the test.
my_summary<-summary(my_test)
my_summary

# Getting the critical values of the test.
my_summary$crit_vals




# Using magrittr :

\dontshow{if (requireNamespace("magrittr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(magrittr)
  imf_example_data \%>\%
  kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                        mode=c(1,2,3,0)) \%>\% narayan()

  # Getting details of the test results using magrittr:
   imf_example_data \%>\%
   kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                           mode=c(1,2,3,0)) \%>\%
   narayan() \%>\% summary()


\dontshow{\}) # examplesIf}
}
\references{
Narayan, P. K. (2005). The saving and investment nexus for China: evidence from cointegration tests. Applied economics, 37(17), 1979-1990.
}
\seealso{
\code{\link{pssf}}  \code{\link{psst}}  \code{\link{ecm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tests.R
\name{psst}
\alias{psst}
\title{Pesaran, Shin, and Smith t Bounds Test}
\usage{
psst(kmodel, case = 3, signif_level = "auto")
}
\arguments{
\item{kmodel}{A fitted KARDL model object of class 'kardl_lm' created using the \code{\link{kardl}} function.}

\item{case}{Numeric or character. Specifies the case of the test to be used in the function.
Acceptable values are 1, 2, 3, 4, 5, and "auto". If "auto" is chosen, the function determines the case automatically based on the model's characteristics. Invalid values will result in an error.
\itemize{
\item \code{1}: No intercept and no trend
\item \code{2}: Restricted intercept and no trend
\item \code{3}: Unrestricted intercept and no trend
\item \code{4}: Unrestricted intercept and restricted trend
\item \code{5}: Unrestricted intercept and unrestricted trend
}}

\item{signif_level}{Character or numeric. Specifies the significance level to be used in the function.
Acceptable values are "auto", "0.10", "0.1", "0.05", "0.025", and "0.01".
If a numeric value is provided, it will be converted to a character string.

When \code{"auto"} is selected, the function determines the significance level
sequentially, starting from the most stringent level (\code{"0.01"}) and proceeding
to \code{"0.025"}, \code{"0.05"}, and \code{"0.10"} until a suitable level is identified.
Invalid values will result in an error.}
}
\value{
The function returns an object of class "htest" containing the following components:
\describe{
\item{statistic}{The calculated t-statistic for the test.}
\item{method}{A description of the test performed.}
\item{data.name}{The name of the data used in the test.}
\item{k}{The number of independent variables in the model.}
\item{notes}{Any notes or warnings related to the test results, such as
sample size considerations or adjustments made to the case based on model characteristics.}
\item{sig}{The significance level used for the test, either specified by the user or determined automatically.}
\item{alternative}{The alternative hypothesis being tested.}
\item{case}{The case used for the test, either specified by the user or determined automatically based on the model's characteristics.}
}
}
\description{
This function performs the Pesaran t Bound test
}
\details{
This function performs the Pesaran, Shin, and Smith (PSS) t Bound test, which is used to detect the existence of a long-term relationship (cointegration) between variables in an autoregressive distributed lag (ARDL) model. The t Bound test specifically focuses on the significance of the coefficient of the lagged dependent variable, helping to assess whether the variable reverts to its long-term equilibrium after short-term deviations. The test provides critical values for both upper and lower bounds. If the t-statistic falls within the appropriate range, it confirms the presence of cointegration. This test is particularly useful when working with datasets containing both stationary and non-stationary variables.
}
\section{Hypothesis testing}{

The PSS t Bound test evaluates the null hypothesis that the long-run coefficients of the model are equal to zero against the alternative hypothesis that at least one of them is non-zero. The test is conducted under different cases, depending on the model specification.

\deqn{
  \Delta {y}_t =  \psi  + \varphi t  + \eta _0   {y}_{t-1}  + \sum_{i=1}^{k} {  \eta _i   {x}_{i,t-1} }  +   \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }+ \sum_{i=1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
}

\deqn{\mathbf{H_{0}:} \eta_0   = 0}
\deqn{\mathbf{H_{1}:} \eta_{0}  \neq 0}
}

\examples{

kardl_model<-kardl(imf_example_data,
                   CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                   mode=c(1,2,3,0))
my_test<-psst(kardl_model)
# Getting the results of the test.
my_test
# Getting details of the test.
my_summary<-summary(my_test)
my_summary

# Getting the critical values of the test.
my_summary$crit_vals




# Using magrittr :

\dontshow{if (requireNamespace("magrittr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(magrittr)
     imf_example_data \%>\%
     kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                           mode=c(1,2,3,0))    \%>\%
                           psst()

   # Getting details of the test results using magrittr:
   imf_example_data \%>\%
   kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                           mode=c(1,2,3,0)) \%>\%
   psst() \%>\% summary()


\dontshow{\}) # examplesIf}
}
\references{
Pesaran, M. H., Shin, Y. and Smith, R. (2001), "Bounds Testing Approaches to the Analysis of Level Relationship", Journal of Applied Econometrics, 16(3), 289-326.
}
\seealso{
\code{\link{pssf}}   \code{\link{ecm}}  \code{\link{narayan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{kardl_get}
\alias{kardl_get}
\title{Get kardl Package Options}
\usage{
kardl_get(...)
}
\arguments{
\item{...}{Option names to retrieve. If no arguments are provided, all options will be returned.}
}
\value{
If no arguments are provided, returns all options as a list. If specific option names are provided, returns their values.
}
\description{
This function retrieves the current settings of the kardl package. Users can specify option names to get their values or call the function without arguments to retrieve all current settings.
}
\examples{

# Get all options
kardl_get()
# Get specific options
kardl_get("maxlag", "mode")

# Note: In interactive use, avoid calling kardl_get() directly to prevent cluttering the console.

a<-kardl_get()
a$AsymSuffix

# To reset options and then get a default option:
mydefaults <- kardl_reset()
mydefaults$maxlag

}
\seealso{
\code{\link{kardl_set}}, \code{\link{kardl_reset}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tests.R
\name{pssf}
\alias{pssf}
\title{Pesaran, Shin, and Smith Bounds F-Test}
\usage{
pssf(kmodel, case = 3, signif_level = "auto")
}
\arguments{
\item{kmodel}{A fitted KARDL model object of class 'kardl_lm' created using the \code{\link{kardl}} function.}

\item{case}{Numeric or character. Specifies the case of the test to be used in the function.
Acceptable values are 1, 2, 3, 4, 5, and "auto". If "auto" is chosen, the function determines the case automatically based on the model's characteristics. Invalid values will result in an error.
\itemize{
\item \code{1}: No intercept and no trend
\item \code{2}: Restricted intercept and no trend
\item \code{3}: Unrestricted intercept and no trend
\item \code{4}: Unrestricted intercept and restricted trend
\item \code{5}: Unrestricted intercept and unrestricted trend
}}

\item{signif_level}{Character or numeric. Specifies the significance level to be used in the function.
Acceptable values are "auto", "0.10", "0.1", "0.05", "0.025", and "0.01".
If a numeric value is provided, it will be converted to a character string.

When \code{"auto"} is selected, the function determines the significance level
sequentially, starting from the most stringent level (\code{"0.01"}) and proceeding
to \code{"0.025"}, \code{"0.05"}, and \code{"0.10"} until a suitable level is identified.
Invalid values will result in an error.}
}
\value{
A list with class "htest" containing the following components:
\itemize{
\item \code{statistic}: The calculated F-statistic for the test.
\item \code{caseTxt}: A character string describing the case used for the test, based on the specified case parameter.
\item \code{alternative}: A character string describing the alternative hypothesis of the test.
\item \code{sample.size}: The number of observations used in the test.
\item \code{varnames}: A character vector containing the names of the dependent variable and independent variables used in the test.
\item \code{k}: The number of independent variables (excluding the dependent variable) included in the test.
\item \code{sig}: The significance level used for the test, either specified by the user or determined automatically.
\item \code{notes}: A character vector containing any notes or warnings related to
the test, such as the suitability of the test for small sample sizes or any adjustments made to the case based on the model's characteristics.
}
}
\description{
This function performs the Pesaran, Shin, and Smith (PSS) F Bound test to assess the presence of a long-term relationship (cointegration) between variables in the context of an autoregressive distributed lag (ARDL) model. The PSS F Bound test examines the joint significance of lagged levels of the variables in the model. It provides critical values for both the upper and lower bounds, which help determine whether the variables are cointegrated. If the calculated F-statistic falls outside these bounds, it indicates the existence of a long-term equilibrium relationship. This test is particularly useful when the underlying data includes a mix of stationary and non-stationary variables.
}
\section{Hypothesis testing}{

The null hypothesis (H0) of the F Bound test is that there is no cointegration among the variables in the model. In other words, it tests whether the long-term relationship between the variables is statistically significant. If the calculated F-statistic exceeds the upper critical value, we reject the null hypothesis and conclude that there is cointegration. Conversely, if the F-statistic falls below the lower critical value, we fail to reject the null hypothesis, indicating no evidence of cointegration. If the F-statistic lies between the two critical values, the result is inconclusive.

\deqn{
  \Delta {y}_t =  \psi  + \varphi t  + \eta _0   {y}_{t-1}  + \sum_{i=1}^{k} {  \eta _i   {x}_{i,t-1} }  +   \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }+ \sum_{i=1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
}

\describe{
\item{Cases 1, 3, 5:}{
\deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k  = 0}
\deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k} \neq 0}
}
\item{Case 2:}{
\deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k =  \psi  = 0}
\deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k}  \neq \psi \neq 0}
}

\item{Case 4:}{
\deqn{\mathbf{H_{0}:} \eta_0  = \eta_1  = \dots = \eta_k = \varphi = 0}
\deqn{\mathbf{H_{1}:} \eta_{0} \neq \eta_{1} \neq \dots \neq \eta_{k}  \neq \varphi \neq 0}
}
}
}

\examples{
kardl_model<-kardl(imf_example_data,
                   CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                   mode=c(1,2,3,0))
my_pssF<-pssf(kardl_model)
# Getting the results of the test.
my_pssF
# Getting details of the test.
my_summary<-summary(my_pssF)
my_summary

# Getting the critical values of the test.
my_summary$crit_vals


# Using magrittr :

\dontshow{if (requireNamespace("magrittr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(magrittr)
   imf_example_data \%>\%
   kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                           mode=c(1,2,3,0)) \%>\% pssf()

   # Getting details of the test results using magrittr:
   imf_example_data \%>\%
   kardl(CPI~ER+PPI+asym(ER)+deterministic(covid)+trend,
                           mode=c(1,2,3,0)) \%>\%
   pssf() \%>\% summary()


\dontshow{\}) # examplesIf}
}
\references{
Pesaran, M. H., Shin, Y. and Smith, R. (2001), "Bounds Testing Approaches to the Analysis of Level Relationship", Journal of Applied Econometrics, 16(3), 289-326.
}
\seealso{
\code{\link{psst}}  \code{\link{ecm}}  \code{\link{narayan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{kardl_set}
\alias{kardl_set}
\title{Set kardl Package Options}
\usage{
kardl_set(...)
}
\arguments{
\item{...}{Named arguments corresponding to the options to be set. Valid option names include those defined in the kardl package settings.}
}
\value{
All current settings as a list after applying any updates from the provided named arguments invisibly.

If no arguments are provided, returns all options as a list. If named arguments are provided, sets those options and returns the updated list.
}
\description{
This function allows users to set options for the kardl package. Users can specify named arguments to set options or call the function without arguments to retrieve all current settings.
}
\examples{
# Get default options
kardl_get("maxlag", "mode")
# Set options
kardl_set(maxlag = 5, mode = "grid")
# Get specific options
kardl_get("maxlag", "mode")

# To have the updated settings available in the global environment, assign the output to a variable:
MySettings<- kardl_set(LongCoef = "LongRun_{varName}", ShortCoef = "ShortRun_{varName}")
# Now MySettings contains the updated settings, and the kardl package
# will use these settings for subsequent operations.
MySettings$LongCoef
MySettings$maxlag

# Reset to defaults after demonstrating custom settings
kardl_reset()

}
\seealso{
\code{\link{kardl_get}}, \code{\link{kardl_reset}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{imf_example_data}
\alias{imf_example_data}
\title{IMF Example Data}
\format{
A data frame with 470 rows and 4 variables:
\describe{
\item{ER}{Numeric. Exchange rate of Turkey.}
\item{CPI}{Numeric. CPI of Turkey.}
\item{PPI}{Numeric. PPI of Turkey.}
\item{covid}{Integer.Covid19 dummy variable.}
}
}
\usage{
imf_example_data
}
\description{
This is an example data set used for testing purposes. It contains monthly data on exchange rates, consumer price index, producer price index, and a dummy variable for the COVID-19 pandemic for Turkey from January 1985 to February 2024.
}
\details{
These data obtained from \pkg{imf.data} package.
The sample data is not updated and obtained by following codes:

\code{install.packages("imf.data")}\cr
\code{library("imf.data")}\cr
\code{IFS <- load_datasets("IFS")}

\itemize{
\item \strong{PCPI_IX}             Prices, Consumer Price Index, All items, Index\cr
\item \strong{AIP_IX}              Economic Activity, Industrial Production, Index\cr
\item \strong{ENDE_XDC_USD_RATE}   Exchange Rates, Domestic Currency per U.S. Dollar, End of Period, Rate}

\code{  trdata<-IFS$get_series(freq = "M", ref_area = "TR", indicator = c("PCPI_IX","AIP_IX","ENDE_XDC_USD_RATE"),start_period = "1985-01",end_period = "2024-02")}\cr
\code{  PeriodRow<-trdata[,1]}\cr
\code{  trdata[,1]<-NULL}\cr
\code{  colnames(trdata)<-c("ER","CPI","PPI")}\cr
\code{  trdata<-log(as.data.frame(lapply(trdata, function(x) as.numeric(x))))}\cr
\code{  rownames(trdata)<-PeriodRow}

\strong{Inserting covid dummy variable}

\code{  trdata<-cbind(trdata,covid=0)}\cr
\code{  trdata[420:470,4]<-1}
}
\examples{
data(imf_example_data)
head(imf_example_data)
}
\seealso{
\code{\link[imf.data]{load_datasets}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kardl.R
\name{kardl}
\alias{kardl}
\title{Estimate ARDL and NARDL Models with Automatic Lag Selection}
\usage{
kardl(
  data = NULL,
  formula = NULL,
  maxlag = NULL,
  mode = NULL,
  criterion = NULL,
  differentAsymLag = NULL,
  batch = NULL,
  ...
)
}
\arguments{
\item{data}{The data of analysis}

\item{formula}{A formula specifying the long-run model equation. This formula defines the relationships
between the dependent variable and explanatory variables, including options for deterministic terms,
asymmetric variables, and a trend component.

\if{html}{\out{<div class="sourceCode">}}\preformatted{   Example formula:
   \code{y ~ x + z + Asymmetric(z) + Lasymmetric(x2 + x3) + Sasymmetric(x3 + x4) + deterministic(dummy1 + dummy2) + trend}
}\if{html}{\out{</div>}}

\strong{\emph{Details}}

The formula allows flexible specification of variables and their roles:

\itemize{
\item \strong{Deterministic variables:}
Deterministic regressors (e.g., dummy variables) can be included using
\code{deterministic()}. Multiple deterministic variables may be supplied
using \code{+}, for example \code{deterministic(dummy1 + dummy2)}.
These variables are treated as fixed components and are not associated
with short-run or long-run dynamics.
\item \strong{Asymmetric variables:}
Asymmetric decompositions can be specified for short-run and/or
long-run dynamics:
\itemize{
\item \strong{Sasymmetric}: Specifies short-run asymmetric variables.
For example, \code{Sasymmetric(x1 + x2)} applies short-run
asymmetric decomposition to \code{x1} and \code{x2}.
\item \strong{Lasymmetric}: Specifies long-run asymmetric variables.
For example, \code{Lasymmetric(x1 + x3)} applies long-run
asymmetric decomposition to \code{x1} and \code{x3}.
\item \strong{Asymmetric}: Specifies variables that enter both
short-run and long-run asymmetric decompositions.
For example, \code{Asymmetric(x1 + x3)} applies asymmetric
decomposition in both dynamics.
}
}

A \strong{trend} term may be included to capture deterministic linear
time trends by simply adding \code{trend} to the formula.

The formula also supports the use of \code{.} to represent all available
regressors in the supplied data (excluding the dependent variable),
following standard R formula conventions.

All of the operators \code{Deterministic()}, \code{Sasymmetric()},
\code{Lasymmetric()}, and \code{Asymmetric()} follow the same usage
rules:

\itemize{
\item They can be freely combined within a single formula, for example:
\preformatted{
  y ~ . +
    Asymmetric(z) +
    Lasymmetric(x2 + x3) +
    Sasymmetric(x3 + x4) +
    deterministic(dummy1 + dummy2) +
    trend
  }

\item They must not be nested within one another. Valid usage:
\code{y ~ x + deterministic(dummy) + Asymmetric(z)}.
Invalid usage (to be avoided):
\code{y ~ x + deterministic(Asymmetric(z))} or
\code{y ~ x + Asymmetric(deterministic(dummy))}.

\item Where applicable, arguments are validated internally using
\code{match.arg()}. Consequently, abbreviated inputs are accepted
provided they uniquely identify a valid option. For example, if
\code{"asymmetric"} is an admissible value, specifying \code{"a"}
is sufficient. For clarity and reproducibility, however, full
argument names are recommended.
}

These components may therefore be combined flexibly to construct
a specification tailored to the empirical analysis.}

\item{maxlag}{An integer specifying the maximum number of lags to be considered for the model.
The default value is \code{4}. This parameter sets an upper limit on the lag length during
the model estimation process.

\strong{\emph{details}}

The \code{maxlag} parameter is crucial for defining the maximum lag length that the model
will evaluate when selecting the optimal lag structure based on the specified \code{criterion}.
It controls the computational effort and helps prevent overfitting by restricting the search
space for lag selection.
\itemize{
\item  If the data has a short time horizon or is prone to overfitting, consider reducing \code{maxlag}.
\item  If the data is expected to have long-term dependencies, increasing \code{maxlag} may be necessary to capture the relevant dynamics.
}

Setting an appropriate value for \code{maxlag} depends on the nature of your dataset and the
context of the analysis:
\itemize{
\item For small datasets or quick tests, use smaller values (e.g., \code{maxlag = 2}).
\item For datasets with more observations or longer-term patterns, larger values (e.g., \code{maxlag = 8})  may be appropriate, though this increases computational time.
}

\strong{\emph{examples}}

Using the default maximum lag (4)

\code{kardl(data, MyFormula, maxlag = 4)}

Reducing the maximum lag to 2 for faster computation

\code{kardl(data, MyFormula, maxlag = 2)}

Increasing the maximum lag to 8 for datasets with longer dependencies

\code{kardl(data, MyFormula, maxlag = 8)}}

\item{mode}{Specifies the mode of estimation and output control. This parameter determines how
the function handles lag estimation and what kind of feedback or control is provided during
the process. The available options are:

\itemize{
\item \strong{"quick"} (default):
Displays progress and messages in the console while the function estimates the optimal lag values.
This mode is suitable for interactive use or for users who want to monitor the estimation process
in real-time. It provides detailed feedback for debugging or observation but may use additional
resources due to verbose output.

\item \strong{"grid"} :
Displays progress and messages in the console while the function estimates the optimal lag values.
This mode is suitable for interactive use or for users who want to monitor the estimation process
in real-time. It provides detailed feedback for debugging or observation but may use additional
resources due to verbose output.

\item \strong{"grid_custom"}:
Suppresses most or all console output, prioritizing faster execution and reduced resource usage
on PCs or servers. This mode is recommended for high-performance scenarios, batch processing,
or when the estimation process does not require user monitoring. Suitable for large-scale or
repeated runs where output is unnecessary.

\item \strong{User-defined vector}:
A numeric vector of lag values specified by the user, allowing full customization of the lag
structure used in model estimation. When a user-defined vector is provided (e.g., \code{c(1, 2, 4, 5)}),
the function skips the lag optimization process and directly uses the specified lags.

Users can define lag values directly as a numeric vector. For example:
\code{mode = c(1, 2, 4, 5)} assigns lags of 1, 2, 4, and 5 to variables in the specified order.
Alternatively, lag values can be assigned to variables by name for clarity and control. For example:
\code{mode = c(CPI = 2, ER_POS = 3, ER_NEG = 1, PPI = 3)} assigns lags to variables explicitly.
Ensure that the lags are correctly designated by verifying the result using
\code{kardl_model$properLag} after estimation.

\strong{\emph{Attention!}}
A function-based criterion or user-defined function can be specified
for model selection, but this is only supported for \code{mode = "grid_custom"}
and \code{mode = "quick"}. The \code{mode = "grid"} option is restricted to
predefined criteria (e.g., AIC or BIC). For more information on available criteria,
see the \code{\link{modelCriterion}} function documentation.
\itemize{
\item When using a numeric vector, ensure the order of lag values matches the variables in your formula.
\item If using named vectors, double-check the variable names to avoid mismatches or unintended results.
\item This mode bypasses the automatic lag optimization and assumes the user-defined lags are correct.
}
}}

\item{criterion}{A string specifying the information criterion to be used for selecting the optimal lag structure.
The available options are:
\itemize{
\item \strong{"AIC"}: Akaike Information Criterion (default). This criterion balances model fit and complexity,
favoring models that explain the data well with fewer parameters.
\item \strong{"BIC"}: Bayesian Information Criterion. This criterion imposes a stronger penalty for model complexity
than AIC, making it more conservative in selecting models with fewer parameters.
\item \strong{"AICc"}: Corrected Akaike Information Criterion. This is an adjusted version of AIC that accounts for small sample sizes,
making it more suitable when the number of observations is limited relative to the number of parameters.
\item \strong{"HQ"}: Hannan-Quinn Information Criterion. This criterion provides a compromise between AIC and BIC,
favoring models that balance fit and complexity without being overly conservative.
}
The criterion can be specified as a string (e.g., \code{"AIC"}) or as a user-defined function that takes a fitted model object.
Please visit the \code{\link{modelCriterion}} function documentation for more details on using custom criteria.}

\item{differentAsymLag}{A logical value indicating whether to allow different lag lengths for positive and negative decompositions.}

\item{batch}{A string specifying the batch processing configuration in the format "current_batch/total_batches".
If a user utilize grid or grid_custom mode and want to split the lag search into multiple batches, this parameter can be used to define the current batch and the total number of batches.
For example, "2/5" indicates that the current batch is the second out of a total of five batches.
The default value is "1/1", meaning that the entire lag search is performed in a single batch.}

\item{...}{Additional arguments that can be passed to the function. These arguments can be used to specify other settings or parameters that are not explicitly defined in the main arguments.}
}
\value{
An object of class \code{kardl_lm} containing the estimated ARDL or NARDL model.
The object includes the following components:
\describe{
\item{argsInfo}{A list of input arguments used for the estimation. It includes the data, formula, maxlag, mode, criterion, differentAsymLag, and batch settings.}
\item{extractedInfo}{A list containing extracted information from the input data and formula, such as variable names, deterministic terms, asymmetric variables, and the prepared dataset for estimation.}
\item{timeInfo}{A list containing timing information for the estimation process, including start time, end time, and total duration.}
\item{lagInfo}{A list containing lag selection information, including the optimal lag configuration and criteria values for different lag combinations.}
\item{estInfo}{A list containing estimation details, such as the type of model, estimation method, model formula, number of parameters (k), number of observations (n), start and end points of the fitted values, and total time span.}
\item{model}{The fitted linear model object of class \code{lm} representing the estimated ARDL or NARDL model.}
}
}
\description{
This function estimates an Autoregressive Distributed Lag (ARDL) or Nonlinear ARDL (NARDL) model based on the provided data and model formula.
It allows for flexible specification of variables, including deterministic terms, asymmetric variables, and trend components.
The function also supports automatic lag selection using various information criteria.
}
\details{
The general formula for the long-run model is specified as follows:
\deqn{
\begin{aligned}
 \Delta {y}_t =  c  +  \eta _0   {y}_{t-1} + \sum_{j=1}^{p} { \gamma_{j}  \Delta {y}_{t-j} }   +\sum_{i=1}^{m} {( \eta ^{+}_i   {x}^{+}_{i,t-1 } + \eta ^{-}_i   {x}^{-}_{i,t-1 } ) }    +  \sum_{i=m+1}^{k} {\eta _i   {x}_{i,t-1 } }
+  \newline  \sum_{i=1}^{m} {\sum_{j=0}^{q^+_i} { \beta^+_{ij}   \Delta {x}^+_{i,t-j} } } + \sum_{i=1}^{m} {\sum_{j=0}^{q^-_i} { \beta^-_{ij}   \Delta {x}^-_{i,t-j} } }  + \sum_{i=m+1}^{k} {\sum_{j=0}^{q_i} { \beta_{ij}   \Delta {x}_{i,t-j} } }+ e_t
\end{aligned}
 }

Where:
\itemize{
\item \eqn{y_t} is the dependent variable at time t.
\item \eqn{c} is the constant term.
\item \eqn{\eta_0} is the coefficient of the lagged dependent variable.
\item \eqn{\gamma_j} are the coefficients of the lagged differences of the dependent variable.
\item \eqn{\eta^+_i} and \eqn{\eta^-_i} are the coefficients of the positive and negative decompositions of the independent variables, respectively.
\item \eqn{\eta_i} are the coefficients of the independent variables that do not have asymmetric decompositions.
\item \eqn{\beta^+_{ij}} and \eqn{\beta^-_{ij}} are the coefficients of the lagged differences of the positive and negative decompositions of the independent variables, respectively.
\item \eqn{\beta_{ij}} are the coefficients of the lagged differences of the independent variables that do not have asymmetric decompositions.
\item \eqn{e_t} is the error term at time t.
\item \eqn{p} is the maximum lag length for the dependent variable.
\item \eqn{q^+_i} and \eqn{q^-_i} are the maximum lag lengths for the positive and negative decompositions of the independent variables, respectively.
\item \eqn{q_i} is the maximum lag length for the independent variables that do not have asymmetric decompositions.
\item \eqn{m} is the number of independent variables with asymmetric decompositions.
\item \eqn{k} is the total number of independent variables.
\item \eqn{\Delta} denotes the first difference operator.
\item \eqn{x^+_{i,t}} and \eqn{x^-_{i,t}} represent the positive and negative decompositions of the independent variable \eqn{x_i} at time t, respectively.
}
}
\section{Notation of reported coefficients}{


In the reported coefficients, the prefix \code{L} denotes lagged variables,
where the accompanying number indicates the lag order, and \code{.d.} denotes
first differences. Accordingly, \code{L1.PPI} represents the first lag of the
level of \code{PPI} (long-run component), while \code{L3.d.PPI} denotes the
third lag of the first-differenced \code{PPI} (short-run component).

In addition, the suffixes \code{_POS} and \code{_NEG} indicate the positive
and negative partial sum components of a variable, respectively. This notation
is used by default and remains valid unless modified through the
\code{kardl_set()} function.
}

\examples{


# Sample article: THE DYNAMICS OF EXCHANGE RATE PASS-THROUGH TO DOMESTIC PRICES IN TURKEY

kardl_set(formula =CPI~ER+PPI+Asymmetr(ER)+deterministic(covid)+trend ,
          data=imf_example_data,
          maxlag=2
) # setting the default values of the kardl function


# using the grid_custom mode with batch processing

kardl_model_grid<-kardl( mode = "grid_custom",batch = "2/3",criterion = "BIC")
kardl_model_grid

kardl_model2<-kardl(mode = c( 2    ,  1    ,  1   ,   3 ))

# Getting the results
kardl_model2

# Getting the summary of the results
summary(kardl_model2)

# using '.' in the formula means that all variables in the data will be used

fit_bic <- kardl(formula=CPI~.+deterministic(covid))
fit_bic

# Setting max lag instead of default value [4]
kardl(imf_example_data,
      CPI~ER+PPI+Lasymmetric(ER),
      maxlag = 3, mode = "grid_custom")

# Using another criterion for finding the best lag
kardl_set(criterion = "HQ") # setting the criterion to HQ
kardl( mode = "grid_custom")

# using default values of lags
kardl( mode=c(1,2,3,0))

# For using different lag values for negative and positive decompositions of non-linear variables
# setting the same lags for positive and negative decompositions.

same<-kardl(formula=CPI~Asymmetric(ER),maxlag=2, mode = "grid_custom",differentAsymLag = FALSE)
dif<-kardl(formula=CPI~Sasymmetric(ER),maxlag=2, mode = "grid_custom",differentAsymLag = TRUE)

same$lagInfo$OptLag
dif$lagInfo$OptLag

# Optional: use magrittr if available
\dontshow{if (requireNamespace("magrittr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
library(magrittr)
  kardl_model_pipe <-  imf_example_data \%>\%
    kardl(mode = "grid_custom")

  kardl_model_pipe
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{ecm}}, \code{\link{kardl_set}}, \code{\link{kardl_get}}, \code{\link{kardl_reset}}, \code{\link{modelCriterion}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multipliers.R
\name{mplier}
\alias{mplier}
\title{Compute Dynamic Multipliers for kardl Models}
\usage{
mplier(kmodel, horizon = 80, minProb = 0)
}
\arguments{
\item{kmodel}{An object of class \code{kardl_lm} produced by the \code{kardl} function.}

\item{horizon}{Integer. Number of periods ahead for which dynamic multipliers are computed.}

\item{minProb}{Numeric. Minimum p-value threshold for including coefficients in the calculation. Coefficients with p-values above this threshold will be set to zero. Default is \code{0} (no threshold).
This parameter allows users to control the inclusion of coefficients in the calculation based on their statistical significance. Setting a threshold can help focus the analysis on more relevant variables, but it may also exclude potentially important effects if set too stringently.}
}
\value{
A list of class \code{kardl_mplier} containing:
\itemize{
\item \strong{mpsi}: Matrix of cumulative dynamic multipliers.
\item \strong{omega}: Vector of omega coefficients (persistence structure).
\item \strong{lambda}: Matrix of short-run dynamic coefficients.
\item \strong{horizon}: Forecast horizon used.
\item \strong{vars}: Extracted model variable structure.
}
}
\description{
Computes cumulative dynamic multipliers based on a model estimated using the
\code{kardl} framework. The function supports different configurations of
linearity and asymmetry in both short-run and long-run dynamics.
}
\details{
The asymmetry structure is determined internally:
\itemize{
\item Variables in \code{extractedInfo$ASvars} are treated as asymmetric in the short run.
\item Variables in \code{extractedInfo$ALvars} are treated as asymmetric in the long run.
}

This allows four possible configurations:
\itemize{
\item \strong{LL}: Linear in both short-run and long-run
\item \strong{NN}: Asymmetric in both short-run and long-run
\item \strong{SA}: Short-run linear, long-run asymmetric
\item \strong{AS}: Short-run asymmetric, long-run linear
}

When a component is linear, the same coefficient path is used for both positive
and negative changes. When asymmetric, separate positive and negative effects
are computed.

The \code{mplier} function computes dynamic multipliers based on the coefficients and lag structure of a model estimated using the \code{kardl} package. The function extracts necessary
information from the model, such as coefficients, lag structure, and variable names, to compute the dynamic multipliers. It calculates the short-run coefficients, Lambda values, and omega values based on the model's parameters and lag structure. The output includes a matrix of dynamic multipliers (mpsi), which can be used for further analysis or visualization.
The dynamic multipliers provide insight into how changes in the independent variables affect the dependent variable over time, allowing for a deeper understanding of the relationships captured by the model. The function also allows users to set a minimum p-value threshold for including coefficients in the calculation, providing flexibility in focusing on statistically significant effects.

The function constructs dynamic multipliers based on the recursive relationship:

\deqn{
\psi_{h}^{+} = \sum_{i=0}^{h} \frac{\partial y_{t+i}}{\partial x_{t}^{+}}, \quad
\psi_{h}^{-} = \sum_{i=0}^{h} \frac{\partial y_{t+i}}{\partial x_{t}^{-}}
}

where \eqn{\psi_h^{+}} and \eqn{\psi_h^{-}} represent cumulative responses to
positive and negative shocks.

The recursion is defined as:

\deqn{
\psi_h = \lambda_h + \sum_{j=1}^{p} \omega_j \psi_{h-j}
}

where \eqn{\lambda_h} captures short-run effects and \eqn{\omega_j} reflects
persistence through lagged dependent variables.

When asymmetry is present, positive and negative shocks are propagated separately.
Otherwise, the same dynamic path is used.
}
\examples{

# This example demonstrates how to use the mplier function to calculate dynamic multipliers
# from a model estimated using the kardl package. The example includes fitting a model with
# the kardl function, calculating the multipliers, and visualizing the results using both
# base R plotting and ggplot2.

 # Calculating dynamic multipliers for a linear model in short and long run (NN)

 kardl_model<-kardl(imf_example_data, CPI~ER )
 m<-mplier(kardl_model,40)
 head(m$mpsi)
 plot(m)

 # Calculating dynamic multipliers for a model with
 # Short-run linear, long-run asymmetric (SA)
 kardl_model<-kardl(imf_example_data, CPI~lasym(ER) )
 m<-mplier(kardl_model,40)
 head(m$mpsi)
 plot(m)

 # Calculating dynamic multipliers for a model with
 # Short-run asymmetric, long-run linear (AS)
 kardl_model<-kardl(imf_example_data, CPI~sasym(ER) )
 m<-mplier(kardl_model,40)
 head(m$mpsi)
 plot(m)

 # Calculating dynamic multipliers for a model with
 # asymmetric effects in both short and long run (NN)
 kardl_model<-kardl(imf_example_data, CPI~asym(ER) )
 m<-mplier(kardl_model,40)
 head(m$mpsi)
 plot(m)

 # The mpsi matrix contains the cumulative dynamic multipliers for each variable and time horizon.
 # The omega vector contains the persistence structure of the model,
 # while the lambda matrix contains the short-run dynamic coefficients.
 # You can inspect these components to understand the dynamics captured by the model.
 kardl_model<-kardl(imf_example_data, CPI~PPI+asym(ER) )
 m<-mplier(kardl_model,40)
 head(m$mpsi)
 head(m$omega)
 head(m$lambda)

 # For plotting specific variables, you can specify them in the plot function. For example,
 # to plot the multipliers for the variable "PPI":

 plot(m, variable = "PPI")


}
\seealso{
\code{\link{bootstrap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kardl.R
\name{ecm}
\alias{ecm}
\title{Perform the Error Correction Model (ECM) test to assess cointegration in the model}
\usage{
ecm(
  data = NULL,
  formula = NULL,
  maxlag = NULL,
  mode = NULL,
  criterion = NULL,
  differentAsymLag = NULL,
  batch = NULL,
  ...
)
}
\arguments{
\item{data}{The data of analysis}

\item{formula}{A formula specifying the long-run model equation. This formula defines the relationships
       between the dependent variable and explanatory variables, including options for deterministic terms,
       asymmetric variables, and a trend component.

       Example formula:
       \code{y ~ x + z + Asymmetric(z) + Lasymmetric(x2 + x3) + Sasymmetric(x3 + x4) + deterministic(dummy1 + dummy2) + trend}



\strong{\emph{Details}}

The formula allows flexible specification of variables and their roles:

\itemize{
\item \strong{Deterministic variables:}
Deterministic regressors (e.g., dummy variables) can be included using
\code{deterministic()}. Multiple deterministic variables may be supplied
using \code{+}, for example \code{deterministic(dummy1 + dummy2)}.
These variables are treated as fixed components and are not associated
with short-run or long-run dynamics.
\item \strong{Asymmetric variables:}
Asymmetric decompositions can be specified for short-run and/or
long-run dynamics:
\itemize{
\item \strong{Sasymmetric}: Specifies short-run asymmetric variables.
For example, \code{Sasymmetric(x1 + x2)} applies short-run
asymmetric decomposition to \code{x1} and \code{x2}.
\item \strong{Lasymmetric}: Specifies long-run asymmetric variables.
For example, \code{Lasymmetric(x1 + x3)} applies long-run
asymmetric decomposition to \code{x1} and \code{x3}.
\item \strong{Asymmetric}: Specifies variables that enter both
short-run and long-run asymmetric decompositions.
For example, \code{Asymmetric(x1 + x3)} applies asymmetric
decomposition in both dynamics.
}
}

A \strong{trend} term may be included to capture deterministic linear
time trends by simply adding \code{trend} to the formula.

The formula also supports the use of \code{.} to represent all available
regressors in the supplied data (excluding the dependent variable),
following standard R formula conventions.

All of the operators \code{Deterministic()}, \code{Sasymmetric()},
\code{Lasymmetric()}, and \code{Asymmetric()} follow the same usage
rules:

\itemize{
\item They can be freely combined within a single formula, for example:
  \preformatted{
  y ~ . +
    Asymmetric(z) +
    Lasymmetric(x2 + x3) +
    Sasymmetric(x3 + x4) +
    deterministic(dummy1 + dummy2) +
    trend
  }

\item They must not be nested within one another. Valid usage:
  \code{y ~ x + deterministic(dummy) + Asymmetric(z)}.
  Invalid usage (to be avoided):
  \code{y ~ x + deterministic(Asymmetric(z))} or
  \code{y ~ x + Asymmetric(deterministic(dummy))}.

\item Where applicable, arguments are validated internally using
  \code{match.arg()}. Consequently, abbreviated inputs are accepted
  provided they uniquely identify a valid option. For example, if
  \code{"asymmetric"} is an admissible value, specifying \code{"a"}
  is sufficient. For clarity and reproducibility, however, full
  argument names are recommended.
}

These components may therefore be combined flexibly to construct
a specification tailored to the empirical analysis.}

\item{maxlag}{An integer specifying the maximum number of lags to be considered for the model.
       The default value is \code{4}. This parameter sets an upper limit on the lag length during
       the model estimation process.


\strong{\emph{details}}


       The \code{maxlag} parameter is crucial for defining the maximum lag length that the model
       will evaluate when selecting the optimal lag structure based on the specified \code{criterion}.
       It controls the computational effort and helps prevent overfitting by restricting the search
       space for lag selection.
\itemize{
\item  If the data has a short time horizon or is prone to overfitting, consider reducing \code{maxlag}.
\item  If the data is expected to have long-term dependencies, increasing \code{maxlag} may be necessary to capture the relevant dynamics.
}

       Setting an appropriate value for \code{maxlag} depends on the nature of your dataset and the
       context of the analysis:
\itemize{
\item For small datasets or quick tests, use smaller values (e.g., \code{maxlag = 2}).
\item For datasets with more observations or longer-term patterns, larger values (e.g., \code{maxlag = 8})  may be appropriate, though this increases computational time.
}


\strong{\emph{examples}}


Using the default maximum lag (4)

\code{kardl(data, MyFormula, maxlag = 4)}

Reducing the maximum lag to 2 for faster computation

\code{kardl(data, MyFormula, maxlag = 2)}

Increasing the maximum lag to 8 for datasets with longer dependencies

\code{kardl(data, MyFormula, maxlag = 8)}}

\item{mode}{Specifies the mode of estimation and output control. This parameter determines how
       the function handles lag estimation and what kind of feedback or control is provided during
       the process. The available options are:

\itemize{
\item \strong{"quick"} (default):
        Displays progress and messages in the console while the function estimates the optimal lag values.
        This mode is suitable for interactive use or for users who want to monitor the estimation process
        in real-time. It provides detailed feedback for debugging or observation but may use additional
        resources due to verbose output.

\item \strong{"grid"} :
        Displays progress and messages in the console while the function estimates the optimal lag values.
        This mode is suitable for interactive use or for users who want to monitor the estimation process
        in real-time. It provides detailed feedback for debugging or observation but may use additional
        resources due to verbose output.

\item \strong{"grid_custom"}:
        Suppresses most or all console output, prioritizing faster execution and reduced resource usage
        on PCs or servers. This mode is recommended for high-performance scenarios, batch processing,
        or when the estimation process does not require user monitoring. Suitable for large-scale or
        repeated runs where output is unnecessary.

\item \strong{User-defined vector}:
        A numeric vector of lag values specified by the user, allowing full customization of the lag
        structure used in model estimation. When a user-defined vector is provided (e.g., `c(1, 2, 4, 5)`),
        the function skips the lag optimization process and directly uses the specified lags.

        - Users can define lag values directly as a numeric vector. For example:
          \code{mode = c(1, 2, 4, 5)} assigns lags of 1, 2, 4, and 5 to variables in the specified order.
        - Alternatively, lag values can be assigned to variables by name for clarity and control. For example:
          \code{mode = c(CPI = 2, ER_POS = 3, ER_NEG = 1, PPI = 3)} assigns lags to variables explicitly.
        - Ensure that the lags are correctly designated by verifying the result using
          \code{kardl_model$properLag} after estimation.

        \strong{\emph{Attention!}}
        -A function-based criterion or user-defined function can be specified
         for model selection, but this is only supported for \code{mode = "grid_custom"}
         and \code{mode = "quick"}. The \code{mode = "grid"} option is restricted to
         predefined criteria (e.g., AIC or BIC). For more information on available criteria,
         see the \code{\link{modelCriterion}} function documentation.
        - When using a numeric vector, ensure the order of lag values matches the variables in your formula.
        - If using named vectors, double-check the variable names to avoid mismatches or unintended results.
        - This mode bypasses the automatic lag optimization and assumes the user-defined lags are correct.
}


       The `mode` parameter provides flexibility for different use cases:
       - Use `"grid"` mode for debugging or interactive use where progress visibility is important.
       - Use `"grid_custom"` mode to minimize overhead in computationally intensive tasks.
       - Specify a user-defined vector to customize the lag structure based on prior knowledge or analysis.

       Selecting the appropriate mode can improve the efficiency and usability of the function depending
       on the user's requirements and the computational environment.}

\item{criterion}{A string specifying the information criterion to be used for selecting the optimal lag structure.
      The available options are:
\itemize{
\item \strong{"AIC"}: Akaike Information Criterion (default). This criterion balances model fit and complexity,
      favoring models that explain the data well with fewer parameters.
\item \strong{"BIC"}: Bayesian Information Criterion. This criterion imposes a stronger penalty for model complexity
      than AIC, making it more conservative in selecting models with fewer parameters.
\item \strong{"AICc"}: Corrected Akaike Information Criterion. This is an adjusted version of AIC that accounts for small sample sizes,
      making it more suitable when the number of observations is limited relative to the number of parameters.
\item \strong{"HQ"}: Hannan-Quinn Information Criterion. This criterion provides a compromise between AIC and BIC,
      favoring models that balance fit and complexity without being overly conservative.
}
      The criterion can be specified as a string (e.g., \code{"AIC"}) or as a user-defined function that takes a fitted model object.
      Please visit the \code{\link{modelCriterion}} function documentation for more details on using custom criteria.}

\item{differentAsymLag}{A logical value indicating whether to allow different lag lengths for positive and negative decompositions.}

\item{batch}{A string specifying the batch processing configuration in the format "current_batch/total_batches".
If a user utilize grid or grid_custom mode and want to split the lag search into multiple batches, this parameter can be used to define the current batch and the total number of batches.
      For example, "2/5" indicates that the current batch is the second out of a total of five batches.
      The default value is "1/1", meaning that the entire lag search is performed in a single batch.}

\item{...}{Additional arguments that can be passed to the function. These arguments can be used to}
}
\value{
A list containing the results of the restricted ECM test, including:
\itemize{
\item \code{ecm}: The estimated ECM model objects including:
\itemize{
\item \code{case}: The case number used in the test (1, 2, 3, 4, or 5).
\item \code{EcmResLagged}: The lagged error correction term used in the ECM model.
\item \code{ecmL}: The estimated long-run model object.
\item \code{shortrunEQ}: The estimated short-run model equation object.
\item \code{longrunEQ}: The estimated long-run model equation object.
}
\item \strong{argsInfo}: A list of input arguments used for the estimation. It includes the data, formula, maxlag, mode, criterion, differentAsymLag, and batch settings.
\item \strong{extractedInfo}: A list containing extracted information from the input data and formula, such as variable names, deterministic terms, asymmetric variables, and the prepared dataset for estimation.
\item \strong{timeInfo}: A list containing timing information for the estimation process, including start time, end time, and total duration.
\item \strong{lagInfo}: A list containing lag selection information, including the optimal lag configuration and criteria values for different lag combinations.
\item \strong{estInfo}: A list containing estimation details, such as the type of model, estimation method, model formula, number of parameters (k), number of observations (n), start and end points of the fitted values, and total time span.
\item \strong{model}: The fitted linear model object of class \code{lm} representing the estimated ARDL or NARDL model.
}
}
\description{
This function is used to perform the Error Correction Model (ECM) test, which is designed to determine whether there is cointegration in the model. Cointegration indicates a long-term equilibrium relationship between variables, despite short-term deviations. The ECM test helps identify if such a long-term relationship exists by examining the short-run dynamics and adjusting for deviations from equilibrium. If the test confirms cointegration, it suggests that the variables move together over time, maintaining a stable long-term relationship. This is critical for ensuring that the model properly captures both short-term fluctuations and long-term equilibrium behavior.
}
\section{Hypothesis testing}{

The restricted ECM test, also known as the PSS t Bound test, is a statistical test used to assess the presence of cointegration in a model. Cointegration refers to a long-term equilibrium relationship between two or more time series variables. The PSS t Bound test is based on the work of Pesaran, Shin, and Smith (2001) and is particularly useful for models with small sample sizes.


The null and alternative hypotheses for the restricted ECM test are as follows:

  \deqn{\mathbf{H_{0}:} \theta  =   0}
  \deqn{\mathbf{H_{1}:} \theta  \neq 0}

  The null hypothesis (\eqn{H_{0}}) states that there is no cointegration in the model, meaning that the long-run relationship between the variables is not significant. The alternative hypothesis (\eqn{H_{1}}) suggests that there is cointegration, indicating a significant long-term relationship between the variables.

  The test statistic is calculated as the t-statistic of the coefficient of the error correction term (\eqn{\theta}) in the ECM model. If the absolute value of the t-statistic exceeds the critical value from the PSS t Bound table, we reject the null hypothesis in favor of the alternative hypothesis, indicating that cointegration is present.

  The cases for the restricted ECM Bound test are defined as follows:



\itemize{
\item \code{case 1}: No constant, no trend.

  This case is used when the model does not include a constant term or a trend term. It is suitable for models where the variables are stationary and do not exhibit any long-term trends.

  The model is specified as follows:

  \deqn{
\begin{aligned}
\Delta y_t =  \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1}  - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
\end{aligned}
}

\item \code{case 2}: Restricted constant, no trend.

  This case is used when the model includes a constant term but no trend term. It is suitable for models where the variables exhibit a long-term relationship but do not have a trend component.
  The model is specified as follows:
  \deqn{
  \begin{aligned}
  \Delta y_t &= \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1} - \alpha_0 - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
  \end{aligned}
  }

\item \code{case 3}: Unrestricted constant, no trend.

  This case is used when the model includes an unrestricted constant term but no trend term. It is suitable for models where the variables exhibit a long-term relationship with a constant but do not have a trend component.

  The model is specified as follows:

  \deqn{
  \begin{aligned}
  \Delta y_t &= \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1} - \alpha_0 - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
  \end{aligned}
    }
\item \code{case 4}: Unrestricted Constant, restricted trend.

  This case is used when the model includes an unrestricted constant term and a restricted trend term. It is suitable for models where the variables exhibit a long-term relationship with a constant and a trend component.

  The model is specified as follows:

  \deqn{
  \begin{aligned}
  \Delta y_t &= \phi + \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1} - \pi (t-1) - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
  \end{aligned}
   }
\item \code{case 5}: Unrestricted constant, unrestricted trend.
}


 The Error Correction Model (ECM) is specified as follows:
\deqn{
\begin{aligned}
\Delta y_t &= \phi + \varphi t +  \sum_{j=1}^{p} \gamma_j \Delta y_{t-j} + \sum_{i=1}^{k} \sum_{j=0}^{q_i} \beta_{ij} \Delta x_{i,t-j} + \theta (y_{t-1}  - \sum_{i=1}^{k} \alpha_i x_{i,t-1} ) + e_t
\end{aligned}
}
}

\examples{

suppressPackageStartupMessages(library(dplyr))
suppressPackageStartupMessages(library(tidyr))
suppressPackageStartupMessages(library(ggplot2))


 # Sample article: THE DYNAMICS OF EXCHANGE RATE PASS-THROUGH TO DOMESTIC PRICES IN TURKEY
 kardl_set(formula=CPI~ER+PPI+asym(ER)+deterministic(covid)+trend ,
           data=imf_example_data ,
           maxlag=3)

 ecm_model_grid<- ecm(mode = "grid")
 ecm_model_grid

 # Checking the cointegration test results using pesaran t test
 psst(ecm_model_grid)
 # Getting the details of psst result
 summary(psst(ecm_model_grid))

 # using the grid_custom mode for faster execution without console output
 ecm_model<- imf_example_data \%>\% ecm(mode = "grid_custom")
 ecm_model

 # Estimating the model with user-defined lag values
 ecm_model2<-ecm(mode = c( 2    ,  1    ,  1   ,   3 ))
 # Getting the results
 ecm_model2
 # Getting the summary of the results
 summary(ecm_model2)
 # OR using pipe operator
 imf_example_data \%>\% ecm(CPI~PPI+asym(ER) +trend,case=4) \%>\% summary()

 # For increasing the performance of finding the most fitted lag vector
 ecm(mode = "grid_custom")
 # Setting max lag instead of default value [4]
 ecm(maxlag = 2, mode = "grid_custom")
 # Using another criterion for finding the best lag
 ecm(criterion = "HQ", mode = "grid_custom")



 # For using different lag values for negative and positive decompositions of non-linear variables

 # setting the same lags for positive and negative decompositions.
 kardl_set(differentAsymLag = FALSE)

 diffAsymLags<-ecm( mode = "grid_custom")
 diffAsymLags$lagInfo$OptLag

 # setting the different lags for positive and negative decompositions
 sameAsymLags<-ecm(differentAsymLag = TRUE , mode = "grid_custom" )
 sameAsymLags$lagInfo$OptLag


 # Setting the preffixes and suffixes for non-linear variables
 kardl_reset()
 kardl_set(AsymPrefix = c("asyP_","asyN_"), AsymSuffix = c("_PP","_NN"))
 customizedNames<-ecm(imf_example_data, CPI~ER+PPI+asym(ER) )
 customizedNames

 # For having the lags plot

 #  ecm_model_grid[["LagCriteria"]] is a matrix, convert it to a data frame
 LagCriteria <- as.data.frame(ecm_model_grid$lagInfo$LagCriteria)
 # Rename columns for easier access and convert relevant columns to numeric
 colnames(LagCriteria) <- c("lag", "AIC", "BIC", "AICc", "HQ")
 LagCriteria <- LagCriteria \%>\%  mutate(across(c(AIC, BIC, HQ), as.numeric))

 # Pivot the data to a long format excluding AICc

 LagCriteria_long <- LagCriteria \%>\%  select(-AICc) \%>\%
   pivot_longer(cols = c(AIC, BIC, HQ), names_to = "Criteria", values_to = "Value")
 # Find the minimum value for each criterion
 min_values <- LagCriteria_long \%>\%  group_by(Criteria) \%>\%
   slice_min(order_by = Value) \%>\%  ungroup()

 # Create the ggplot with lines, highlight minimum values, and add labels
 ggplot(LagCriteria_long, aes(x = lag, y = Value, color = Criteria, group = Criteria)) +
   geom_line() +
   geom_point(data = min_values, aes(x = lag, y = Value), color = "red", size = 3, shape = 8) +
   geom_text(data = min_values, aes(x = lag, y = Value, label = lag),
             vjust = 1.5, color = "black", size = 3.5) +
   labs(title = "Lag Criteria Comparison", x = "Lag Configuration",  y = "Criteria Value") +
   theme_minimal() +
   theme(axis.text.x = element_text(angle = 45, hjust = 1))


}
\seealso{
\code{\link{kardl}} \code{\link{pssf}}  \code{\link{psst}}   \code{\link{ecm}}  \code{\link{narayan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kardl.R
\name{modelCriterion}
\alias{modelCriterion}
\title{Model Selection Criteria}
\usage{
modelCriterion(estModel, cr, ...)
}
\arguments{
\item{estModel}{An object containing the fitted model. The object should include at least:
\itemize{
\item \code{estModel$model} - the actual fitted model object (e.g., from \code{lm}, \code{glm}).
\item \code{k} - the number of estimated parameters.
\item \code{n} - the sample size.
}}

\item{cr}{A character string specifying the criterion to compute.
Options are \code{"AIC"}, \code{"BIC"}, \code{"AICc"}, and \code{"HQ"}. Alternatively,
a user-defined function can be provided. See details below for more information on using custom criteria.}

\item{...}{Additional arguments passed to the user-defined criterion function if \code{cr} is a function.}
}
\value{
A numeric value representing the selected criterion, normalized by the sample size if one of the predefined options is used.
}
\description{
Computes a model selection criterion (AIC, BIC, AICc, or HQ) or applies a user-defined function
to evaluate a statistical model.
}
\details{
This function returns model selection criteria used to compare the quality of different models.
All criteria are defined such that \strong{lower values indicate better models} (i.e., the goal is minimization).

If you wish to compare models using a maximization approach (e.g., log-likelihood),
you can multiply the result by \code{-1}.

Note: The predefined string options (e.g., \code{"AIC"}) are \strong{not} the same as the built-in R functions \code{AIC()} or \code{BIC()}.
In particular, the values returned by this function are adjusted by dividing by the sample size \code{n}
(i.e., normalized AIC/BIC), which makes it more comparable across datasets of different sizes.

The function returns:
\itemize{
\item \strong{"AIC"}:  \eqn{ \frac{2k - 2\ell}{n} } Akaike Information Criterion divided by \code{n}.
\item \strong{"BIC"}:  \eqn{ \frac{\log(n) \cdot k - 2\ell}{n} } Bayesian Information Criterion divided by \code{n}.
\item \strong{"AICc"}: \eqn{ \frac{2k(k+1)}{n - k - 1} + \frac{2k - 2\ell}{n} } Corrected Akaike Information Criterion divided by \code{n}.
\item \strong{"HQ"}: \eqn{ \frac{2 \log(\log(n)) \cdot k - 2\ell}{n} } Hannan-Quinn Criterion divided by \code{n}.
}

where:
\itemize{
\item \eqn{k} is the number of parameters,
\item \eqn{n} is the sample size,
\item \eqn{\ell} is the log-likelihood of the model.
}

If \code{cr} is a function, it is called with the fitted model and any additional arguments passed through \code{...}.
}
\examples{

# Example usage of modelCriterion function with a simple linear model
mylm<- lm(mpg ~ wt + hp, data = mtcars)
modelCriterion(mylm, AIC )
modelCriterion(mylm, "AIC" )

 # Example usage of modelCriterion function with a kardl model
 kardl_model <- kardl(imf_example_data,
                      CPI ~ ER + PPI + asym(ER) + deterministic(covid) + trend,
                      mode = c(1, 2, 3, 0))

 # Using AIC as the kardl package's built-in criterion function which is different
 # from the base R AIC function.
 modelCriterion(kardl_model, "AIC")

 # Using the base R AIC function directly on the fitted model object
 modelCriterion(kardl_model, AIC)
 # Using the base R AIC function outside of modelCriterion to compute AIC for the fitted model
 AIC(kardl_model)

 # Using BIC as the criterion for the kardl model which is different from the base R BIC function.
 modelCriterion(kardl_model, "BIC")

 # Using a custom criterion function that divides AIC by the sample size
 my_cr_fun <- function(mod, ...) { AIC(mod) / length(mod$model[[1]]) }
 modelCriterion(kardl_model, my_cr_fun)

}
\seealso{
\code{\link{kardl}}
}
